@@ -3,6 +3,9 @@
  * $Date: 2005-11-10 09:52:44 -0600 (Thu, 10 Nov 2005) $
  * $Revision: 4255 $
  *
+ * Some portions of this file have been modified by Robert Hanson hansonr.at.stolaf.edu 2012-2017
+ * for use in SwingJS via transpilation into JavaScript using Java2Script.
+ *
  * Copyright (C) 2003-2005  The Jmol Development Team
  *
  * Contact: jmol-developers@lists.sf.net
@@ -24,7 +27,6 @@
 package javajs.util;
 
 import javajs.api.EigenInterface;
-
 import javajs.api.Interface;
 
 final public class Measure {
@@ -163,6 +165,10 @@ final public class Measure {
                                               T3 pointB,
                                               T3 pointC, V3 vNorm,
                                               V3 vAB, P4 plane) {
+    if (vNorm == null)
+      vNorm = new V3();
+    if (vAB == null)
+      vAB = new V3();
     float w = getNormalThroughPoints(pointA, pointB, pointC, vNorm, vAB);
     plane.set4(vNorm.x, vNorm.y, vNorm.z, w);
     return plane;
@@ -244,12 +250,23 @@ final public class Measure {
     return -vTemp.dot(vNorm);
   }
 
-  public static void getPlaneProjection(P3 pt, P4 plane, P3 ptProj, V3 vNorm) {
+  /**
+   * Project a point onto a plane, also returning the normal vector and the directed distance to the plane.
+   * 
+   * @param pt
+   * @param plane
+   * @param retPtProj  returned pt (can be pt)
+   * @param retNorm returned normal vector
+   * @return directed distance to plane
+   */
+  public static float getPlaneProjection(T3 pt, P4 plane, T3 retPtProj, V3 retNorm) {
     float dist = distanceToPlane(plane, pt);
-    vNorm.set(plane.x, plane.y, plane.z);
-    vNorm.normalize();
-    vNorm.scale(-dist);
-    ptProj.add2(pt, vNorm);
+    retNorm.set(plane.x, plane.y, plane.z);
+    retNorm.normalize();
+    if (dist > 0)
+      retNorm.scale(-1);
+    retPtProj.scaleAdd2(-dist, retNorm, pt);
+    return dist;
   }
 
   /**
@@ -259,7 +276,7 @@ final public class Measure {
    * @param ptB
    * @param ptC
    * @param isOutward
-   * @param normal set to be opposite to direction of ptCenter from ABC
+   * @param normal set to be opposite to direction of ptCenter from abd
    * @param vTemp
    * @return true if winding is CCW; false if CW
    */
@@ -291,23 +308,41 @@ final public class Measure {
     vTemp.normalize();
     getPlaneThroughPoint(ptTemp, vTemp, plane);
     }
-    
-  public static void projectOntoAxis(P3 point, P3 axisA,
+  
+  /**
+   * project point onto a line containing ptA and having axis unitVector axisUnitVector,
+   * returning the result in point and a vector from ptA to pt in vectorProjection  
+   * @param pt input pt to be projected, returns projected
+   * @param ptA input point on line
+   * @param axisUnitVector input unit vector
+   * @param vectorProjection return for pt.sub(ptA)
+   * @return projected distance
+   */
+  public static float projectOntoAxis(P3 pt, P3 ptA,
                                      V3 axisUnitVector,
                                      V3 vectorProjection) {
-    vectorProjection.sub2(point, axisA);
+    vectorProjection.sub2(pt, ptA);
     float projectedLength = vectorProjection.dot(axisUnitVector);
-    point.scaleAdd2(projectedLength, axisUnitVector, axisA);
-    vectorProjection.sub2(point, axisA);
+    pt.scaleAdd2(projectedLength, axisUnitVector, ptA);
+    vectorProjection.sub2(pt, ptA);
+    return projectedLength;
   }
-  
-  public static void calcBestAxisThroughPoints(P3[] points, P3 axisA,
+
+  /**
+   * 
+   * @param points
+   * @param nPoints 
+   * @param axisA
+   * @param axisUnitVector
+   * @param vectorProjection
+   * @param nTriesMax
+   */
+  public static void calcBestAxisThroughPoints(P3[] points, int nPoints, P3 axisA,
                                                V3 axisUnitVector,
                                                V3 vectorProjection,
                                                int nTriesMax) {
     // just a crude starting point.
 
-    int nPoints = points.length;
     axisA.setT(points[0]);
     axisUnitVector.sub2(points[nPoints - 1], axisA);
     axisUnitVector.normalize();
@@ -451,6 +486,7 @@ final public class Measure {
 
 
   /**
+   * Calculate the line that is the intersection of two planes.
    * 
    * @param plane1
    * @param plane2
@@ -505,6 +541,8 @@ final public class Measure {
   }
 
   /**
+   * 
+   * Calculate the intersection of a line with a plane.
    * 
    * @param pt1  point on line
    * @param v    unit vector of line
@@ -528,125 +566,125 @@ final public class Measure {
     return ptRet;
   }
 
-	/*
-	 * public static Point3f getTriangleIntersection(Point3f a1, Point3f a2,
-	 * Point3f a3, Point4f plane, Point3f b1, Point3f b2, Point3f b3, Vector3f
-	 * vNorm, Vector3f vTemp, Point3f ptRet, Point3f ptTemp, Vector3f vTemp2,
-	 * Point4f pTemp, Vector3f vTemp3) {
-	 * 
-	 * if (getTriangleIntersection(b1, b2, a1, a2, a3, vTemp, plane, vNorm,
-	 * vTemp2, vTemp3, ptRet, ptTemp)) return ptRet; if
-	 * (getTriangleIntersection(b2, b3, a1, a2, a3, vTemp, plane, vNorm, vTemp2,
-	 * vTemp3, ptRet, ptTemp)) return ptRet; if (getTriangleIntersection(b3, b1,
-	 * a1, a2, a3, vTemp, plane, vNorm, vTemp2, vTemp3, ptRet, ptTemp)) return
-	 * ptRet; return null; }
-	 */
-	/*
-	 * public static boolean getTriangleIntersection(Point3f b1, Point3f b2,
-	 * Point3f a1, Point3f a2, Point3f a3, Vector3f vTemp, Point4f plane, Vector3f
-	 * vNorm, Vector3f vTemp2, Vector3f vTemp3, Point3f ptRet, Point3f ptTemp) {
-	 * if (distanceToPlane(plane, b1) * distanceToPlane(plane, b2) >= 0) return
-	 * false; vTemp.sub(b2, b1); vTemp.normalize(); if (getIntersection(b1, vTemp,
-	 * plane, ptRet, vNorm, vTemp2) != null) { if (isInTriangle(ptRet, a1, a2, a3,
-	 * vTemp, vTemp2, vTemp3)) return true; } return false; } private static
-	 * boolean isInTriangle(Point3f p, Point3f a, Point3f b, Point3f c, Vector3f
-	 * v0, Vector3f v1, Vector3f v2) { // from
-	 * http://www.blackpawn.com/texts/pointinpoly/default.html // Compute
-	 * barycentric coordinates v0.sub(c, a); v1.sub(b, a); v2.sub(p, a); float
-	 * dot00 = v0.dot(v0); float dot01 = v0.dot(v1); float dot02 = v0.dot(v2);
-	 * float dot11 = v1.dot(v1); float dot12 = v1.dot(v2); float invDenom = 1 /
-	 * (dot00 * dot11 - dot01 * dot01); float u = (dot11 * dot02 - dot01 * dot12)
-	 * * invDenom; float v = (dot00 * dot12 - dot01 * dot02) * invDenom; return (u
-	 * > 0 && v > 0 && u + v < 1); }
-	 */
+  /*
+   * public static Point3f getTriangleIntersection(Point3f a1, Point3f a2,
+   * Point3f a3, Point4f plane, Point3f b1, Point3f b2, Point3f b3, Vector3f
+   * vNorm, Vector3f vTemp, Point3f ptRet, Point3f ptTemp, Vector3f vTemp2,
+   * Point4f pTemp, Vector3f vTemp3) {
+   * 
+   * if (getTriangleIntersection(b1, b2, a1, a2, a3, vTemp, plane, vNorm,
+   * vTemp2, vTemp3, ptRet, ptTemp)) return ptRet; if
+   * (getTriangleIntersection(b2, b3, a1, a2, a3, vTemp, plane, vNorm, vTemp2,
+   * vTemp3, ptRet, ptTemp)) return ptRet; if (getTriangleIntersection(b3, b1,
+   * a1, a2, a3, vTemp, plane, vNorm, vTemp2, vTemp3, ptRet, ptTemp)) return
+   * ptRet; return null; }
+   */
+  /*
+   * public static boolean getTriangleIntersection(Point3f b1, Point3f b2,
+   * Point3f a1, Point3f a2, Point3f a3, Vector3f vTemp, Point4f plane, Vector3f
+   * vNorm, Vector3f vTemp2, Vector3f vTemp3, Point3f ptRet, Point3f ptTemp) {
+   * if (distanceToPlane(plane, b1) * distanceToPlane(plane, b2) >= 0) return
+   * false; vTemp.sub(b2, b1); vTemp.normalize(); if (getIntersection(b1, vTemp,
+   * plane, ptRet, vNorm, vTemp2) != null) { if (isInTriangle(ptRet, a1, a2, a3,
+   * vTemp, vTemp2, vTemp3)) return true; } return false; } private static
+   * boolean isInTriangle(Point3f p, Point3f a, Point3f b, Point3f c, Vector3f
+   * v0, Vector3f v1, Vector3f v2) { // from
+   * http://www.blackpawn.com/texts/pointinpoly/default.html // Compute
+   * barycentric coordinates v0.sub(c, a); v1.sub(b, a); v2.sub(p, a); float
+   * dot00 = v0.dot(v0); float dot01 = v0.dot(v1); float dot02 = v0.dot(v2);
+   * float dot11 = v1.dot(v1); float dot12 = v1.dot(v2); float invDenom = 1 /
+   * (dot00 * dot11 - dot01 * dot01); float u = (dot11 * dot02 - dot01 * dot12)
+   * * invDenom; float v = (dot00 * dot12 - dot01 * dot02) * invDenom; return (u
+   * > 0 && v > 0 && u + v < 1); }
+   */
 
-	/**
-	 * Closed-form solution of absolute orientation requiring 1:1 mapping of
-	 * positions.
-	 * 
-	 * @param centerAndPoints
-	 * @param retStddev
-	 * @return unit quaternion representation rotation
-	 * 
-	 * @author hansonr Bob Hanson
-	 * 
-	 */
-	public static Quat calculateQuaternionRotation(P3[][] centerAndPoints,
-			float[] retStddev) {
-		/*
-		 * see Berthold K. P. Horn,
-		 * "Closed-form solution of absolute orientation using unit quaternions" J.
-		 * Opt. Soc. Amer. A, 1987, Vol. 4, pp. 629-642
-		 * http://www.opticsinfobase.org/viewmedia.cfm?uri=josaa-4-4-629&seq=0
-		 * 
-		 * 
-		 * A similar treatment was developed independently (and later!) by G.
-		 * Kramer, in G. R. Kramer,
-		 * "Superposition of Molecular Structures Using Quaternions" Molecular
-		 * Simulation, 1991, Vol. 7, pp. 113-119.
-		 * 
-		 * In that treatment there is a lot of unnecessary calculation along the
-		 * trace of matrix M (eqn 20). I'm not sure why the extra x^2 + y^2 + z^2 +
-		 * x'^2 + y'^2 + z'^2 is in there, but they are unnecessary and only
-		 * contribute to larger numerical averaging errors and additional processing
-		 * time, as far as I can tell. Adding aI, where a is a scalar and I is the
-		 * 4x4 identity just offsets the eigenvalues but doesn't change the
-		 * eigenvectors.
-		 * 
-		 * and Lydia E. Kavraki, "Molecular Distance Measures"
-		 * http://cnx.org/content/m11608/latest/
-		 */
+  /**
+   * Closed-form solution of absolute orientation requiring 1:1 mapping of
+   * positions.
+   * 
+   * @param centerAndPoints
+   * @param retStddev
+   * @return unit quaternion representation rotation
+   * 
+   * @author hansonr Bob Hanson
+   * 
+   */
+  public static Quat calculateQuaternionRotation(P3[][] centerAndPoints,
+      float[] retStddev) {
+    /*
+     * see Berthold K. P. Horn,
+     * "Closed-form solution of absolute orientation using unit quaternions" J.
+     * Opt. Soc. Amer. A, 1987, Vol. 4, pp. 629-642
+     * http://www.opticsinfobase.org/viewmedia.cfm?uri=josaa-4-4-629&seq=0
+     * 
+     * 
+     * A similar treatment was developed independently (and later!) by G.
+     * Kramer, in G. R. Kramer,
+     * "Superposition of Molecular Structures Using Quaternions" Molecular
+     * Simulation, 1991, Vol. 7, pp. 113-119.
+     * 
+     * In that treatment there is a lot of unnecessary calculation along the
+     * trace of matrix M (eqn 20). I'm not sure why the extra x^2 + y^2 + z^2 +
+     * x'^2 + y'^2 + z'^2 is in there, but they are unnecessary and only
+     * contribute to larger numerical averaging errors and additional processing
+     * time, as far as I can tell. Adding aI, where a is a scalar and I is the
+     * 4x4 identity just offsets the eigenvalues but doesn't change the
+     * eigenvectors.
+     * 
+     * and Lydia E. Kavraki, "Molecular Distance Measures"
+     * http://cnx.org/content/m11608/latest/
+     */
 
 
-		retStddev[1] = Float.NaN;
-		Quat q = new Quat();
-		P3[] ptsA = centerAndPoints[0];
-		P3[] ptsB = centerAndPoints[1];
-		int nPts = ptsA.length - 1;
-		if (nPts < 2 || ptsA.length != ptsB.length)
-			return q;
-		double Sxx = 0, Sxy = 0, Sxz = 0, Syx = 0, Syy = 0, Syz = 0, Szx = 0, Szy = 0, Szz = 0;
-		P3 ptA = new P3();
-		P3 ptB = new P3();
-		P3 ptA0 = ptsA[0];
-		P3 ptB0 = ptsB[0];
-		for (int i = nPts + 1; --i >= 1;) {
-			ptA.sub2(ptsA[i], ptA0);
-			ptB.sub2(ptsB[i], ptB0);
-			Sxx += (double) ptA.x * (double) ptB.x;
-			Sxy += (double) ptA.x * (double) ptB.y;
-			Sxz += (double) ptA.x * (double) ptB.z;
-			Syx += (double) ptA.y * (double) ptB.x;
-			Syy += (double) ptA.y * (double) ptB.y;
-			Syz += (double) ptA.y * (double) ptB.z;
-			Szx += (double) ptA.z * (double) ptB.x;
-			Szy += (double) ptA.z * (double) ptB.y;
-			Szz += (double) ptA.z * (double) ptB.z;
-		}
-		retStddev[0] = getRmsd(centerAndPoints, q);
-		double[][] N = new double[4][4];
-		N[0][0] = Sxx + Syy + Szz;
-		N[0][1] = N[1][0] = Syz - Szy;
-		N[0][2] = N[2][0] = Szx - Sxz;
-		N[0][3] = N[3][0] = Sxy - Syx;
+    retStddev[1] = Float.NaN;
+    Quat q = new Quat();
+    P3[] ptsA = centerAndPoints[0];
+    P3[] ptsB = centerAndPoints[1];
+    int nPts = ptsA.length - 1;
+    if (nPts < 2 || ptsA.length != ptsB.length)
+      return q;
+    double Sxx = 0, Sxy = 0, Sxz = 0, Syx = 0, Syy = 0, Syz = 0, Szx = 0, Szy = 0, Szz = 0;
+    P3 ptA = new P3();
+    P3 ptB = new P3();
+    P3 ptA0 = ptsA[0];
+    P3 ptB0 = ptsB[0];
+    for (int i = nPts + 1; --i >= 1;) {
+      ptA.sub2(ptsA[i], ptA0);
+      ptB.sub2(ptsB[i], ptB0);
+      Sxx += (double) ptA.x * (double) ptB.x;
+      Sxy += (double) ptA.x * (double) ptB.y;
+      Sxz += (double) ptA.x * (double) ptB.z;
+      Syx += (double) ptA.y * (double) ptB.x;
+      Syy += (double) ptA.y * (double) ptB.y;
+      Syz += (double) ptA.y * (double) ptB.z;
+      Szx += (double) ptA.z * (double) ptB.x;
+      Szy += (double) ptA.z * (double) ptB.y;
+      Szz += (double) ptA.z * (double) ptB.z;
+    }
+    retStddev[0] = getRmsd(centerAndPoints, q);
+    double[][] N = new double[4][4];
+    N[0][0] = Sxx + Syy + Szz;
+    N[0][1] = N[1][0] = Syz - Szy;
+    N[0][2] = N[2][0] = Szx - Sxz;
+    N[0][3] = N[3][0] = Sxy - Syx;
 
-		N[1][1] = Sxx - Syy - Szz;
-		N[1][2] = N[2][1] = Sxy + Syx;
-		N[1][3] = N[3][1] = Szx + Sxz;
+    N[1][1] = Sxx - Syy - Szz;
+    N[1][2] = N[2][1] = Sxy + Syx;
+    N[1][3] = N[3][1] = Szx + Sxz;
 
-		N[2][2] = -Sxx + Syy - Szz;
-		N[2][3] = N[3][2] = Syz + Szy;
+    N[2][2] = -Sxx + Syy - Szz;
+    N[2][3] = N[3][2] = Syz + Szy;
 
-		N[3][3] = -Sxx - Syy + Szz;
+    N[3][3] = -Sxx - Syy + Szz;
 
-		// this construction prevents JavaScript from requiring preloading of Eigen
+    // this construction prevents JavaScript from requiring preloading of Eigen
 
-		float[] v = ((EigenInterface) Interface.getInterface("javajs.util.Eigen"))
-				.setM(N).getEigenvectorsFloatTransposed()[3];
-		q = Quat.newP4(P4.new4(v[1], v[2], v[3], v[0]));
-		retStddev[1] = getRmsd(centerAndPoints, q);
-		return q;
-	}
+    float[] v = ((EigenInterface) Interface.getInterface("javajs.util.Eigen"))
+        .setM(N).getEigenvectorsFloatTransposed()[3];
+    q = Quat.newP4(P4.new4(v[1], v[2], v[3], v[0]));
+    retStddev[1] = getRmsd(centerAndPoints, q);
+    return q;
+  }
 
   /**
    * Fills a 4x4 matrix with rotation-translation of mapped points A to B.
@@ -687,18 +725,18 @@ final public class Measure {
    * @param vPts
    * @return  array of points with first point center
    */
-	public static P3[] getCenterAndPoints(Lst<P3> vPts) {
-	  int n = vPts.size();
-	  P3[] pts = new P3[n + 1];
-	  pts[0] = new P3();
-	  if (n > 0) {
-	    for (int i = 0; i < n; i++) {
-	      pts[0].add(pts[i + 1] = vPts.get(i));
-	    }
-	    pts[0].scale(1f / n);
-	  }
-	  return pts;
-	}
+  public static P3[] getCenterAndPoints(Lst<P3> vPts) {
+    int n = vPts.size();
+    P3[] pts = new P3[n + 1];
+    pts[0] = new P3();
+    if (n > 0) {
+      for (int i = 0; i < n; i++) {
+        pts[0].add(pts[i + 1] = vPts.get(i));
+      }
+      pts[0].scale(1f / n);
+    }
+    return pts;
+  }
 
   public static float getRmsd(P3[][] centerAndPoints, Quat q) {
     double sum2 = 0;
@@ -717,4 +755,301 @@ final public class Measure {
     return (float) Math.sqrt(sum2 / n);
   }
 
+  /**
+   * Get the endpoints of the best line through N points, where N >= 2
+   * 
+   * @param points
+   * @param nPoints
+   * @return end points
+   */
+  public static P3[] getBestLineThroughPoints(P3[] points, int nPoints) {
+    if (nPoints <= 0)
+      nPoints = points.length;
+    if (nPoints <= 2) {
+      return points;
+    }
+    P3 ptA = new P3();
+    V3 unitVector = new V3();
+    V3 vTemp = new V3();
+    calcBestAxisThroughPoints(points, nPoints, ptA, unitVector,
+        vTemp, 8);
+    return getProjectedLineSegment(points, nPoints, ptA, unitVector, vTemp);
+  }
+
+  public static P3[] getProjectedLineSegment(P3[] points, int nPoints,
+                                              P3 ptA, V3 unitVector,
+                                              V3 vTemp) {
+    if (nPoints < 0)
+      nPoints = points.length;
+    if (vTemp == null)
+      vTemp = new V3();
+    P3 pmin = null, pmax = null, p; 
+    float dmin = Float.MAX_VALUE, dmax = -Float.MAX_VALUE;
+    for (int i = 0; i < points.length; i++) {
+      projectOntoAxis(p = P3.newP(points[i]), ptA, unitVector, vTemp);
+      float d = unitVector.dot(vTemp);
+      if (d < dmin) {
+        dmin = d;
+        pmin = p;
+      }
+      if (d > dmax) {
+        dmax = d;
+        pmax = p;
+      }
+    }
+    return new P3[] { pmin, pmax };
+  }
+
+  public static boolean isInTriangle(P3 p, P3 a, P3 b, P3 c, V3 v0, V3 v1, V3 v2) { 
+    // from http: //www.blackpawn.com/texts/pointinpoly/default.html 
+    // Compute   barycentric coordinates 
+    v0.sub2(c, a);
+    v1.sub2(b, a);
+    v2.sub2(p, a);
+    float dot00 = v0.dot(v0);
+    float dot01 = v0.dot(v1);
+    float dot02 = v0.dot(v2);
+    float dot11 = v1.dot(v1);
+    float dot12 = v1.dot(v2);
+    float invDenom = 1 / (dot00 * dot11 - dot01 * dot01);
+    float u = (dot11 * dot02 - dot01 * dot12) * invDenom;
+    float v = (dot00 * dot12 - dot01 * dot02) * invDenom;
+    return (u >= 0 && v >= 0 && u + v <= 1);
+  }
+  
+  /**
+   * Calculate the best ax + by + cz + d = 0 plane through a number of points
+   * using a three-step check for the best plane based on normal distance. this
+   * simple calculation isn't perfect, but it does the job quickly and with no
+   * need for a full singular value decomposition.
+   * 
+   * @param points
+   * @param nPoints
+   * @param plane
+   * @return RMSD for the best plane along with filling the plane itself
+   * 
+   * @author hansonr 2022.01.26
+   */
+  public static float calcBestPlaneThroughPoints(P3[] points, int nPoints,
+                                                 P4 plane) {
+    if (nPoints <= 0) {
+      nPoints = points.length;
+    }
+    if (nPoints == 3) {
+      getPlaneThroughPoints(points[0], points[1], points[2], null, null, plane);
+      return 0;
+    }
+    P4 pmin = plane;
+    P4 plane2 = new P4();
+    P4 plane3;
+    float rmsd = calcPlaneForMode(points, nPoints, plane, 'z');
+    if (rmsd < 1e-6)
+      return rmsd;
+    float f2 = calcPlaneForMode(points, nPoints, plane2, 'y');
+    if (f2 < rmsd) {
+      rmsd = f2;
+      pmin = plane2;
+      plane3 = plane;
+    } else {
+      plane3 = plane2;
+    }
+    if (rmsd >= 1e-6) {
+      f2 = calcPlaneForMode(points, nPoints, plane3, 'x');
+      if (f2 < rmsd) {
+        rmsd = f2;
+        pmin = plane3;
+      }
+    }
+    if (pmin != plane) {
+      plane.setT(pmin);
+      plane.w = pmin.w;
+    }
+    return rmsd;
+  }
+
+  /**
+   * Compact calculation of the best pane using a simple method discussed at 
+   *    https://stackoverflow.com/questions/12299540/plane-fitting-to-4-or-more-xyz-points
+   *    
+   * (A^T A)^-1 A^T B
+   *
+   * run three times to ensure that at least one is not perpendicular.
+
+   * @param points
+   * @param nPoints
+   * @param plane filled with best plane
+   * @param mode
+   * @return rmsd
+   */
+  public static float calcPlaneForMode(P3[] points, int nPoints, P4 plane, char mode) {
+    
+    double[][] A = new double[nPoints][3];
+    double[][] AT = new double[3][nPoints];
+    double[][] ATAT = new double[3][nPoints];
+    double[][] ATA1 = new double[3][3];
+    double[] B = new double[nPoints];
+    
+    for (int i = nPoints; --i >= 0;) {
+      P3 p = points[i];
+      A[i][0] = AT[0][i] = (mode == 'x' ? p.z : p.x);
+      A[i][1] = AT[1][i] = (mode == 'y' ? p.z : p.y);
+      A[i][2] = AT[2][i] = 1;
+      B[i] = -(mode == 'y' ? p.y : mode == 'x' ? p.x : p.z);
+    }
+    M3 m = new M3();
+    for (int i = 3; --i >= 0;) {
+      for (int j = 3; --j >= 0;) {
+        double d = 0;
+        for (int k = nPoints; --k >= 0;) {
+          d += AT[i][k] * A[k][j];
+        }
+        m.set33(i, j, (float) d);
+      }
+    }
+    m.invert();
+    for (int i = 3; --i >= 0;) {
+      for (int j = 3; --j >= 0;) {
+        ATA1[i][j] = m.get33(i, j);
+      }
+    }
+    for (int i = 3; --i >= 0;) {
+      for (int k = nPoints; --k >= 0;) {
+        double d = 0;
+      for (int j = 3; --j >= 0;) {
+          d += ATA1[i][j] * AT[j][k];
+        }
+      ATAT[i][k] = d;
+      }
+    }
+    switch (mode) {
+    case 'x':
+      plane.x = 1;
+      break;
+    case 'y':
+      plane.y = 1;
+      break;
+    case 'z':
+      plane.z = 1;
+      break;
+    }
+    float len2 = 1;
+    for (int j = 3; --j >= 0;) {
+      double v = 0;
+      for (int k = nPoints; --k >= 0;) {
+        v += ATAT[j][k]*B[k];
+      }
+      switch (j) {
+      case 0:
+        len2 += v * v;
+        if (mode == 'x')
+          plane.z = (float) v;
+        else
+          plane.x = (float) v;
+        break;
+      case 1:
+        len2 += v * v;
+        if (mode == 'y')
+          plane.z = (float) v;
+        else 
+          plane.y = (float) v;
+        break;
+      case 2:
+        plane.w = (float) v;
+      }
+    }
+    float f = (float) Math.sqrt(len2);
+    plane.scale4((1/plane.w > 0 ? 1 : -1)/f);
+    float sum2 = 0;
+    for (int i = 0; i < nPoints; i++) {
+      float d = distanceToPlane(plane, points[i]);
+      sum2 += d*d;
+    }
+    float ret = (float) Math.sqrt(sum2 / nPoints);
+    return ret;
+  }
+
+  static P3 rndPt() {
+    return P3.new3((float) Math.random()*20,(float) (Math.random()*20),(float) (Math.random()*20) );
+  }
+  static void testRnd() {
+    P4 plane = P4.new4((float) Math.random()*20, (float) Math.random()*20, (float) Math.random()*20, (float) Math.random()*20);
+    plane.scale4(1/plane.length());
+    System.out.println("\n==========\n ");
+    System.out.println("plane is " + plane);
+    P3 ptProj = new P3();
+    V3 vNorm = new V3();
+    P3[] pts = new P3[4];
+    for (int i = 0; i < pts.length; i++) {
+      pts[i] = new P3();
+      P3 p = rndPt();
+      getPlaneProjection(p, plane, ptProj, vNorm );
+      pts[i].setT(ptProj);
+      float d = (float)Math.random()*0.1f;
+      pts[i].scaleAdd2(d, vNorm, ptProj);
+      System.out.println(pts[i] + " d=" + d);
+    }
+    P4 p2 = new P4();
+    float f = calcBestPlaneThroughPoints(pts, -1, p2);
+    System.out.println("found " + p2 + " rmsd = " + f);
+  }
+  
+  
+  
+  static public void test() {
+    for (int i = 0; i < 10; i++)
+      testRnd();
+    System.exit(0);
+  }
+
+  /**
+   * Based on a set of centering points, returns the list of points on a given
+   * plane.
+   * 
+   * @param pts
+   *        initial list centering points such as {1/2 1/2 0}
+   * @param plane
+   * @return non-null list
+   */
+  public static Lst<P3> getPointsOnPlane(P3[] pts, P4 plane) {
+    Lst<P3> ret = new Lst<P3>();
+    for (int i = pts.length; --i >= 0;) {
+      float d = Math.abs(Measure.distanceToPlane(plane, pts[i]));
+      if (d < 0.001f) {
+        ret.addLast(pts[i]);
+      }
+    }
+    return ret;
+  }
+
+  /**
+   * Based on a set of centering points, creates a list of lattice points in place.
+   * 
+   * @param cpts centering points such as {1/2 1/2 0}
+   * @param h
+   * @param k
+   * @param l
+   * @return non-null list
+*/
+  public static Lst<P3> getLatticePoints(Lst<P3> cpts, int h, int k, int l) {
+    cpts.addLast(new P3());
+    h = (h == 0 ? 1 : Math.abs(h));
+    k = (k == 0 ? 1 : Math.abs(k));
+    l = (l == 0 ? 1 : Math.abs(l));
+    int n = cpts.size();
+    for (int ih = -h; ih <= h; ih++) {
+      for (int ik = -k; ik <= k; ik++) {
+        for (int il = -l; il <= l; il++) {
+          for (int i = 0; i < n; i++) {
+            P3 pt = P3.new3(ih, ik, il);
+            pt.add(cpts.get(i));
+            cpts.addLast(pt);
+          }
+        }
+      }
+    }
+    for (int i = n; --i >= 0;)
+      cpts.removeItemAt(i);
+    return cpts;
+  }
+
 }
