@@ -215,7 +215,14 @@ public class PT {
   0.000001f, 
   0.0000001f, 
   0.00000001f, 
-  0.000000001f
+  0.000000001f, 
+  // added for JavaScript to have full double precision if specified
+  0.0000000001f,
+  0.00000000001f,
+  0.000000000001f,
+  0.0000000000001f,
+  0.00000000000001f,
+  0.000000000000001f,
   };
   public static boolean checkTrailingText(String str, int ich, int ichMax) {
     //number must be pure -- no additional characters other than white space or ;
@@ -784,26 +791,20 @@ public class PT {
     // because in JavaScript those would be false for unwrapped primitives
     // coming from equivalent of Array.get()
     // Strings will need their own escaped processing
-    
-    return info instanceof Number || info instanceof Boolean;
-  }
-
-  private static Object arrayGet(Object info, int i) {
     /**
-     * 
-     * Note that info will be a primitive in JavaScript
-     * but a wrapped primitive in Java.
-     * 
      * @j2sNative
      * 
-     *            return info[i];
+     * if(typeof info == "number" || typeof info == "boolean") {
+     * return true;
+     * }
+     * 
+     * 
      */
-    {
-      return Array.get(info, i);
-    }
+    {}
+     return info instanceof Number || info instanceof Boolean;
   }
   
-  @SuppressWarnings("unchecked")
+  @SuppressWarnings({ "null", "unused", "unchecked" })
   public static String toJSON(String infoType, Object info) {
     if (info == null)
       return packageJSON(infoType, null);
@@ -817,11 +818,12 @@ public class PT {
         /**
          * @j2sNative
          * 
-         * if (typeof s == "undefined") s = "null"
+         *            if (typeof s == "undefined") s = "null"
          * 
          */
-        {}
-        
+        {
+        }
+
         if (s.indexOf("{\"") != 0) {
           //don't doubly fix JSON strings when retrieving status
           // what about  \1 \2 \3 etc.?
@@ -840,6 +842,8 @@ public class PT {
         sb.append("{ ");
         String sep = "";
         for (String key : ((Map<String, ?>) info).keySet()) {
+          if (key == null)
+            key = "null";
           sb.append(sep).append(
               packageJSON(key, toJSON(null, ((Map<?, ?>) info).get(key))));
           sep = ",";
@@ -877,10 +881,20 @@ public class PT {
       if (s == null) {
         sb.append("[");
         int n = AU.getLength(info);
-        for (int i = 0; i < n; i++) {
-          if (i > 0)
-            sb.appendC(',');
-          sb.append(toJSON(null, arrayGet(info, i)));
+        Object o = null;
+        /** @j2sNative 
+         *  o = info[0];
+         *  typeof o != "number" && typeof 0 != "boolean" && (o = null);
+         */
+        {}
+        if (o != null) {
+          sb.appendO(info);
+        } else {
+          for (int i = 0; i < n; i++) {
+            if (i > 0)
+              sb.appendC(',');
+            sb.append(toJSON(null, arrayGet(info, i)));
+          }
         }
         sb.append("]");
         break;
@@ -890,6 +904,22 @@ public class PT {
     return packageJSON(infoType, (s == null ? sb.toString() : s));
   }
 
+  private static Object arrayGet(Object info, int i) {
+    /**
+     * 
+     * Note that info will be a primitive in JavaScript
+     * but a wrapped primitive in Java.
+     * 
+     * @j2sNative
+     * 
+     *            return info[i];
+     */
+    {
+      return Array.get(info, i);
+    }
+  }
+  
+
   /**
    * Checks to see if an object is an array (including typed arrays), and if it is, returns null;
    * otherwise it returns the string equivalent of that object.
@@ -989,10 +1019,11 @@ public class PT {
    */
   public static String escF(float f) {
     String sf = "" + f;
+    // NaN, Infinity
     /**
      * @j2sNative
      * 
-     * if (sf.indexOf(".") < 0 && sf.indexOf("e") < 0)
+     * if (sf.indexOf(".") < 0 && sf.indexOf("e") < 0 && sf.indexOf("N") < 0 && sf.indexOf("n") < 0)
      *   sf += ".0";
      */
     {
@@ -1142,12 +1173,16 @@ public class PT {
         if (strFormat.charAt(ich) == '.') {
           ++ich;
           if ((ch = strFormat.charAt(ich)) == '-') {
-            isExponential = (strT == null);
+            isExponential = true;//(strT == null);
             ++ich;
           } 
           if ((ch = strFormat.charAt(ich)) >= '0' && ch <= '9') {
             precision = ch - '0';
             ++ich;
+            if ((ch = strFormat.charAt(ich)) >= '0' && ch <= '9') {
+              precision = 10*precision + (ch - '0');
+              ++ich;
+            }
           }
           if (isExponential)
             precision = -precision;
@@ -1163,7 +1198,7 @@ public class PT {
           strLabel += formatF(floatT, width, precision, alignLeft,
               zeroPad);
         else if (strT != null)  // 'd' 'i' or 's'
-          strLabel += formatS(strT, width, precision, alignLeft,
+          strLabel += formatS(strT, width, precision < 0 ? precision - 1 : precision, alignLeft,
               zeroPad);
         else if (!Double.isNaN(doubleT)) // 'e'
           strLabel += formatD(doubleT, width, precision - 1, alignLeft,
