@@ -28,13 +28,13 @@ import java.util.Map;
 import java.util.Map.Entry;
 
 import org.jmol.adapter.smarter.Atom;
-import org.jmol.adapter.smarter.AtomSetCollection;
 import org.jmol.adapter.smarter.AtomSetCollectionReader;
 import org.jmol.api.JmolAdapter;
 import org.jmol.api.SymmetryInterface;
 import org.jmol.script.T;
 import org.jmol.util.Logger;
 import org.jmol.util.Vibration;
+import org.jmol.viewer.JC;
 
 import javajs.api.GenericCifDataParser;
 import javajs.util.BS;
@@ -75,6 +75,9 @@ import javajs.util.V3;
 public class CifReader extends AtomSetCollectionReader {
 
 
+  protected static final String CELL_TYPE_MAGNETIC_PARENT = "parent";
+  protected static final String CELL_TYPE_MAGNETIC_STANDARD = "standard";
+
   /**
    * Allows checking specific blocks 
    * 
@@ -94,6 +97,7 @@ public class CifReader extends AtomSetCollectionReader {
   Parser subParser; // TopoCifParser
   
   private static final String titleRecords = "__citation_title__publ_section_title__active_magnetic_irreps_details__";
+
   private MSCifParser modr; // Modulated Structure subreader
 //  private MagCifRdr magr;// Magnetic CIF subreader - not necessary
 
@@ -139,7 +143,6 @@ public class CifReader extends AtomSetCollectionReader {
   protected Map<String, String> htGroup1;
   protected int nAtoms0;
   private int titleAtomSet = 1;
-  private int intTableNo;
 
   @Override
   public void initializeReader() throws Exception {
@@ -161,7 +164,7 @@ public class CifReader extends AtomSetCollectionReader {
     asc.checkSpecial = !checkFilterKey("NOSPECIAL");
     allowRotations = !checkFilterKey("NOSYM");
     if (strSupercell != null && strSupercell.indexOf(",") >= 0)
-      addCellType("conventional", strSupercell, true);
+      addCellType(CELL_TYPE_CONVENTIONAL, strSupercell, true);
     if (binaryDoc != null)
       return; // mmtf
     readCifData();
@@ -202,7 +205,7 @@ public class CifReader extends AtomSetCollectionReader {
   }
 
   private boolean continueWith(String key) {
-    boolean ret = (key != null && !key.equals("_shelx_hkl_file"));
+    boolean ret = (key != null && (ac == 0 || !key.equals("_shelx_hkl_file")));
     return ret;
   }
 
@@ -279,6 +282,7 @@ public class CifReader extends AtomSetCollectionReader {
         modDim = parseIntStr(data);
         if (modr != null)
           modr.setModDim(modDim);
+      } else if (skipKey(key)) {
       } else if (key.startsWith("_cell_") && key.indexOf("_commen_") < 0) {
         processCellParameter();
       } else if (key.startsWith("_atom_sites_fract_tran")) {
@@ -323,17 +327,28 @@ public class CifReader extends AtomSetCollectionReader {
       } else if (key.startsWith("_aflow_")) {
         isAFLOW = true;
       } else if (key.equals("_symmetry_int_tables_number")) {
-        intTableNo = parseIntStr(data);
+        int intTableNo = parseIntStr(data);
         rotateHexCell = (isAFLOW && (intTableNo >= 143 && intTableNo <= 194)); // trigonal or hexagonal
       } else if (key.equals("_entry_id")) {
         pdbID = data;
       } else if (key.startsWith("_topol_")){
         getTopologyParser().ProcessRecord(key, data);
+      } else {
+        // see MMCifReader or MSRdr
+        processSubclassEntry();
       }
     }
     return true;
   }
 
+  private boolean skipKey(String key) {
+    return key.startsWith("_shelx_")
+    || key.startsWith("_reflns_")
+    || key.startsWith("_diffrn_");
+  }
+
+
+
   private void addModelTitle(String key) {
     if (asc.atomSetCount > titleAtomSet)
       appendLoadNote("\nMODEL: " + (titleAtomSet = asc.atomSetCount));
@@ -345,6 +360,13 @@ public class CifReader extends AtomSetCollectionReader {
       getModulationReader().processEntry();
   }
 
+  /**
+   * (magnetic CIF only)
+   * 
+   * Process the unit cell transformation as indicated by _parent_space_group or
+   *  _space_group_magn (or older _magentic_space_group)
+   * 
+   */
   private void processUnitCellTransform() {
     data = PT.replaceAllCharacters(data, " ", "");
     
@@ -363,18 +385,23 @@ public class CifReader extends AtomSetCollectionReader {
     // _space_group_magn.transform_OG_Pp_abc     '-a-c,-b,1/2c;0,0,0'   -- no interest to us
     // _parent_space_group.transform_Pp_abc   'a,b,c;0,0,0'             -- no interest to us
 
-    
-      
-    
     if (key.contains("_from_parent") || key.contains("child_transform"))
-      addCellType("parent", data, true);
+      addCellType(CELL_TYPE_MAGNETIC_PARENT, data, true);
     else if (key.contains("_to_standard") || key.contains("transform_bns_pp_abc"))
-      addCellType("standard", data, false);
+      addCellType(CELL_TYPE_MAGNETIC_STANDARD, data, false);
     appendLoadNote(key + ": " + data);
   }
 
   private Map<String, String> htCellTypes;
 
+  /**
+   * (magnetic CIF or LOAD ... SUPERCELL)
+   * 
+   * Add a cell type such as "conventional"
+   * @param type
+   * @param data if starting with "!" then "opposite of"
+   * @param isFrom TRUE for SUPERCELL or "_from_parent" or "child_transform"
+   */
   private void addCellType(String type, String data, boolean isFrom) {
     if (htCellTypes == null)
       htCellTypes = new Hashtable<String, String>();
@@ -386,7 +413,7 @@ public class CifReader extends AtomSetCollectionReader {
     htCellTypes.put(type, cell);
     if (type.equalsIgnoreCase(strSupercell)) {
       strSupercell = cell;
-      htCellTypes.put("conventional", (isFrom ? "" : "!") + data);
+      htCellTypes.put(CELL_TYPE_CONVENTIONAL, (isFrom ? "" : "!") + data);
     }
   }
 
@@ -490,7 +517,7 @@ public class CifReader extends AtomSetCollectionReader {
           a.formalCharge = Math.round(charge);
         }
         if (!Float.isNaN(radius)) {
-          a.bondRadius = radius;
+          a.bondingRadius = radius;
         }
       }
     }
@@ -569,7 +596,8 @@ public class CifReader extends AtomSetCollectionReader {
     SymmetryInterface sym = (haveSymmetry ? asc.getXSymmetry()
         .getBaseSymmetry() : null);
     if (sym != null && sym.getSpaceGroup() == null) {
-      appendLoadNote("Invalid or missing space group operations!");
+      if (!isBinary) // ignore this for MMTF
+        appendLoadNote("Invalid or missing space group operations!");
       sym = null;
     }
     if (modDim > 0 && sym != null) {
@@ -820,7 +848,11 @@ public class CifReader extends AtomSetCollectionReader {
     key = (String) cifParser.getTokenPeeked();
     if (!continueWith(key))
       return false;
-    data = cifParser.getNextToken();
+    if (skipKey(key)) {
+      data = cifParser.skipNextToken();
+    } else {
+      data = cifParser.getNextToken();
+    }
     if (debugging && data != null && data.length() > 0 && data.charAt(0) != '\0')
       Logger.debug(">> " + key  + " " + data);
     if (data == null) {
@@ -1187,7 +1219,7 @@ public class CifReader extends AtomSetCollectionReader {
       "*_label_atom_id" // 73 mCIF dev 
       };
 
-  final private static String singleAtomID = atomFields[CC_COMP_ID];
+//  final private static String singleAtomID = atomFields[CC_COMP_ID];
 
   /* to: hansonr@stolaf.edu
    * from: Zukang Feng zfeng@rcsb.rutgers.edu
@@ -1200,6 +1232,7 @@ public class CifReader extends AtomSetCollectionReader {
 
   /**
    * reads atom data in any order
+   * 
    * @param isLigand 
    * 
    * @return TRUE if successful; FALS if EOF encountered
@@ -1248,17 +1281,41 @@ public class CifReader extends AtomSetCollectionReader {
           continue;
       }
       Atom atom = null;
-      if (haveCoord) {
-        atom = new Atom();
-      } else {
-        if (fieldProperty(key2col[ANISO_LABEL]) != NONE
-            || fieldProperty(key2col[ANISO_MMCIF_ID]) != NONE
-            || fieldProperty(key2col[MOMENT_LABEL]) != NONE) {
-          if ((atom = asc.getAtomFromName(field)) == null)
-            continue; // atom has been filtered out
+      if (isMMCIF) {
+        if (haveCoord) {
+          atom = new Atom();
         } else {
-          continue;
+          if (fieldProperty(key2col[ANISO_LABEL]) != NONE
+              || fieldProperty(key2col[ANISO_MMCIF_ID]) != NONE
+              || fieldProperty(key2col[MOMENT_LABEL]) != NONE) {
+            if ((atom = asc.getAtomFromName(field)) == null)
+              continue; // atom has been filtered out
+          } else {
+            continue;
+          }
         }
+      } else {
+      // check for atom reference before atom definition
+        int f = NONE;
+        if ((f = fieldProperty(key2col[LABEL])) != NONE
+            || (f = fieldProperty(key2col[CC_ATOM_ID])) != NONE
+            || (f = fieldProperty(key2col[LABEL_ATOM_ID])) != NONE
+            || (f = fieldProperty(key2col[ANISO_LABEL])) != NONE
+            || (f = fieldProperty(key2col[ANISO_MMCIF_ID])) != NONE
+            || (f = fieldProperty(key2col[MOMENT_LABEL])) != NONE
+            ) {
+        atom = asc.getAtomFromName(field);
+        }
+      if (atom == null) {
+        atom = new Atom();
+          if (f != NONE) {
+            if (asc.iSet < 0) {
+              nextAtomSet();   
+              asc.newAtomSet();
+            }
+            asc.atomSymbolicMap.put(field, atom);
+        }
+      }
       }
       String componentId = null;
       String id = null;
@@ -1703,6 +1760,8 @@ public class CifReader extends AtomSetCollectionReader {
               field = field.replace(';', ' ');
               symops.addLast(field);
               setSymmetryOperator(field);
+              if (modulated && modDim == 0)
+                modDim = getModulationReader().modDim;
             }
           break;
         case SYM_MAGN_CENTERING:
@@ -1957,9 +2016,9 @@ public class CifReader extends AtomSetCollectionReader {
       }
       asc.setCurrentModelInfo("unitCellParams", null);
       if (nMolecular++ == asc.iSet) {
-        asc.clearGlobalBoolean(AtomSetCollection.GLOBAL_FRACTCOORD);
-        asc.clearGlobalBoolean(AtomSetCollection.GLOBAL_SYMMETRY);
-        asc.clearGlobalBoolean(AtomSetCollection.GLOBAL_UNITCELLS);
+        asc.clearGlobalBoolean(JC.GLOBAL_FRACTCOORD);
+        asc.clearGlobalBoolean(JC.GLOBAL_SYMMETRY);
+        asc.clearGlobalBoolean(JC.GLOBAL_UNITCELLS);
       }
 
     }
