@@ -885,7 +885,6 @@ public class TopoCifParser implements Parser {
     String label;
     String specialDetails;
 
-    @SuppressWarnings("unused")
     int idx;
     boolean hasAtoms;
 
@@ -1053,7 +1052,7 @@ public class TopoCifParser implements Parser {
   static void setTAtom(Atom a, Atom b) {
     b.setT(a);
     b.formalCharge = a.formalCharge;
-    b.bondRadius = a.bondRadius;
+    b.bondingRadius = a.bondingRadius;
   }
 
   /**
@@ -1093,7 +1092,7 @@ public class TopoCifParser implements Parser {
     }
   }
 
-  private final static P3 ZERO = new P3();
+  final static P3 ZERO = new P3();
   private class TNode extends Atom implements TPoint {
 
     public String id;
@@ -1428,9 +1427,9 @@ public class TopoCifParser implements Parser {
       finalized = true;
     }
 
-    public String getMolecularFormula() {
-      return (mf == null ? (mf = getMF(tatoms)) : mf);
-    }
+//    public String getMolecularFormula() {
+//      return (mf == null ? (mf = getMF(tatoms)) : mf);
+//    }
 
     /**
      * 
