@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2021-11-14 13:32:58 -0600 (Sun, 14 Nov 2021) $
- * $Revision: 22254 $
+ * $Date: 2022-06-01 06:55:35 +0200 (Wed, 01 Jun 2022) $
+ * $Revision: 22422 $
  *
  * Copyright (C) 2003-2005  Miguel, Jmol Development, www.jmol.org
  *
@@ -40,6 +40,7 @@ import javajs.util.BS;
 
 import org.jmol.util.BSUtil;
 import org.jmol.util.Logger;
+import org.jmol.viewer.JC;
 
 @SuppressWarnings("unchecked")
 public class AtomSetCollection {
@@ -60,29 +61,16 @@ public class AtomSetCollection {
 
   public Map<String, Object> atomSetInfo = new Hashtable<String, Object>();
 
-  private final static String[] globalBooleans = {
-      "someModelsHaveFractionalCoordinates", "someModelsHaveSymmetry",
-      "someModelsHaveUnitcells", "someModelsHaveCONECT", "isPDB", "someModelsHaveDomains", "someModelsHaveValidations" };
-
-  public final static int GLOBAL_FRACTCOORD = 0;
-  public final static int GLOBAL_SYMMETRY = 1;
-  public final static int GLOBAL_UNITCELLS = 2;
-  public final static int GLOBAL_CONECT = 3;
-  public final static int GLOBAL_ISPDB = 4;
-  public final static int GLOBAL_DOMAINS = 5;
-  public final static int GLOBAL_VALIDATIONS = 6;
-  
-
   public void clearGlobalBoolean(int globalIndex) {
-    atomSetInfo.remove(globalBooleans[globalIndex]);
+    atomSetInfo.remove(JC.getBoolName(globalIndex));
   }
 
   public void setGlobalBoolean(int globalIndex) {
-    setInfo(globalBooleans[globalIndex], Boolean.TRUE);
+    setInfo(JC.getBoolName(globalIndex), Boolean.TRUE);
   }
 
   boolean getGlobalBoolean(int globalIndex) {
-    return (atomSetInfo.get(globalBooleans[globalIndex]) == Boolean.TRUE);
+    return (atomSetInfo.get(JC.getBoolName(globalIndex)) == Boolean.TRUE);
   }
 
   public Atom[] atoms = new Atom[256];
@@ -118,6 +106,8 @@ public class AtomSetCollection {
 
   public boolean atomMapAnyCase;
 
+  private int fixedSite;
+
   public AtomSetCollection(String fileTypeName, AtomSetCollectionReader reader,
       AtomSetCollection[] array, Lst<?> list) {
 
@@ -131,9 +121,14 @@ public class AtomSetCollection {
     p.put("PATH_KEY", SmarterJmolAdapter.PATH_KEY);
     p.put("PATH_SEPARATOR", SmarterJmolAdapter.PATH_SEPARATOR);
     setInfo("properties", p);
-    Integer modelIndex = (reader == null ? null : (Integer) reader.htParams.get("appendToModelIndex"));
-    if (modelIndex != null)
-      this.setInfo("appendToModelIndex",modelIndex);
+    if (reader != null) {
+    Integer ii = (Integer) reader.htParams.get("appendToModelIndex");   
+    if (ii != null)
+      this.setInfo("appendToModelIndex",ii);
+    ii = (Integer) reader.htParams.get("fixedSite");   
+    if (ii != null)
+      fixedSite = ii.intValue();    
+    }
     if (array != null) {
       int n = 0;
       readerList = new Lst<AtomSetCollectionReader>();
@@ -269,7 +264,7 @@ public class AtomSetCollection {
           bond.atomIndex2 + existingAtomsCount, bond.order);
     }
     // Set globals
-    for (int i = globalBooleans.length; --i >= 0;)
+    for (int i = JC.globalBooleans.length; --i >= 0;)
       if (collection.getGlobalBoolean(i))
         setGlobalBoolean(i);
 
@@ -440,8 +435,7 @@ public class AtomSetCollection {
     for (int i = ac; --i >= 0;)
       atoms[i] = null;
     ac = 0;
-    atomSymbolicMap.clear();
-    atomMapAnyCase = false;
+    clearMap();
     atomSetCount = 0;
     iSet = -1;
     for (int i = atomSetAuxiliaryInfo.length; --i >= 0;) {
@@ -546,7 +540,7 @@ public class AtomSetCollection {
     atom.index = ac;
     atoms[ac++] = atom;    
     atom.atomSetIndex = iSet;
-    atom.atomSite = atomSetAtomCounts[iSet]++;
+    atom.atomSite = (fixedSite > 0 ? fixedSite - 1 : atomSetAtomCounts[iSet]++);
     return atom;
   }
 
@@ -596,14 +590,13 @@ public class AtomSetCollection {
     return b;
   }
 
-  public void addNewBondFromNames(String atomName1, String atomName2, int order) {
-    addNewBondWithOrderA(getAtomFromName(atomName1), getAtomFromName(atomName2), order);
+  public Bond addNewBondFromNames(String atomName1, String atomName2, int order) {
+    return addNewBondWithOrderA(getAtomFromName(atomName1), getAtomFromName(atomName2), order);
   }
 
-  public void addNewBondWithOrderA(Atom atom1, Atom atom2,
+  public Bond addNewBondWithOrderA(Atom atom1, Atom atom2,
                                     int order) {
-    if (atom1 != null && atom2 != null)
-      addNewBondWithOrder(atom1.index, atom2.index, order);
+    return (atom1 != null && atom2 != null ? addNewBondWithOrder(atom1.index, atom2.index, order) : null);
   }
 
   public void addBond(Bond bond) {
@@ -688,7 +681,7 @@ public class AtomSetCollection {
     coordinatesAreFractional = tf;
     setCurrentModelInfo("coordinatesAreFractional", Boolean.valueOf(tf));
     if (tf)
-      setGlobalBoolean(GLOBAL_FRACTCOORD);
+      setGlobalBoolean(JC.GLOBAL_FRACTCOORD);
   }
 
   public boolean haveAnisou;
@@ -875,15 +868,17 @@ public class AtomSetCollection {
       atomSetNumbers[iSet] = atomSetCount;
     }
     if (doClearMap) { // false for CASTEP reader only
-      atomSymbolicMap.clear();
-      atomMapAnyCase = false;
-    }
+        clearMap();
+      }
     setCurrentModelInfo("title", collectionName);
   }
 
+  private void clearMap() {
+    atomSymbolicMap.clear();
+    atomMapAnyCase = false;
+  }
+
   public int getAtomSetAtomIndex(int i) {
-    if  (i < 0)
-      System.out.println("??");
     return atomSetAtomIndexes[i];
   }
 
