@@ -28,8 +28,20 @@ package org.jmol.adapter.smarter;
 import java.io.BufferedReader;
 import java.util.Map;
 
+import org.jmol.api.Interface;
+import org.jmol.api.JmolAdapter;
+import org.jmol.api.SymmetryInterface;
+import org.jmol.script.SV;
+import org.jmol.util.BSUtil;
+import org.jmol.util.Logger;
+import org.jmol.util.SimpleUnitCell;
+import org.jmol.viewer.FileManager;
+import org.jmol.viewer.JC;
+import org.jmol.viewer.Viewer;
+
 import javajs.api.GenericBinaryDocument;
 import javajs.api.GenericLineReader;
+import javajs.util.BS;
 import javajs.util.Lst;
 import javajs.util.M3;
 import javajs.util.M4;
@@ -42,16 +54,6 @@ import javajs.util.T3;
 import javajs.util.T4;
 import javajs.util.V3;
 
-import org.jmol.api.Interface;
-import org.jmol.api.JmolAdapter;
-import org.jmol.api.SymmetryInterface;
-import javajs.util.BS;
-import org.jmol.script.SV;
-import org.jmol.symmetry.Symmetry;
-import org.jmol.util.BSUtil;
-import org.jmol.util.Logger;
-import org.jmol.viewer.Viewer;
-
 
 /*
  * Notes 9/2006 Bob Hanson
@@ -134,7 +136,10 @@ import org.jmol.viewer.Viewer;
 
 public abstract class AtomSetCollectionReader implements GenericLineReader {
 
-  public final static float ANGSTROMS_PER_BOHR = 0.5291772f; // used by SpartanArchive
+  public final static float ANGSTROMS_PER_BOHR = 0.5291772f; // used by SpartanArchive and others
+
+  protected static final String CELL_TYPE_CONVENTIONAL = "conventional";
+  protected static final String CELL_TYPE_PRIMITIVE = "primitive";
 
   public boolean isBinary;
   public boolean debugging;
@@ -214,7 +219,7 @@ public abstract class AtomSetCollectionReader implements GenericLineReader {
   private SB loadNote = new SB();
   public boolean doConvertToFractional;
   boolean fileCoordinatesAreFractional;
-  boolean merging;
+  protected boolean merging;
   float symmetryRange;
   private int[] firstLastStep;
   private int lastModelNumber = Integer.MAX_VALUE;
@@ -229,8 +234,8 @@ public abstract class AtomSetCollectionReader implements GenericLineReader {
   public T3 paramsLattice;
   public boolean paramsCentroid;
   private boolean paramsPacked;
-  
-  
+
+
   protected String filePath;
   protected String fileName;
 
@@ -253,7 +258,8 @@ public abstract class AtomSetCollectionReader implements GenericLineReader {
       return;
     debugging = Logger.debugging;
     this.htParams = htParams;
-    filePath = "" + htParams.get("fullPathName");
+    // check for drag-drop or coerced type
+    filePath = FileManager.stripTypePrefix("" + htParams.get("fullPathName"));
     int i = filePath.lastIndexOf('/');
     fileName = filePath.substring(i + 1);
     if (readerOrDocument instanceof BufferedReader)
@@ -294,7 +300,10 @@ public abstract class AtomSetCollectionReader implements GenericLineReader {
 
   private void fixBaseIndices() {
     try {
-    int baseModelIndex = ((Integer) htParams.get("baseModelIndex")).intValue();
+    Integer ii = (Integer) htParams.get("baseModelIndex");
+    if (ii == null)
+      return;
+    int baseModelIndex = ii.intValue();
     baseAtomIndex += asc.ac;
     baseBondIndex += asc.bondCount;
     baseModelIndex += asc.atomSetCount;
@@ -394,6 +403,8 @@ public abstract class AtomSetCollectionReader implements GenericLineReader {
 
   protected boolean isFinalized;
 
+  protected boolean noPack;
+
   protected void finalizeReaderASCR() throws Exception {
     isFinalized = true;
     if (asc.atomSetCount > 0) {
@@ -407,10 +418,11 @@ public abstract class AtomSetCollectionReader implements GenericLineReader {
         asc.centralize();
       if (fillRange != null)// && previousUnitCell == null)
         asc.setInfo("boundbox", fillRange);
+      
       Map<String, Object> info = asc.getAtomSetAuxiliaryInfo(0);
       if (info != null) {
         if (domains != null) {
-          asc.setGlobalBoolean(AtomSetCollection.GLOBAL_DOMAINS);
+          asc.setGlobalBoolean(JC.GLOBAL_DOMAINS);
           String s = ((SV) domains).getMapKeys(2, true);
           int pt = s.indexOf("{ ", 2);
           if (pt >= 0)
@@ -444,7 +456,7 @@ public abstract class AtomSetCollectionReader implements GenericLineReader {
       }
     }
     if (!fixJavaFloat)
-      asc.setInfo("legacyJavaFloat", Boolean.TRUE);
+      asc.setInfo("highPrecision", Boolean.TRUE);
     setLoadNote();
   }
 
@@ -458,7 +470,7 @@ public abstract class AtomSetCollectionReader implements GenericLineReader {
   }
 
   public void setIsPDB() {
-    asc.setGlobalBoolean(AtomSetCollection.GLOBAL_ISPDB);
+    asc.setGlobalBoolean(JC.GLOBAL_ISPDB);
     if (htParams.get("pdbNoHydrogens") != null)
       asc.setInfo("pdbNoHydrogens",
           htParams.get("pdbNoHydrogens"));
@@ -468,10 +480,10 @@ public abstract class AtomSetCollectionReader implements GenericLineReader {
 
   protected void setModelPDB(boolean isPDB) {
     if (isPDB)
-      asc.setGlobalBoolean(AtomSetCollection.GLOBAL_ISPDB);
+      asc.setGlobalBoolean(JC.GLOBAL_ISPDB);
     else  
-      asc.clearGlobalBoolean(AtomSetCollection.GLOBAL_ISPDB);
-    asc.setCurrentModelInfo("isPDB", isPDB ? Boolean.TRUE : null);
+      asc.clearGlobalBoolean(JC.GLOBAL_ISPDB);
+    asc.setCurrentModelInfo(JC.getBoolName(JC.GLOBAL_ISPDB), isPDB ? Boolean.TRUE : null);
   }
 
   private Object finish() {
@@ -505,7 +517,7 @@ public abstract class AtomSetCollectionReader implements GenericLineReader {
     if (asc.errorMessage != null)
       return asc.errorMessage + "\nfor file " + filePath
           + "\ntype " + name;
-    if ((asc.bsAtoms == null ? asc.ac == 0
+    if (!merging && (asc.bsAtoms == null ? asc.ac == 0
         : asc.bsAtoms.nextSetBit(0) < 0)
         && fileType.indexOf("DataOnly") < 0 && asc.atomSetInfo.get("dataOnly") == null)
       return "No atoms found\nfor file " + filePath + "\ntype " + name;
@@ -540,7 +552,7 @@ public abstract class AtomSetCollectionReader implements GenericLineReader {
     Object o = htParams.get("packingError");
     if (o != null)
       packingError = ((Float) o).floatValue();
-    else if (htParams.get("legacyJavaFloat") != null) {
+    else if (htParams.get("highPrecision") != null) {
       // earlier versions were not fully JavaScript compatible
       // because XtalSymmetry.isWithinUnitCell was giving different answers
       // for floats (Java) as for doubles (JavaScript).
@@ -562,14 +574,19 @@ public abstract class AtomSetCollectionReader implements GenericLineReader {
     bsFilter = (requiresBSFilter ? (BS) htParams.get("bsFilter") : null);
     setFilter(null);
     fillRange = htParams.get("fillRange");
-    if (strSupercell != null) {
-      if (!checkFilterKey("NOPACK"))
+    paramsLattice = (T3) htParams.get("lattice");
+    o = htParams.get("supercell");
+    // noPack does not work as advertised
+    noPack = checkFilterKey("NOPACK");
+    if (strSupercell != null && !noPack) {
+      // only for filter cell=
         forcePacked = true;
     }
-    o = htParams.get("supercell");
     if (o instanceof P3) {
-      P3 s = ptSupercell = (P3) o; // only used by CASTEP phonon reader now
+      P3 s = ptSupercell = (P3) o; 
+      if (s.length() != 1) {
       strSupercell = ((int) s.x) + "a," +((int) s.y) + "b," + ((int) s.z) + "c"; 
+      }
     } else if (o instanceof String) {
       strSupercell = (String) o;
     }
@@ -617,7 +634,6 @@ public abstract class AtomSetCollectionReader implements GenericLineReader {
 
     symmetryRange = (htParams.containsKey("symmetryRange") ? ((Float) htParams
         .get("symmetryRange")).floatValue() : 0);
-    paramsLattice = (T3) htParams.get("lattice");
     paramsCentroid = htParams.containsKey("centroid");
     paramsPacked = htParams.containsKey("packed");
     initializeSymmetryOptions();
@@ -733,14 +749,14 @@ public abstract class AtomSetCollectionReader implements GenericLineReader {
     previousUnitCell = unitCellParams;
     iHaveUnitCell = ignoreFileUnitCell;
     if (!ignoreFileUnitCell) {
-      unitCellParams = new float[26];
+      unitCellParams = new float[SimpleUnitCell.PARAM_COUNT];
       //0-5 a b c alpha beta gamma
       //6-21 m00 m01... m33 cartesian-->fractional
       //22-24 supercell.x supercell.y supercell.z
       //25 scaling
-      for (int i = 25; --i >= 0;)
+      for (int i = SimpleUnitCell.PARAM_COUNT; --i >= 0;)
         unitCellParams[i] = Float.NaN;
-      unitCellParams[25] = latticeScaling;
+      unitCellParams[SimpleUnitCell.PARAM_SCALE] = latticeScaling;
       symmetry = null;
     }
     if (!ignoreFileSpaceGroupName)
@@ -776,7 +792,7 @@ public abstract class AtomSetCollectionReader implements GenericLineReader {
     if (ignoreFileSpaceGroupName || name == null)
       return;
     String s = name.trim();
-    if (s.equals(sgName))
+    if (s.length() == 0 || s.equals("HM:") || s.equals(sgName))
       return;
     if (!s.equals("P1"))
       Logger.info("Setting space group name to " + s);
@@ -797,19 +813,20 @@ public abstract class AtomSetCollectionReader implements GenericLineReader {
 
   private void initializeCartesianToFractional() {
     for (int i = 0; i < 16; i++)
-      if (!Float.isNaN(unitCellParams[6 + i]))
+      if (!Float.isNaN(unitCellParams[SimpleUnitCell.PARAM_M4 + i]))
         return; //just do this once
+    // set the matrix to the identity matrix
     for (int i = 0; i < 16; i++)
-      unitCellParams[6 + i] = ((i % 5 == 0 ? 1 : 0));
+      unitCellParams[SimpleUnitCell.PARAM_M4 + i] = ((i % 5 == 0 ? 1 : 0));
     nMatrixElements = 0;
   }
 
   public void clearUnitCell() {
     if (ignoreFileUnitCell)
       return;
-    for (int i = 6; i < 22; i++)
+    for (int i = SimpleUnitCell.PARAM_STD; i < SimpleUnitCell.PARAM_SUPERCELL; i++)
       unitCellParams[i] = Float.NaN;
-    checkUnitCell(6);
+    checkUnitCell(SimpleUnitCell.PARAM_STD);
   }
 
   public float[] ucItems;
@@ -818,23 +835,23 @@ public abstract class AtomSetCollectionReader implements GenericLineReader {
       return;
     if (i == 0 && x == 1 && !allow_a_len_1  || i == 3 && x == 0) {
       if (ucItems == null)
-        ucItems = new float[6];
+        ucItems = new float[SimpleUnitCell.PARAM_STD];
       ucItems[i] = x;
       return;
     }
-    if (ucItems != null && i < 6)
+    if (ucItems != null && i < SimpleUnitCell.PARAM_STD)
       ucItems[i] = x;
 
-    if (!Float.isNaN(x) && i >= 6 && Float.isNaN(unitCellParams[6]))
+    if (!Float.isNaN(x) && i >= SimpleUnitCell.PARAM_M4 && Float.isNaN(unitCellParams[SimpleUnitCell.PARAM_M4]))
       initializeCartesianToFractional();
     unitCellParams[i] = x;
     if (debugging) {
       Logger.debug("setunitcellitem " + i + " " + x);
     }
-    if (i < 6 || Float.isNaN(x))
-      iHaveUnitCell = checkUnitCell(6);
+    if (i < SimpleUnitCell.PARAM_STD || Float.isNaN(x))
+      iHaveUnitCell = checkUnitCell(SimpleUnitCell.PARAM_STD);
     else if (++nMatrixElements == 12)
-      iHaveUnitCell = checkUnitCell(22);
+      iHaveUnitCell = checkUnitCell(SimpleUnitCell.PARAM_M4+16);
   }
 
   protected M3 matUnitCellOrientation;
@@ -853,38 +870,43 @@ public abstract class AtomSetCollectionReader implements GenericLineReader {
       unitCellParams[4] = beta;
     if (gamma != 0)
       unitCellParams[5] = gamma;
-    iHaveUnitCell = checkUnitCell(6);
+    iHaveUnitCell = checkUnitCell(SimpleUnitCell.PARAM_STD);
   }
 
   public void addExplicitLatticeVector(int i, float[] xyz, int i0) {
     if (ignoreFileUnitCell)
       return;
     if (i == 0)
-      for (int j = 0; j < 6; j++)
+      for (int j = 0; j < SimpleUnitCell.PARAM_STD; j++)
         unitCellParams[j] = 0;
     i = 6 + i * 3;
     unitCellParams[i++] = xyz[i0++];
     unitCellParams[i++] = xyz[i0++];
     unitCellParams[i] = xyz[i0];
     if (Float.isNaN(unitCellParams[0])) {
-      for (i = 0; i < 6; i++)
+      for (i = 0; i < SimpleUnitCell.PARAM_STD; i++)
         unitCellParams[i] = -1;
     }
-    iHaveUnitCell = checkUnitCell(15);
+    iHaveUnitCell = checkUnitCell(SimpleUnitCell.PARAM_VABC+9);
+    if (iHaveUnitCell) {
+      if (slabXY || polymerX)
+        unitCellParams[2] = -1;
+      if (polymerX)
+        unitCellParams[1] = -1;
+    }
   }
 
   private boolean checkUnitCell(int n) {
     for (int i = 0; i < n; i++)
       if (Float.isNaN(unitCellParams[i]))
         return false;
-    if (n == 22 && unitCellParams[0] == 1) {
+    if (n == SimpleUnitCell.PARAM_M4+16 && unitCellParams[0] == 1) {
       if (unitCellParams[1] == 1 
           && unitCellParams[2] == 1 
-          && unitCellParams[6] == 1 
-          && unitCellParams[11] == 1 
-          && unitCellParams[16] == 1 
-          )
-        return false; 
+          && unitCellParams[SimpleUnitCell.PARAM_M4] == 1 
+          && unitCellParams[SimpleUnitCell.PARAM_M4+5] == 1 
+          && unitCellParams[SimpleUnitCell.PARAM_M4+10] == 1 
+          ) {
       // this is an mmCIF or PDB case for NMR models having
       // CRYST1    1.000    1.000    1.000  90.00  90.00  90.00 P 1           1          
       // ORIGX1      1.000000  0.000000  0.000000        0.00000                         
@@ -893,6 +915,17 @@ public abstract class AtomSetCollectionReader implements GenericLineReader {
       // SCALE1      1.000000  0.000000  0.000000        0.00000                         
       // SCALE2      0.000000  1.000000  0.000000        0.00000                         
       // SCALE3      0.000000  0.000000  1.000000        0.00000 
+        return false; 
+      }
+    }
+    if (n == SimpleUnitCell.PARAM_STD && Float.isNaN(unitCellParams[SimpleUnitCell.PARAM_VABC])) {
+      if (slabXY && unitCellParams[2] > 0) {
+        SimpleUnitCell.addVectors(unitCellParams);
+        unitCellParams[2] = -1;
+      } else if (polymerX && unitCellParams[1] > 0) {
+        SimpleUnitCell.addVectors(unitCellParams);
+        unitCellParams[1] = unitCellParams[2] = -1;
+    }
     }
     if (doApplySymmetry) {
       getSymmetry();
@@ -941,7 +974,7 @@ public abstract class AtomSetCollectionReader implements GenericLineReader {
     setFractionalCoordinates(toFrac);
   }
   protected SymmetryInterface getNewSymmetry() {
-    return symmetry = (Symmetry) getInterface("org.jmol.symmetry.Symmetry");
+    return symmetry = (SymmetryInterface) getInterface("org.jmol.symmetry.Symmetry");
   }
 
   public void setFractionalCoordinates(boolean TF) {
@@ -952,7 +985,7 @@ public abstract class AtomSetCollectionReader implements GenericLineReader {
   /////////// FILTER /////////////////
 
   protected BS bsFilter;
-  public String filter;
+  public String filter, filterCased;
   public boolean haveAtomFilter;
   private boolean filterAltLoc;
   private boolean filterGroup3;
@@ -963,6 +996,7 @@ public abstract class AtomSetCollectionReader implements GenericLineReader {
   private String filterAtomNameTerminator = ";";  
   private boolean filterElement;
   protected boolean filterHetero;
+  protected boolean filterAllHetero;
   private boolean filterEveryNth;
   String filterSymop;
   private int filterN;
@@ -984,6 +1018,12 @@ public abstract class AtomSetCollectionReader implements GenericLineReader {
 
   public boolean allow_a_len_1 = false;
 
+  public boolean slabXY;
+
+  private boolean polymerX;
+
+  // xtal structures -- SLAB
+  
   // ALL:  "CENTER" "REVERSEMODELS"
   // ALL:  "SYMOP=n"
   // MANY: "NOVIB" "NOMO"
@@ -1014,7 +1054,10 @@ public abstract class AtomSetCollectionReader implements GenericLineReader {
     } else {
       // from PDB REMARK350()
       bsFilter = null;
+      filterCased = null;
     }
+    if (filterCased == null)
+      filterCased = (filter0 == null ? null : filter0 + ";");
     if (filter0 != null)
       filter0 = filter0.toUpperCase();
     filter = filter0;
@@ -1027,20 +1070,25 @@ public abstract class AtomSetCollectionReader implements GenericLineReader {
     useAltNames = checkFilterKey("ALTNAME");
     reverseModels = checkFilterKey("REVERSEMODELS");
     allow_a_len_1 =checkFilterKey("TOPOS");
+    slabXY = checkFilterKey("SLABXY");
+    polymerX = !slabXY && checkFilterKey("POLYMERX");
 
     if (filter == null)
       return;
     if (checkFilterKey("HETATM")) {
       filterHetero = true;
       filter = PT.rep(filter, "HETATM", "HETATM-Y");
+      filterCased = PT.rep(filterCased, "HETATM", "HETATM-Y");
     }
     if (checkFilterKey("ATOM")) {
       filterHetero = true;
       filter = PT.rep(filter, "ATOM", "HETATM-N");
+      filterCased = PT.rep(filterCased, "ATOM", "HETATM-N");
     }
     
     // can't use getFilter() here because form includes a semicolon:
     // cell=a+b,a-b,c;0,1/2,1/2 
+    // also allows for NOPACKCELL by documentation 14.0
     if (checkFilterKey("CELL="))  
       strSupercell = filter.substring(filter.indexOf("CELL=") + 5).toLowerCase(); // must be last filter option
     nameRequired = PT.getQuotedAttribute(filter, "NAME");
@@ -1053,13 +1101,14 @@ public abstract class AtomSetCollectionReader implements GenericLineReader {
       filter0 = filter = PT.rep(filter, "NAME=", "");
     }
     filterAtomName = checkFilterKey("*.") || checkFilterKey("!.");
-    if (filter.startsWith("_") || filter.indexOf(";_") >= 0)
+    if (filter.startsWith("_") || filter.startsWith("!_") || filter.indexOf(";_") >= 0)
       filterElement = checkFilterKey("_");
 
     filterGroup3 = checkFilterKey("[");
     filterChain = checkFilterKey(":");
     filterAltLoc = checkFilterKey("%");
     filterEveryNth = checkFilterKey("/=");
+    filterAllHetero = checkFilterKey("ALLHET");
     if (filterEveryNth)
       filterN = parseIntAt(filter, filter.indexOf("/=") + 2);
     else if (filter.startsWith("=") || filter.indexOf(";=") >= 0)
@@ -1078,24 +1127,32 @@ public abstract class AtomSetCollectionReader implements GenericLineReader {
       htParams.put("bsFilter", bsFilter);
       filter = (";" + filter + ";").replace(',', ';');
       String s = getFilter("LATTICESCALING=");
-      if (s != null && unitCellParams.length > 25) 
-        unitCellParams[25] = latticeScaling = parseFloatStr(s); 
+      if (s != null && unitCellParams.length > SimpleUnitCell.PARAM_SCALE) 
+        unitCellParams[SimpleUnitCell.PARAM_SCALE] = latticeScaling = parseFloatStr(s); 
       s = getFilter("SYMOP=");
       if (s != null)
         filterSymop = " " + s + " ";
       Logger.info("filtering with " + filter);
       if (haveAtomFilter) {
         int ipt;
-        filter1 = filter;
+        filter1Cased = filterCased;
+        filter2Cased = "";
         if ((ipt = filter.indexOf("|")) >= 0) {
-          filter1 = filter.substring(0, ipt).trim() + ";";
-          filter2 = ";" + filter.substring(ipt).trim();
+          filter1Cased = filter.substring(0, ipt).trim() + ";";
+          filter2Cased = ";" + filter.substring(ipt).trim();
         }
+        filter1 = filter1Cased.toUpperCase();
+        filter2 = (filter2Cased.length() == 0 ? null : filter2Cased.toUpperCase());
       }
     }
   }
 
-  private String filter1, filter2;
+  private String filter1, filter2, filter1Cased, filter2Cased;
+
+  public String getFilterWithCase(String key) {
+    int pt = (filterCased == null ? -1 : filterCased.toUpperCase().indexOf(key.toUpperCase()));
+    return (pt < 0 ? null : filterCased.substring(pt + key.length(), filterCased.indexOf(";", pt)));
+  }
 
   public String getFilter(String key) {
     int pt = (filter == null ? -1 : filter.indexOf(key));
@@ -1132,10 +1189,10 @@ public abstract class AtomSetCollectionReader implements GenericLineReader {
     if (!haveAtomFilter)
       return true;
     // cif, mdtop, pdb, gromacs, pqr
-    boolean isOK = checkFilter(atom, filter1);
+    boolean isOK = checkFilter(atom, filter1, filter1Cased);
     if (filter2 != null)
-      isOK |= checkFilter(atom, filter2);
-    if (isOK && filterEveryNth)
+      isOK |= checkFilter(atom, filter2, filter2Cased);
+    if (isOK && filterEveryNth && (!atom.isHetero || !filterAllHetero))
       isOK = (((nFiltered++) % filterN) == 0);
     bsFilter.setBitTo(iAtom >= 0 ? iAtom : asc.ac, isOK);
     return isOK;
@@ -1145,9 +1202,12 @@ public abstract class AtomSetCollectionReader implements GenericLineReader {
    * 
    * @param atom
    * @param f
+   * @param fCased 
    * @return  true if a filter is found
    */
-  private boolean checkFilter(Atom atom, String f) {
+  private boolean checkFilter(Atom atom, String f, String fCased) {
+    if (atom.isHetero && filterAllHetero)
+      return true;
     return (!filterGroup3 || atom.group3 == null || !filterReject(f, "[",
         atom.group3.toUpperCase() + "]"))
         && (!filterAtomName || allowAtomName(atom.atomName, f))
@@ -1155,7 +1215,7 @@ public abstract class AtomSetCollectionReader implements GenericLineReader {
             || atom.atomName.toUpperCase().indexOf("\0" + filterAtomTypeStr) >= 0)
         && (!filterElement || atom.elementSymbol == null || !filterReject(f, "_",
             atom.elementSymbol.toUpperCase() + ";"))
-        && (!filterChain || atom.chainID == 0 || !filterReject(f, ":", ""
+        && (!filterChain || atom.chainID == 0 || !filterReject(fCased, ":", ""
             + vwr.getChainIDStr(atom.chainID)))
         && (!filterAltLoc || atom.altLoc == '\0' || !filterReject(
             f, "%", "" + atom.altLoc))
@@ -1317,6 +1377,12 @@ public abstract class AtomSetCollectionReader implements GenericLineReader {
     finalizeSubclassSymmetry(sym != null);
     //if (sym != null && ptSupercell != null)
       //asc.getXSymmetry().finalizeUnitCell(ptSupercell);
+    
+    if (merging && sym != null && iHaveFractionalCoordinates && iHaveUnitCell && iHaveSymmetryOperators) {
+      // only do this if XtalSymmetry has not done it already
+      fractionalizeCoordinates(false);
+      addJmolScript("modelkit spacegroup P1");
+    }
     initializeSymmetry();
     return sym;
   }
