@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2021-09-24 10:29:11 -0500 (Fri, 24 Sep 2021) $
- * $Revision: 22231 $
+ * $Date: 2022-07-19 09:55:17 +0530 (Tue, 19 Jul 2022) $
+ * $Revision: 22431 $
  *
  * Copyright (C) 2003-2005  Miguel, Jmol Development, www.jmol.org
  *
@@ -30,17 +30,16 @@ import java.io.InputStream;
 import java.util.Map;
 import java.util.StringTokenizer;
 
-import javajs.api.GenericBinaryDocument;
-import javajs.util.LimitedLineReader;
-import javajs.util.PT;
-import javajs.util.Rdr;
-
 import org.jmol.api.Interface;
 import org.jmol.util.Logger;
 import org.jmol.viewer.FileManager;
 import org.jmol.viewer.JC;
 import org.jmol.viewer.Viewer;
 
+import javajs.api.GenericBinaryDocument;
+import javajs.util.LimitedLineReader;
+import javajs.util.PT;
+import javajs.util.Rdr;
 
 public class Resolver {
 
@@ -54,11 +53,11 @@ public class Resolver {
                  "WebMO;MO;", // MO is for XmlMolpro 
     "pdb.", ";Pdb;Pqr;P2n;JmolData;",
     "pymol.", ";PyMOL;",
-    "simple.", ";Alchemy;Ampac;Cube;FoldingXyz;GhemicalMM;HyperChem;Jme;JSON;Mopac;MopacArchive;Tinker;Input;FAH;",
+    "simple.", ";Alchemy;Ampac;Cube;FoldingXyz;GhemicalMM;HyperChem;Jme;JSON;Mopac;MopacArchive;Tinker;Input;FAH;Orca;",
     "spartan.", ";Spartan;SpartanSmol;Odyssey;",
     "xtal.", ";Abinit;Aims;Bilbao;Castep;Cgd;Crystal;Dmol;Espresso;Gulp;Jana;Magres;Shelx;Siesta;VaspOutcar;" +
-             "VaspPoscar;Wien2k;Xcrysden;",
-    "xml.",  ";XmlArgus;XmlCml;XmlChem3d;XmlMolpro;XmlOdyssey;XmlXsd;XmlVasp;XmlQE;",
+             "VaspPoscar;Wien2k;Xcrysden;PWmat;Optimade;",
+    "xml.",  ";XmlChemDraw;XmlArgus;XmlCml;XmlChem3d;XmlMolpro;XmlOdyssey;XmlXsd;XmlVasp;XmlQE;",
   };
   
 
@@ -84,7 +83,7 @@ public class Resolver {
    */
   public static String getFileType(BufferedReader br) {
     try {
-      return determineAtomSetCollectionReader(br, false);
+      return determineAtomSetCollectionReader(br, null);
     } catch (Exception e) {
       return null;
     }
@@ -110,9 +109,7 @@ public class Resolver {
     fullName = FileManager.fixDOSName(fullName);
     String errMsg = null;
     if (type == null) {
-      type = (String) htParams.get("filter");
-      int pt = (type == null ? -1 : type.toLowerCase().indexOf("filetype"));
-      type = (pt < 0 ? null : type.substring(pt + 8, (type+ ";").indexOf(";", pt)).replace('=', ' ').trim());
+      type = SmarterJmolAdapter.getFileTypefromFilter((String) htParams.get("filter"));
     }      
     if (type != null) {
       readerName = getReaderFromType(type);
@@ -123,7 +120,7 @@ public class Resolver {
       else
         Logger.info("The Resolver assumes " + readerName);
     } else {
-      readerName = determineAtomSetCollectionReader(readerOrDocument, true);
+      readerName = determineAtomSetCollectionReader(readerOrDocument, htParams);
       if (readerName.charAt(0) == '\n') {
         type = (String) htParams.get("defaultType");
         if (type != null) {
@@ -179,9 +176,10 @@ public class Resolver {
 
     String set;
     int pt;
-    for (int i = readerSets.length; --i >= 0;)
+    for (int i = readerSets.length; --i >= 0;) {
       if ((pt = (set = readerSets[i--]).toLowerCase().indexOf(type)) >= 0)
         return set.substring(pt + 1, set.indexOf(";", pt + 2));
+    }
     return null;
   }
   
@@ -226,12 +224,12 @@ public class Resolver {
    * involved -- molecular dynamics coordinate files, for instance (mdcrd).
    * 
    * @param readerOrDocument
-   * @param returnLines
+   * @param htParams
    * @return readerName or a few lines, if requested, or null
    * @throws Exception
    */
   private static String determineAtomSetCollectionReader(Object readerOrDocument,
-                                                         boolean returnLines)
+                                                         Map<String, Object> htParams)
       throws Exception {
 
     // We must do this in a very specific order. DON'T MESS WITH THIS!
@@ -241,17 +239,19 @@ public class Resolver {
     if (readerOrDocument instanceof GenericBinaryDocument) {
       GenericBinaryDocument doc = (GenericBinaryDocument) readerOrDocument;
       readerName = getBinaryType(doc.getInputStream());
-      return (readerName == null ? "binary file type not recognized" : readerName);
+      return (readerName == null ? "binary file type not recognized"
+          : readerName);
     }
     if (readerOrDocument instanceof InputStream) {
       readerName = getBinaryType((InputStream) readerOrDocument);
       if (readerName != null)
         return readerName;
-      readerOrDocument = Rdr.getBufferedReader(new BufferedInputStream((InputStream) readerOrDocument), null);
+      readerOrDocument = Rdr.getBufferedReader(
+          new BufferedInputStream((InputStream) readerOrDocument), null);
     }
 
-    LimitedLineReader llr = new LimitedLineReader(
-        (BufferedReader) readerOrDocument, 16384);
+    BufferedReader rdr = (BufferedReader) readerOrDocument;
+    LimitedLineReader llr = new LimitedLineReader(rdr, 16384);
 
     String leader = llr.getHeader(LEADER_CHAR_MAX).trim();
 
@@ -274,10 +274,12 @@ public class Resolver {
       return (readerName.equals("Xml") ? getXmlType(llr.getHeader(0))
           : readerName);
     }
-
+   
     // now allow identification in first 16 lines
     // excluding those starting with "#"
 
+    String msg;
+    boolean isJSONMap = (leader.charAt(0) == '{');
     String[] lines = new String[16];
     int nLines = 0;
     for (int i = 0; i < lines.length; ++i) {
@@ -298,18 +300,34 @@ public class Resolver {
 
     // Test 5. check content of initial 16K bytes of file 
 
-    if ((readerName = checkHeaderContains(llr.getHeader(0))) != null)
+    if ((readerName = checkHeaderContains(llr.getHeader(0))) != null) {
       return readerName;
+    }
 
     // Test 6. check special file formats (pass 2) 
 
     if ((readerName = checkSpecial2(lines)) != null)
       return readerName;
 
+    if (isJSONMap) {
+      // unfortunately, we need to read this data fully from the reader.
+      //
+      String json = rdr.readLine();
+      if ((readerName = checkJSONContains(json)) != null) {
+        if (htParams != null)
+          htParams.put("fileData", json);
+        return readerName;
+      }
+      msg = (htParams == null ? null
+          : json.substring(0, Math.min(100, json.length())));
+    } else {
+      msg = (htParams == null ? null
+          : "\n" + lines[0] + "\n" + lines[1] + "\n" + lines[2] + "\n");
+    }
+
     // Failed to identify file type
 
-    return (returnLines ? "\n" + lines[0] + "\n" + lines[1] + "\n" + lines[2]
-        + "\n" : null);
+    return msg;
   }
 
   ////////////////////////////////////////////////////////////////
@@ -411,7 +429,6 @@ public class Resolver {
 
     if (nLines == 1 && lines[0].length() > 0 && PT.isDigit(lines[0].charAt(0)))
       return "Jme"; //only one line, and that line starts with a number 
-
     if (checkMopacGraphf(lines))
       return "MopacGraphf"; //must be prior to checkFoldingXyz and checkMol
     if (checkOdyssey(lines))
@@ -428,11 +445,15 @@ public class Resolver {
       return "Xyz";
     case 2:
       return "Bilbao";
+    case 3: 
+      return "PWmat";
     }
     if (checkAlchemy(lines[0]))
       return "Alchemy";
     if (checkFoldingXyz(lines))
       return "FoldingXyz";
+    if (checkXSF(lines))
+      return "Xcrysden";
     if (checkCube(lines))
       return "Cube";
     if (checkWien2k(lines))
@@ -444,6 +465,14 @@ public class Resolver {
     return null;
   }
   
+  private static boolean checkXSF(String[] lines) {
+    int i = 0;
+    while (lines[i].length() == 0) {
+      i++;
+    }
+    return (lines[i].startsWith("ANIMSTEPS ") || lines[i].equals("ATOMS\n") && PT.parseInt(lines[i + 1]) > 0);
+  }
+
   private static boolean checkAims(String[] lines) {
 
     // use same tokenizing mechanism as in AimsReader.java to also recognize
@@ -473,7 +502,7 @@ public class Resolver {
     11 ATOMS,    12 BONDS,     0 CHARGES
     */
     int pt;
-    if ((pt = line.indexOf("ATOMS")) >= 0 && line.indexOf("BONDS") > pt) {
+    if ((pt = line.indexOf("ATOMS")) > 0 && line.indexOf("BONDS") > pt) {
         int n = PT.parseInt(line.substring(0, pt).trim());
         return (n > 0);
       }
@@ -594,9 +623,22 @@ public class Resolver {
  
   private static int checkXyz(String[] lines) {
     // first and third lines numerical --> Bilbao format
-    if (isInt(lines[0].trim()))
-      return (isInt(lines[2].trim()) ? 2 : 1);
-    return (lines[0].indexOf("Bilabao Crys") >= 0 ? 2 : 0);
+    // first int and line[5] starts with "POSITION" (case insensitice) --> PWmat atom.config
+
+    boolean checkPWM = false;
+    int i = PT.parseInt(lines[0]);
+    if (i >= 0 && lines[0].trim().equals("" + i)) {
+      if (isInt(lines[2]))
+        return 2; 
+      checkPWM = true;
+    }
+    if (lines[0].indexOf("Bilbao Crys") >= 0)
+      return 2;
+    String s;
+    if ((checkPWM || lines.length > 5 && i > 0)
+        && ((s = lines[1].trim().toUpperCase()).startsWith("LATTICE VECTOR") || s.equals("LATTICE")))
+      return 3;
+    return (checkPWM ? 1 : 0);
   }
   
   ////////////////////////////////////////////////////////////////
@@ -662,13 +704,16 @@ public class Resolver {
   private final static String[] vaspOutcarLineStartRecords = 
   { "VaspOutcar", " vasp.", " INCAR:" };
 
+  private final static String[] orcaInputLineStartRecords = 
+  { "Orca", "* xyz", "*xyz" };
+
   private final static String[][] lineStartsWithRecords =
   { mmcifLineStartRecords, cifLineStartRecords,
     pdbLineStartRecords, cgdLineStartRecords, shelxLineStartRecords, 
     ghemicalMMLineStartRecords, jaguarLineStartRecords, 
     mdlLineStartRecords, spartanSmolLineStartRecords, csfLineStartRecords, 
     mol2Records, mdTopLineStartRecords, hyperChemLineStartRecords,
-    vaspOutcarLineStartRecords
+    vaspOutcarLineStartRecords, orcaInputLineStartRecords
     };
 
  
@@ -678,7 +723,24 @@ public class Resolver {
 
   private static String checkHeaderContains(String header) throws Exception {
     for (int i = 0; i < headerContainsRecords.length; ++i) {
-      String[] recordTags = headerContainsRecords[i];
+      String fileType = checkHeaderRecords(header, headerContainsRecords[i]);
+      if (fileType != null)
+        return fileType;
+    }
+    return null;
+  }
+
+  private static String checkJSONContains(String header) throws Exception {
+    for (int i = 0; i < jsonContainsRecords.length; ++i) {
+      String fileType = checkHeaderRecords(header, jsonContainsRecords[i]);
+      if (fileType != null)
+        return fileType;
+    }
+    return null;
+  }
+
+  private static String checkHeaderRecords(String header, String[] recordTags) throws Exception {
+    
       for (int j = 1; j < recordTags.length; ++j) {
         String recordTag = recordTags[j];
         if (header.indexOf(recordTag) < 0)
@@ -696,7 +758,6 @@ public class Resolver {
               ? getXmlType(header) 
             : null);
       }
-    }
     return null;
   }
 
@@ -710,6 +771,9 @@ public class Resolver {
     if (header.indexOf("C3XML") >= 0) {
       return "XmlChem3d";
     }
+    if (header.indexOf("CDXML") >= 0) {
+      return "XmlChemDraw";
+    }
     if (header.indexOf("arguslab") >= 0) {
       return "XmlArgus";
     }
@@ -809,6 +873,12 @@ public class Resolver {
   private final static String[] qcJsonContainsRecords = 
   { "QCJSON", "\"QCJSON" };
 
+  private final static String[] optimadeContainsRecords = 
+  { "Optimade", "\"cartesian_site_positions\":", "\"api_version\":", "optimade"};
+
+  private final static String[] orcaContainsRecords = 
+  { "Orca", "* O   R   C   A *" };
+
   /*
   private final static String[] gaussianWfnRecords =
   { "GaussianWfn", "MO ORBITALS" };
@@ -838,7 +908,11 @@ public class Resolver {
     espressoContainsRecords, siestaContainsRecords, xcrysDenContainsRecords,
     mopacArchiveContainsRecords,abinitContainsRecords,gaussianFchkContainsRecords,
     inputContainsRecords, aflowContainsRecords, magCifContainsRecords, 
-    qcJsonContainsRecords    
+    qcJsonContainsRecords, optimadeContainsRecords, orcaContainsRecords,
+  };
+
+  private final static String[][] jsonContainsRecords =
+  { optimadeContainsRecords 
   };
   
   ////////////////////////////////////////////////////////////////
