@@ -28,6 +28,17 @@ import java.util.Hashtable;
 import java.util.Map;
 import java.util.Map.Entry;
 
+import org.jmol.api.JmolModulationSet;
+import org.jmol.api.SymmetryInterface;
+import org.jmol.symmetry.Symmetry;
+import org.jmol.symmetry.SymmetryOperation;
+import org.jmol.util.BSUtil;
+import org.jmol.util.SimpleUnitCell;
+import org.jmol.util.Tensor;
+import org.jmol.util.Vibration;
+import org.jmol.viewer.JC;
+
+import javajs.util.BS;
 import javajs.util.Lst;
 import javajs.util.M3;
 import javajs.util.M4;
@@ -38,16 +49,6 @@ import javajs.util.SB;
 import javajs.util.T3;
 import javajs.util.V3;
 
-import org.jmol.api.JmolModulationSet;
-import org.jmol.api.SymmetryInterface;
-import javajs.util.BS;
-import org.jmol.symmetry.Symmetry;
-import org.jmol.symmetry.SymmetryOperation;
-import org.jmol.util.BSUtil;
-import org.jmol.util.SimpleUnitCell;
-import org.jmol.util.Tensor;
-import org.jmol.util.Vibration;
-
 /**
  * 
  * A class used by AtomSetCollection for building the symmetry of a model and
@@ -59,6 +60,7 @@ public class XtalSymmetry {
   private final static int PARTICLE_NONE = 0;
   private final static int PARTICLE_CHAIN = 1;
   private final static int PARTICLE_SYMOP = 2;
+  private static final float MAX_INTERCHAIN_BOND_2 = 25; // allowing for hydrogen bonds
 
   private AtomSetCollectionReader acr;
   private AtomSetCollection asc;
@@ -131,7 +133,7 @@ public class XtalSymmetry {
   public SymmetryInterface symmetry;
 
   public SymmetryInterface getSymmetry() {
-    return (symmetry == null ?(symmetry = (Symmetry) acr.getInterface("org.jmol.symmetry.Symmetry")) : symmetry);
+    return (symmetry == null ?(symmetry = (SymmetryInterface) acr.getInterface("org.jmol.symmetry.Symmetry")) : symmetry);
   }
 
   SymmetryInterface setSymmetry(SymmetryInterface symmetry) {
@@ -145,11 +147,6 @@ public class XtalSymmetry {
 
   private void setLatticeCells() {
 
-    //    int[] latticeCells, boolean applySymmetryToBonds,
-    //  }
-    //                       boolean doPackUnitCell, boolean doCentroidUnitCell,
-    //                       boolean centroidPacked, String strSupercell,
-    //                       P3 ptSupercell) {
     //set when unit cell is determined
     // x <= 555 and y >= 555 indicate a range of cells to load
     // AROUND the central cell 555 and that
@@ -170,8 +167,6 @@ public class XtalSymmetry {
     doCentroidUnitCell = acr.doCentroidUnitCell;
     centroidPacked = acr.centroidPacked;
     filterSymop = acr.filterSymop;
-    //if (acr.strSupercell == null)
-      //setSupercellFromPoint(acr.ptSupercell);
   }
 
   private void setUnitCell(float[] info, M3 matUnitCellOrientation,
@@ -189,7 +184,7 @@ public class XtalSymmetry {
       }
       trajectoryUnitCells.addLast(unitCellParams);
     }
-    asc.setGlobalBoolean(AtomSetCollection.GLOBAL_UNITCELLS);
+    asc.setGlobalBoolean(JC.GLOBAL_UNITCELLS);
     getSymmetry().setUnitCell(unitCellParams, false);
     // we need to set the auxiliary info as well, because 
     // ModelLoader creates a new symmetry object.
@@ -263,7 +258,7 @@ public class XtalSymmetry {
   }
 
   public void setSpaceGroupFrom(SymmetryInterface readerSymmetry) {
-    getSymmetry().setSpaceGroupFrom(readerSymmetry);
+    getSymmetry().setSpaceGroupTo(readerSymmetry.getSpaceGroup());
   }
 
   private void setAtomSetSpaceGroupName(String spaceGroupName) {
@@ -298,7 +293,7 @@ public class XtalSymmetry {
     rmaxx = rmaxy = rmaxz = -Float.MAX_VALUE;
     P3 pt0 = null;
     if (acr.latticeType == null)
-      acr.latticeType = symmetry.getLatticeType();
+      acr.latticeType = "" + symmetry.getLatticeType();
     if (acr.isPrimitive) {
       asc.setCurrentModelInfo("isprimitive", Boolean.TRUE);
       if (!"P".equals(acr.latticeType) || acr.primitiveToCrystal != null) {
@@ -311,10 +306,10 @@ public class XtalSymmetry {
       if (acr.fillRange instanceof String) {
 
         String type = (String) acr.fillRange; // conventional or primitive
-        if (type.equals("conventional")) {
+        if (type.equals(AtomSetCollectionReader.CELL_TYPE_CONVENTIONAL)) {
           acr.fillRange = symmetry.getConventionalUnitCell(acr.latticeType,
               acr.primitiveToCrystal);
-        } else if (type.equals("primitive")) {
+        } else if (type.equals(AtomSetCollectionReader.CELL_TYPE_PRIMITIVE)) {
           acr.fillRange = symmetry.getUnitCellVectors();
           symmetry.toFromPrimitive(true, acr.latticeType.charAt(0),
               (T3[]) acr.fillRange, acr.primitiveToCrystal);
@@ -351,7 +346,9 @@ public class XtalSymmetry {
         sym2.toFractional(pt0, false);
         if (acr.fixJavaFloat)
           PT.fixPtFloats(pt0, PT.FRACTIONAL_PRECISION);
-        if (!isWithinCell(ndims, pt0, 0, 1, 0, 1, 0, 1, packingError))
+        if (acr.noPack
+            ? !removePacking(ndims, pt0, 0, 1, 0, 1, 0, 1, packingError)
+            : !isWithinCell(ndims, pt0, 0, 1, 0, 1, 0, 1, packingError))
           bsAtoms.clear(i);
 
       }
@@ -367,8 +364,12 @@ public class XtalSymmetry {
     if (isSuper) {
       // expand range to accommodate this alternative cell
       // oabc will be cartesian
-      oabc = symmetry.getV0abc(supercell);
-      if (oabc != null) {
+      M4 m = new M4();
+      if (mident == null)
+        mident = new M4();
+      oabc = symmetry.getV0abc(supercell, m);
+      if (oabc != null && !m.equals(mident)) {
+        // flag this to set symmetry to P1 in the end
         // set the bounds for atoms in the new unit cell
         // in terms of the old unit cell
         setMinMax(dim, kcode, maxX, maxY, maxZ);
@@ -390,6 +391,7 @@ public class XtalSymmetry {
       supercell = null;
       oabc = null;
     } else {
+      asc.setGlobalBoolean(JC.GLOBAL_SUPERCELL);
       boolean doPack0 = doPackUnitCell;
       doPackUnitCell = doPack0;//(doPack0 || oabc != null && acr.forcePacked);
       bsAtoms = updateBSAtoms();
@@ -437,13 +439,13 @@ public class XtalSymmetry {
     setMinMax(dim, kcode, maxX, maxY, maxZ);
     if (oabc == null) {
       applyAllSymmetry(acr.ms, bsAtoms);
-      if (!applySymmetryToBonds || !acr.doPackUnitCell)
+      if (!acr.noPack && (!applySymmetryToBonds || !acr.doPackUnitCell))
         return;
       // fall through if packed and there are bonds, that is, when we did not shift atoms
       // so we reset to the original min and max and then trim.
       setMinMax(dim, kcode, maxX, maxY, maxZ);
     }
-    if (acr.forcePacked || acr.doPackUnitCell) {
+    if (acr.forcePacked || acr.doPackUnitCell || acr.noPack) {
       trimToUnitCell(iAtomFirst);
     }
 
@@ -460,13 +462,22 @@ public class XtalSymmetry {
     // trim atom set based on current min/max
     Atom[] atoms = asc.atoms;
     BS bs = updateBSAtoms();
+    if (acr.noPack) {
     for (int i = bs.nextSetBit(iAtomFirst); i >= 0; i = bs
         .nextSetBit(i + 1)) {
+        if (!removePacking(ndims, atoms[i], minXYZ.x, maxXYZ.x, minXYZ.y,
+            maxXYZ.y, minXYZ.z, maxXYZ.z, packingError))
+          bs.clear(i);
+      }
+    } else {
+      for (int i = bs.nextSetBit(iAtomFirst); i >= 0; i = bs
+          .nextSetBit(i + 1)) {
       if (!isWithinCell(ndims, atoms[i], minXYZ.x, maxXYZ.x, minXYZ.y,
           maxXYZ.y, minXYZ.z, maxXYZ.z, packingError))
         bs.clear(i);
     }
   }
+  }
 
   /**
    * Update asc.bsAtoms to include all atoms, or at least all
@@ -570,6 +581,15 @@ public class XtalSymmetry {
         && (ndims < 3 || pt.z > minZ - slop && pt.z < maxZ + slop));
   }
   
+  public boolean removePacking(int ndims, P3 pt, float minX, float maxX,
+                              float minY, float maxY, float minZ, float maxZ,
+                              float slop) {
+    return (pt.x > minX - slop && pt.x < maxX - slop
+        && (ndims < 2 || pt.y > minY - slop && pt.y < maxY - slop) 
+        && (ndims < 3 || pt.z > minZ - slop && pt.z < maxZ - slop));
+  }
+
+  
   //  /**
   //   * A problem arises when converting to JavaScript, because JavaScript numbers are all
   //   * doubles, while here we have floats. So what we do is to multiply by a number that
@@ -626,12 +646,12 @@ public class XtalSymmetry {
     applySymmetryToBonds = acr.applySymmetryToBonds;
     doPackUnitCell = acr.doPackUnitCell && !applySymmetryToBonds;
     bondCount0 = asc.bondCount;
+    ndims = (int) symmetry.getUnitCellInfoType(SimpleUnitCell.INFO_DIMENSIONS);
     finalizeSymmetry(symmetry);
     int operationCount = symmetry.getSpaceGroupOperationCount();
     BS excludedOps = (acr.thisBiomolecule == null ? null : new BS());
     if (excludedOps != null)
       asc.checkSpecial = true;
-    ndims = (int) symmetry.getUnitCellInfoType(SimpleUnitCell.INFO_DIMENSIONS);
     SimpleUnitCell.setMinMaxLatticeParameters(ndims, minXYZ, maxXYZ, 0);
     latticeOp = symmetry.getLatticeOp();
     latticeOnly = (asc.checkLatticeOnly && latticeOp >= 0); // CrystalReader
@@ -731,8 +751,8 @@ public class XtalSymmetry {
         for (int j = 0; j < nn; j++) {
           M4 m = M4.newM4(lstNCS.get(j));
           m.mul2(m1, m);
-          if (doNormalize)
-            SymmetryOperation.setOffset(m, atoms, firstAtom, noSymmetryCount);
+          if (doNormalize && noSymmetryCount > 0)
+            SymmetryOperation.normalizeOperationToCentroid(3, m, atoms, firstAtom, noSymmetryCount);
           lstNCS.addLast(m);
         }
       }
@@ -777,7 +797,7 @@ public class XtalSymmetry {
             if (doPackUnitCell) {
 
               // 555 cell 
-              sym.toUnitCell(c, ptOffset);
+              sym.toUnitCellRnd(c, ptOffset);
               pttemp.setT(c);
               sym.toFractional(pttemp, false);
               if (acr.fixJavaFloat)
@@ -851,7 +871,8 @@ public class XtalSymmetry {
   private int symmetryAddAtoms(int transX, int transY, int transZ,
                                int baseCount, int pt, int iCellOpPt,
                                P3[] cartesians, MSInterface ms, BS excludedOps,
-                               int[] atomMap) throws Exception {
+                               int[] atomMap)
+      throws Exception {
     boolean isBaseCell = (baseCount == 0);
     boolean addBonds = (atomMap != null);
     if (doPackUnitCell)
@@ -910,8 +931,8 @@ public class XtalSymmetry {
 
       int pt0 = firstAtom + (checkSpecial || excludedOps != null ? pt
           : checkRange111 ? baseCount : 0);
-      float spinOp = (iSym >= nOp ? 0 : asc.vibScale == 0 ? sym
-          .getSpinOp(iSym) : asc.vibScale);
+      float spinOp = (iSym >= nOp ? 0
+          : asc.vibScale == 0 ? sym.getSpinOp(iSym) : asc.vibScale);
       int i0 = Math.max(firstAtom,
           (bsAtoms == null ? 0 : bsAtoms.nextSetBit(0)));
       boolean checkDistance = checkDistances;
@@ -927,12 +948,12 @@ public class XtalSymmetry {
           continue;
 
         if (ms == null) {
-          sym.newSpaceGroupPoint(iSym, a, pttemp, transX, transY, transZ,
-              (iSym >= nOp ? lstNCS.get(iSym - nOp) : null));
+          sym.newSpaceGroupPoint(a, iSym,
+              (iSym >= nOp ? lstNCS.get(iSym - nOp) : null), transX, transY,
+              transZ, pttemp);
         } else {
           sym = ms.getAtomSymmetry(a, this.symmetry);
-          sym.newSpaceGroupPoint(iSym, a, pttemp, transX, transY, transZ,
-              null);
+          sym.newSpaceGroupPoint(a, iSym, null, transX, transY, transZ, pttemp);
           // COmmensurate structures may use a symmetry operator
           // to changes space groups.
           code = sym.getSpaceGroupOperationCode(iSym);
@@ -948,7 +969,7 @@ public class XtalSymmetry {
         P3 c = P3.newP(pttemp); // cartesian position
         sym.toCartesian(c, false);
         if (doPackUnitCell) {
-          sym.toUnitCell(c, ptOffset);
+          sym.toUnitCellRnd(c, ptOffset);
           pttemp.setT(c);
           sym.toFractional(pttemp, false);
           if (acr.fixJavaFloat)
@@ -963,9 +984,8 @@ public class XtalSymmetry {
         if (checkDistance) {
           // for range checking, we first make sure we are not out of range
           // for the cartesian
-          if (checkSymmetryRange
-              && (c.x < rminx || c.y < rminy || c.z < rminz || c.x > rmaxx
-                  || c.y > rmaxy || c.z > rmaxz))
+          if (checkSymmetryRange && (c.x < rminx || c.y < rminy || c.z < rminz
+              || c.x > rmaxx || c.y > rmaxy || c.z > rmaxz))
             continue;
           float minDist2 = Float.MAX_VALUE;
           // checkAll means we have to check against operations that have
@@ -980,7 +1000,6 @@ public class XtalSymmetry {
             if (pc == null)
               continue;
             float d2 = c.distanceSquared(pc);
-            //System.out.println(iSym + " " +  i + " " + j + " " + pc + " " + c + " " + d2);
             if (checkSpecial && d2 < d0) {
               /* checkSpecial indicates that we are looking for atoms with (nearly) the
                * same cartesian position.  
@@ -1105,7 +1124,7 @@ public class XtalSymmetry {
 
   private void finalizeSymmetry(SymmetryInterface symmetry) {
     String name = (String) asc.getAtomSetAuxiliaryInfoValue(-1, "spaceGroup");
-    symmetry.setFinalOperations(name, asc.atoms, firstAtom,
+    symmetry.setFinalOperations(ndims, name, asc.atoms, firstAtom,
         noSymmetryCount, doNormalize, filterSymop);
     if (filterSymop != null || name == null || name.equals("unspecified!"))
       setAtomSetSpaceGroupName(symmetry.getSpaceGroupName());
@@ -1133,36 +1152,52 @@ public class XtalSymmetry {
     return (maxXYZ0 == null ? V3.new3(maxXYZ.x - minXYZ.x, maxXYZ.y - minXYZ.y,
         maxXYZ.z - minXYZ.z) : V3.newVsub(maxXYZ0, minXYZ0));
   }
+
+  static M4 mident;
   
   @SuppressWarnings("unchecked")
   public void applySymmetryBio(Map<String, Object> thisBiomolecule,
                                boolean applySymmetryToBonds, String filter) {
     Lst<M4> biomts = (Lst<M4>) thisBiomolecule.get("biomts");
-    if (biomts.size() < 2)
-      return;
+    int len = biomts.size();
+    // this was a bad idea - biomt can be just the identity matrix, in which case this is just a selection set -- 7OJP
+    //    if (len < 2)
+    //      return;
+    if (mident == null) {
+      mident = new M4();
+      mident.setIdentity();
+    }
     acr.lstNCS = null; // disable NCS
     setLatticeCells();
-    int[] lc = (latticeCells != null && latticeCells[0] != 0 ? new int[3] : null);
+    int[] lc = (latticeCells != null && latticeCells[0] != 0 ? new int[3]
+        : null);
     if (lc != null)
       for (int i = 0; i < 3; i++)
         lc[i] = latticeCells[i];
     latticeCells = null;
+
+    String bmChains = acr.getFilterWithCase("BMCHAINS");
+    int fixBMChains = (bmChains == null ? -1
+        : bmChains.length() < 2 ? 0 : PT.parseInt(bmChains.substring(1)));
+    if (fixBMChains == Integer.MIN_VALUE) {
+      fixBMChains = -(int) bmChains.charAt(1);
+    }
     int particleMode = (filter.indexOf("BYCHAIN") >= 0 ? PARTICLE_CHAIN
         : filter.indexOf("BYSYMOP") >= 0 ? PARTICLE_SYMOP : PARTICLE_NONE);
     doNormalize = false;
     Lst<String> biomtchains = (Lst<String>) thisBiomolecule.get("chains");
-    if (biomtchains.get(0).equals(biomtchains.get(1)))
-      biomtchains = null;
+    // Q: Why this? I think each biomt MUST have a chain
+    //    if (biomtchains.get(0).equals(biomtchains.get(1)))
+    //      biomtchains = null;
     symmetry = null;
     // it's not clear to me why you would do this:
     //if (!Float.isNaN(unitCellParams[0])) // PDB can do this; 
-      //setUnitCell(unitCellParams, null, unitCellOffset);
+    //setUnitCell(unitCellParams, null, unitCellOffset);
     getSymmetry().setSpaceGroup(doNormalize);
     //symmetry.setUnitCell(null);
     addSpaceGroupOperation("x,y,z", false);
     String name = (String) thisBiomolecule.get("name");
     setAtomSetSpaceGroupName(acr.sgName = name);
-    int len = biomts.size();
     this.applySymmetryToBonds = applySymmetryToBonds;
     bondCount0 = asc.bondCount;
     firstAtom = asc.getLastAtomSetAtomIndex();
@@ -1221,25 +1256,39 @@ public class XtalSymmetry {
     }
     Map<String, BS> assemblyIdAtoms = (Map<String, BS>) thisBiomolecule
         .get("asemblyIdAtoms");
+
     if (filter.indexOf("#<") >= 0) {
+      // ??
       len = Math.min(len,
           PT.parseInt(filter.substring(filter.indexOf("#<") + 2)) - 1);
       filter = PT.rep(filter, "#<", "_<");
     }
-    for (int iAtom = firstAtom; iAtom < atomMax; iAtom++)
-      atoms[iAtom].bsSymmetry = BSUtil.newAndSetBit(0);
+
+    int maxChain = 0;
+    for (int iAtom = firstAtom; iAtom < atomMax; iAtom++) {
+      atoms[iAtom].bsSymmetry = new BS(); //TODO was set 0, but this is not necessarily true now
+      int chainID = atoms[iAtom].chainID;
+      if (chainID > maxChain)
+        maxChain = chainID;
+    }
     BS bsAtoms = asc.bsAtoms;
     int[] atomMap = (addBonds ? new int[asc.ac] : null);
-    for (int i = (biomtchains == null ? 1 : 0); i < len; i++) {
+    // allow for filtering BIOMT number
+    // len >= 2, so I don't know what is going on here -- no chains? 
+    for (int imt = (biomtchains == null ? 1 : 0); imt < len; imt++) {
       if (filter.indexOf("!#") >= 0) {
-        if (filter.indexOf("!#" + (i + 1) + ";") >= 0)
+        if (filter.indexOf("!#" + (imt + 1) + ";") >= 0)
           continue;
       } else if (filter.indexOf("#") >= 0
-          && filter.indexOf("#" + (i + 1) + ";") < 0) {
+          && filter.indexOf("#" + (imt + 1) + ";") < 0) {
         continue;
       }
-      M4 mat = biomts.get(i);
-      String chains = (biomtchains == null ? null : biomtchains.get(i));
+      M4 mat = biomts.get(imt);
+      boolean notIdentity = !mat.equals(mident);
+
+      // if asym_id is given, that is what is being referred to, not author chains
+      // we just set bsAtoms to match
+      String chains = (biomtchains == null ? null : biomtchains.get(imt));
       if (chains != null && assemblyIdAtoms != null) {
         // must use label_asym_id, not auth_asym_id // bug fix 11/18/2015 
         bsAtoms = new BS();
@@ -1250,12 +1299,18 @@ public class XtalSymmetry {
           bsAtoms.and(asc.bsAtoms);
         chains = null;
       }
+
+      int lastID = -1, id;
+      boolean skipping = false;
       for (int iAtom = firstAtom; iAtom < atomMax; iAtom++) {
-        if (bsAtoms != null
-            && !bsAtoms.get(iAtom)
-            || chains != null
-            && chains.indexOf(":" + acr.vwr.getChainIDStr(atoms[iAtom].chainID)
-                + ";") < 0)
+        if (bsAtoms != null) {
+          skipping = !bsAtoms.get(iAtom);
+        } else if (chains != null 
+            && (id = atoms[iAtom].chainID) != lastID) {
+          skipping = (chains
+              .indexOf(":" + acr.vwr.getChainIDStr(lastID = id) + ";") < 0);
+        }
+        if (skipping)
           continue;
         try {
           int atomSite = atoms[iAtom].atomSite;
@@ -1263,26 +1318,30 @@ public class XtalSymmetry {
           if (addBonds)
             atomMap[atomSite] = asc.ac;
           atom1 = asc.newCloneAtom(atoms[iAtom]);
+          atom1.bondingRadius = imt; // temporary only -- to distinguish transforms
+          asc.atomSymbolicMap.put("" + atom1.atomSerial, atom1);
           if (asc.bsAtoms != null)
             asc.bsAtoms.set(atom1.index);
           atom1.atomSite = atomSite;
-          mat.rotTrans(atom1);
-          atom1.bsSymmetry = BSUtil.newAndSetBit(i);
+          if (notIdentity)
+            mat.rotTrans(atom1);
+          atom1.bsSymmetry = BSUtil.newAndSetBit(imt);
         } catch (Exception e) {
           asc.errorMessage = "appendAtomCollection error: " + e;
         }
       }
-      if (i > 0) {
+      // not the identity, which is always the first entry (set by Jmol in reader)
+      // symmetry always has first operation x,y,z
+      if (notIdentity)
         symmetry.addBioMoleculeOperation(mat, false);
-        if (addBonds) {
-          // Clone bonds
-          for (int bondNum = asc.bondIndex0; bondNum < bondCount0; bondNum++) {
-            Bond bond = asc.bonds[bondNum];
-            int iAtom1 = atomMap[atoms[bond.atomIndex1].atomSite];
-            int iAtom2 = atomMap[atoms[bond.atomIndex2].atomSite];
-//            if (iAtom1 >= atomMax || iAtom2 >= atomMax)
-            asc.addNewBondWithOrder(iAtom1, iAtom2, bond.order);
-          }
+      if (addBonds) {
+        // Clone bonds
+        for (int bondNum = asc.bondIndex0; bondNum < bondCount0; bondNum++) {
+          Bond bond = asc.bonds[bondNum];
+          int iAtom1 = atomMap[atoms[bond.atomIndex1].atomSite];
+          int iAtom2 = atomMap[atoms[bond.atomIndex2].atomSite];
+          //            if (iAtom1 >= atomMax || iAtom2 >= atomMax)
+          asc.addNewBondWithOrder(iAtom1, iAtom2, bond.order);
         }
       }
     }
@@ -1290,25 +1349,142 @@ public class XtalSymmetry {
       if (asc.bsAtoms == null)
         asc.bsAtoms = BSUtil.newBitSet2(0, asc.ac);
       asc.bsAtoms.clearBits(firstAtom, atomMax);
-    }
+      if (particleMode == PARTICLE_NONE) {
+        // check for BMCHAINS filter 
+        if (fixBMChains != -1) {
+          boolean assignABC = (fixBMChains != 0);
+          BS bsChains = (assignABC ? new BS() : null);
+          atoms = asc.atoms;
+          int firstNew = 0;
+          if (assignABC) {
+            // tested for 1k28 and 1auy
+            //For PDB and mmCIF, convert chains for symmetry-generated atoms to unique
+            // ids. Three options:
+            // bmChains or bmChains=0: append symmetry operator number to chain ID
+            // bmChains=q: start with 'q'
+            // bmChains= 3 : start with last chain ID + 3
+            firstNew = (fixBMChains < 0 ? Math.max(-fixBMChains, maxChain + 1)
+                : Math.max(maxChain + fixBMChains, 0 + 'A'));
+            bsChains.setBits(0, firstNew - 1);
+            bsChains.setBits(1 + 'Z', 0 + 'a');
+            bsChains.setBits(1 + 'z', 200);
+          }
+          BS[] bsAll = (asc.structureCount == 1 ? asc.structures[0].bsAll
+              : null);
+          Map<String, Integer> chainMap = new Hashtable<String, Integer>();
+          Map<Integer, Integer> knownMap = new Hashtable<Integer, Integer>();
+          Map<Integer, int[]> knownAtomMap = (bsAll == null ? null
+              : new Hashtable<Integer, int[]>());
+          int[] lastKnownAtom = null;
+          for (int i = atomMax, n = asc.ac; i < n; i++) {
+            int ic = atoms[i].chainID;
+            int isym = atoms[i].bsSymmetry.nextSetBit(0);
+            String ch0 = acr.vwr.getChainIDStr(ic);
+            String ch = (isym == 0 ? ch0 : ch0 + isym);
+            Integer known = chainMap.get(ch);
+            if (known == null) {
+              if (assignABC && isym != 0) {
+                int pt = (firstNew < 200 ? bsChains.nextClearBit(firstNew)
+                    : 200);
+                if (pt < 200) {
+                  bsChains.set(pt);
+                  // have A-Z or a-z
+                  known = Integer
+                      .valueOf(acr.vwr.getChainID("" + (char) pt, true));
+                  firstNew = pt;
+                } else {
+                  // 1auy will do this
+                }
+              }
+              if (known == null)
+                known = Integer.valueOf(acr.vwr.getChainID(ch, true));
+              if (ch != ch0) {
+                knownMap.put(known, Integer.valueOf(ic));
+                if (bsAll != null) {
+                  if (lastKnownAtom != null)
+                    lastKnownAtom[1] = i;
+                  knownAtomMap.put(known, lastKnownAtom = new int[] { i, n });
+                }
+              }
+              chainMap.put(ch, known);
+            }
+            atoms[i].chainID = known.intValue();
+          }
+          if (asc.structureCount > 0) {
+            // update structures
+            Structure[] strucs = asc.structures;
+            int nStruc = asc.structureCount;
+            for (Entry<Integer, Integer> e : knownMap.entrySet()) {
+              Integer known = e.getKey();
+              int ch1 = known.intValue();
+              int ch0 = e.getValue().intValue();
+              for (int i = 0; i < nStruc; i++) {
+                Structure s = strucs[i];
+                if (s.bsAll != null) {
+                  // MMTFReader processes bsAll[] in addStructureSymmetry()
+                } else if (s.startChainID == s.endChainID) {
+                  if (s.startChainID == ch0) {
+                    Structure s1 = s.clone();
+                    s1.startChainID = s1.endChainID = ch1;
+                    asc.addStructure(s1);
+                  }
+                } else {
+                  System.err.println(
+                      "XtalSymmetry not processing biomt chain structure "
+                          + acr.vwr.getChainIDStr(ch0) + " to "
+                          + acr.vwr.getChainIDStr(ch1));
+                  // TODO now what??
+                }
+              }
 
+            }
+          }
+        }
+
+        // clean interchain CONECT 
+        Lst<int[]> vConnect = (Lst<int[]>) asc.getAtomSetAuxiliaryInfoValue(-1,
+            "PDB_CONECT_bonds");
+        if (!addBonds && vConnect != null) {
+          for (int i = vConnect.size(); --i >= 0;) {
+            int[] bond = vConnect.get(i);
+            Atom a = asc.getAtomFromName("" + bond[0]);
+            Atom b = asc.getAtomFromName("" + bond[1]);
+            // bondingRadius here just being used for BIOMT 
+            if (a != null && b != null && a.bondingRadius != b.bondingRadius
+                && (bsAtoms == null
+                    || bsAtoms.get(a.index) && bsAtoms.get(b.index))
+                && a.distanceSquared(b) > MAX_INTERCHAIN_BOND_2) {
+              vConnect.removeItemAt(i);
+              System.out.println("long interchain bond removed for @"
+                  + a.atomSerial + "-@" + b.atomSerial);
+            }
+          }
+        }
+      }
+      // reset bondingRadius to NaN
+      for (int i = atomMax, n = asc.ac; i < n; i++) {
+        asc.atoms[i].bondingRadius = Float.NaN;
+      }
+
+    }
     noSymmetryCount = atomMax - firstAtom;
-    asc.setCurrentModelInfo("presymmetryAtomIndex",
-        Integer.valueOf(firstAtom));
+    asc.setCurrentModelInfo("presymmetryAtomIndex", Integer.valueOf(firstAtom));
     asc.setCurrentModelInfo("presymmetryAtomCount",
         Integer.valueOf(noSymmetryCount));
     asc.setCurrentModelInfo("biosymmetryCount", Integer.valueOf(len));
     asc.setCurrentModelInfo("biosymmetry", symmetry);
     finalizeSymmetry(symmetry);
     setSymmetryOps();
+
     reset();
+
     //TODO: need to clone bonds
   }
 
   private void reset() {
     asc.coordinatesAreFractional = false;
     asc.setCurrentModelInfo("hasSymmetry", Boolean.TRUE);
-    asc.setGlobalBoolean(AtomSetCollection.GLOBAL_SYMMETRY);
+    asc.setGlobalBoolean(JC.GLOBAL_SYMMETRY);
   }
 
   public Tensor addRotatedTensor(Atom a, Tensor t, int iSym, boolean reset,
@@ -1355,7 +1531,6 @@ public class XtalSymmetry {
         if (v.modDim > 0) {
           ((JmolModulationSet) v).setMoment();
         } else {
-          //System.out.println("xytalsym v=" + v + "  "+ i + "  ");
           v = (Vibration) v.clone(); // this could be a modulation set
           sym.toCartesian(v, true);
           asc.atoms[i].vib = v;
