@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2021-09-07 18:56:03 -0500 (Tue, 07 Sep 2021) $
- * $Revision: 22229 $
+ * $Date: 2022-03-11 11:09:21 -0600 (Fri, 11 Mar 2022) $
+ * $Revision: 22365 $
 
  *
  * Copyright (C) 2003-2005  The Jmol Development Team
@@ -77,11 +77,11 @@ public class Atom extends Point3fi implements Node {
 
   public char altloc = '\0';
   public byte atomID;
-  public int atomSite;
+  int atomSite;
   public Group group;
   private float userDefinedVanDerWaalRadius;
   byte valence;
-  private short atomicAndIsotopeNumber;
+  short atomicAndIsotopeNumber;
   public BS atomSymmetry;
 
   private int formalChargeAndFlags; //  cccc CIP_ _CIP --hv
@@ -755,18 +755,20 @@ public class Atom extends Point3fi implements Node {
     return this;
   }
   
-  public P3 getFractionalCoordPt(boolean fixJavaFloat, boolean ignoreOffset, P3 pt) {
+  public P3 getFractionalCoordPt(boolean fixJavaFloat, boolean ignoreOffset,
+                                 P3 pt) {
     // ignoreOffset TRUE uses the original unshifted matrix
     SymmetryInterface c = getUnitCell();
-    if (c == null) 
-      return this;
     if (pt == null)
       pt = P3.newP(this);
     else
       pt.setT(this);
+    if (c != null) {
+      c = c.getUnitCellMultiplied();
     c.toFractional(pt, ignoreOffset);
     if (fixJavaFloat)
       PT.fixPtFloats(pt, PT.FRACTIONAL_PRECISION);
+    }
     return pt;
   }
   
@@ -787,12 +789,13 @@ public class Atom extends Point3fi implements Node {
    */
   P3 getFractionalUnitCoordPt(boolean fixJavaFloat, boolean asCartesian, P3 pt) {
     SymmetryInterface c = getUnitCell();
-    if (c == null)
-      return this;
     if (pt == null)
       pt = P3.newP(this);
     else
       pt.setT(this);
+    if (c == null)
+      return pt;
+    c = c.getUnitCellMultiplied();
     if (group.chain.model.isJmolDataFrame) {
       c.toFractional(pt, false);
       if (asCartesian)
@@ -896,19 +899,25 @@ public class Atom extends Point3fi implements Node {
    */
    
   public String getInfo() {
-    return getIdentity(true);
+    return getIdentity(ID_ALL);
   } 
 
-  public String getIdentityXYZ(boolean allInfo, P3 pt) {
-    pt = (group.chain.model.isJmolDataFrame ? getFractionalCoordPt(!group.chain.model.ms.vwr.g.legacyJavaFloat, false, pt) : this);
-    return getIdentity(allInfo) 
-        + " " + PT.formatF(pt.x, 0, 3, true, true) 
+  public final static int ID_U    = 1;
+  public final static int ID_ALL  = 2;
+  public final static int ID_XTAL = 3;
+  public final static int ID_CHIME = 4;
+  
+  public String getIdentityXYZ(P3 pt, int mode) {
+    pt = (mode == ID_XTAL || group.chain.model.isJmolDataFrame ? getFractionalCoordPt(!group.chain.model.ms.vwr.g.legacyJavaFloat, false, pt) : this);
+    String s = (mode == ID_XTAL ? "" : getIdentity(mode) + " ")
+        + PT.formatF(pt.x, 0, 3, true, true) 
         + " " + PT.formatF(pt.y, 0, 3, true, true) 
         + " " + PT.formatF(pt.z, 0, 3, true, true) 
         ;  
+    return s;
   }
   
-  String getIdentity(boolean allInfo) {
+  String getIdentity(int mode) {
     SB info = new SB();
     String group3 = getGroup3(true);
     if (group3 != null && group3.length() > 0 && (!group3.equals("UNK") || group.chain.model.isBioModel)) {
@@ -926,7 +935,7 @@ public class Atom extends Point3fi implements Node {
           s = PT.esc(s);
         info.append(s);
       }
-      if (!allInfo)
+      if (mode != ID_ALL)
         return info.toString();
       info.append(".");
     }
@@ -1381,7 +1390,7 @@ public class Atom extends Point3fi implements Node {
     case T.element:
       return getElementSymbolIso(true);
     case T.identify:
-      return getIdentity(true);
+      return getIdentity(ID_ALL);
     case T.insertion:
       ch = group.getInsertionCode();
       return (ch == '\0' ? "" : "" + ch);
@@ -1532,4 +1541,19 @@ public class Atom extends Point3fi implements Node {
     return (m.isBioModel && !m.isPdbWithMultipleBonds && m.hydrogenCount == 0);
   }
 
+  public void setSymop(int isym, boolean andClear) {
+    if (atomSymmetry == null)
+      atomSymmetry = new BS();
+    if (andClear)
+    atomSymmetry.clearAll();
+    if (isym > 0)
+      atomSymmetry.set(isym - 1);
+  }
+
+  @Override
+  public int getExplicitHydrogenCount() {
+    // SMILES only [CH2]
+    return 0;
+  }
+
 }
