@@ -45,7 +45,6 @@ import org.jmol.c.VDW;
 import org.jmol.modelsetbio.BioModelSet;
 import org.jmol.script.T;
 import org.jmol.util.BSUtil;
-import org.jmol.util.Edge;
 import org.jmol.util.Elements;
 import org.jmol.util.GData;
 import org.jmol.util.Logger;
@@ -166,7 +165,7 @@ abstract public class AtomCollection {
     if (userSettableValues == null)
       userSettableValues = ("atomName atomType coord element formalCharge hydrophobicity " +
           "ionic occupancy partialCharge temperature valence vanderWaals vibrationVector " +
-          "atomNo seqID resNo chain").split(" ");
+          "atomNo seqID resNo chain site").split(" ");
   }
 
   final public static int TAINT_ATOMNAME = 0;
@@ -186,7 +185,8 @@ abstract public class AtomCollection {
   final public static int TAINT_SEQID = 14;
   final public static int TAINT_RESNO = 15;
   final public static int TAINT_CHAIN = 16;
-  final public static int TAINT_MAX = 17; // 1 more than last number, above
+  final public static int TAINT_SITE = 17;
+  final public static int TAINT_MAX = 18; // 1 more than last number, above
   
 
   String[] atomNames;
@@ -206,6 +206,7 @@ abstract public class AtomCollection {
   
   
   protected void releaseModelSetAC() {
+    ac = 0;
     at = null;
     vwr = null;
     g3d = null;
@@ -319,7 +320,7 @@ abstract public class AtomCollection {
     //definitely want FIRST (model) not last here
     for (int i = 0; i < ac; i++) {
       Atom atom = at[i];
-      if (atom != null && atom.getAtomNumber() == atomNumber && bsVisibleFrames.get(atom.mi))
+      if (!isDeleted(atom) && atom.getAtomNumber() == atomNumber && bsVisibleFrames.get(atom.mi))
         return i;
     }
     return -1;
@@ -338,7 +339,7 @@ abstract public class AtomCollection {
   public float[] getAtomicCharges() {
     float[] charges = new float[ac];
     for (int i = ac; --i >= 0; )
-      charges[i] = (at[i] == null ? 0 : at[i].getElementNumber());
+      charges[i] = (isDeleted(at[i]) ? 0 : at[i].getElementNumber());
     return charges;
   }
 
@@ -358,7 +359,7 @@ abstract public class AtomCollection {
     float r;
     for (int i = ac; --i >= 0;) {
       Atom atom = at[i];
-      if (atom == null)
+      if (isDeleted(atom))
         continue;
       if ((r = atom.getBondingRadius()) > maxBondingRadius)
         maxBondingRadius = r;
@@ -387,7 +388,7 @@ abstract public class AtomCollection {
   }
 
   private void setBf(int i) {
-    if (at[i] == null)
+    if (isDeleted(at[i]))
       return;
     int bf = at[i].getBfactor100();
     if (bf < bfactor100Lo)
@@ -463,7 +464,7 @@ abstract public class AtomCollection {
     float radiusAdjust = (envelopeRadius == Float.MAX_VALUE ? 0 : envelopeRadius);
     for (int i = 0; i < ac; i++) {
       //surfaceDistance100s[i] = Integer.MIN_VALUE;
-      if (bsSurface.get(i) || at[i] == null) {
+      if (bsSurface.get(i) || isDeleted(at[i])) {
         surfaceDistance100s[i] = 0;
       } else {
         float dMin = Float.MAX_VALUE;
@@ -521,9 +522,9 @@ abstract public class AtomCollection {
             setAtomCoord(i, xyz.x, xyz.y, xyz.z);
             break;
           case T.fracxyz:
-            at[i].setFractionalCoordTo(xyz, true);
-            taintAtom(i, TAINT_COORD);
-            break;
+//            at[i].setFractionalCoordTo(xyz, true);
+//            taintAtom(i, TAINT_COORD);
+//            break;
           case T.fuxyz:
             at[i].setFractionalCoordTo(xyz, false);
             taintAtom(i, TAINT_COORD);
@@ -631,9 +632,9 @@ abstract public class AtomCollection {
       case T.fracx:
       case T.fracy:
       case T.fracz:
-        atom.setFractionalCoord(tok, fValue, true);
-        taintAtom(i, TAINT_COORD);
-        break;
+//        atom.setFractionalCoord(tok, fValue, true);
+//        taintAtom(i, TAINT_COORD);
+//        break;
       case T.fux:
       case T.fuy:
       case T.fuz:
@@ -714,7 +715,7 @@ abstract public class AtomCollection {
    * also handles modulation info
    * 
    * @param atomIndex
-   * @param c
+   * @param c generally x, y, z, but for modulation could be 1, 2, 3, or O (oh, not zero)
    * @return value or NaN
    */
   public float getVibCoord(int atomIndex, char c) {
@@ -747,11 +748,11 @@ abstract public class AtomCollection {
     case 'Z':
       return v.z;
     case 'O':
-      return ((Float) ms.getModulation('O', null)).floatValue();
+      return ((Float) ms.getModulation('O', null, true)).floatValue();
     case '1':
     case '2':
     case '3':
-      T3 t = (T3) ms.getModulation('T', null);
+      T3 t = (T3) ms.getModulation('T', null, true);
       float x = (c == '1' ? t.x : c == '2' ? t.y : t.z);
       return (float) (x - Math.floor(x));
     default:
@@ -861,6 +862,15 @@ abstract public class AtomCollection {
       taintAtom(atom.i, TAINT_ELEMENT);
   }
 
+  public void setSite(Atom atom, int site, boolean doTaint) {
+    if (atom.atomSite == site)
+      return;
+    atom.atomSite = site;
+    if (doTaint)
+      taintAtom(atom.i, TAINT_SITE);
+  }
+
+
   private void resetPartialCharges() {
     partialCharges = null;
     bsPartialCharges = null;
@@ -900,6 +910,10 @@ abstract public class AtomCollection {
       taintAtom(atomIndex, TAINT_OCCUPANCY);
   }
   
+  public float getOccupancyFloat(int i) {
+    return (occupancies == null || i >= occupancies.length? 100 : occupancies[i]);
+  }
+  
   protected void setPartialCharge(int atomIndex, float partialCharge, boolean doTaint) {
     if (Float.isNaN(partialCharge))
       return;
@@ -1008,6 +1022,9 @@ abstract public class AtomCollection {
         case TAINT_CHAIN:
           setChainID(atomIndex, tokens[pt]);
           break;
+        case TAINT_SITE:
+          atom.atomSite = (int) x;
+          break;
         case TAINT_ELEMENT:
           atom.setAtomicAndIsotopeNumber((int)x);
           atom.paletteID = PAL.CPK.id;
@@ -1041,7 +1058,7 @@ abstract public class AtomCollection {
         vwr.setData(name, new Object[] {name, fData, bs, Integer.valueOf(JmolDataManager.DATA_TYPE_AF)}, 0, 0, 0, 0, 0);
         
     } catch (Exception e) {
-      Logger.error("AtomCollection.loadData error: " + e);
+      Logger.error("loadData error: " + e);
     }  
   }
   
@@ -1226,7 +1243,7 @@ abstract public class AtomCollection {
     boolean isMultiModel = ((mode & AtomData.MODE_FILL_MULTIMODEL) != 0);
     for (int i = 0; i < ac; i++) {
       Atom atom = at[i];
-      if (atom == null || atom.isDeleted() || !isMultiModel && atomData.modelIndex >= 0
+      if (isDeleted(atom) || !isMultiModel && atomData.modelIndex >= 0
           && atom.mi != atomData.firstModelIndex) {
         if (atomData.bsIgnored == null)
           atomData.bsIgnored = new BS();
@@ -1279,6 +1296,7 @@ abstract public class AtomCollection {
   public final static int CALC_H_HAVEH = 0x400;
   public final static int CALC_H_QUICK = Viewer.MIN_QUICK;
   public static final int CALC_H_IGNORE_H = 0x800;
+  public static final int CALC_H_ALLOW_H = 0x1000;
 
   
   /**
@@ -1297,6 +1315,7 @@ abstract public class AtomCollection {
     boolean justCarbon = ((flags & CALC_H_JUSTC) == CALC_H_JUSTC);
     boolean isQuick = ((flags & CALC_H_QUICK) == CALC_H_QUICK);
     boolean ignoreH = ((flags & CALC_H_IGNORE_H) == CALC_H_IGNORE_H);
+    boolean allowH = ((flags & CALC_H_ALLOW_H) == CALC_H_ALLOW_H);
     V3 z = new V3();
     V3 x = new V3();
     P3[][] hAtoms = new P3[ac][];
@@ -1323,7 +1342,7 @@ abstract public class AtomCollection {
           break;
         case 6:
         }
-        int n = (doAll || ignoreH ? atom.getCovalentHydrogenCount() : 0);
+        int n = (doAll || ignoreH || allowH ? atom.getCovalentHydrogenCount() : 0);
         if (doAll && n > 0 || ignoreH && n == 0)
           continue;
         int nMissing = getMissingHydrogenCount(atom, false);
@@ -1441,7 +1460,7 @@ abstract public class AtomCollection {
                   (hybridization == 2 || atomicNumber == 5 
                   || atomicNumber == 6 &&  aaRet[1] == 1
                   || atomicNumber == 7 
-                  && (atom.group.getNitrogenAtom() == atom || isAdjacentSp2(atom))
+                  && (atom.group.getNitrogenAtom() == atom & atom.getFormalCharge() == 0 || isAdjacentSp2(atom))
                   ? "sp2c"
                   : "sp3d"), true, false, isQuick) != null) {
                 pt = P3.newP(z);
@@ -1486,11 +1505,9 @@ abstract public class AtomCollection {
     for (int i = 0; i < bonds.length; i++) {
       Bond[] b2 = bonds[i].getOtherAtom(atom).bonds;
       for (int j = 0; j < b2.length; j++)
-        switch (b2[j].order) {
-        case Edge.BOND_AROMATIC:
-        case Edge.BOND_AROMATIC_DOUBLE:
-        case Edge.BOND_COVALENT_DOUBLE:
-        case Edge.BOND_COVALENT_TRIPLE:
+        switch (b2[j].getCovalentOrder()) {
+        case 2:
+        case 3:
           return true;
         }
     }
@@ -1500,6 +1517,8 @@ abstract public class AtomCollection {
   int[] aaRet;
   
   public int getMissingHydrogenCount(Atom atom, boolean allowNegative) {
+    if (aaRet == null)
+      aaRet = new int[5];
     int targetCount = atom.getTargetValence();
     if (targetCount < 0)
       return 0;
@@ -1507,8 +1526,6 @@ abstract public class AtomCollection {
     int valence = atom.getValence();
     Model model = ((ModelSet) this).am[atom.mi];
     String s = (model.isBioModel && !model.isPdbWithMultipleBonds ? atom.group.getGroup3() : null);
-    if (aaRet == null)
-      aaRet = new int[5];
     aaRet[0] = targetCount;
     aaRet[1] = charge;
     aaRet[2] = 0; // hybridization
@@ -2209,7 +2226,7 @@ abstract public class AtomCollection {
       boolean isType = (tokType == T.atomtype);
       String names = "," + specInfo + ",";
       for (int i = ac; --i >= 0;) {
-        if (at[i] == null)
+        if (isDeleted(at[i]))
           continue;
         String s = (isType ? at[i].getAtomType() : at[i].getAtomName());
         if (names.indexOf("," + s + ",") >= 0)
@@ -2218,13 +2235,13 @@ abstract public class AtomCollection {
       return bs;
     case T.atomno:
       for (int i = ac; --i >= 0;) {
-        if (at[i] != null && at[i].getAtomNumber() == iSpec)
+        if (!isDeleted(at[i]) && at[i].getAtomNumber() == iSpec)
           bs.set(i);
       }
       return bs;
     case T.bonded:
       for (int i = ac; --i >= 0;) {
-        if (at[i] != null && at[i].getCovalentBondCount() > 0)
+        if (!isDeleted(at[i]) && at[i].getCovalentBondCount() > 0)
           bs.set(i);
       }
       return bs;
@@ -2243,13 +2260,13 @@ abstract public class AtomCollection {
       //$FALL-THROUGH$
     case T.elemno:
       for (int i = ac; --i >= 0;) {
-        if (at[i] != null && at[i].getElementNumber() == iSpec)
+        if (!isDeleted(at[i]) && at[i].getElementNumber() == iSpec)
           bs.set(i);
       }
       return bs;
     case T.hetero:
       for (int i = ac; --i >= 0;) {
-        if (at[i] != null && at[i].isHetero())
+        if (!isDeleted(at[i]) && at[i].isHetero())
           bs.set(i);
       }
       return bs;
@@ -2257,7 +2274,7 @@ abstract public class AtomCollection {
       return getIdentifierOrNull((String) specInfo);
     case T.leadatom:
       for (int i = ac; --i >= 0;) {
-        if (at[i] != null && at[i].isLeadAtom())
+        if (!isDeleted(at[i]) && at[i].isLeadAtom())
           bs.set(i);
       }
       return bs;
@@ -2268,7 +2285,7 @@ abstract public class AtomCollection {
     case T.resno:
       // could be PDB type file with UNK
       for (int i = ac; --i >= 0;) {
-        if (at[i] != null && at[i].getResno() == iSpec)
+        if (!isDeleted(at[i]) && at[i].getResno() == iSpec)
           bs.set(i);
       }
       return bs;
@@ -2284,7 +2301,7 @@ abstract public class AtomCollection {
       int[] hs = new int[2];
       Atom a;
       for (int i = ac; --i >= 0;) {
-        if (at[i] == null)
+        if (isDeleted(at[i]))
           continue;
         int g = at[i].group.groupID;
         if (g >= JC.GROUPID_WATER && g < JC.GROUPID_SOLVENT_MIN) {
@@ -2310,7 +2327,7 @@ abstract public class AtomCollection {
       // could be PDB type file with UNK or a modulated CIF file subsystem
       String spec = (String) specInfo;
       for (int i = ac; --i >= 0;)
-        if (at[i] != null && isAltLoc(at[i].altloc, spec))
+        if (!isDeleted(at[i]) && isAltLoc(at[i].altloc, spec))
           bs.set(i);
       return bs;
     case T.spec_atom:
@@ -2322,7 +2339,7 @@ abstract public class AtomCollection {
       if (allowStar)
         atomSpec = atomSpec.substring(1);
       for (int i = ac; --i >= 0;)
-        if (at[i] != null && isAtomNameMatch(at[i], atomSpec, allowStar, allowStar))
+        if (!isDeleted(at[i]) && isAtomNameMatch(at[i], atomSpec, allowStar, allowStar))
           bs.set(i);
       return bs;
     case T.spec_chain:
@@ -2332,19 +2349,19 @@ abstract public class AtomCollection {
     case T.spec_resid:
       // could be PDB type file with UNK
       for (int i = ac; --i >= 0;)
-        if (at[i] != null && at[i].group.groupID == iSpec)
+        if (!isDeleted(at[i]) && at[i].group.groupID == iSpec)
           bs.set(i);
       return bs;
     case T.spec_seqcode:
       return BSUtil.copy(getSeqcodeBits(iSpec, true));
     case T.inscode:
       for (int i = ac; --i >= 0;)
-        if (at[i] != null && at[i].group.getInsCode() == iSpec)
+        if (!isDeleted(at[i]) && at[i].group.getInsCode() == iSpec)
           bs.set(i);
       return bs;
     case T.symop:
       for (int i = ac; --i >= 0;)
-        if (at[i] != null && at[i].getSymOp() == iSpec)
+        if (!isDeleted(at[i]) && at[i].getSymOp() == iSpec)
           bs.set(i);
       return bs;
     }
@@ -2377,7 +2394,7 @@ abstract public class AtomCollection {
       for (int i = i0; i >= 0; i = bsInfo.nextSetBit(i + 1))
         bsTemp.set(at[i].getElementNumber());
       for (int i = ac; --i >= 0;)
-        if (at[i] != null && bsTemp.get(at[i].getElementNumber()))
+        if (!isDeleted(at[i]) && bsTemp.get(at[i].getElementNumber()))
           bs.set(i);
       return bs;
     case T.group:
@@ -2394,7 +2411,7 @@ abstract public class AtomCollection {
       for (int i = i0; i >= 0; i = bsInfo.nextSetBit(i + 1))
         bsTemp.set(at[i].atomSite);
       for (int i = ac; --i >= 0;)
-        if (at[i] != null && bsTemp.get(at[i].atomSite))
+        if (!isDeleted(at[i]) && bsTemp.get(at[i].atomSite))
           bs.set(i);
       return bs;
     }
@@ -2405,7 +2422,7 @@ abstract public class AtomCollection {
   public BS getChainBits(int chainID) {
     boolean caseSensitive = vwr.getBoolean(T.chaincasesensitive);
     if (caseSensitive) {
-    	// check for script compiler run before setting chain ids
+      // check for script compiler run before setting chain ids
       if (chainID >= 97 && chainID <= 122)
         chainID += 159;    
     } else {
@@ -2416,7 +2433,7 @@ abstract public class AtomCollection {
     BS bsDone = BS.newN(ac);
     int id;
     for (int i = bsDone.nextClearBit(0); i < ac; i = bsDone.nextClearBit(i + 1)) {
-      if (at[i] == null)
+      if (isDeleted(at[i]))
         continue;
       Chain chain = at[i].group.chain;
       if (chainID == (id = chain.chainID) || !caseSensitive 
@@ -2456,7 +2473,7 @@ abstract public class AtomCollection {
     switch (insCode) {
     case '?':
       for (int i = ac; --i >= 0;) {
-        if (at[i] == null)
+        if (isDeleted(at[i]))
           continue;
         int atomSeqcode = at[i].group.seqcode;
         if ((!haveSeqNumber 
@@ -2469,7 +2486,7 @@ abstract public class AtomCollection {
       break;
     default:
       for (int i = ac; --i >= 0;) {
-        if (at[i] == null)
+        if (isDeleted(at[i]))
           continue;
         int atomSeqcode = at[i].group.seqcode;
         if (seqcode == atomSeqcode || 
@@ -2543,7 +2560,7 @@ abstract public class AtomCollection {
         name = name.substring(1);
 
     for (int i = ac; --i >= 0;) {
-      if (at[i] == null)
+      if (isDeleted(at[i]))
         continue;
       String g3 = at[i].getGroup3(true);
       if (g3 != null && g3.length() > 0) {
@@ -2551,8 +2568,10 @@ abstract public class AtomCollection {
           if (bs == null)
             bs = BS.newN(i + 1);
           bs.set(i);
-          while (--i >= 0 && at[i].getGroup3(true).equals(g3))
+          while (--i >= 0) {
+            if (!isDeleted(at[i]) && at[i].getGroup3(true).equals(g3))
             bs.set(i);
+          }
           i++;
         }
       } else if (isAtomNameMatch(at[i], name, checkStar, allowInitialStar)) {
@@ -2586,7 +2605,7 @@ abstract public class AtomCollection {
     BS bsResult = new BS();
     for (int i = ac; --i >= 0;) {
       Atom atom = at[i];
-      if (atom == null)
+      if (isDeleted(atom))
         continue;
       float d = Measure.distanceToPlane(plane, atom);
       if (distance > 0 && d >= -0.1 && d <= distance || distance < 0
@@ -2605,7 +2624,7 @@ abstract public class AtomCollection {
     clearVisibleSets();
     bsAtoms.clearAll();
     for (int i = ac; --i >= 0;)
-      if (at[i] != null && at[i].isVisible(Atom.ATOM_INFRAME))
+      if (!isDeleted(at[i]) && at[i].isVisible(Atom.ATOM_INFRAME))
         bsAtoms.set(i);
   }
 
@@ -2618,7 +2637,7 @@ abstract public class AtomCollection {
     }
     bsVisible.clearAll();
     for (int i = ac; --i >= 0;)
-      if (at[i] != null && at[i].checkVisible())
+      if (!isDeleted(at[i]) && at[i].checkVisible())
         bsVisible.set(i);
     if (vwr.shm.bsSlabbedInternal != null)
       bsVisible.andNot(vwr.shm.bsSlabbedInternal);
@@ -2633,7 +2652,7 @@ abstract public class AtomCollection {
       return bsClickable;
     bsClickable.clearAll();
     for (int i = ac; --i >= 0;)
-      if (at[i] != null && at[i].isClickable())
+      if (!isDeleted(at[i]) && at[i].isClickable())
         bsClickable.set(i);
     haveBSClickable = true;
     return bsClickable;
@@ -2873,6 +2892,10 @@ abstract public class AtomCollection {
 
   }
 
+  public static boolean isDeleted(Atom atom) {
+    return (atom == null || atom.isDeleted());
+  }
+
 
 }
 
