@@ -302,7 +302,7 @@ abstract public class BondCollection extends AtomCollection {
    * @param minFrac
    * @param maxfrac
    * @param isFractional
-   * @return
+   * @return true if in range
    */
   protected boolean isInRange(Atom atom1, Atom atom2, float minD, float maxD,
                             boolean minFrac, boolean maxfrac,
@@ -799,7 +799,7 @@ abstract public class BondCollection extends AtomCollection {
     boolean nonbonded = (min == 0);
     for (i = ac; --i >= 0;) {
       int n = nBonded[i];
-      if (n < min || n > max)
+      if (at[i] == null || n < min || n > max)
         bsResult.clear(i);
       else if (nonbonded && n == 0)
         bsResult.set(i);
