@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2020-05-26 14:23:08 -0500 (Tue, 26 May 2020) $
- * $Revision: 22014 $
+ * $Date: 2022-03-30 10:19:01 -0500 (Wed, 30 Mar 2022) $
+ * $Revision: 22389 $
  *
  * Copyright (C) 2002-2005  The Jmol Development Team
  *
@@ -85,6 +85,7 @@ public class Measurement {
   private String newUnits;
   public float fixedValue = Float.NaN;
   private boolean isPending;
+  public boolean inFront;
   
   public boolean isTainted() {
     return (tainted && !(tainted = false));
@@ -241,7 +242,7 @@ public class Measurement {
   /**
    * 
    * @param units from MEASURE or measure()
-   * @return
+   * @return format
    */
   private String formatDistance(String units) {
     String label = getLabelString();
@@ -273,18 +274,6 @@ public class Measurement {
     return formatString(f, newUnits, label);
   }
 
-  private static String fixUnits(String units) {
-    if (units.equals("nanometers"))
-      return "nm";
-    else if (units.equals("picometers"))
-      return "pm";
-    else if (units.equals("angstroms"))
-      return "\u00C5";
-    else if (units.equals("vanderwaals") || units.equals("vdw"))
-      return "%";
-    return units;
-  }
-
   /**
    * 
    * @param units  final units
@@ -337,21 +326,7 @@ public class Measurement {
             units = "pm";
         }
       }
-      if (Float.isNaN(dist))
-        return Float.NaN;
-      if (units.equals("hz"))
-        return (andRound ? Math.round(dist * 10) / 10f : dist);
-      if (units.equals("noe"))
-        return (andRound ? Math.round(dist * 100) / 100f : dist);
-      if (units.equals("nm"))
-        return (andRound ? Math.round(dist * 100) / 1000f : dist / 10);
-      if (units.equals("pm"))
-        return (andRound ? Math.round(dist * 1000) / 10f : dist * 100);
-      if (units.equals("au"))
-        return (andRound ? Math.round(dist / JC.ANGSTROMS_PER_BOHR * 1000) / 1000f
-            : dist / JC.ANGSTROMS_PER_BOHR);
-      if (units.endsWith("khz"))
-        return (andRound ? Math.round(dist / 10) / 100f : dist / 1000);
+      return toUnits(dist, units, andRound);
     }
     return (andRound ? Math.round(dist * 100) / 100f : dist);
   }
@@ -607,4 +582,51 @@ public class Measurement {
       || s.indexOf(" ") < 0 && s.endsWith("hz"));
   }
 
+  public static float toUnits(float dist, String units, boolean andRound) {
+    if (Float.isNaN(dist))
+      return Float.NaN;
+    if (units.equals("hz"))
+      return (andRound ? Math.round(dist * 10) / 10f : dist);
+    if (units.equals("nm"))
+      return (andRound ? Math.round(dist * 100) / 1000f : dist / 10);
+    if (units.equals("pm"))
+      return (andRound ? Math.round(dist * 1000) / 10f : dist * 100);
+    if (units.equals("au"))
+      return (andRound ? Math.round(dist / JC.ANGSTROMS_PER_BOHR * 1000) / 1000f
+          : dist / JC.ANGSTROMS_PER_BOHR);
+    if (units.endsWith("khz"))
+      return (andRound ? Math.round(dist / 10) / 100f : dist / 1000);
+//    if (units.equals("noe"))
+//      return (andRound ? Math.round(dist * 100) / 100f : dist);
+    return (andRound ? Math.round(dist * 100) / 100f : dist);
+  }
+
+  public static float fromUnits(float dist, String units) {
+    if (units.equals("nm"))
+      return dist * 10;
+    if (units.equals("pm"))
+      return dist / 100;
+    if (units.equals("au"))
+      return dist * JC.ANGSTROMS_PER_BOHR;
+    if (units.equals("\u00C5"))
+      return dist;
+    return 0;
+  }
+
+  public static String fixUnits(String u) {
+    String units = (u.endsWith("s") ? u.substring(0, u.length() - 1) : u);
+    if (units.equals("nanometer"))
+      return "nm";
+    else if (units.equals("bohr") || units.equals("atomicunits") || units.equals("atomic"))
+      return "au";
+    else if (units.equals("picometer"))
+      return "pm";
+    else if (units.equals("\u00E5") || units.equals("angstrom") || units.equals("a") || units.equals("ang"))
+      return "\u00C5";
+    else if (units.equals("vanderwaal") || units.equals("vdw"))
+      return "%";
+    return u;
+  }
+
+
 }
