@@ -153,6 +153,7 @@ public final class ModelLoader {
   public Group[] groups;
   private int groupCount;
   private P3 modulationTUV;
+  private boolean isSupercell;
   
 
   @SuppressWarnings("unchecked")
@@ -164,7 +165,7 @@ public final class ModelLoader {
     ms.msInfo = info;
     ms.modelSetProperties = (Properties) ms.getInfoM("properties");
     //isMultiFile = getModelSetAuxiliaryInfoBoolean("isMultiFile"); -- no longer necessary
-    ms.haveBioModels = ms.getMSInfoB("isPDB");
+    ms.haveBioModels = ms.getMSInfoB(JC.getBoolName(JC.GLOBAL_ISPDB));
     isMutate = ms.getMSInfoB("isMutate");
     if (ms.haveBioModels)
       jbr = vwr.getJBR().setLoader(this);
@@ -177,15 +178,14 @@ public final class ModelLoader {
     isPyMOLsession = ms.getMSInfoB("isPyMOL");
     doAddHydrogens = (jbr != null && !isTrajectory && !isPyMOLsession
         && !ms.getMSInfoB("pdbNoHydrogens") && (ms
-        .getMSInfoB("pdbAddHydrogens") || vwr
-        .getBooleanProperty("pdbAddHydrogens")));
+        .getMSInfoB("pdbAddHydrogens") || vwr.getBoolean(T.pdbaddhydrogens)));
     if (info != null) {
       info.remove("pdbNoHydrogens");
       info.remove("pdbAddHydrogens");
       info.remove("trajectorySteps");
       if (isTrajectory)
         ms.vibrationSteps = (Lst<V3[]>) info.remove("vibrationSteps");
-      if (info.containsKey("legacyJavaFloat")) {
+      if (info.containsKey("highPrecision")) {
         // we must RESET this, because 'ZAP' has unset it in the script
         vwr.setBooleanProperty("legacyJavaFloat", true);
       }
@@ -198,32 +198,33 @@ public final class ModelLoader {
     }
     noAutoBond = ms.getMSInfoB("noAutoBond");
     is2D = ms.getMSInfoB("is2D");
-    doMinimize = is2D && ms.getMSInfoB("doMinimize");
+    doMinimize = (is2D || ms.getMSInfoB("minimize3D")) && ms.getMSInfoB("doMinimize");
     adapterTrajectoryCount = (isTrajectory ? ms.trajectory.steps.size() : 0);
-    ms.someModelsHaveSymmetry = ms.getMSInfoB("someModelsHaveSymmetry");
-    someModelsHaveUnitcells = ms.getMSInfoB("someModelsHaveUnitcells");
-    someModelsAreModulated = ms.getMSInfoB("someModelsAreModulated");
+    ms.someModelsHaveSymmetry = ms.getMSInfoB(JC.getBoolName(JC.GLOBAL_SYMMETRY));
+    someModelsHaveUnitcells = ms.getMSInfoB(JC.getBoolName(JC.GLOBAL_UNITCELLS));
+    someModelsAreModulated = ms.getMSInfoB(JC.getBoolName(JC.GLOBAL_MODULATED));
+    isSupercell = ms.getMSInfoB(JC.getBoolName(JC.GLOBAL_SUPERCELL));
     ms.someModelsHaveFractionalCoordinates = ms
-        .getMSInfoB("someModelsHaveFractionalCoordinates");
+        .getMSInfoB(JC.getBoolName(JC.GLOBAL_SYMMETRY));
     if (merging) {
       ms.haveBioModels |= modelSet0.haveBioModels;
       ms.bioModelset = modelSet0.bioModelset;
       if (ms.bioModelset != null)
         ms.bioModelset.set(vwr, ms);
       ms.someModelsHaveSymmetry |= modelSet0
-          .getMSInfoB("someModelsHaveSymmetry");
+          .getMSInfoB(JC.getBoolName(JC.GLOBAL_SYMMETRY));
       someModelsHaveUnitcells |= modelSet0
-          .getMSInfoB("someModelsHaveUnitcells");
+          .getMSInfoB(JC.getBoolName(JC.GLOBAL_UNITCELLS));
       ms.someModelsHaveFractionalCoordinates |= modelSet0
-          .getMSInfoB("someModelsHaveFractionalCoordinates");
+          .getMSInfoB(JC.getBoolName(JC.GLOBAL_FRACTCOORD));
       ms.someModelsHaveAromaticBonds |= modelSet0.someModelsHaveAromaticBonds;
-      ms.msInfo.put("someModelsHaveSymmetry",
+      ms.msInfo.put(JC.getBoolName(JC.GLOBAL_SYMMETRY),
           Boolean.valueOf(ms.someModelsHaveSymmetry));
-      ms.msInfo.put("someModelsHaveUnitcells",
+      ms.msInfo.put(JC.getBoolName(JC.GLOBAL_UNITCELLS),
           Boolean.valueOf(someModelsHaveUnitcells));
-      ms.msInfo.put("someModelsHaveFractionalCoordinates",
+      ms.msInfo.put(JC.getBoolName(JC.GLOBAL_FRACTCOORD),
           Boolean.valueOf(ms.someModelsHaveFractionalCoordinates));
-      ms.msInfo.put("someModelsHaveAromaticBonds",
+      ms.msInfo.put(JC.getBoolName(JC.GLOBAL_AROMATICBONDS),
           Boolean.valueOf(ms.someModelsHaveAromaticBonds));
     }
   }
@@ -279,17 +280,17 @@ public final class ModelLoader {
     return ms.ac;
   }
 
-  private void createModelSet(JmolAdapter adapter, Object asc,
-                              BS bsNew) {
+  private void createModelSet(JmolAdapter adapter, Object asc, BS bsNew) {
     int nAtoms = (adapter == null ? 0 : adapter.getAtomCount(asc));
     if (nAtoms > 0)
       Logger.info("reading " + nAtoms + " atoms");
-    adapterModelCount = (adapter == null ? 1 : adapter
-        .getAtomSetCount(asc));
+    adapterModelCount = (adapter == null ? 1 : adapter.getAtomSetCount(asc));
     // cannot append a trajectory into a previous model
-    appendToModelIndex = (ms.msInfo == null ? null : ((Integer) ms.msInfo.get("appendToModelIndex")));
-    appendNew = !isMutate && (!merging || adapter == null || adapterModelCount > 1
-        || isTrajectory || vwr.getBoolean(T.appendnew) && appendToModelIndex == null);
+    appendToModelIndex = (ms.msInfo == null ? null
+        : ((Integer) ms.msInfo.get("appendToModelIndex")));
+    appendNew = !isMutate
+        && (!merging || adapter == null || adapterModelCount > 1 || isTrajectory
+            || vwr.getBoolean(T.appendnew) && appendToModelIndex == null);
     htAtomMap.clear();
     chainOf = new Chain[defaultGroupCount];
     group3Of = new String[defaultGroupCount];
@@ -321,19 +322,25 @@ public final class ModelLoader {
       Quat q = (Quat) ms.getInfoM("defaultOrientationQuaternion");
       if (q != null) {
         Logger.info("defaultOrientationQuaternion = " + q);
-        Logger
-            .info("Use \"set autoLoadOrientation TRUE\" before loading or \"restore orientation DEFAULT\" after loading to view this orientation.");
+        Logger.info(
+            "Use \"set autoLoadOrientation TRUE\" before loading or \"restore orientation DEFAULT\" after loading to view this orientation.");
       }
       iterateOverAllNewModels(adapter, asc);
-      iterateOverAllNewAtoms(adapter, asc);
-      iterateOverAllNewBonds(adapter, asc);
+      JmolAdapterBondIterator iterBond = adapter.getBondIterator(asc);
+      boolean haveBonds = (iterBond != null);
+      iterateOverAllNewAtoms(adapter, asc, haveBonds);
+      short mad = vwr.getMadBond();
+      if (haveBonds) {
+        ms.defaultCovalentMad = (jmolData == null ? mad : 0);
+        iterateOverAllNewBonds(iterBond);
+      }
+      ms.defaultCovalentMad = mad;
       if (merging && !appendNew) {
-        Map<String, Object> info = adapter.getAtomSetAuxiliaryInfo(
-            asc, 0);
-        ms.setInfo(baseModelIndex, "initialAtomCount", info
-            .get("initialAtomCount"));
-        ms.setInfo(baseModelIndex, "initialBondCount", info
-            .get("initialBondCount"));
+        Map<String, Object> info = adapter.getAtomSetAuxiliaryInfo(asc, 0);
+        ms.setInfo(baseModelIndex, "initialAtomCount",
+            info.get("initialAtomCount"));
+        ms.setInfo(baseModelIndex, "initialBondCount",
+            info.get("initialBondCount"));
       }
       initializeUnitCellAndSymmetry();
       initializeBonding();
@@ -367,11 +374,16 @@ public final class ModelLoader {
     Model[] models = ms.am;
     for (int i = models[baseModelIndex].firstAtomIndex; i < ac; i++)
       if (atoms[i] != null)
-      models[atoms[i].mi].bsAtoms.set(i);
+        models[atoms[i].mi].bsAtoms.set(i);
 
     freeze();
     finalizeShapes();
     vwr.setModelSet(ms);
+    if (isSupercell && appendNew) {
+      for (int i = baseModelIndex; i < ms.mc; i++) {
+        vwr.assignSpaceGroup(null, "P1", i);
+      }
+    }
     setAtomProperties();
     if (adapter != null)
       adapter.finish(asc);    
@@ -462,9 +474,10 @@ public final class ModelLoader {
         }
         key = "property_" + key.toLowerCase();
         Logger.info("creating " + key + " for model " + ms.getModelName(i));
-        vwr.setData(key, new Object[] { key, value, bs, Integer.valueOf(JmolDataManager.DATA_TYPE_UNKNOWN), Boolean.FALSE }, 
+        vwr.setData(key, new Object[] { key, value, bs, Integer.valueOf(JmolDataManager.DATA_TYPE_UNKNOWN), Boolean.FALSE, entry }, 
             ms.ac, 0, 0, Integer.MAX_VALUE, 0);
       }
+      ms.setInfo(i, "atomProperties", null);
     }
   }
 
@@ -553,8 +566,11 @@ public final class ModelLoader {
       if (modelAuxiliaryInfo.containsKey("modelID"))
         modelAuxiliaryInfo.put("modelID0", modelAuxiliaryInfo.get("modelID"));
       Properties modelProperties = (Properties) modelAuxiliaryInfo.get("modelProperties");
-      vwr.setStringProperty("_fileType", (String) modelAuxiliaryInfo
-          .get("fileType"));
+      if (!merging || appendNew) {
+        String ftype = (String) modelAuxiliaryInfo.get("fileType");
+        vwr.setStringProperty("_fileType", ftype);
+        vwr.fm.setFileType(ftype);
+      }
       if (modelName == null)
         modelName = (jmolData != null && jmolData.indexOf(";") > 2 ? jmolData.substring(jmolData
             .indexOf(":") + 2, jmolData.indexOf(";"))
@@ -609,7 +625,7 @@ public final class ModelLoader {
                                             String jmolData) {
     if (appendNew) {
       boolean modelIsPDB = (modelAuxiliaryInfo != null && Boolean.TRUE == modelAuxiliaryInfo
-          .get("isPDB"));
+          .get(JC.getBoolName(JC.GLOBAL_ISPDB)));
       ms.am[modelIndex] = (modelIsPDB ? jbr.getBioModel(modelIndex,
           trajectoryBaseIndex, jmolData, modelProperties, modelAuxiliaryInfo)
           : new Model().set(ms, modelIndex, trajectoryBaseIndex, jmolData,
@@ -644,7 +660,7 @@ public final class ModelLoader {
     boolean isModelKit = (ms.modelSetName != null
         && ms.modelSetName.startsWith("Jmol Model Kit")
         || modelName.startsWith("Jmol Model Kit") || "Jme".equals(ms.getInfo(
-        modelIndex, "fileType")));
+        modelIndex, "fileType")) && is2D);
     models[modelIndex].isModelKit = isModelKit;
   }
 
@@ -768,7 +784,7 @@ public final class ModelLoader {
     }
   }
 
-  private void iterateOverAllNewAtoms(JmolAdapter adapter, Object asc) {
+  private void iterateOverAllNewAtoms(JmolAdapter adapter, Object asc, boolean haveBonds) {
     // atom is created, but not all methods are safe, because it
     // has no group -- this is only an issue for debugging
     int iLast = -1;
@@ -776,6 +792,7 @@ public final class ModelLoader {
     boolean addH = false;
     boolean isLegacyHAddition = false;//vwr.getBoolean(T.legacyhaddition);
     JmolAdapterAtomIterator iterAtom = adapter.getAtomIterator(asc);
+    int nAtoms = adapter.getAtomCount(asc);
     ms.setCapacity(adapter.getAtomCount(asc));
     int nRead = 0;
     Model[] models = ms.am;
@@ -791,6 +808,8 @@ public final class ModelLoader {
         model = models[modelIndex];
         currentChainID = Integer.MAX_VALUE;
         isNewChain = true;
+        // set the internal array for model.bsAtoms to be large enough
+        model.bsAtoms.set(ms.ac + nAtoms);
         model.bsAtoms.clearAll();
         model.isOrderly = (appendToModelIndex == null);
         isPdbThisModel = model.isBioModel;
@@ -801,16 +820,16 @@ public final class ModelLoader {
       }
       String group3 = iterAtom.getGroup3();
       int chainID = iterAtom.getChainID();
-      checkNewGroup(adapter, chainID, group3, iterAtom.getSequenceNumber(), 
+      checkNewGroup(adapter, chainID, group3, iterAtom.getSequenceNumber(),
           iterAtom.getInsertionCode(), addH, isLegacyHAddition);
       int isotope = iterAtom.getElementNumber();
       if (addH && Elements.getElementNumber(isotope) == 1)
         jbr.setHaveHsAlready(true);
-      String name = iterAtom.getAtomName(); 
+      String name = iterAtom.getAtomName();
       int charge = (addH ? getPdbCharge(group3, name) : iterAtom.getFormalCharge());
-      addAtom(isPdbThisModel, iterAtom.getSymmetry(),
+      
+      Atom atom = addAtom(isPdbThisModel, iterAtom.getSymmetry(),
           iterAtom.getAtomSite(),
-          iterAtom.getUniqueID(),
           isotope,
           name,
           charge, 
@@ -828,6 +847,8 @@ public final class ModelLoader {
           iterAtom.getRadius(), 
           iterAtom.getBondRadius()
           );
+      if (haveBonds)
+        htAtomMap.put(iterAtom.getUniqueID(), atom);
     }
     if (groupCount > 0 && addH) {
       jbr.addImplicitHydrogenAtoms(adapter, groupCount - 1,
@@ -840,7 +861,9 @@ public final class ModelLoader {
     for (int i = 0; i < ms.ac; i++) {
       if (atoms[i] != null && atoms[i].mi > iLast) {
         iLast = atoms[i].mi;
-   models[iLast].firstAtomIndex = i;
+        Model m = models[iLast];
+        m.firstAtomIndex = i;
+        m.isOrderly = (m.act == m.bsAtoms.length()); 
         VDW vdwtype = ms.getDefaultVdwType(iLast);
         if (vdwtype != vdwtypeLast) {
           Logger.info("Default Van der Waals type for model" + " set to "
@@ -849,7 +872,7 @@ public final class ModelLoader {
         }
       }
     }
-    Logger.info(nRead + " atoms created");    
+    Logger.info(nRead + " atoms created");
   }
 
   private void addJmolDataProperties(Model m,
@@ -906,8 +929,7 @@ public final class ModelLoader {
             : 0);
   }
 
-  private void addAtom(boolean isPDB, BS atomSymmetry, int atomSite,
-                       Object atomUid, int atomicAndIsotopeNumber,
+  private Atom addAtom(boolean isPDB, BS atomSymmetry, int atomSite, int atomicAndIsotopeNumber,
                        String atomName, int formalCharge, float partialCharge,
                        Lst<Object> tensors, float occupancy, float bfactor,
                        P3 xyz, boolean isHetero, int atomSerial, int atomSeqID,
@@ -936,7 +958,7 @@ public final class ModelLoader {
         partialCharge, occupancy, bfactor, tensors, isHetero, specialAtomID,
         atomSymmetry, bondRadius);
     atom.altloc = alternateLocationID;
-    htAtomMap.put(atomUid, atom);
+    return atom;
   }
 
   private void checkNewGroup(JmolAdapter adapter, int chainID,
@@ -986,20 +1008,14 @@ public final class ModelLoader {
     return model.chains[model.chainCount++] = new Chain(model, chainID, (chainID == 0 || chainID == 32 ? 0 : ++iChain));
   }
 
-  private void iterateOverAllNewBonds(JmolAdapter adapter,
-                                      Object asc) {
-    JmolAdapterBondIterator iterBond = adapter
-        .getBondIterator(asc);
-    if (iterBond == null)
-      return;
-    short mad = vwr.getMadBond();
-    ms.defaultCovalentMad = (jmolData == null ? mad : 0);
+  private void iterateOverAllNewBonds(JmolAdapterBondIterator iterBond) {
+    boolean force1 = isMutate && !vwr.getBoolean(T.pdbaddhydrogens);
     boolean haveMultipleBonds = false;
     while (iterBond.hasNext()) {
       int iOrder = iterBond.getEncodedOrder();
-      short order = (short) iOrder;
-      Bond b = bondAtoms(iterBond.getAtomUniqueID1(), iterBond
-          .getAtomUniqueID2(), order);
+      short order = (force1 ? 1 : (short) iOrder);
+      Bond b = bondAtoms(iterBond.getAtomUniqueID1(),
+          iterBond.getAtomUniqueID2(), order);
       if (b != null) {
         if (order > 1 && order != Edge.BOND_STEREO_NEAR
             && order != Edge.BOND_STEREO_FAR)
@@ -1015,9 +1031,8 @@ public final class ModelLoader {
     }
     if (haveMultipleBonds && ms.someModelsHaveSymmetry
         && !vwr.getBoolean(T.applysymmetrytobonds))
-      Logger
-          .info("ModelSet: use \"set appletSymmetryToBonds TRUE \" to apply the file-based multiple bonds to symmetry-generated atoms.");
-    ms.defaultCovalentMad = mad;
+      Logger.info(
+          "ModelSet: use \"set appletSymmetryToBonds TRUE \" to apply the file-based multiple bonds to symmetry-generated atoms.");
   }
   
   private Lst<Bond> vStereo;
@@ -1088,7 +1103,8 @@ public final class ModelLoader {
     if (someModelsHaveUnitcells) {
       ms.unitCells = new SymmetryInterface[ms.mc];
       ms.haveUnitCells = true;
-      boolean haveMergeCells = (modelSet0 != null && modelSet0.unitCells != null);
+      boolean haveMergeCells = (modelSet0 != null
+          && modelSet0.unitCells != null);
       for (int i = 0, pt = 0; i < ms.mc; i++) {
         if (haveMergeCells && i < baseModelCount) {
           ms.unitCells[i] = modelSet0.unitCells[i];
@@ -1101,7 +1117,8 @@ public final class ModelLoader {
             if (lst != null)
               notionalCell = lst.get(pt++);
           }
-          ms.unitCells[i].setSymmetryInfo(i, ms.getModelAuxiliaryInfo(i), notionalCell);
+          ms.unitCells[i].setSymmetryInfo(i, ms.getModelAuxiliaryInfo(i),
+              notionalCell);
         }
       }
     }
@@ -1111,11 +1128,13 @@ public final class ModelLoader {
       for (int iAtom = baseAtomIndex, iModel = -1, i0 = 0; iAtom < ms.ac; iAtom++) {
         if (atoms[iAtom].mi != iModel) {
           iModel = atoms[iAtom].mi;
-          i0 = baseAtomIndex
-              + ms.getInfoI(iModel,
-                  "presymmetryAtomIndex")
-              + ms.getInfoI(iModel,
-                  "presymmetryAtomCount");
+          i0 = baseAtomIndex + ms.getInfoI(iModel, "presymmetryAtomIndex");
+          if (i0 < 0)
+            break;
+          int n = ms.getInfoI(iModel, "presymmetryAtomCount");
+          ms.am[iModel].auxiliaryInfo.put("presymmetryAtomIndex", Integer.valueOf(i0));
+          ms.setInfo(iModel, "asymmetricUnit", ms.am[iModel].bsAsymmetricUnit = BSUtil.newBitSet2(i0,  i0 + n));
+          i0 += n;
         }
         if (iAtom >= i0)
           ms.bsSymmetry.set(iAtom);
@@ -1126,7 +1145,7 @@ public final class ModelLoader {
       int modelIndex = -1;
       SymmetryInterface c = null;
       boolean isFractional = false;
-      boolean roundCoords = !vwr.g.legacyJavaFloat;
+      boolean roundCoords = (!vwr.getBoolean(T.doubleprecision) && !vwr.getBoolean(T.legacyjavafloat));
       for (int i = baseAtomIndex; i < ms.ac; i++) {
         if (atoms[i].mi != modelIndex) {
           modelIndex = atoms[i].mi;
@@ -1139,7 +1158,7 @@ public final class ModelLoader {
           c = atoms[i].getUnitCell();
           c.toCartesian(c.toSupercell(atoms[i]), false);
           if (roundCoords)
-            PT.fixPtFloats(atoms[i], PT.CARTESIAN_PRECISION);          
+            PT.fixPtFloats(atoms[i], PT.CARTESIAN_PRECISION);
         }
       }
       for (int imodel = baseModelIndex; imodel < ms.mc; imodel++)
@@ -1159,7 +1178,7 @@ public final class ModelLoader {
     int modelAtomCount = 0;
     BS bsExclude = (BS) ms.getInfoM("bsExcludeBonding");
     if (bsExclude == null) {
-      bsExclude = (ms.getInfoM("someModelsHaveCONECT") == null ? null
+      bsExclude = (ms.getInfoM(JC.getBoolName(JC.GLOBAL_CONECT)) == null ? null
           : new BS());
       if (bsExclude != null)
         ms.setPdbConectBonding(baseAtomIndex, baseModelIndex, bsExclude);
@@ -1289,7 +1308,7 @@ public final class ModelLoader {
     group.groupIndex = groupIndex;
 
     for (int i = lastAtomIndex + 1; --i >= firstAtomIndex;)
-      ms.at[i].group = group;
+        ms.at[i].group = group;
 
   }
 
@@ -1339,7 +1358,7 @@ public final class ModelLoader {
         bondsCache[j] = null;
     }
 
-    ms.setAtomNamesAndNumbers(0, baseAtomIndex, modelSet0);
+    ms.setAtomNamesAndNumbers(0, baseAtomIndex, modelSet0, false);
 
     // find elements for the popup menus
 
@@ -1399,6 +1418,8 @@ public final class ModelLoader {
           Atom a2 = b2.getOtherAtom(a1);
           v.sub2(a2, a1);
           //System.out.println(b2 + " " + v);
+          // in the case where we have a vertical stereochemical indicator, 
+          // South and wedge or North and hash sets to invert stereo??
           if (Math.abs(v.x) < 0.1) {
             if ((b.order == Edge.BOND_STEREO_NEAR) == (v.y < 0))
               stereodir = -1;
@@ -1487,6 +1508,7 @@ public final class ModelLoader {
    * @param v 
    * @param v0 
    * @param v1 
+   * @param dir 
    * @return   atom bitset
    */
   private BS getBranch2dZ(int atomIndex, int atomIndexNot, BS bs0, 
@@ -1547,6 +1569,7 @@ public final class ModelLoader {
    * @param v
    * @param v0
    * @param v1
+   * @param dir 
    */
   private static void setAtom2dZ(Atom atomRef, Atom atom2, V3 v, V3 v0, V3 v1, int dir) {
     v.sub2(atom2, atomRef);
@@ -1566,7 +1589,7 @@ public final class ModelLoader {
     ms.sm.setModelSet(ms);
     ms.setBsHidden(vwr.slm.getHiddenSet());
     if (!merging)
-      ms.sm.resetShapes();
+      ms.sm.resetShapes(false);
     ms.sm.loadDefaultShapes(ms);
     if (ms.someModelsHaveAromaticBonds && vwr.getBoolean(T.smartaromatic))      
       ms.assignAromaticBondsBs(false, null);
