@@ -23,18 +23,17 @@
  */
 package org.jmol.modelset;
 
+import org.jmol.shape.Shape;
+import org.jmol.util.C;
+import org.jmol.util.Font;
+import org.jmol.viewer.JC;
+import org.jmol.viewer.Viewer;
+
+import javajs.util.BS;
 import javajs.util.P3;
 import javajs.util.P3i;
 import javajs.util.PT;
 import javajs.util.SB;
-import javajs.util.BS;
-
-import org.jmol.shape.Shape;
-import org.jmol.util.C;
-import org.jmol.util.Font;
-import org.jmol.util.Txt;
-import org.jmol.viewer.JC;
-import org.jmol.viewer.Viewer;
 
 public class Text {
  
@@ -43,10 +42,10 @@ public class Text {
   public boolean doFormatText;
 
   public Font font;
-  private byte fid;
+  private int fid;
   private int ascent;
   public int descent;
-  private int lineHeight;
+  public int lineHeight;
 
   protected int offsetX; // Labels only
   protected int offsetY; // Labels only
@@ -63,6 +62,9 @@ public class Text {
   public Object image;
   public float imageScale = 1;
 
+  public int barPixels;
+
+  public float barDistance;
 
 
   public Text() {
@@ -104,11 +106,11 @@ public class Text {
     t.zSlab = Integer.MIN_VALUE;
     return t;
   }
-
+  
   private void set(Font font, short colix, int align,
                    float scalePixelsPerMicron) {
     this.scalePixelsPerMicron = scalePixelsPerMicron;
-    this.isEcho = isEcho;
+    //this.isEcho = isEcho;
     this.colix = colix;
     this.align = align;
     this.setFont(font, !isEcho);
@@ -128,7 +130,7 @@ public class Text {
     lineHeight = ascent + descent;
   }
 
-  public void setFontFromFid(byte fid) { //labels only
+  public void setFontFromFid(int fid) { //labels only
     if (this.fid == fid)
       return;
     fontScale = 0;
@@ -136,9 +138,11 @@ public class Text {
   }
 
   public void setText(String text) {
-    if (image != null)
+    if (image != null) {
       getFontMetrics();
-    image = null;
+      image = null;
+    }
+    barPixels = 0;
     if (text != null && text.length() == 0)
       text = null;
     if (this.text != null && this.text.equals(text))
@@ -218,8 +222,16 @@ public class Text {
     else if (fontScale != imageFontScaling)
       setFontScale(imageFontScaling);
     if (doFormatText) {
-      text = (isEcho ? Txt.formatText(vwr, textUnformatted) : textUnformatted);
+      text = (isEcho ? vwr.formatText(textUnformatted) : textUnformatted);
       recalc();
+    } else {
+      if (textUnformatted != null && textUnformatted.startsWith("%SCALE")) {
+        float[] ret = new float[2];
+        text = vwr.getScaleText(textUnformatted.substring(6).trim(), vwr.antialiased, (xyz == null ? 15 : 8), ret);
+        barPixels = (int) (ret[0] * imageFontScaling);
+        barDistance = ret[1];
+      recalc();
+    }
     }
     float dx = offsetX * imageFontScaling;
     float dy = offsetY * imageFontScaling;
@@ -258,7 +270,7 @@ public class Text {
          * 
          */
         // dx and dy are the overall object offset, with text
-        dx = getPymolXYOffset(pymolOffset[1], textWidth, pixelsPerAngstrom);
+        dx = (xyz != null && barPixels > 0 ? 0 : getPymolXYOffset(pymolOffset[1], textWidth, pixelsPerAngstrom));
         int dh = ascent - descent;
         dy = -getPymolXYOffset(-pymolOffset[2], dh, pixelsPerAngstrom)
             - (textHeight + dh) / 2;
@@ -290,7 +302,9 @@ public class Text {
           //$FALL-THROUGH$
         case JC.TEXT_ALIGN_LEFT:
           dy = 0;
+          break;
         }
+      // don't apply an X-offset if this is a scale and set on a point
       //System.out.println(dx + " Text " + dy + " " + boxWidth + " " + boxHeight);
       setBoxXY(boxWidth, boxHeight, dx, dy, boxXY, isAbsolute);
     } else {
@@ -333,22 +347,26 @@ public class Text {
       float x = (movableXPercent != Integer.MAX_VALUE ? movableXPercent
           * windowWidth / 100 : is3dEcho ? movableX : movableX * scale);
       float offsetX = this.offsetX * scale;
-      xLeft = xRight = xCenter = x + offsetX;
+      xLeft = xRight = xCenter = x + offsetX + barPixels;
     } else {
       xLeft = 5 * scale;
-      xCenter = windowWidth / 2;
+      xCenter = (windowWidth + barPixels) / 2;
       xRight = windowWidth - xLeft;
+      xLeft += barPixels;
     }
 
     // set box X from alignments
 
-    boxXY[0] = xLeft;
     switch (align) {
     case JC.TEXT_ALIGN_CENTER:
       boxXY[0] = xCenter - boxWidth / 2;
       break;
     case JC.TEXT_ALIGN_RIGHT:
       boxXY[0] = xRight - boxWidth;
+      break;
+    default:
+      boxXY[0] = xLeft;
+      break;
     }
 
     // set box Y from alignments
@@ -524,6 +542,8 @@ public class Text {
 
   public float scalePixelsPerMicron;
 
+  public int barPixelsXYZ;
+
   public void setScalePixelsPerMicron(float scalePixelsPerMicron) {
     fontScale = 0;//fontScale * this.scalePixelsPerMicron / scalePixelsPerMicron;
     this.scalePixelsPerMicron = scalePixelsPerMicron;
@@ -712,4 +732,8 @@ public class Text {
     setScalePixelsPerMicron(sppm);
   }
 
+  @Override
+  public String toString() {
+    return textUnformatted;
+  }
 }
