@@ -119,9 +119,9 @@ abstract public class JmolPopup extends JmolGenericPopup {
   
   @Override
   public void jpiDispose() {
-    super.jpiDispose();
     helper.menuClearListeners(frankPopup);
     frankPopup = null;
+    super.jpiDispose();
   }
 
   @Override
@@ -168,7 +168,7 @@ abstract public class JmolPopup extends JmolGenericPopup {
       updateHeteroComputedMenu(vwr.ms.getHeteroList(modelIndex));
       updateSurfMoComputedMenu((Map<String, Object>) modelInfo.get("moData"));
       updateFileTypeDependentMenus();
-      updatePDBComputedMenus();
+      updatePDBResidueComputedMenus();
       updateMode = UPDATE_CONFIG;
       updateConfigurationComputedMenu();
       updateSYMMETRYComputedMenus();
@@ -338,9 +338,9 @@ abstract public class JmolPopup extends JmolGenericPopup {
     modelInfo = vwr.ms.getModelAuxiliaryInfo(modelIndex);
     if (modelInfo == null)
       modelInfo = new Hashtable<String, Object>();
-    isPDB = checkBoolean("isPDB");
+    isPDB = checkBoolean(JC.getBoolName(JC.GLOBAL_ISPDB));
     isMultiFrame = (modelCount > 1);
-    hasSymmetry = modelInfo.containsKey("hasSymmetry");
+    hasSymmetry = !isPDB && modelInfo.containsKey("hasSymmetry");
     isUnitCell = modelInfo.containsKey("unitCellParams");
     fileHasUnitCell = (isPDB && isUnitCell || checkBoolean("fileHasUnitCell"));
     isLastFrame = (modelIndex == modelCount - 1);
@@ -585,25 +585,28 @@ abstract public class JmolPopup extends JmolGenericPopup {
     menuEnable(menu, true);
   }
 
-  protected void updatePDBComputedMenus() {
-
+  protected void updatePDBResidueComputedMenus() {
+    boolean haveMenu = false;
     SC menu3 = htMenus.get("PDBaaResiduesComputedMenu");
     if (menu3 != null) {
       menuRemoveAll(menu3, 0);
       menuEnable(menu3, false);
+      haveMenu = true;
     }
     SC menu1 = htMenus.get("PDBnucleicResiduesComputedMenu");
     if (menu1 != null) {
       menuRemoveAll(menu1, 0);
       menuEnable(menu1, false);
+      haveMenu = true;
     }
 
     SC menu2 = htMenus.get("PDBcarboResiduesComputedMenu");
     if (menu2 != null) {
       menuRemoveAll(menu2, 0);
       menuEnable(menu2, false);
+      haveMenu = true;
     }
-    if (modelSetInfo == null)
+    if (modelSetInfo == null || !haveMenu)
       return;
     int n = (modelIndex < 0 ? 0 : modelIndex + 1);
     String[] lists = ((String[]) modelSetInfo.get("group3Lists"));
