@@ -3,22 +3,10 @@ package org.jmol.script;
 import java.util.Hashtable;
 import java.util.Map;
 
-import javajs.util.BArray;
-import javajs.util.CU;
-import javajs.util.Lst;
-import javajs.util.M34;
-import javajs.util.M4;
-import javajs.util.Measure;
-import javajs.util.P3;
-import javajs.util.P4;
-import javajs.util.PT;
-import javajs.util.SB;
-import javajs.util.T3;
-
 import org.jmol.api.Interface;
 import org.jmol.api.JmolDataManager;
-import javajs.util.BS;
 import org.jmol.modelset.Atom;
+import org.jmol.modelset.AtomCollection;
 import org.jmol.modelset.Bond;
 import org.jmol.modelset.BondSet;
 import org.jmol.modelset.Group;
@@ -33,6 +21,19 @@ import org.jmol.util.Elements;
 import org.jmol.util.Escape;
 import org.jmol.viewer.JC;
 
+import javajs.util.BArray;
+import javajs.util.BS;
+import javajs.util.CU;
+import javajs.util.Lst;
+import javajs.util.M34;
+import javajs.util.M4;
+import javajs.util.Measure;
+import javajs.util.P3;
+import javajs.util.P4;
+import javajs.util.PT;
+import javajs.util.SB;
+import javajs.util.T3;
+
 /**
  * The ScriptExpr class holds the main functions for 
  * processing mathematical and atom selection expressions.
@@ -407,7 +408,8 @@ abstract class ScriptExpr extends ScriptParam {
           }
           v = getAssocArray(i);
         } else {
-          v = getPointOrPlane(i, false, true, true, false, 3, 4, true);
+          v = getPointOrPlane(i, MODE_P34 
+              | MODE_P_ALLOW_FRACTIONAL | MODE_P_CONVERT_TO_CARTESIAN);
         }
         i = iToken;
         break;
@@ -481,6 +483,7 @@ abstract class ScriptExpr extends ScriptParam {
             if (tokAt(iToken + 3) == T.per && tokAt(iToken + 4) == T.bin)
               tok2 = T.selectedfloat;
             //$FALL-THROUGH$
+          case T.pivot:
           case T.min:
           case T.max:
           case T.stddev:
@@ -488,8 +491,8 @@ abstract class ScriptExpr extends ScriptParam {
           case T.sum2:
           case T.average:
             allowMathFunc = (isUserFunction || var.intValue == T.distance
-                || tok2 == T.minmaxmask || tok2 == T.selectedfloat);
-            var.intValue |= tok2;
+                || tok2 == T.minmaxmask || tok2 == T.selectedfloat || tok2 == T.pivot);
+            var.intValue |= tok2 & T.minmaxmask;
             getToken(iToken + 2);
           }
         }
@@ -786,12 +789,12 @@ abstract class ScriptExpr extends ScriptParam {
         break;
       case T.hkl:
         rpn.addX(SV.newT(instruction));
-        rpn.addX(SV.newV(T.point4f, hklParameter(pc + 2, false)));
+        rpn.addX(SV.newV(T.point4f, hklParameter(pc + 2, null, true)));
         pc = iToken;
         break;
       case T.plane:
         rpn.addX(SV.newT(instruction));
-        rpn.addX(SV.newV(T.point4f, planeParameter(pc + 2)));
+        rpn.addX(SV.newV(T.point4f, planeParameter(pc + 2, false)));
         pc = iToken;
         break;
       case T.coord:
@@ -818,7 +821,7 @@ abstract class ScriptExpr extends ScriptParam {
         rpn.addX(SV.newT(instruction));
         // note that the compiler has changed all within() types to strings.
         if (s.equals("hkl")) {
-          rpn.addX(SV.newV(T.point4f, hklParameter(pc + 2, false)));
+          rpn.addX(SV.newV(T.point4f, hklParameter(pc + 2, null, true)));
           pc = iToken;
         }
         break;
@@ -858,7 +861,7 @@ abstract class ScriptExpr extends ScriptParam {
         rpn.addXBs(BSUtil.copy(vwr.slm.getHiddenSet()));
         break;
       case T.fixed:
-        rpn.addXBs(BSUtil.copy(vwr.getMotionFixedAtoms()));
+        rpn.addXBs(vwr.getMotionFixedAtoms());
         break;
       case T.displayed:
         rpn.addXBs(BSUtil.copyInvert(vwr.slm.getHiddenSet(), ac));
@@ -958,9 +961,7 @@ abstract class ScriptExpr extends ScriptParam {
         break;
       case T.centroid:
       case T.cell:
-        P3 pt = (P3) value;
-        rpn.addXBs(getAtomBits(
-            instruction.tok, (P3) value));
+        rpn.addXBs(getAtomBits(instruction.tok, value));
         break;
       case T.thismodel:
         rpn.addXBs(vwr.am.cmi < 0 ? vwr.getFrameAtoms() : vwr.getModelUndeletedAtomsBitSet(vwr.am.cmi));
@@ -1280,7 +1281,7 @@ abstract class ScriptExpr extends ScriptParam {
     for (int i = ac; --i >= 0;) {
       boolean match = false;
       Atom atom = atoms[i];
-      if (atom == null)
+      if (AtomCollection.isDeleted(atom))
         continue;
       if (isProp) {
         if (data == null || data.length <= i)
@@ -1297,19 +1298,26 @@ abstract class ScriptExpr extends ScriptParam {
   }
 
   protected boolean compareFloat(int tokOperator, float a, float b) {
+	// simple NaN check all comparisons are false
+    // Jmol DOES allow NaN == NaN
+	// check for one but not the other being NaN
+	// which is always false
+    if ((a == a) != (b == b))
+      return (tokOperator == T.opNE);
+    // either both are NaN or both are not NaN
     switch (tokOperator) {
     case T.opLT:
       return a < b;
     case T.opLE:
-      return a <= b;
+      return a <= b || a != a;
     case T.opGE:
-      return a >= b;
+      return a >= b || a != a;
     case T.opGT:
       return a > b;
     case T.opEQ:
-      return a == b;
+      return a == b || a != a; // a is NaN and thus b is also NaN
     case T.opNE:
-      return a != b && !Float.isNaN(a);
+      return a != b && a == a; // opposite of the above
     }
     return false;
   }
@@ -1324,6 +1332,8 @@ abstract class ScriptExpr extends ScriptParam {
     if (!isCaseSensitive)
       comparisonString = comparisonString.toLowerCase();
     for (int i = ac; --i >= 0;) {
+      if (atoms[i] == null)
+        continue;
       String propertyString = atoms[i]
           .atomPropertyString(vwr, tokWhat);
       if (!isCaseSensitive)
@@ -1401,7 +1411,7 @@ abstract class ScriptExpr extends ScriptParam {
     for (int i = 0; i < ac; ++i) {
       boolean match = false;
       Atom atom = atoms[i];
-      if (atom == null)
+      if (AtomCollection.isDeleted(atom))
         continue;
       switch (tokWhat) {
       default:
@@ -1532,6 +1542,7 @@ abstract class ScriptExpr extends ScriptParam {
   private SV getBitsetPropertySelector(int i, int xTok) throws ScriptException {
     int tok = getToken(i).tok;
     switch (tok) {
+    case T.pivot:
     case T.min:
     case T.max:
     case T.average:
@@ -1572,6 +1583,9 @@ abstract class ScriptExpr extends ScriptParam {
 
     int minmaxtype = tok & T.minmaxmask;
     boolean selectedFloat = (minmaxtype == T.selectedfloat);
+    boolean isPivot = (minmaxtype == T.apivot);
+    if (isPivot)
+      minmaxtype = T.minmaxmask;
     int ac = vwr.ms.ac;
     float[] fout = (minmaxtype == T.allfloat ? new float[ac] : null);
     boolean isExplicitlyAll = (minmaxtype == T.minmaxmask || selectedFloat);
@@ -1733,7 +1747,14 @@ abstract class ScriptExpr extends ScriptParam {
       for (int i = i0; i >= 0 && i < i1; i = (haveBitSet ? bs.nextSetBit(i + 1)
           : i + 1)) {
         n++;
-        Atom atom = (pts == null ? modelSet.at[i] : null);
+        Atom atom;
+        if (pts == null) {
+          atom = modelSet.at[i];
+          if (AtomCollection.isDeleted(atom))
+            continue;
+        } else {
+          atom = null;
+        }
         switch (mode) {
         case 0: // float
           float fv = Float.MAX_VALUE;
@@ -1744,7 +1765,7 @@ abstract class ScriptExpr extends ScriptParam {
             bsAtom.clear(i);
             break;
           case T.property:
-            fv = (data == null ? 0 : data[i]);
+            fv = (data == null || i >= data.length ? 0 : data[i]);
             break;
           case T.distance:
             if (planeRef != null)
@@ -1928,8 +1949,12 @@ abstract class ScriptExpr extends ScriptParam {
     if (minmaxtype == T.allfloat)
       return fout;
     if (minmaxtype == T.all) {
-      if (asVectorIfAll)
+      if (asVectorIfAll) {
+        if (isPivot) {
+          return getMathExt().getMinMax(vout, T.pivot, false);
+        }
         return vout;
+      }
       int len = vout.size();
       if ((isString || isHash) && !isExplicitlyAll && len == 1)
         return vout.get(0);
@@ -2061,6 +2086,9 @@ abstract class ScriptExpr extends ScriptParam {
     Lst<SV> lst = (Lst<SV>) obj;
     if (lst.size() == 0)
       return "";
+    SV v0 = lst.get(0);
+    if (SV.ptValue(v0) != null)
+      return obj;
     if (lst.get(0).asString().contains("|"))
       return vwr.ms.getAtoms(T.sequence, SV.newV(T.varray, lst).asString());
     BS bs = SV.unEscapeBitSetArray(lst, true);
@@ -2434,7 +2462,7 @@ abstract class ScriptExpr extends ScriptParam {
       if (st[i].tok == T.define)
         break;
     }
-    if (i == slen)// || isScriptCheck)
+    if (i == slen || chk)
       return i == slen;
     switch (st[0].tok) {
     case T.parallel:
@@ -2627,7 +2655,7 @@ abstract class ScriptExpr extends ScriptParam {
 
     return true;
   }
-  
+
   protected boolean isFunction(String sf) {
     return (getFunction(sf) != null);
   }
@@ -2645,7 +2673,7 @@ abstract class ScriptExpr extends ScriptParam {
 
   protected ScriptFunction getFunction(String sf) {
     ScriptFunction f = (privateFuncs == null ? null : privateFuncs.get(sf));
-    return (ScriptFunction) (f == null ? vwr.getFunction(sf) : f);
+    return (f == null ? (ScriptFunction) vwr.getFunction(sf) : f);
   }
 
 
