@@ -28,7 +28,6 @@ import java.io.BufferedReader;
 import java.io.IOException;
 import java.util.Hashtable;
 import java.util.Map;
-import java.util.zip.ZipInputStream;
 
 import javax.swing.JOptionPane;
 
@@ -366,8 +365,7 @@ public class ScriptManager implements JmolScriptManager {
       if (!isQuiet)
         vwr.setScriptStatus(null, strScript, -2 - (++scriptIndex), null);
       eval.evaluateCompiledScript(vwr.isSyntaxCheck, vwr.isSyntaxAndFileCheck,
-          historyDisabled, vwr.listCommands, outputBuffer,
-          isQueued || !vwr.isSingleThreaded);
+          historyDisabled, vwr.listCommands, outputBuffer, isQueued);
     } else {
       vwr.scriptStatus(strErrorMessage);
       vwr.setScriptStatus("Jmol script terminated", strErrorMessage, 1,
@@ -398,15 +396,19 @@ public class ScriptManager implements JmolScriptManager {
 
   private String checkScriptExecution(String strScript, boolean isInsert) {
     String str = strScript;
-    if (str.indexOf("\1##") >= 0)
-      str = str.substring(0, str.indexOf("\1##"));
+    int pt = str.indexOf(JC.SCRIPT_GUI);
+    if (pt >= 0)
+      str = str.substring(0, pt);
+    else if ((pt = str.indexOf(JC.SCRIPT_EXT)) >= 0)
+      str = str.substring(0, pt);
+
     if (checkResume(str))
       return "script processing resumed";
     if (checkStepping(str))
       return "script processing stepped";
     if (checkHalt(str, isInsert))
       return "script execution halted";
-    wasmHack(strScript);
+    vwr.wasmInchiHack(strScript);
     return null;
   }
 
@@ -466,7 +468,7 @@ public class ScriptManager implements JmolScriptManager {
 
   @Override
   public boolean checkHalt(String str, boolean isInsert) {
-    if (str.equalsIgnoreCase("pause")) {
+    if (str.equalsIgnoreCase("pause") || str.equalsIgnoreCase("pause" + JC.SCRIPT_EXT)) {
       vwr.pauseScriptExecution();
       if (vwr.scriptEditorVisible)
         vwr.setScriptStatus("", "paused -- type RESUME to continue", 0, null);
@@ -520,21 +522,11 @@ public class ScriptManager implements JmolScriptManager {
     if (strScript.indexOf(")") == 0 || strScript.indexOf("!") == 0) // history
       // disabled
       strScript = strScript.substring(1);
-    strScript = wasmHack(strScript);
+    strScript = vwr.wasmInchiHack(strScript);
     ScriptContext sc = newScriptEvaluator().checkScriptSilent(strScript);
     return (returnContext || sc.errorMessage == null ? sc : sc.errorMessage);
   }
 
-  public String wasmHack(String cmd) {
-    if (Viewer.isJS
-        && (cmd.indexOf("find('inchi')") >= 0
-            || cmd.indexOf("find(\"inchi\")") >= 0)
-        || cmd.indexOf(".inchi(") >= 0) {
-      vwr.getInchi(null, null, null);
-    }
-    return cmd;
-  }
-
   //////////////////////// open file async ///////////////////////
 
   /**
@@ -543,27 +535,34 @@ public class ScriptManager implements JmolScriptManager {
    * 
    * @param fname
    * @param flags
-   *        1=pdbCartoons, 2=no scripting, 4=append, 8=noAutoPlay
+   * 
+   *        1=pdbCartoons, 
+   *        2=no scripting, 
+   *        4=append, 
+   *        8=no autoplay, 
+   *        16=file dropped, 
+   *        32=script only (dropped into console)
+   *        64=check dims for resize 
    * 
    */
   @Override
-  public void openFileAsync(String fname, int flags, boolean checkDims) {
-    if (checkDims && FileManager.isEmbeddable(fname)) 
+  public void openFileAsync(String fname, int flags, String type) {
+    boolean scriptOnly = ((flags & SCRIPT_ONLY) != 0);
+    if (!scriptOnly && (flags & CHECK_DIMS) != 0 && FileManager.isEmbeddable(fname)) 
       checkResize(fname);
     boolean noScript = ((flags & NO_SCRIPT) != 0);
     boolean noAutoPlay = ((flags & NO_AUTOPLAY) != 0);
 
     String cmd = null;
-    fname = fname.trim();
-    if (fname.startsWith("\t")) {
-      noScript = true;
-      fname = fname.substring(1);
-    }
-    fname = fname.replace('\\', '/');
+    fname = fname.trim().replace('\\', '/');
     boolean isCached = fname.startsWith("cache://");
     if (vwr.isApplet && fname.indexOf("://") < 0)
       fname = "file://" + (fname.startsWith("/") ? "" : "/") + fname;
     try {
+      if (scriptOnly) {
+        cmd = "script " + PT.esc(fname);
+        return;
+      }
       // using finally... here on return
       if (fname.endsWith(".pse")) {
         cmd = (isCached ? "" : "zap;") + "load SYNC " + PT.esc(fname)
@@ -573,7 +572,10 @@ public class ScriptManager implements JmolScriptManager {
       if (fname.endsWith("jvxl")) {
         cmd = "isosurface ";
       } else if (!fname.toLowerCase().endsWith(".spt")) {
-        String type = getDragDropFileTypeName(fname);
+        if (type == null)
+          type = getDragDropFileTypeName(fname);
+        else if (!type.endsWith("::"))
+          type += "::";
         if (type == null) {
           try {
             BufferedInputStream bis = vwr.getBufferedInputStream(fname);
@@ -586,9 +588,13 @@ public class ScriptManager implements JmolScriptManager {
           } catch (IOException e) {
             return;
           }
+          if (type == "MENU") {
+            cmd = "load MENU " + PT.esc(fname);
+          } else {
           cmd = "if (_filetype == 'Pdb') { isosurface sigma 1.0 within 2.0 {*} "
               + PT.esc(fname) + " mesh nofill }; else; { isosurface "
               + PT.esc(fname) + "}";
+          }
           return;
         }
         // these next will end with the escaped file name
@@ -677,7 +683,7 @@ public class ScriptManager implements JmolScriptManager {
     Object br = vwr.fm.getUnzippedReaderOrStreamFromName(fileName, null, true,
         false, true, true, null);
     String modelType = null;
-    if (br instanceof ZipInputStream) {
+    if (vwr.fm.isZipStream(br)) {
       String zipDirectory = vwr.getZipDirectoryAsString(fileName);
       if (zipDirectory.indexOf("JmolManifest") >= 0)
         return "Jmol";
@@ -712,7 +718,8 @@ public class ScriptManager implements JmolScriptManager {
           int ver = vwr.stateScriptVersionInt = main * 10000 + sub * 100
               + minor;
           vwr.setBooleanProperty("legacyautobonding", (ver < 110924));
-          vwr.g.legacyHAddition = (ver < 130117);
+          vwr.setBooleanProperty("legacyHAddition", (ver < 130117));
+          if (!vwr.getBoolean(T.doubleprecision))
           vwr.setBooleanProperty("legacyjavafloat",
               (ver < 140206 || ver >= 140300 && ver < 140306));
           vwr.setIntProperty("bondingVersion", ver < 140111 ? 0 : 1);
@@ -724,24 +731,32 @@ public class ScriptManager implements JmolScriptManager {
     }
     vwr.setIntProperty("bondingVersion", prevCovalentVersion);
     vwr.setBooleanProperty("legacyautobonding", false);
-    vwr.g.legacyHAddition = false;
+    vwr.setBooleanProperty("legacyHAddition", false);
     vwr.stateScriptVersionInt = Integer.MAX_VALUE;
   }
 
   /**
    * Add hydrogens to a model
    * 
-   * @param bsAtoms at least one atom, for identification of a model index
-   * @param vConnections return list of atoms added
-   * @param pts list of point positions for the added hydrogens?
+   * @param bsAtoms
+   *        at least one atom, for identification of a model index
+   * @param vConnections
+   *        return list of atoms added
+   * @param pts
+   *        list of point positions for the added hydrogens?
    */ 
   @Override
-  public BS addHydrogensInline(BS bsAtoms, Lst<Atom> vConnections, P3[] pts)
+  public BS addHydrogensInline(BS bsAtoms, Lst<Atom> vConnections, P3[] pts,
+                               Map<String, Object> htParams)
       throws Exception {
     int iatom = bsAtoms.nextSetBit(0);
+    if (htParams == null)
+      htParams = new Hashtable<String, Object>();
     int modelIndex = (iatom < 0 ? vwr.am.cmi : vwr.ms.at[iatom].mi);
     if (modelIndex < 0)
       modelIndex = vwr.ms.mc - 1;
+    htParams.put("appendToModelIndex", Integer.valueOf(modelIndex));
+    boolean siteFixed = (htParams.containsKey("fixedSite"));
 //    if (!vwr.ms.isAtomInLastModel(iatom))
 //      return new BS();
 
@@ -752,12 +767,18 @@ public class ScriptManager implements JmolScriptManager {
     vwr.g.appendNew = false;
     // BitSet bsB = getAtomBits(Token.hydrogen, null);
     // bsA.andNot(bsB);
-    int atomIndex = vwr.ms.ac;
-    int atomno = vwr.ms.getAtomCountInModel(modelIndex);// BH! not quite right if deleted
+    int atomno = 0;
+    for (int i = bsA.nextSetBit(0); i >= 0; i = bsA.nextSetBit(i + 1)) {
+      int an = vwr.ms.at[i].getAtomNumber();
+      if (an > atomno)
+         atomno = an;
+    }
+    
     SB sbConnect = new SB();
-    for (int i = 0; i < vConnections.size(); i++) {
+    for (int i = 0, atomIndex = vwr.ms.ac; i < vConnections.size(); i++, atomIndex++) {
       Atom a = vConnections.get(i);
-      sbConnect.append(";  connect 0 100 ").append("({" + (atomIndex++) + "}) ")
+      if (a != null)
+        sbConnect.append(";  connect 0 100 ").append("({" + (atomIndex) + "}) ")
           .append("({" + a.i + "}) group;");
     }
     SB sb = new SB();
@@ -768,13 +789,16 @@ public class ScriptManager implements JmolScriptManager {
           .append(" ").appendF(pts[i].z).append(" - - - - ").appendI(++atomno)
           .appendC('\n');
     
-    Map<String, Object> htParams = new Hashtable<String, Object>();
-    htParams.put("appendToModelIndex", Integer.valueOf(modelIndex));
     vwr.openStringInlineParamsAppend(sb.toString(), htParams, true);
-    eval.runScriptBuffer(sbConnect.toString(), null, false);
+    vwr.runScript(sbConnect.toString());
     BS bsB = vwr.getModelUndeletedAtomsBitSet(modelIndex);
     bsB.andNot(bsA);
     vwr.g.appendNew = wasAppendNew;
+    if (!siteFixed) {
+    bsA = vwr.ms.am[modelIndex].bsAsymmetricUnit;
+    if (bsA != null)
+      bsA.or(bsB);
+    }
     return bsB;
   }
 
