@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2021-12-04 23:29:43 -0600 (Sat, 04 Dec 2021) $
- * $Revision: 22271 $
+ * $Date: 2022-07-24 20:37:43 +0530 (Sun, 24 Jul 2022) $
+ * $Revision: 22433 $
  *
  * Copyright (C) 2002-2005  The Jmol Development Team
  *
@@ -45,7 +45,6 @@ import org.jmol.viewer.JC;
 
 import java.util.Hashtable;
 
-
 import java.util.Map;
 
 public class Labels extends AtomShape {
@@ -53,7 +52,7 @@ public class Labels extends AtomShape {
   public String[] strings;
   public String[] formats;
   public short[] bgcolixes;
-  public byte[] fids;
+  public int[] fids;
   public int[] offsets;
 
   private Map<Integer, Text> atomLabels = new Hashtable<Integer, Text>();
@@ -65,16 +64,20 @@ public class Labels extends AtomShape {
   public int defaultOffset;
   public int defaultAlignment;
   public int defaultZPos;
-  public byte defaultFontId;
+  public int defaultFontId;
   public short defaultColix;
   public short defaultBgcolix;
   public byte defaultPaletteID;
   public int defaultPointer;
-  public byte zeroFontId;
 
-  private boolean defaultsOnlyForNone = true;
+  public int zeroFontId;
+
+  //  private boolean defaultsOnlyForNone = true;
+  /**
+   * defaults are set after giving SELECT NONE;
+   */
   private boolean setDefaults = false;
-  
+
   //labels
 
   @Override
@@ -92,9 +95,10 @@ public class Labels extends AtomShape {
 
   @SuppressWarnings("unchecked")
   @Override
-  public void setProperty(String propertyName, Object value, BS bsSelected) {
+  public void setProperty(String propertyName, Object value, BS bs) {
     isActive = true;
-
+    Atom[] atoms = ms.at;
+    int ac = ms.ac;
     if ("setDefaults" == propertyName) {
       setDefaults = ((Boolean) value).booleanValue();
       return;
@@ -103,15 +107,14 @@ public class Labels extends AtomShape {
     if ("color" == propertyName) {
       byte pid = PAL.pidOf(value);
       short colix = C.getColixO(value);
-      if (!setDefaults) {
-        int n = checkColixLength(colix, bsSelected.length());
-        for (int i = bsSelected.nextSetBit(0); i >= 0 && i < n; i = bsSelected
-            .nextSetBit(i + 1))
-          setLabelColix(i, colix, pid);
-      }
-      if (setDefaults || !defaultsOnlyForNone) {
+      if (setDefaults) {// || !defaultsOnlyForNone) {
         defaultColix = colix;
         defaultPaletteID = pid;
+      } else {
+        int n = checkColixLength(colix, bs.length());
+        for (int i = bs.nextSetBit(0); i >= 0
+            && i < n; i = bs.nextSetBit(i + 1))
+          setLabelColix(i, colix, pid);
       }
       return;
     }
@@ -122,8 +125,8 @@ public class Labels extends AtomShape {
       float val = ((Float) value).floatValue();
       float scalePixelsPerMicron = (val == 0 ? 0 : 10000f / val);
       int n = Math.min(ac, strings.length);
-      for (int i = bsSelected.nextSetBit(0); i >= 0 && i < n; i = bsSelected
-          .nextSetBit(i + 1)) {
+      for (int i = bs.nextSetBit(0); i >= 0
+          && i < n; i = bs.nextSetBit(i + 1)) {
         Text text = getLabel(i);
         if (text == null) {
           text = Text.newLabel(vwr, null, strings[i], C.INHERIT_ALL, (short) 0,
@@ -137,20 +140,21 @@ public class Labels extends AtomShape {
     }
 
     if ("label" == propertyName) {
-      boolean isPicked = (isPickingMode() && bsSelected.cardinality() == 1 && bsSelected.nextSetBit(0) == lastPicked);
+      boolean isPicked = (isPickingMode() && bs.cardinality() == 1
+          && bs.nextSetBit(0) == lastPicked);
       setScaling();
       LabelToken[][] tokens = null;
-      int nbs = checkStringLength(bsSelected.length());
+      int nbs = checkStringLength(bs.length());
       if (defaultColix != C.INHERIT_ALL || defaultPaletteID != 0)
-        checkColixLength(defaultColix, bsSelected.length());
+        checkColixLength(defaultColix, bs.length());
       if (defaultBgcolix != C.INHERIT_ALL)
-        checkBgColixLength(defaultBgcolix, bsSelected.length());
+        checkBgColixLength(defaultBgcolix, bs.length());
       if (value instanceof Lst) {
         Lst<SV> list = (Lst<SV>) value;
         int n = list.size();
         tokens = new LabelToken[][] { null };
-        for (int pt = 0, i = bsSelected.nextSetBit(0); i >= 0 && i < nbs; i = bsSelected
-            .nextSetBit(i + 1)) {
+        for (int pt = 0, i = bs.nextSetBit(0); i >= 0
+            && i < nbs; i = bs.nextSetBit(i + 1)) {
           if (pt >= n) {
             setLabel(nullToken, "", i, !isPicked);
             continue;
@@ -162,8 +166,8 @@ public class Labels extends AtomShape {
         String strLabel = (String) value;
         tokens = (strLabel == null || strLabel.length() == 0 ? nullToken
             : new LabelToken[][] { null });
-        for (int i = bsSelected.nextSetBit(0); i >= 0 && i < ac; i = bsSelected
-            .nextSetBit(i + 1))
+        for (int i = bs.nextSetBit(0); i >= 0
+            && i < ac; i = bs.nextSetBit(i + 1))
           setLabel(tokens, strLabel, i, !isPicked);
       }
       return;
@@ -178,8 +182,8 @@ public class Labels extends AtomShape {
       String label = propertyName.substring(6);
       if (label.length() == 0)
         label = null;
-      setLabel(new LabelToken[][] { null }, label,
-          ((Integer) value).intValue(), false);
+      setLabel(new LabelToken[][] { null }, label, ((Number) value).intValue(),
+          false);
       return;
     }
 
@@ -198,14 +202,14 @@ public class Labels extends AtomShape {
       if (bsBgColixSet == null)
         bsBgColixSet = BS.newN(ac);
       short bgcolix = C.getColixO(value);
-      if (!setDefaults) {
-        int n = checkBgColixLength(bgcolix, bsSelected.length());
-        for (int i = bsSelected.nextSetBit(0); i >= 0 && i < n; i = bsSelected
-            .nextSetBit(i + 1))
+      if (setDefaults) { // || !defaultsOnlyForNone)
+        defaultBgcolix = bgcolix;
+      } else {
+        int n = checkBgColixLength(bgcolix, bs.length());
+        for (int i = bs.nextSetBit(0); i >= 0
+            && i < n; i = bs.nextSetBit(i + 1))
           setBgcolix(i, bgcolix);
       }
-      if (setDefaults || !defaultsOnlyForNone)
-        defaultBgcolix = bgcolix;
       return;
     }
 
@@ -220,45 +224,50 @@ public class Labels extends AtomShape {
         fids = null;
         return;
       }
-      byte fid = vwr.gdata.getFontFid(fontsize);
-      if (!setDefaults)
-        for (int i = bsSelected.nextSetBit(0); i >= 0 && i < ac; i = bsSelected
-            .nextSetBit(i + 1))
-          setFont(i, fid);
-      if (setDefaults || !defaultsOnlyForNone)
-        defaultFontId = fid;
+      Font f;
+      if (setDefaults) { // || !defaultsOnlyForNone)
+        f = Font.getFont3D(defaultFontId);
+        defaultFontId = vwr.getFont3D(f.fontFace, f.fontStyle, fontsize).fid;
+      } else {
+        for (int i = bs.nextSetBit(0); i >= 0
+            && i < ac; i = bs.nextSetBit(i + 1)) {
+          f = Font.getFont3D(
+              fids == null || i >= fids.length ? fids[i] : defaultFontId);
+          setFont(i, vwr.getFont3D(f.fontFace, f.fontStyle, fontsize).fid);
+        }
+      }
       return;
     }
 
     if ("font" == propertyName) {
-      byte fid = ((Font) value).fid;
-      if (!setDefaults)
-        for (int i = bsSelected.nextSetBit(0); i >= 0 && i < ac; i = bsSelected
-            .nextSetBit(i + 1))
-          setFont(i, fid);
-      if (setDefaults || !defaultsOnlyForNone)
+      int fid = ((Font) value).fid;
+      if (setDefaults) {// || !defaultsOnlyForNone)
         defaultFontId = fid;
+      } else {
+        for (int i = bs.nextSetBit(0); i >= 0
+            && i < ac; i = bs.nextSetBit(i + 1))
+          setFont(i, fid);
+      }
       return;
     }
 
     if ("offset" == propertyName) {
-      if (!(value instanceof Integer)) {
-        if (!setDefaults) {
-          checkColixLength((short)-1, ac);
-          for (int i = bsSelected.nextSetBit(0); i >= 0 && i < ac; i = bsSelected
-              .nextSetBit(i + 1))
-            setPymolOffset(i, (float[]) value);
+      if (value instanceof Integer) {
+        int offset = ((Integer) value).intValue();
+        if (setDefaults) {// || !defaultsOnlyForNone)
+          defaultOffset = offset;
+        } else {
+          for (int i = bs.nextSetBit(0); i >= 0
+              && i < ac; i = bs.nextSetBit(i + 1))
+            setOffsets(i, offset);
         }
-        return;
+      } else if (!setDefaults) {
+        // pymol offset only
+        checkColixLength((short) -1, ac);
+        for (int i = bs.nextSetBit(0); i >= 0
+            && i < ac; i = bs.nextSetBit(i + 1))
+          setPymolOffset(i, (float[]) value);
       }
-
-      int offset = ((Integer) value).intValue();
-      if (!setDefaults)
-        for (int i = bsSelected.nextSetBit(0); i >= 0 && i < ac; i = bsSelected
-            .nextSetBit(i + 1))
-          setOffsets(i, offset);
-      if (setDefaults || !defaultsOnlyForNone)
-        defaultOffset = offset;
       return;
     }
 
@@ -268,51 +277,56 @@ public class Labels extends AtomShape {
       int hAlignment = (type.equalsIgnoreCase("right") ? JC.TEXT_ALIGN_RIGHT
           : type.equalsIgnoreCase("center") ? JC.TEXT_ALIGN_CENTER
               : JC.TEXT_ALIGN_LEFT);
-      for (int i = bsSelected.nextSetBit(0); i >= 0 && i < ac; i = bsSelected
-          .nextSetBit(i + 1))
-        setHorizAlignment(i, hAlignment);
-      if (setDefaults || !defaultsOnlyForNone)
+      if (setDefaults) {// || !defaultsOnlyForNone)
         defaultAlignment = hAlignment;
+      } else {
+        for (int i = bs.nextSetBit(0); i >= 0
+            && i < ac; i = bs.nextSetBit(i + 1))
+          setHorizAlignment(i, hAlignment);
+      }
       return;
     }
 
     if ("pointer" == propertyName) {
       int pointer = ((Integer) value).intValue();
-      if (!setDefaults)
-        for (int i = bsSelected.nextSetBit(0); i >= 0 && i < ac; i = bsSelected
-            .nextSetBit(i + 1))
-          setPointer(i, pointer);
-      if (setDefaults || !defaultsOnlyForNone)
+      if (setDefaults) {// || !defaultsOnlyForNone)
         defaultPointer = pointer;
+      } else {
+        for (int i = bs.nextSetBit(0); i >= 0
+            && i < ac; i = bs.nextSetBit(i + 1))
+          setPointer(i, pointer);
+      }
       return;
     }
 
     if ("front" == propertyName) {
       boolean TF = ((Boolean) value).booleanValue();
-      if (!setDefaults)
-        for (int i = bsSelected.nextSetBit(0); i >= 0 && i < ac; i = bsSelected
-            .nextSetBit(i + 1))
-          setZPos(i, JC.LABEL_ZPOS_FRONT, TF);
-      if (setDefaults || !defaultsOnlyForNone)
+      if (setDefaults) {// || !defaultsOnlyForNone)
         defaultZPos = (TF ? JC.LABEL_ZPOS_FRONT : 0);
+      } else {
+        for (int i = bs.nextSetBit(0); i >= 0
+            && i < ac; i = bs.nextSetBit(i + 1))
+          setZPos(i, JC.LABEL_ZPOS_FRONT, TF);
+      }
       return;
     }
 
     if ("group" == propertyName) {
       boolean TF = ((Boolean) value).booleanValue();
-      if (!setDefaults)
-        for (int i = bsSelected.nextSetBit(0); i >= 0 && i < ac; i = bsSelected
-            .nextSetBit(i + 1))
-          setZPos(i, JC.LABEL_ZPOS_GROUP, TF);
-      if (setDefaults || !defaultsOnlyForNone)
+      if (setDefaults) {// || !defaultsOnlyForNone)
         defaultZPos = (TF ? JC.LABEL_ZPOS_GROUP : 0);
+      } else {
+        for (int i = bs.nextSetBit(0); i >= 0
+            && i < ac; i = bs.nextSetBit(i + 1))
+          setZPos(i, JC.LABEL_ZPOS_GROUP, TF);
+      }
       return;
     }
 
     if ("display" == propertyName || "toggleLabel" == propertyName) {
       // toggle
-      int mode = ("toggleLabel" == propertyName ? 0 : ((Boolean) value)
-          .booleanValue() ? 1 : -1);
+      int mode = ("toggleLabel" == propertyName ? 0
+          : ((Boolean) value).booleanValue() ? 1 : -1);
       if (mads == null)
         mads = new short[ac];
       String strLabelPDB = null;
@@ -321,12 +335,12 @@ public class Labels extends AtomShape {
       LabelToken[] tokensUNK = null;
       String strLabel;
       LabelToken[] tokens;
-      int nstr = checkStringLength(bsSelected.length());
+      int nstr = checkStringLength(bs.length());
       short bgcolix = defaultBgcolix;
-      int nbg = checkBgColixLength(bgcolix, bsSelected.length());
+      int nbg = checkBgColixLength(bgcolix, bs.length());
       short thisMad = (short) (mode >= 0 ? 1 : -1);
-      for (int i = bsSelected.nextSetBit(0); i >= 0 && i < ac; i = bsSelected
-          .nextSetBit(i + 1)) {
+      for (int i = bs.nextSetBit(0); i >= 0
+          && i < ac; i = bs.nextSetBit(i + 1)) {
         Atom atom = atoms[i];
         if (i < nstr && strings[i] != null) {
           // an old string -- toggle
@@ -363,7 +377,7 @@ public class Labels extends AtomShape {
     }
 
     if ("pymolLabels" == propertyName) {
-      setPymolLabels((Map<Integer, Text>) value, bsSelected);
+      setPymolLabels((Map<Integer, Text>) value, bs);
       return;
     }
 
@@ -371,7 +385,7 @@ public class Labels extends AtomShape {
       labelBoxes = null;
       int firstAtomDeleted = ((int[]) ((Object[]) value)[2])[1];
       int nAtomsDeleted = ((int[]) ((Object[]) value)[2])[2];
-      fids = (byte[]) AU.deleteElements(fids, firstAtomDeleted, nAtomsDeleted);
+      fids = (int[]) AU.deleteElements(fids, firstAtomDeleted, nAtomsDeleted);
       bgcolixes = (short[]) AU.deleteElements(bgcolixes, firstAtomDeleted,
           nAtomsDeleted);
       offsets = (int[]) AU.deleteElements(offsets, firstAtomDeleted,
@@ -380,20 +394,22 @@ public class Labels extends AtomShape {
           nAtomsDeleted);
       strings = (String[]) AU.deleteElements(strings, firstAtomDeleted,
           nAtomsDeleted);
-      BSUtil.deleteBits(bsFontSet, bsSelected);
-      BSUtil.deleteBits(bsBgColixSet, bsSelected);
+      BSUtil.deleteBits(bsFontSet, bs);
+      BSUtil.deleteBits(bsBgColixSet, bs);
       // pass to super
     }
 
-    setPropAS(propertyName, value, bsSelected);
+    setPropAS(propertyName, value, bs);
 
   }
 
   private boolean isPickingMode() {
-    return (vwr.getPickingMode() == ActionManager.PICKING_LABEL && labelBoxes != null);
+    return (vwr.getPickingMode() == ActionManager.PICKING_LABEL
+        && labelBoxes != null);
   }
 
   private int checkStringLength(int n) {
+    int ac = ms.ac;
     n = Math.min(ac, n);
     if (strings == null || n > strings.length) {
       formats = AU.ensureLengthS(formats, n);
@@ -405,27 +421,27 @@ public class Labels extends AtomShape {
   }
 
   private int checkBgColixLength(short colix, int n) {
-    n = Math.min(ac, n);
+    n = Math.min(ms.ac, n);
     if (colix == C.INHERIT_ALL)
       return (bgcolixes == null ? 0 : bgcolixes.length);
     if (bgcolixes == null || n > bgcolixes.length)
       bgcolixes = AU.ensureLengthShort(bgcolixes, n);
     return n;
   }
-  
+
   private void setPymolLabels(Map<Integer, Text> labels, BS bsSelected) {
     // from PyMOL reader
     setScaling();
-    int n = checkStringLength(ac);
-    checkColixLength((short)-1, n);
-    for (int i = bsSelected.nextSetBit(0); i >= 0 && i < n; i = bsSelected
-        .nextSetBit(i + 1))
+    int n = checkStringLength(ms.ac);
+    checkColixLength((short) -1, n);
+    for (int i = bsSelected.nextSetBit(0); i >= 0
+        && i < n; i = bsSelected.nextSetBit(i + 1))
       setPymolLabel(i, labels.get(Integer.valueOf(i)), null);
   }
 
   /**
-   * Sets offset using PyMOL standard array;
-   * only operates in cases where label is already defined
+   * Sets offset using PyMOL standard array; only operates in cases where label
+   * is already defined
    * 
    * @param i
    * @param value
@@ -436,11 +452,12 @@ public class Labels extends AtomShape {
     if (text == null) {
       if (strings == null || i >= strings.length || strings[i] == null)
         return;
-      byte fid = (bsFontSet != null && bsFontSet.get(i) ? fids[i] : -1);
+      int fid = (bsFontSet != null && bsFontSet.get(i) ? fids[i] : -1);
       if (fid < 0)
         setFont(i, fid = defaultFontId);
+      Atom a = ms.at[i];
       text = Text.newLabel(vwr, Font.getFont3D(fid), strings[i],
-          getColix2(i, atoms[i], false), getColix2(i, atoms[i], true), 0,
+          getColix2(i, a, false), getColix2(i, a, true), 0,
           scalePixelsPerMicron);
       setPymolLabel(i, text, formats[i]);
     }
@@ -451,22 +468,26 @@ public class Labels extends AtomShape {
   private boolean isScaled;
   private float scalePixelsPerMicron;
   private P3 ptTemp = new P3();
-  
+
   private void setScaling() {
     isActive = true;
     if (bsSizeSet == null)
-      bsSizeSet = BS.newN(ac);
+      bsSizeSet = BS.newN(ms.ac);
     isScaled = vwr.getBoolean(T.fontscaling);
-    scalePixelsPerMicron = (isScaled ? vwr
-        .getScalePixelsPerAngstrom(false) * 10000f : 0);
+    scalePixelsPerMicron = (isScaled
+        ? vwr.getScalePixelsPerAngstrom(false) * 10000f
+        : 0);
   }
-  
+
   private void setPymolLabel(int i, Text t, String format) {
     if (t == null)
       return;
     String label = t.text;
-    Atom atom = atoms[i];
-    addString(atom, i, label, format == null ? PT.rep(label, "%", "%%") : format);
+    Atom atom = ms.at[i];
+    if (atom == null)
+      return;
+    addString(atom, i, label,
+        format == null ? PT.rep(label, "%", "%%") : format);
     atom.setShapeVisibility(vf, true);
     if (t.colix >= 0)
       setLabelColix(i, t.colix, PAL.UNKNOWN.id);
@@ -474,19 +495,22 @@ public class Labels extends AtomShape {
     putLabel(i, t);
   }
 
-  private void setLabel(LabelToken[][] temp, String strLabel, int i, boolean doAll) {
+  private void setLabel(LabelToken[][] temp, String strLabel, int i,
+                        boolean doAll) {
     // checkStringLength must be first
-    Atom atom = atoms[i];
+    Atom atom = ms.at[i];
     LabelToken[] tokens = temp[0];
     if (tokens == null)
       tokens = temp[0] = LabelToken.compile(vwr, strLabel, '\0', null);
-    String label = (tokens == null ? null : LabelToken.formatLabelAtomArray(
-        vwr, atom, tokens, '\0', null, ptTemp ));
+    String label = (tokens == null ? null
+        : LabelToken.formatLabelAtomArray(vwr, atom, tokens, '\0', null,
+            ptTemp));
     boolean isNew = addString(atom, i, label, strLabel);
     doAll |= isNew || label == null;
     Text text = getLabel(i);
     if (isScaled && doAll) {
-      text = Text.newLabel(vwr, null, label, C.INHERIT_ALL, (short) 0, 0, scalePixelsPerMicron);
+      text = Text.newLabel(vwr, null, label, C.INHERIT_ALL, (short) 0, 0,
+          scalePixelsPerMicron);
       putLabel(i, text);
     } else if (text != null && label != null) {
       text.setText(label);
@@ -525,11 +549,13 @@ public class Labels extends AtomShape {
 
   @Override
   public Object getProperty(String property, int index) {
+    if (property.equals("font"))
+      return Font.getFont3D(defaultFontId);
     if (property.equals("offsets"))
       return offsets;
     if (property.equals("label"))
-      return (strings != null && index < strings.length && strings[index] != null 
-          ? strings[index] : "");
+      return (strings != null && index < strings.length
+          && strings[index] != null ? strings[index] : "");
     return null;
   }
 
@@ -538,7 +564,7 @@ public class Labels extends AtomShape {
       atomLabels.remove(Integer.valueOf(i));
     else {
       atomLabels.put(Integer.valueOf(i), text);
-      text.textUnformatted = formats[i]; 
+      text.textUnformatted = formats[i];
     }
   }
 
@@ -548,7 +574,7 @@ public class Labels extends AtomShape {
 
   public void putBox(int i, float[] boxXY) {
     if (labelBoxes == null)
-      labelBoxes = new Hashtable<Integer, float[]>(); 
+      labelBoxes = new Hashtable<Integer, float[]>();
     labelBoxes.put(Integer.valueOf(i), boxXY);
   }
 
@@ -557,7 +583,7 @@ public class Labels extends AtomShape {
       return null;
     return labelBoxes.get(Integer.valueOf(i));
   }
-  
+
   private void setLabelColix(int i, short colix, byte pid) {
     setColixAndPalette(colix, pid, i);
     // text is only created by labelsRenderer
@@ -575,11 +601,11 @@ public class Labels extends AtomShape {
   }
 
   private void setOffsets(int i, int offset) {
-    
+
     if (offsets == null || i >= offsets.length) {
       if (offset == JC.LABEL_DEFAULT_OFFSET)
         return;
-      offsets = AU.ensureLengthI(offsets, ac);
+      offsets = AU.ensureLengthI(offsets, ms.ac);
     }
     offsets[i] = (offsets[i] & JC.LABEL_FLAGS) | offset;
 
@@ -595,7 +621,7 @@ public class Labels extends AtomShape {
       case JC.TEXT_ALIGN_LEFT:
         return;
       }
-      offsets = AU.ensureLengthI(offsets, ac);
+      offsets = AU.ensureLengthI(offsets, ms.ac);
     }
     if (hAlign == JC.TEXT_ALIGN_NONE)
       hAlign = JC.TEXT_ALIGN_LEFT;
@@ -609,7 +635,7 @@ public class Labels extends AtomShape {
     if (offsets == null || i >= offsets.length) {
       if (pointer == JC.LABEL_POINTER_NONE)
         return;
-      offsets = AU.ensureLengthI(offsets, ac);
+      offsets = AU.ensureLengthI(offsets, ms.ac);
     }
     offsets[i] = JC.setPointer(offsets[i], pointer);
     Text text = getLabel(i);
@@ -621,16 +647,16 @@ public class Labels extends AtomShape {
     if (offsets == null || i >= offsets.length) {
       if (!TF)
         return;
-      offsets = AU.ensureLengthI(offsets, ac);
+      offsets = AU.ensureLengthI(offsets, ms.ac);
     }
     offsets[i] = JC.setZPosition(offsets[i], TF ? flag : 0);
   }
 
-  private void setFont(int i, byte fid) {
+  private void setFont(int i, int fid) {
     if (fids == null || i >= fids.length) {
       if (fid == zeroFontId)
         return;
-      fids = AU.ensureLengthByte(fids, ac);
+      fids = AU.ensureLengthI(fids, ms.ac);
     }
     fids[i] = fid;
     bsFontSet.set(i);
@@ -652,21 +678,22 @@ public class Labels extends AtomShape {
     }
   }
 
-//  @Override
-//  public String getShapeState() {
-//    // not implemented -- see org.jmol.viewer.StateCreator
-//    return null;
-//  }
+  //  @Override
+  //  public String getShapeState() {
+  //    // not implemented -- see org.jmol.viewer.StateCreator
+  //    return null;
+  //  }
 
   private int pickedAtom = -1;
   private int lastPicked = -1;
   private int pickedOffset = 0;
   private int pickedX;
   private int pickedY;
-  
-  
+
   @Override
-  public Map<String, Object> checkObjectClicked(int x, int y, int modifiers, BS bsVisible, boolean drawPicking) {
+  public Map<String, Object> checkObjectClicked(int x, int y, int modifiers,
+                                                BS bsVisible,
+                                                boolean drawPicking) {
     if (!isPickingMode())
       return null;
     int iAtom = findNearestLabel(x, y);
@@ -695,8 +722,9 @@ public class Labels extends AtomShape {
         vwr.acm.setDragAtomIndex(iAtom);
         pickedX = x;
         pickedY = y;
-        pickedOffset = (offsets == null || pickedAtom >= offsets.length ? 
-            JC.LABEL_DEFAULT_OFFSET : offsets[pickedAtom]);
+        pickedOffset = (offsets == null || pickedAtom >= offsets.length
+            ? JC.LABEL_DEFAULT_OFFSET
+            : offsets[pickedAtom]);
         return true;
       }
       return false;
@@ -709,7 +737,7 @@ public class Labels extends AtomShape {
     move2D(pickedAtom, x, y);
     return true;
   }
-                         
+
   private int findNearestLabel(int x, int y) {
     if (labelBoxes == null)
       return -1;
@@ -717,16 +745,20 @@ public class Labels extends AtomShape {
     int imin = -1;
     float zmin = Float.MAX_VALUE;
     float afactor = (vwr.antialiased ? 2 : 1);
+    Atom[] atoms = ms.at;
     for (Map.Entry<Integer, float[]> entry : labelBoxes.entrySet()) {
-      if (!atoms[entry.getKey().intValue()].isVisible(vf | Atom.ATOM_INFRAME_NOTHIDDEN))
+      if (!atoms[entry.getKey().intValue()]
+          .isVisible(vf | Atom.ATOM_INFRAME_NOTHIDDEN))
         continue;
       float[] boxXY = entry.getValue();
-      float dx = (x - boxXY[0])*afactor;
-      float dy = (y - boxXY[1])*afactor;
-      if (dx <= 0 || dy <= 0 || dx >= boxXY[2] || dy >= boxXY[3] || boxXY[4] > zmin)
+      float dx = (x - boxXY[0]) * afactor;
+      float dy = (y - boxXY[1]) * afactor;
+      if (dx <= 0 || dy <= 0 || dx >= boxXY[2] || dy >= boxXY[3]
+          || boxXY[4] > zmin)
         continue;
       zmin = boxXY[4];
-      float d = Math.min(Math.abs(dx - boxXY[2]/2), Math.abs(dy - boxXY[3]/2));
+      float d = Math.min(Math.abs(dx - boxXY[2] / 2),
+          Math.abs(dy - boxXY[3] / 2));
       if (d <= dmin) {
         dmin = d;
         imin = entry.getKey().intValue();
@@ -737,7 +769,7 @@ public class Labels extends AtomShape {
 
   private void move2D(int pickedAtom, int x, int y) {
     int xOffset = JC.getXOffset(pickedOffset);
-    int yOffset = JC.getYOffset(pickedOffset);        
+    int yOffset = JC.getYOffset(pickedOffset);
     xOffset += x - pickedX;
     yOffset -= y - pickedY;
     int offset = JC.getOffset(xOffset, yOffset, true);
@@ -756,5 +788,5 @@ public class Labels extends AtomShape {
     }
     return colix;
   }
-  
+
 }
