@@ -36,6 +36,7 @@ import java.util.Map;
 import org.jmol.util.BSUtil;
 import org.jmol.util.C;
 import org.jmol.util.Escape;
+import org.jmol.util.Font;
 import org.jmol.util.Logger;
 import org.jmol.util.MeshSurface;
 
@@ -50,6 +51,7 @@ import javajs.util.V3;
 import org.jmol.viewer.ActionManager;
 import org.jmol.viewer.JC;
 import javajs.util.BS;
+
 import org.jmol.script.SV;
 import org.jmol.script.T;
 import org.jmol.shape.Mesh;
@@ -60,6 +62,12 @@ public class Draw extends MeshCollection {
 
   // bob hanson hansonr@stolaf.edu 3/2006
   
+  private final static int PT_COORD = 1;
+  private final static int PT_IDENTIFIER = 2;
+  private final static int PT_BITSET = 3;
+  private final static int PT_MODEL_INDEX = 4;
+  private final static int PT_MODEL_BASED_POINTS = 5;
+
   public Draw() {
     // from reflection
     htObjects = new Hashtable<String, Mesh>();
@@ -111,6 +119,7 @@ public void initShape() {
   private boolean isCylinder;
   private boolean isVertices;
   private boolean isPlane;
+//  private boolean isBest;
   private boolean isReversed;
   private boolean isRotated45;
   private boolean isCrossed;
@@ -128,14 +137,12 @@ public void initShape() {
   
   private Lst<Object[]> vData;
   private String intersectID;
-  private P3[] boundBox;
   
   private Lst<P3[]> lineData;
-  private final static int PT_COORD = 1;
-  private final static int PT_IDENTIFIER = 2;
-  private final static int PT_BITSET = 3;
-  private final static int PT_MODEL_INDEX = 4;
-  private final static int PT_MODEL_BASED_POINTS = 5;
+  public int defaultFontId0 = -1;
+  public int defaultFontId = -1;
+  private int thisFontID = -1;
+  private Integer titleColor;
 
   @SuppressWarnings("unchecked")
   @Override
@@ -147,6 +154,20 @@ public void initShape() {
       return;
     }
 
+    if ("font" == propertyName) {
+      defaultFontId = (value == null ? -1 : ((Font) value).fid);
+      defaultFontSize = (value == null ? JC.DRAW_DEFAULT_FONTSIZE : ((Font) value).fontSize);
+      return;
+    }
+    
+    if ("myfont" == propertyName) {
+      thisFontID = ((Font) value).fid;
+      if (thisMesh != null) {
+        thisMesh.fontID = thisFontID; 
+      }
+      return;
+    }
+    
     if ("length" == propertyName) {
       length = ((Float) value).floatValue();
       return;
@@ -158,10 +179,7 @@ public void initShape() {
     }
 
     if ("intersect" == propertyName) {
-      if (value instanceof String)
         intersectID = (String) value;
-      else
-        boundBox = (P3[]) value;
       return;
     }
 
@@ -200,7 +218,7 @@ public void initShape() {
     if ("modelIndex" == propertyName) {
       //from saved state -- used to set modelVertices
       indicatedModelIndex = ((Integer) value).intValue();
-      if (indicatedModelIndex < 0 || indicatedModelIndex >= vwr.ms.mc)
+      if (indicatedModelIndex < 0 || indicatedModelIndex >= ms.mc)
         return;
       vData.addLast(new Object[] { Integer.valueOf(PT_MODEL_INDEX),
           (modelInfo = new int[] { indicatedModelIndex, 0 }) });
@@ -209,7 +227,7 @@ public void initShape() {
 
     if ("planedef" == propertyName) {
       plane = (P4) value;
-      if (intersectID != null || boundBox != null || slabData != null)
+      if (intersectID != null || slabData != null)
         return;
       if (isCircle || isArc)
         isPlane = true;
@@ -372,7 +390,7 @@ public void initShape() {
       vData.addLast(new Object[] { Integer.valueOf(PT_BITSET), bsAtoms });
       //nbitsets++;
       if (isCircle && diameter == 0 && width == 0)
-        width = vwr.ms.calcRotationRadiusBs(bsAtoms) * 2.0f;
+        width = ms.calcRotationRadiusBs(bsAtoms) * 2.0f;
       return;
     }
 
@@ -385,7 +403,28 @@ public void initShape() {
       addPoints(PT_MODEL_BASED_POINTS, value);
       return;
     }
+    
+    if ("title" == propertyName) {
+      if (thisMesh != null) {
+        thisMesh.title = setTitle(value);
+        return;
+      }
+    }
 
+    if ("titlecolor" == propertyName) {
+      Integer c = ((Integer) value);
+      titleColor = c;
+      if (thisMesh != null) {
+        thisMesh.titleColor = c;
+      }
+      return;
+    }
+
+//    if ("best" == propertyName) {
+//      isBest = ((Boolean) value).booleanValue();
+//      return;
+//    }
+//    
     if ("set" == propertyName) {
       if (thisMesh == null) {
         allocMesh(null, null);
@@ -400,6 +439,8 @@ public void initShape() {
         thisMesh.initialize(T.fullylit, null, null);
         setAxes(thisMesh);
         thisMesh.title = title;
+        thisMesh.titleColor = titleColor;
+        thisMesh.fontID = thisFontID;
         thisMesh.visible = true;
       }
       nPoints = -1; // for later scaling
@@ -416,65 +457,28 @@ public void initShape() {
     setPropertySuper(propertyName, value, bs);
   }
 
-  private void addPoints(int type, Object value) {
-    @SuppressWarnings("unchecked")
-    Lst<SV> pts = (Lst<SV>) value;
-    Integer key = Integer.valueOf(type);
-    boolean isModelPoints = (type == PT_MODEL_BASED_POINTS);
-    if (isModelPoints)
-      vData.addLast(new Object[] { key, pts });
-    for (int i = 0, n = pts.size(); i < n; i++) {
-      Object o = pts.get(i);
-      P3 pt;
-      if (o instanceof P3) {
-        // from Draw HKL
-        pt = (P3) o;
-      } else {
-        SV v = (SV) o;
-        switch (v.tok) {
-        case T.bitset:
-          if (!isModelPoints && ((BS) v.value).isEmpty())
-            continue;
-          pt = vwr.ms.getAtomSetCenter((BS) v.value);
-          break;
-        case T.point3f:
-          if (isModelPoints)
-            continue;
-          //$FALL-THROUGH$
-        default:
-          pt = SV.ptValue(v);
-        }
+  private void deleteModels(int modelIndex) {
+    //int firstAtomDeleted = ((int[])((Object[])value)[2])[1];
+    //int nAtomsDeleted = ((int[])((Object[])value)[2])[2];
+    for (int i = meshCount; --i >= 0;) {
+      DrawMesh m = dmeshes[i];
+      if (m == null)
+        continue;
+      boolean deleteMesh = (m.modelIndex == modelIndex);
+      if (m.modelFlags != null) {
+        m.deleteAtoms(modelIndex);
+        deleteMesh = (m.modelFlags.length() == 0);
+        if (!deleteMesh)
+          continue;
       }
-      if (isModelPoints) {
-        pts.set(i, SV.getVariable(pt));
-      } else {
-        vData.addLast(new Object[] { key, pt });
+      if (deleteMesh) {
+        meshCount--;
+        deleteMeshElement(i);
+      } else if (meshes[i].modelIndex > modelIndex) {
+        meshes[i].modelIndex--;
       }
     }
-  }
-
-private void deleteModels(int modelIndex) {
-   //int firstAtomDeleted = ((int[])((Object[])value)[2])[1];
-   //int nAtomsDeleted = ((int[])((Object[])value)[2])[2];
-   for (int i = meshCount; --i >= 0;) {
-     DrawMesh m = dmeshes[i];
-     if (m == null)
-       continue;
-     boolean deleteMesh = (m.modelIndex == modelIndex);
-     if (m.modelFlags != null) {
-       m.deleteAtoms(modelIndex);
-       deleteMesh = (m.modelFlags.length() == 0);
-       if (!deleteMesh)
-         continue;
-     } 
-     if (deleteMesh) {
-       meshCount--;
-       deleteMeshElement(i);
-     } else if (meshes[i].modelIndex > modelIndex) {
-       meshes[i].modelIndex--;
-     }
-   }
-   resetObjects();
+    resetObjects();
   }
 
  private void deleteMeshElement(int i) {
@@ -485,15 +489,16 @@ private void deleteModels(int modelIndex) {
 }
 
 private void initDraw() {
-   boundBox = null;
    bsAllModels = null;
    colix = C.ORANGE;
    color = 0xFFFFFFFF;
    diameter = 0;
    explicitID = false;
+   thisFontID = -1;
    indicatedModelIndex = -1;
    intersectID = null;
    isCurve = isArc = isArrow = isPlane = isCircle = isCylinder = isLine = false;
+   //isBest = 
    isFixed = isReversed = isRotated45 = isCrossed = noHead = isBarb = false;
    isPerpendicular = isVertices = isVector = false;
    isValid = true;
@@ -505,6 +510,7 @@ private void initDraw() {
    plane = null;
    polygon = null;
    slabData = null;
+   titleColor = null;
    vData = new  Lst<Object[]>();
    width = 0;
    setPropertySuper("thisID", MeshCollection.PREVIOUS_MESH_ID, null);
@@ -540,6 +546,15 @@ private void initDraw() {
     DrawMesh m = thisMesh;
     if (index >= 0 && (index >= meshCount || (m = (DrawMesh) meshes[index]) == null))
       return null;
+    if (property.equals("font")) {
+      if (defaultFontId < 0) {
+        setProperty("font", vwr.gdata.getFont3DFSS(JC.DEFAULT_FONTFACE,
+            JC.DEFAULT_FONTSTYLE, vwr.getFloat(T.drawfontsize)), null);
+        defaultFontId0 = defaultFontId;
+      }
+      return Font.getFont3D(index < 0 || m.fontID < 0 ? defaultFontId : m.fontID);
+    }
+
     if (property == "command")
       return getCommand(m);
     if (property == "type")
@@ -590,13 +605,8 @@ private void initDraw() {
     thisMesh.clear("draw");
     thisMesh.diameter = diameter;
     thisMesh.width = width;
-    if (intersectID != null || boundBox != null) {
-      if (boundBox != null) {
-        // TODO
-        if (plane == null) {
-
-        }
-      } else if (plane != null && intersectID != null) {
+    if (plane != null) {
+      if (intersectID != null) {
         Lst<P3[]> vData = new Lst<P3[]>();
         Object[] data = new Object[] { intersectID, plane, vData, null };
         vwr.shm.getShapePropertyData(JC.SHAPE_ISOSURFACE, "intersectPlane",
@@ -605,32 +615,31 @@ private void initDraw() {
           indicatedModelIndex = ((Integer) data[3]).intValue();
           lineData = vData;
         }
+      } else if (slabData != null) {
+        slabData.getMeshSlicer().getIntersection(0, plane, null, null, null,
+            null, null, false, true, T.plane, false);
+        polygon = new Lst<Object>();
+        polygon.addLast(slabData.vs);
+        polygon.addLast(slabData.pis);
       }
-    } else if (slabData != null && plane != null) {
-      slabData.getMeshSlicer().getIntersection(0, plane, null, null, null,
-          null, null, false, true, T.plane, false);
-      polygon = new Lst<Object>();
-      polygon.addLast(slabData.vs);
-      polygon.addLast(slabData.pis);
     }
     if (polygon == null
         && (lineData != null ? lineData.size() == 0
-            : (vData.size() == 0) == (connections == null)) || !isArrow
-        && connections != null)
+            : (vData.size() == 0) == (connections == null))
+        || !isArrow && connections != null)
       return false; // connections only for arrows at this point
-    int modelCount = vwr.ms.mc;
-    if (polygon != null
-        || lineData != null
-        || indicatedModelIndex < 0
-        && (isFixed || isArrow || isCurve || isCircle || isCylinder || modelCount == 1)) {
+    int modelCount = ms.mc;
+    if (polygon != null || lineData != null
+        || indicatedModelIndex < 0 && (isFixed || isArrow || isCurve
+            || isCircle || isCylinder || modelCount == 1)) {
       // make just ONE copy 
-      
+
       // arrows and curves simply can't be handled as
       // multiple frames yet
       thisMesh.modelIndex = (lineData == null ? vwr.am.cmi
           : indicatedModelIndex);
-      thisMesh.isFixed = (isFixed || lineData == null
-          && thisMesh.modelIndex < 0 && modelCount > 1);
+      thisMesh.isFixed = (isFixed
+          || lineData == null && thisMesh.modelIndex < 0 && modelCount > 1);
       if (isFixed && modelCount > 1)
         thisMesh.modelIndex = -1;
       else if (lineData == null && thisMesh.modelIndex < 0)
@@ -702,7 +711,7 @@ private void initDraw() {
     thisMesh.isBarb = isBarb;
     thisMesh.width = (thisMesh.drawType == EnumDrawType.CYLINDER
         || thisMesh.drawType == EnumDrawType.CIRCULARPLANE ? -Math.abs(width)
-        : width);
+            : width);
     thisMesh.setCenter(-1);
     if (offset != null)
       thisMesh.offset(offset);
@@ -724,6 +733,43 @@ private void initDraw() {
 
   MeshSurface slabData;
   
+  @SuppressWarnings("unchecked")
+  private void addPoints(int type, Object value) {
+    Lst<?> pts = (Lst<?>) value;
+    Integer key = Integer.valueOf(type);
+    boolean isModelPoints = (type == PT_MODEL_BASED_POINTS);
+    if (isModelPoints)
+      vData.addLast(new Object[] { key, pts });
+    for (int i = 0, n = pts.size(); i < n; i++) {
+      Object o = pts.get(i);
+      P3 pt;
+      if (o instanceof P3) {
+        // from Draw HKL
+        pt = (P3) o;
+      } else {
+        SV v = (SV) o;
+        switch (v.tok) {
+        case T.bitset:
+          if (!isModelPoints && ((BS) v.value).isEmpty())
+            continue;
+          pt = ms.getAtomSetCenter((BS) v.value);
+          break;
+        case T.point3f:
+          if (isModelPoints)
+            continue;
+          //$FALL-THROUGH$
+        default:
+          pt = SV.ptValue(v);
+        }
+      }
+      if (isModelPoints) {
+        ((Lst<SV>) pts).set(i, SV.getVariable(pt));
+      } else {
+        vData.addLast(new Object[] { key, pt });
+      }
+    }
+  }
+
   private void addPoint(T3 newPt, int iModel) {
     if (makePoints) {
       if (newPt == null || iModel >= 0 && !bsAllModels.get(iModel))
@@ -794,7 +840,7 @@ private void initDraw() {
         if (bsModel != null)
           bs.and(bsModel);
         if (bs.length() > 0)
-          addPoint(vwr.ms.getAtomSetCenter(bs), (makePoints ? iModel : -1));
+          addPoint(ms.getAtomSetCenter(bs), (makePoints ? iModel : -1));
         break;
       case PT_IDENTIFIER:
         int[] idInfo = (int[]) info[1];
@@ -845,7 +891,7 @@ private void initDraw() {
               if (bsModel != null)
                 bs.and(bsModel);
               if (bs.length() > 0)
-                addPoint(vwr.ms.getAtomSetCenter(bs), j);
+                addPoint(ms.getAtomSetCenter(bs), j);
             } else if (point instanceof SV) {
               addPoint(SV.ptValue((SV) point), j);
             }
@@ -971,7 +1017,8 @@ private void initDraw() {
             vAB);
         center = new P3();
         Measure.calcAveragePointN(ptList, nVertices, center);
-        dist = (length == Float.MAX_VALUE ? ptList[0].distance(center) : length);
+        dist = (length == Float.MAX_VALUE ? ptList[0].distance(center)
+            : length);
         normal.scale(dist);
         ptList[0].setT(center);
         ptList[1].add2(center, normal);
@@ -979,7 +1026,8 @@ private void initDraw() {
       } else if (nVertices == 2 && isPerpendicular) {
         // perpendicular line to line or plane to line
         Measure.calcAveragePoint(ptList[0], ptList[1], center);
-        dist = (length == Float.MAX_VALUE ? ptList[0].distance(center) : length);
+        dist = (length == Float.MAX_VALUE ? ptList[0].distance(center)
+            : length);
         if (isPlane && length != Float.MAX_VALUE)
           dist /= 2f;
         if (isPlane && isRotated45)
@@ -1032,9 +1080,10 @@ private void initDraw() {
         ptList[0].sub2(center, normal);
         ptList[1].add2(center, normal);
       }
-      if (nVertices > 4)
+      if (nVertices > 4) {
+        // even 4 is problematic
         nVertices = 4; // for now
-
+      }
       switch (nVertices) {
       case -2:
         nVertices = 2;
@@ -1042,12 +1091,15 @@ private void initDraw() {
       case 1:
         break;
       case 2:
+        //isBest = false;
         drawType = (isArc ? EnumDrawType.ARC
             : isPlane && isCircle ? EnumDrawType.CIRCULARPLANE
-                : isCylinder ? EnumDrawType.CYLINDER
-                    : EnumDrawType.LINE);
+                : isCylinder ? EnumDrawType.CYLINDER : EnumDrawType.LINE);
         break;
       default:
+//        if (isBest) {
+//          drawType = (isPlane ? EnumDrawType.PLANE : EnumDrawType.LINE);
+//        }
         drawType = (thisMesh.connectedAtoms == null ? EnumDrawType.PLANE
             : EnumDrawType.ARROW);
       }
@@ -1065,8 +1117,7 @@ private void initDraw() {
     thisMesh.setPolygonCount(nPoly + 1);
     thisMesh.pis[nPoly] = new int[npoints];
     for (int i = 0; i < npoints; i++) {
-      thisMesh.pis[nPoly][i] = nVertices0
-          + (i < nVertices ? i : nVertices - 1);
+      thisMesh.pis[nPoly][i] = nVertices0 + (i < nVertices ? i : nVertices - 1);
     }
     return;
   }
@@ -1171,6 +1222,7 @@ private void initDraw() {
   private final static int MAX_OBJECT_CLICK_DISTANCE_SQUARED = 10 * 10;
 
   private final P3i ptXY = new P3i();
+  private float defaultFontSize = JC.DRAW_DEFAULT_FONTSIZE;
   
   @Override
   public Map<String, Object> checkObjectClicked(int x, int y, int action,
@@ -1276,6 +1328,7 @@ private void initDraw() {
       x <<= 1;
       y <<= 1;
     }
+    
     int action = moveAll ? ActionManager.ACTION_dragDrawObject : ActionManager.ACTION_dragDrawPoint;
     if (vwr.acm.userActionEnabled(action) 
         && !vwr.acm.userAction(action, new Object[] { mesh.thisID, new int[] {x, y, iVertex} } ))
@@ -1332,7 +1385,7 @@ private void initDraw() {
       DrawMesh m = dmeshes[i];
       if (m.visibilityFlags != 0) {
         int mCount = (m.isDrawPolygon ? m.pc
-            : m.modelFlags == null ? 1 : vwr.ms.mc);
+            : m.modelFlags == null ? 1 : ms.mc);
         for (int iModel = mCount; --iModel >= 0;) {
           if (m.modelFlags != null
               && !m.modelFlags.get(iModel)
@@ -1382,14 +1435,12 @@ private void initDraw() {
 
   private String getCommand2(Mesh mesh, int iModel) {
     DrawMesh dmesh = (DrawMesh) mesh;
-    if (!dmesh.isValid || 
-        dmesh.drawType == EnumDrawType.NONE  
-        && dmesh.lineData == null 
-        && dmesh.drawVertexCount == 0 && dmesh.drawVertexCounts == null
-        )
+    if (!dmesh.isValid
+        || dmesh.drawType == EnumDrawType.NONE && dmesh.lineData == null
+            && dmesh.drawVertexCount == 0 && dmesh.drawVertexCounts == null)
       return "";
     SB str = new SB();
-    int modelCount = vwr.ms.mc;
+    int modelCount = ms.mc;
     if (!dmesh.isFixed && iModel >= 0 && modelCount > 1)
       appendCmd(str, "frame " + vwr.getModelNumberDotted(iModel));
     str.append("  draw ID ").append(PT.esc(dmesh.thisID));
@@ -1402,14 +1453,17 @@ private void initDraw() {
     else if (dmesh.isBarb)
       str.append(" barb");
     if (dmesh.scale != 1 && dmesh.isScaleSet
-        && (dmesh.haveXyPoints || dmesh.connectedAtoms != null || dmesh.drawType == EnumDrawType.CIRCLE || dmesh.drawType == EnumDrawType.ARC))
+        && (dmesh.haveXyPoints || dmesh.connectedAtoms != null
+            || dmesh.drawType == EnumDrawType.CIRCLE
+            || dmesh.drawType == EnumDrawType.ARC))
       str.append(" scale ").appendF(dmesh.scale);
     if (dmesh.width != 0)
       str.append(" diameter ").appendF(
           (dmesh.drawType == EnumDrawType.CYLINDER ? Math.abs(dmesh.width)
-              : dmesh.drawType == EnumDrawType.CIRCULARPLANE ? Math
-                  .abs(dmesh.width * dmesh.scale) : dmesh.width));
-    else if (dmesh.diameter > 0)
+              : dmesh.drawType == EnumDrawType.CIRCULARPLANE
+                  ? Math.abs(dmesh.width * dmesh.scale)
+                  : dmesh.width));
+    else if (dmesh.diameter != 0)
       str.append(" diameter ").appendI(dmesh.diameter);
     if (dmesh.lineData != null) {
       str.append("  lineData [");
@@ -1426,9 +1480,12 @@ private void initDraw() {
       }
       str.append("]");
     } else {
-      int nVertices = dmesh.drawVertexCount > 0  || dmesh.drawVertexCounts == null ? dmesh.drawVertexCount
-          : dmesh.drawVertexCounts[iModel >= 0 ? iModel : 0];
-      switch (dmesh.drawTypes == null || dmesh.drawTypes[iModel] == null ? dmesh.drawType : dmesh.drawTypes[iModel]) {
+      int nVertices = dmesh.drawVertexCount > 0
+          || dmesh.drawVertexCounts == null ? dmesh.drawVertexCount
+              : dmesh.drawVertexCounts[iModel >= 0 ? iModel : 0];
+      switch (dmesh.drawTypes == null || dmesh.drawTypes[iModel] == null
+          ? dmesh.drawType
+          : dmesh.drawTypes[iModel]) {
       case NONE:
       case MULTIPLE:
         break;
@@ -1500,11 +1557,18 @@ private void initDraw() {
       dmesh.mat4.getTranslation(v);
       str.append(" offset ").append(Escape.eP(v));
     }
+    if (dmesh.titleColor != null) {
+      str.append(" title color ")
+          .append(Escape.escapeColor(dmesh.titleColor.intValue()));
+    }
     if (dmesh.title != null) {
       String s = "";
       for (int i = 0; i < dmesh.title.length; i++)
         s += "|" + dmesh.title[i];
-      str.append(PT.esc(s.substring(1)));
+      str.append(" ").append(PT.esc(s.substring(1)));
+    }
+    if (dmesh.fontID >= 0) {
+      str.append(" font " + Font.getFont3D(dmesh.fontID).getInfo());
     }
     str.append(";\n");
     appendCmd(str, dmesh.getState("draw"));
@@ -1561,7 +1625,7 @@ private void initDraw() {
       info.put("scale", Float.valueOf(mesh.scale));
       if (mesh.drawType == EnumDrawType.MULTIPLE) {
         Lst<Map<String, Object>> m = new  Lst<Map<String,Object>>();
-        int modelCount = vwr.ms.mc;
+        int modelCount = ms.mc;
         for (int k = 0; k < modelCount; k++) {
           if (mesh.ptCenters[k] == null)
             continue;
@@ -1608,6 +1672,9 @@ private void initDraw() {
     SB s = new SB();
     s.append("\n");
     appendCmd(s, myType + " delete");
+    if (defaultFontId >= 0 && (
+        defaultFontId != defaultFontId0 || defaultFontSize != JC.DRAW_DEFAULT_FONTSIZE))
+      appendCmd(s, getFontCommand("draw", (Font) getProperty("font", -1)));
     for (int i = 0; i < meshCount; i++) {
       DrawMesh mesh = dmeshes[i];
       if (mesh.vc == 0 && mesh.lineData == null)
