@@ -70,7 +70,7 @@ public class SmilesGenerator {
    * 0x1000 CH2 explicit
    * 0x2000 CH2 and CH3 explicit
    */
-  private int explicitH;
+  private int explicitHydrogen;
   
   private Lst<BS> ringSets;
 
@@ -80,7 +80,7 @@ public class SmilesGenerator {
   private int nPairs, nPairsMax;
   private BS bsBondsUp = new BS();
   private BS bsBondsDn = new BS();
-  private BS bsToDo, bsIgnoreH = new BS();
+  private BS bsToDo, bsIgnoreHydrogen = new BS();
   private SimpleNode prevAtom;
   private SimpleNode[] prevSp2Atoms;
   private SimpleNode[] alleneStereo;
@@ -105,15 +105,28 @@ public class SmilesGenerator {
   private SmilesMatcher sm;
   private int iHypervalent;
   private boolean is2D;
+  private boolean haveSmilesAtoms;
 
   // generation of SMILES strings
 
+  /**
+   * The main entry point from SmilesMatcher
+   * @param sm
+   * @param atoms
+   * @param ac
+   * @param bsSelected
+   * @param comment
+   * @param flags
+   * @return SMILES
+   * @throws InvalidSmilesException
+   */
   String getSmiles(SmilesMatcher sm, Node[] atoms, int ac, BS bsSelected,
                    String comment, int flags)
       throws InvalidSmilesException {
     int ipt = bsSelected.nextSetBit(0);
     if (ipt < 0)
       return "";
+    haveSmilesAtoms = (atoms[ipt] instanceof SmilesAtom && ((SmilesAtom) atoms[ipt]).definesStereo());
     this.sm = sm;
     this.flags = flags;
     this.atoms = atoms;
@@ -134,7 +147,7 @@ public class SmilesGenerator {
     aromaticDouble = ((flags
         & JC.SMILES_AROMATIC_DOUBLE) == JC.SMILES_AROMATIC_DOUBLE);
 
-    explicitH = ((flags
+    explicitHydrogen = ((flags
         & JC.SMILES_GEN_EXPLICIT_H2_ONLY) == JC.SMILES_GEN_EXPLICIT_H2_ONLY
             ? JC.SMILES_GEN_EXPLICIT_H2_ONLY
             : (flags
@@ -142,7 +155,7 @@ public class SmilesGenerator {
                     ? JC.SMILES_GEN_EXPLICIT_H_ALL
                     : 0);
 
-    if (explicitH == JC.SMILES_GEN_EXPLICIT_H2_ONLY) {
+    if (explicitHydrogen == JC.SMILES_GEN_EXPLICIT_H2_ONLY) {
       BS bsHa = new BS();
       for (int i = bsSelected.nextSetBit(0); i >= 0; i = bsSelected
           .nextSetBit(i + 1)) {
@@ -160,7 +173,7 @@ public class SmilesGenerator {
                 break;
             }
             if (doIgnore) {
-              bsIgnoreH.set(i);
+              bsIgnoreHydrogen.set(i);
               bsSelected.andNot(bsHa);
             }
           }
@@ -340,9 +353,7 @@ public class SmilesGenerator {
                                     boolean forceBrackets)
       throws InvalidSmilesException {
 
-    if (explicitH == 0 && atom.getAtomicAndIsotopeNumber() == 1
-        && atom.getEdges().length > 0)
-      atom = atoms[atom.getBondedAtomIndex(0)]; // don't start with H
+    atom = checkFirstAtom(atom);
     
     bsSelected = JmolMolecule.getBranchBitSet(atoms, atom.getIndex(),
         BSUtil.copy(bs), null, -1, true, allowBioResidues);
@@ -353,12 +364,15 @@ public class SmilesGenerator {
       if (atoms[i].getCovalentBondCount() > 4 || isPolyhedral)
         iHypervalent = i;
     bsIncludingH = BSUtil.copy(bsSelected);
-    if (explicitH == 0)
+    if (explicitHydrogen == 0)
       for (int j = bsSelected.nextSetBit(0); j >= 0; j = bsSelected
           .nextSetBit(j + 1)) {
         Node a = atoms[j];
         if (a.getAtomicAndIsotopeNumber() == 1 
-            && a.getBondCount() > 0 && a.getBondedAtomIndex(0) != iHypervalent)
+            && a.getBondCount() > 0 
+            && a.getBondedAtomIndex(0) != iHypervalent
+            && !isExplicitOnly(atoms[a.getBondedAtomIndex(0)])
+            )
           bsSelected.clear(j);
       }
     bsAromatic = new BS();
@@ -421,6 +435,34 @@ public class SmilesGenerator {
     return s;
   }
 
+  /**
+   * Don't start with H or central cumulene (in case there is symmetry)
+   * 
+   * @param atom
+   * @return starting node atom
+   */
+  private Node checkFirstAtom(Node atom) {
+    if (explicitHydrogen == 0 && atom.getAtomicAndIsotopeNumber() == 1
+        && atom.getEdges().length > 0)
+      atom = atoms[atom.getBondedAtomIndex(0)]; // don't start with H
+    Node a = atom;
+    Node aprev = null;
+    while (true) {
+      if (a.getCovalentBondCount() != 2)
+        break;
+      Edge[] bonds = a.getEdges();
+      if (bonds[0].getBondType() != Edge.BOND_COVALENT_DOUBLE
+          || bonds[1].getBondType() != Edge.BOND_COVALENT_DOUBLE)
+        break;
+      Node anext = (Node) bonds[0].getOtherNode(a);
+      if (anext == aprev)
+        anext = (Node) bonds[1].getOtherNode(a);
+      aprev = a;
+      a = anext;
+    }
+    return (a == null ? atom : a);
+  }
+
   /**
    * 
    * get aromaticity, ringSets, and aromaticRings fields so that we can
@@ -458,23 +500,25 @@ public class SmilesGenerator {
     int i = bond.index;
     boolean isFirst = (atomFrom == null || bond.getAtomIndex1() == atomFrom
         .getIndex());
-    return (bsBondsUp.get(i) ? (isFirst ? '/' : '\\')
-        : bsBondsDn.get(i) ? (isFirst ? '\\' : '/') : '\0');
+    return (bsBondsUp.get(i) ? 
+        (isFirst ? '/' : '\\')
+        : bsBondsDn.get(i) ? 
+            (isFirst ? '\\' : '/') : '\0');
   }
 
   /**
-   * Creates global BitSets bsBondsUp and bsBondsDown. Noniterative. 
+   * Creates global BitSets bsBondsUp and bsBondsDown. Noniterative.
    *
    */
   private void setBondDirections() {
     BS bsDone = new BS();
     Edge[][] edges = new Edge[2][3];
-    
+
     // We don't assume a bond list, just an atom list, so we
     // loop through all the bonds of all the atoms, flagging them
     // as having been done already so as not to do twice. 
     // The bonds we are marking will be bits in bsBondsUp or bsBondsDn
-    
+
     for (int i = bsSelected.nextSetBit(0); i >= 0; i = bsSelected
         .nextSetBit(i + 1)) {
       Node atom1 = atoms[i];
@@ -484,7 +528,8 @@ public class SmilesGenerator {
         int index = bond.index;
         SimpleNode atom2;
         if (bsDone.get(index) || bond.getCovalentOrder() != 2
-            || SmilesSearch.isRingBond(ringSets, null, i, (atom2 = bond.getOtherNode(atom1)).getIndex()))
+            || SmilesSearch.isRingBond(ringSets, null, i,
+                (atom2 = bond.getOtherNode(atom1)).getIndex()))
           continue;
         bsDone.set(index);
         int nCumulene = 0;
@@ -504,19 +549,22 @@ public class SmilesGenerator {
         int i0 = 0;
         SimpleNode[] atom12 = new SimpleNode[] { atom1, atom2 };
         int edgeCount = 1;
-        
+
         // OK, so we have a double bond. Only looking at single bonds around it.
-        
+
         // First pass: just see if there is an already-assigned bond direction
         // and collect the edges in an array. 
-        
+
         for (int j = 0; j < 2 && edgeCount > 0 && edgeCount < 3; j++) {
           edgeCount = 0;
           SimpleNode atomA = atom12[j];
           Edge[] bb = ((Node) atomA).getEdges();
+          boolean explicitH = explicitHydrogen > 0 || isExplicitOnly(atomA);
           for (int b = 0; b < bb.length; b++) {
             SimpleNode other;
-            if (bb[b].getCovalentOrder() != 1 || explicitH == 0 && (other = bb[b].getOtherNode(atomA)).getElementNumber() == 1
+            if (bb[b].getCovalentOrder() != 1 
+                || !explicitH
+                && (other = bb[b].getOtherNode(atomA)).getElementNumber() == 1
                 && other.getIsotopeNumber() == 0)
               continue;
             edges[j][edgeCount++] = bb[b];
@@ -528,15 +576,15 @@ public class SmilesGenerator {
         }
         if (edgeCount == 3 || edgeCount == 0)
           continue;
-        
+
         // If no bond around this double bond is already marked, we assign it UP.
-        
+
         if (b0 == null) {
           i0 = 0;
           b0 = edges[i0][0];
           bsBondsUp.set(b0.index);
         }
-        
+
         // The character '/' or '\\' is assigned based on a
         // geometric reference to the reference bond. Initially
         // this comes in in reference to the double bond, but
@@ -552,45 +600,68 @@ public class SmilesGenerator {
         // "more than 90 degrees apart" (ab, and cd)
         // Parity errors would be caught here, but I doubt you
         // could ever get that with a real molecule. 
-        
-        char c0 = getBondStereochemistry(b0, atom12[i0]);
-        a0 = b0.getOtherNode(atom12[i0]);
+        SimpleNode aA = atom12[i0];
+        char c0 = getBondStereochemistry(b0, aA);
+        a0 = b0.getOtherNode(aA);
         if (a0 == null)
           continue;
-        for (int j = 0; j < 2; j++)
+        for (int j = 0; j < 2; j++) {
+          SimpleNode aB = atom12[j];
           for (int jj = 0; jj < 2; jj++) {
             Edge b1 = edges[j][jj];
             if (b1 == null || b1 == b0)
               continue;
             int bi = b1.index;
-            SimpleNode a1 = b1.getOtherNode(atom12[j]);
+            SimpleNode a1 = b1.getOtherNode(aB);
             if (a1 == null)
               continue;
-            char c1 = getBondStereochemistry(b1, atom12[j]);
+            char c1 = getBondStereochemistry(b1, aB);
 
             //   c1 is FROM the double bond:
             //    
             //     a0    a1
             //      \   /
-            //    [i0]=[j]       /a /b  \c \d
-            //   
-            boolean isOpposite = SmilesStereo.isDiaxial(atom12[i0], atom12[j],
-                a0, a1, vTemp, 0);
-            if (c1 == '\0' || (c1 != c0) == isOpposite) {
-              boolean isUp = (c0 == '\\' && isOpposite || c0 == '/'
-                  && !isOpposite);
-              if (isUp == (b1.getAtomIndex1() != a1.getIndex()))
-                bsBondsUp.set(bi);
-              else
-                bsBondsDn.set(bi);
+            //      aA=aB       /a /b  \c \d
+            //  
+            // or   a0
+            //       \
+            //       aA=aB   when aA == aB  (j == 0)
+            //       /
+            //      a1
+            boolean isOpposite;
+
+            if (haveSmilesAtoms) {
+              Boolean isop = ((SmilesAtom) a0).isStereoOpposite(a1.getIndex(), aA.getIndex(), aB.getIndex());
+              if (isop == null) {
+                if (Logger.debugging)
+                  Logger.debug("SmilesGenerator could not find stereo for " + a0 + "/" + a1);
+                continue;
+              }
+              isOpposite = isop.booleanValue();
             } else {
-              Logger.error("BOND STEREOCHEMISTRY ERROR");
+              isOpposite = SmilesStereo.isDiaxial(aA, aB, a0, a1,
+                  vTemp, 0);
+            }
+
+            if (c1 == '\0' || (c1 != c0) == isOpposite) {
+              boolean isUp = (c0 == '\\' && isOpposite
+                  || c0 == '/' && !isOpposite);
+              if (isUp == (b1.getAtomIndex1() != a1.getIndex())) {
+                bsBondsUp.set(bi);
+              } else {
+                bsBondsDn.set(bi);
+              }
+            } else {
+              bsBondsUp.clear(bi);
+              bsBondsDn.clear(bi);
+              Logger.error("BOND STEREOCHEMISTRY PARITY ERROR-stereochemistry for bond " + bi + "");
             }
             if (Logger.debugging)
-              Logger.debug(getBondStereochemistry(b0, atom12[0]) + " "
+              Logger.debug(getBondStereochemistry(b0, aA) + " "
                   + a0.getIndex() + " " + a1.getIndex() + " "
-                  + getBondStereochemistry(b1, atom12[j]));
+                  + getBondStereochemistry(b1, aB));
           }
+        }
       }
     }
   }
@@ -605,7 +676,11 @@ public class SmilesGenerator {
       return null;
     ptAtom++;
     bsToDo.clear(atomIndex);
-    boolean includeHs = (atomIndex == iHypervalent || explicitH != 0 && !bsIgnoreH.get(atomIndex));
+    boolean includeHs = (
+        atomIndex == iHypervalent 
+        || explicitHydrogen != 0 && !bsIgnoreHydrogen.get(atomIndex)
+        );
+    boolean explicitHs = isExplicitOnly(atom);
     boolean isExtension = (!bsSelected.get(atomIndex));
     int prevIndex = (prevAtom == null ? -1 : prevAtom.getIndex());
     boolean isAromatic = bsAromatic.get(atomIndex);
@@ -644,8 +719,8 @@ public class SmilesGenerator {
           continue;
         }
         
-        boolean isH = !includeHs
-            && (atom1.getElementNumber() == 1 && atom1.getIsotopeNumber() == 0);
+        boolean isH = !includeHs && !explicitHs
+            && (atom1.getElementNumber() == 1 && atom1.getIsotopeNumber() <= 0);
         if (!bsIncludingH.get(index1)) {
           if (!isH && allowConnectionsToOutsideWorld
               && bsSelected.get(atomIndex))
@@ -708,7 +783,7 @@ public class SmilesGenerator {
         Edge bond = v.get(i);
         SimpleNode a = bond.getOtherNode(atom);
         int n = a.getCovalentBondCount()
-            - (includeHs ? 0 : ((Node) a).getCovalentHydrogenCount());
+            - (includeHs || isExplicitOnly(a) ? 0 : ((Node) a).getCovalentHydrogenCount());
         int order = bond.getCovalentOrder();
         if (n == 1 && (bondNext != null || i < nBonds - 1)) {
           bsBranches.set(bond.index);
@@ -740,9 +815,16 @@ public class SmilesGenerator {
     if (stereoFlag < 7 && nH == 1)
       stereo[stereoFlag++] = aH;
 
-    boolean deferStereo = (orderNext == 1 && sp2Atoms == null);
+    boolean 
+// I guess I thought that we should not put a / before a ring number
+// for a future ring, but that is not true. fixed 2022.03.12 Jmol 14.32.34
+//
+//    deferStereo = (orderNext == 1 && sp2Atoms == null);
+//    if (deferStereo) {
+      deferStereo = false;
+//    }
+    
     char chBond = getBondStereochemistry(bondPrev, prevAtom);
-
     if (strPrev != null || chBond != '\0') {
       if (chBond != '\0')
         strPrev = "" + chBond;
@@ -809,9 +891,8 @@ public class SmilesGenerator {
         continue;
       SimpleNode a = bond.getOtherNode(atom);
       strPrev = getBondOrder(bond, atomIndex, a.getIndex(), isAromatic);
-      if (!deferStereo) {
-        chBond = getBondStereochemistry(bond, atom);
-        if (chBond != '\0')
+      chBond = getBondStereochemistry(bond, atom);
+      if (!deferStereo && chBond != '\0') {
           strPrev = "" + chBond;
       }
       sbRings.append(strPrev);
@@ -878,6 +959,15 @@ public class SmilesGenerator {
       }
       prevStereo = null;
     }
+    
+    
+    if (haveSmilesAtoms && atat == null && (
+    		// TODO [S@] stereoFlag == 3 || 
+    		stereoFlag == 4)) {
+      atat = ((SmilesAtom) atom).getStereoAtAt(stereo);
+    }
+    
+    
     int charge = atom.getFormalCharge();
     int isotope = atom.getIsotopeNumber();
     int valence = atom.getValence();
@@ -945,6 +1035,11 @@ public class SmilesGenerator {
     return atomNext;
   }
 
+  private boolean isExplicitOnly(SimpleNode atom) {
+    // imine hack here
+    return (atom.getElementNumber() == 7 || atom.getElementNumber() == 6 && atom.getIsotopeNumber() == 17);
+  }
+
   private SimpleNode[] atemp;
   /**
    * swap slices of an array [i0 i1) with [i1 i2)
@@ -1119,7 +1214,7 @@ public class SmilesGenerator {
                                   SimpleNode[] stereo, int stereoFlag, boolean isFirst) {
     if (stereoFlag == 10 || stereoFlag < (is2D ? 3 : 4))
       return "";
-    if (explicitH == 0 && atomIndex >= 0 && stereoFlag == 4
+    if (explicitHydrogen == 0 && atomIndex >= 0 && stereoFlag == 4
         && (atom.getElementNumber()) == 6) {
       // do a quick check for two of the same group for tetrahedral carbon only
       String s = "";
@@ -1185,10 +1280,9 @@ public class SmilesGenerator {
                                 BS bsDone) {
     if (bsDone != null)
       bsDone.set(atomIndex);
-    //System.out.println("level=" + level + " atomIndex=" + atomIndex + " atom=" + atom + " s=" + s);
     int n = ((Node)atom).getAtomicAndIsotopeNumber();
     int nx = atom.getCovalentBondCount();
-    int nh = (n == 6 && explicitH != 0 ? ((Node) atom).getCovalentHydrogenCount() : 0);
+    int nh = (n == 6 && explicitHydrogen != 0 ? ((Node) atom).getCovalentHydrogenCount() : 0);
     // only carbon or singly-connected atoms are checked
     // for C we use nh -- CH3, for example.
     // for other atoms, we use number of bonds.
@@ -1290,13 +1384,5 @@ public class SmilesGenerator {
     return Math.min(i0, i1) + "_" + Math.max(i0, i1);
   }
 
-//static {
-//  T3 atom = P3.new3(0,  0.0001f,  -1);
-//  T3 atomPrev = P3.new3(0,  0 , 1);
-//  T3 ref = P3.new3(0,  0 , 0);
-//  T3 a = P3.new3(1,  0,  0);
-// System.out.println(Measure.computeTorsion((T3) atom, (T3)atomPrev, ref, (T3) a, true));  
-//}
-
 }
 
