@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2021-01-26 17:46:08 -0600 (Tue, 26 Jan 2021) $
- * $Revision: 22103 $
+ * $Date: 2022-01-17 18:01:15 -0600 (Mon, 17 Jan 2022) $
+ * $Revision: 22302 $
  *
  * Copyright (C) 2005  The Jmol Development Team
  *
@@ -1170,7 +1170,7 @@ public class SmilesParser {
       if (!isBracketed)
         setBondAtom(bond, null, newAtom, search);
       if (branchLevel == 0
-          && (bond.order == SmilesBond.TYPE_AROMATIC || bond.order == SmilesBond.TYPE_BIO_CROSSLINK))
+          && (bond.getBondType() == SmilesBond.TYPE_AROMATIC || bond.order == SmilesBond.TYPE_BIO_CROSSLINK))
         branchLevel++;
     }
     // if (Logger.debugging)
