@@ -40,6 +40,7 @@ import org.jmol.util.JmolMolecule;
 import org.jmol.util.Logger;
 import org.jmol.util.SimpleUnitCell;
 import org.jmol.util.Tensor;
+import org.jmol.viewer.JC;
 import org.jmol.viewer.Viewer;
 
 import javajs.util.BS;
@@ -48,38 +49,40 @@ import javajs.util.M3;
 import javajs.util.M4;
 import javajs.util.Matrix;
 import javajs.util.P3;
+import javajs.util.PT;
 import javajs.util.Quat;
 import javajs.util.SB;
 import javajs.util.T3;
 import javajs.util.V3;
 
+/* Symmetry is a wrapper class that allows access to the package-local
+ * classes PointGroup, SpaceGroup, SymmetryInfo, and UnitCell.
+ * 
+ * When symmetry is detected in ANY model being loaded, a SymmetryInterface
+ * is established for ALL models.
+ * 
+ * The SpaceGroup information could be saved with each model, but because this 
+ * depends closely on what atoms have been selected, and since tracking that with atom
+ * deletion is a bit complicated, instead we just use local instances of that class.
+ * 
+ * The three PointGroup methods here could be their own interface; they are just here
+ * for convenience.
+ * 
+ * The file readers use SpaceGroup and UnitCell methods
+ * 
+ * The modelSet and modelLoader classes use UnitCell and SymmetryInfo 
+ * 
+ */
 public class Symmetry implements SymmetryInterface {
   // NOTE: THIS CLASS IS VERY IMPORTANT.
   // IN ORDER TO MODULARIZE IT, IT IS REFERENCED USING 
   // xxxx = Interface.getSymmetry();
 
-  /* Symmetry is a wrapper class that allows access to the package-local
-   * classes PointGroup, SpaceGroup, SymmetryInfo, and UnitCell.
-   * 
-   * When symmetry is detected in ANY model being loaded, a SymmetryInterface
-   * is established for ALL models.
-   * 
-   * The SpaceGroup information could be saved with each model, but because this 
-   * depends closely on what atoms have been selected, and since tracking that with atom
-   * deletion is a bit complicated, instead we just use local instances of that class.
-   * 
-   * The three PointGroup methods here could be their own interface; they are just here
-   * for convenience.
-   * 
-   * The file readers use SpaceGroup and UnitCell methods
-   * 
-   * The modelSet and modelLoader classes use UnitCell and SymmetryInfo 
-   * 
-   */
-  private PointGroup pointGroup;
   SpaceGroup spaceGroup;
+  private PointGroup pointGroup;
   private SymmetryInfo symmetryInfo;
   private UnitCell unitCell;
+  private CIPChirality cip;
   private boolean isBio;
 
   @Override
@@ -95,15 +98,16 @@ public class Symmetry implements SymmetryInterface {
   }
 
   @Override
-  public SymmetryInterface setPointGroup(SymmetryInterface siLast,
-                                         T3 center, T3[] atomset,
-                                         BS bsAtoms,
+  public SymmetryInterface setPointGroup(SymmetryInterface siLast, T3 center,
+                                         T3[] atomset, BS bsAtoms,
                                          boolean haveVibration,
                                          float distanceTolerance,
-                                         float linearTolerance, boolean localEnvOnly) {
-    pointGroup = PointGroup.getPointGroup(siLast == null ? null
-        : ((Symmetry) siLast).pointGroup, center, atomset, bsAtoms,
-        haveVibration, distanceTolerance, linearTolerance, localEnvOnly);
+                                         float linearTolerance,
+                                         boolean localEnvOnly) {
+    pointGroup = PointGroup.getPointGroup(
+        siLast == null ? null : ((Symmetry) siLast).pointGroup, center, atomset,
+        bsAtoms, haveVibration, distanceTolerance, linearTolerance,
+        localEnvOnly);
     return this;
   }
 
@@ -113,9 +117,8 @@ public class Symmetry implements SymmetryInterface {
   }
 
   @Override
-  public Object getPointGroupInfo(int modelIndex, String drawID,
-                                  boolean asInfo, String type, int index,
-                                  float scale) {
+  public Object getPointGroupInfo(int modelIndex, String drawID, boolean asInfo,
+                                  String type, int index, float scale) {
     if (drawID == null && !asInfo && pointGroup.textInfo != null)
       return pointGroup.textInfo;
     else if (drawID == null && pointGroup.isDrawType(type, index, scale))
@@ -155,32 +158,31 @@ public class Symmetry implements SymmetryInterface {
     return spaceGroup;
   }
 
-  @Override
-  public void setSpaceGroupFrom(SymmetryInterface symmetry) {
-    spaceGroup = (SpaceGroup) symmetry.getSpaceGroup();
-  }
-
   /**
    * 
    * @param desiredSpaceGroupIndex
    * @param name
-   * @param data a Lst<SymmetryOperation> or Lst<M4> 
-   * @param modDim in [3+d] modulation dimension
+   * @param data
+   *        a Lst<SymmetryOperation> or Lst<M4>
+   * @param modDim
+   *        in [3+d] modulation dimension
    * @return true if a known space group
    */
   @Override
   public boolean createSpaceGroup(int desiredSpaceGroupIndex, String name,
                                   Object data, int modDim) {
-    spaceGroup = SpaceGroup.createSpaceGroup(desiredSpaceGroupIndex, name,
-        data, modDim);
+    spaceGroup = SpaceGroup.createSpaceGroup(desiredSpaceGroupIndex, name, data,
+        modDim);
     if (spaceGroup != null && Logger.debugging)
       Logger.debug("using generated space group " + spaceGroup.dumpInfo());
     return spaceGroup != null;
   }
 
   @Override
-  public Object getSpaceGroupInfoObj(String name, SymmetryInterface cellInfo, boolean isFull) {
-    return SpaceGroup.getInfo(spaceGroup, name, cellInfo, isFull);
+  public Object getSpaceGroupInfoObj(String name, SymmetryInterface cellInfo,
+                                     boolean isFull, boolean addNonstandard) {
+    return SpaceGroup.getInfo(spaceGroup, name, cellInfo, isFull,
+        addNonstandard);
   }
 
   @Override
@@ -189,9 +191,9 @@ public class Symmetry implements SymmetryInterface {
   }
 
   @Override
-  public void setFinalOperations(String name, P3[] atoms, int iAtomFirst,
-                                 int noSymmetryCount, boolean doNormalize,
-                                 String filterSymop) {
+  public void setFinalOperations(int dim, String name, P3[] atoms,
+                                 int iAtomFirst, int noSymmetryCount,
+                                 boolean doNormalize, String filterSymop) {
     if (name != null && (name.startsWith("bio") || name.indexOf(" *(") >= 0)) // filter SYMOP
       spaceGroup.name = name;
     if (filterSymop != null) {
@@ -203,16 +205,16 @@ public class Symmetry implements SymmetryInterface {
       spaceGroup = SpaceGroup.createSpaceGroup(-1,
           name + " *(" + filterSymop.trim() + ")", lst, -1);
     }
-    spaceGroup.setFinalOperations(atoms, iAtomFirst, noSymmetryCount,
-        doNormalize);
+    spaceGroup.setFinalOperationsForAtoms(dim, atoms, iAtomFirst,
+        noSymmetryCount, doNormalize);
   }
 
   @Override
   public M4 getSpaceGroupOperation(int i) {
     return (spaceGroup == null || spaceGroup.operations == null // bio 
         || i >= spaceGroup.operations.length ? null
-        : spaceGroup.finalOperations == null ? spaceGroup.operations[i]
-            : spaceGroup.finalOperations[i]);
+            : spaceGroup.finalOperations == null ? spaceGroup.operations[i]
+                : spaceGroup.finalOperations[i]);
   }
 
   @Override
@@ -226,23 +228,24 @@ public class Symmetry implements SymmetryInterface {
   }
 
   @Override
-  public void newSpaceGroupPoint(int i, P3 atom1, P3 atom2, int transX,
-                                 int transY, int transZ, M4 o) {
+  public void newSpaceGroupPoint(P3 pt, int i, M4 o, int transX,
+                                 int transY, int transZ, P3 retPoint) {
     if (o == null && spaceGroup.finalOperations == null) {
-      SymmetryOperation op = spaceGroup.operations[i]; 
+      SymmetryOperation op = spaceGroup.operations[i];
       // temporary spacegroups don't have to have finalOperations
       if (!op.isFinalized)
         op.doFinalize();
-      SymmetryOperation.newPoint(op, atom1, atom2, transX, transY, transZ);
-      return;
+      o = op;
     }
-    SymmetryOperation.newPoint((o == null ? spaceGroup.finalOperations[i] : o), atom1, atom2, transX, transY, transZ);
+    newPoint((o == null ? spaceGroup.finalOperations[i] : o), pt, transX,
+        transY, transZ, retPoint);
   }
 
   @Override
   public V3[] rotateAxes(int iop, V3[] axes, P3 ptTemp, M3 mTemp) {
-    return (iop == 0 ? axes : spaceGroup.finalOperations[iop].rotateAxes(axes,
-        unitCell, ptTemp, mTemp));
+    return (iop == 0 ? axes
+        : spaceGroup.finalOperations[iop].rotateAxes(axes, unitCell, ptTemp,
+            mTemp));
   }
 
   @Override
@@ -270,6 +273,11 @@ public class Symmetry implements SymmetryInterface {
     return spaceGroup.latticeOp;
   }
 
+  @Override
+  public Lst<P3> getLatticeCentering() {
+    return SymmetryOperation.getLatticeCentering(getSymmetryOperations());
+  }
+
   @Override
   public Matrix getOperationRsVs(int iop) {
     return (spaceGroup.finalOperations == null ? spaceGroup.operations
@@ -283,11 +291,14 @@ public class Symmetry implements SymmetryInterface {
 
   @Override
   /**
-   * @param rot is a full (3+d)x(3+d) array of epsilons
-   * @param trans is a (3+d)x(1) array of translations
+   * @param rot
+   *        is a full (3+d)x(3+d) array of epsilons
+   * @param trans
+   *        is a (3+d)x(1) array of translations
    * @return Jones-Faithful representation
    */
-  public String addOp(String code, Matrix rs, Matrix vs, Matrix sigma) {
+  public String addSubSystemOp(String code, Matrix rs, Matrix vs,
+                               Matrix sigma) {
     spaceGroup.isSSG = true;
     String s = SymmetryOperation.getXYZFromRsVs(rs, vs, false);
     int i = spaceGroup.addSymmetry(s, -1, true);
@@ -308,45 +319,47 @@ public class Symmetry implements SymmetryInterface {
   // in lieu of saving the actual unit cell read in the reader. Not perfect.
   // The idea was to be able to create the unit cell from "scratch" independent
   // of the reader. 
-  
+
   @Override
   public String getSpaceGroupName() {
     return (symmetryInfo != null ? symmetryInfo.sgName
-        : spaceGroup != null ? spaceGroup.getName() : unitCell != null
-            && unitCell.name.length() > 0 ? "cell=" + unitCell.name : "");
+        : spaceGroup != null ? spaceGroup.getName()
+            : unitCell != null && unitCell.name.length() > 0
+                ? "cell=" + unitCell.name
+                : "");
+  }
+
+  @Override
+  public String getSpaceGroupNameType(String type) {
+    return (spaceGroup == null ? null : spaceGroup.getNameType(type, this));
   }
 
   @Override
   public void setSpaceGroupName(String name) {
-    if (spaceGroup != null) 
+    if (spaceGroup != null)
       spaceGroup.setName(name);
   }
 
   @Override
   public int getSpaceGroupOperationCount() {
-    return (symmetryInfo != null ? symmetryInfo.symmetryOperations.length
-        : spaceGroup != null && spaceGroup.finalOperations != null ? spaceGroup.finalOperations.length
+    return (symmetryInfo != null 
+        && symmetryInfo.symmetryOperations != null ? // null here for PDB 
+            symmetryInfo.symmetryOperations.length
+        : spaceGroup != null && spaceGroup.finalOperations != null
+            ? spaceGroup.finalOperations.length
             : 0);
   }
 
   @Override
-  public String getLatticeType() {
-    return (symmetryInfo != null ? symmetryInfo.latticeType 
-        : spaceGroup == null ? "P" 
-            : spaceGroup.latticeType);
-  }
-
-  @Override
-  public void setLatticeType(String type) {
-    if (spaceGroup != null)
-      spaceGroup.latticeType = type;
+  public char getLatticeType() {
+    return (symmetryInfo != null ? symmetryInfo.latticeType
+        : spaceGroup == null ? 'P' : spaceGroup.latticeType);
   }
 
   @Override
   public String getIntTableNumber() {
-    return (symmetryInfo != null ? symmetryInfo.intlTableNo 
-        : spaceGroup == null ? null 
-            : spaceGroup.intlTableNumber);
+    return (symmetryInfo != null ? symmetryInfo.intlTableNo
+        : spaceGroup == null ? null : spaceGroup.intlTableNumber);
   }
 
   @Override
@@ -361,21 +374,29 @@ public class Symmetry implements SymmetryInterface {
 
   @Override
   public String getSymmetryInfoStr() {
-    return (symmetryInfo == null ? "" : symmetryInfo.infoStr);
+    if (symmetryInfo != null)
+      return symmetryInfo.infoStr;
+    if (spaceGroup == null)
+      return "";
+    symmetryInfo = new SymmetryInfo();
+    symmetryInfo.setSymmetryInfo(null, getUnitCellParams(), spaceGroup);
+    return symmetryInfo.infoStr;
   }
 
   @Override
-  public M4[] getSymmetryOperations() {
+  public SymmetryOperation[] getSymmetryOperations() {
     if (symmetryInfo != null)
       return symmetryInfo.symmetryOperations;
     if (spaceGroup == null)
       spaceGroup = SpaceGroup.getNull(true, false, true);
+    spaceGroup.setFinalOperations();
     return spaceGroup.finalOperations;
   }
 
   @Override
   public boolean isSimple() {
-    return (symmetryInfo == null || symmetryInfo.symmetryOperations == null);
+    return (spaceGroup == null
+        && (symmetryInfo == null || symmetryInfo.symmetryOperations == null));
   }
 
   /**
@@ -390,7 +411,7 @@ public class Symmetry implements SymmetryInterface {
                                            float[] unitCellParams) {
     symmetryInfo = new SymmetryInfo();
     float[] params = symmetryInfo.setSymmetryInfo(modelAuxiliaryInfo,
-        unitCellParams);
+        unitCellParams, null);
     if (params != null) {
       setUnitCell(params, modelAuxiliaryInfo.containsKey("jmolData"));
       unitCell.moreInfo = (Lst<String>) modelAuxiliaryInfo
@@ -403,7 +424,7 @@ public class Symmetry implements SymmetryInterface {
         initializeOrientation(matUnitCellOrientation);
       if (Logger.debugging)
         Logger.debug("symmetryInfos[" + modelIndex + "]:\n"
-            + unitCell.dumpInfo(true));
+            + unitCell.dumpInfo(true, true));
     }
     return this;
   }
@@ -416,18 +437,10 @@ public class Symmetry implements SymmetryInterface {
   }
 
   @Override
-  public boolean checkUnitCell(SymmetryInterface uc, P3 cell, P3 ptTemp,
-                               boolean isAbsolute) {
-    uc.toFractional(ptTemp, isAbsolute);
-    // {1 1 1} here is the original cell
-    return (ptTemp.x >= cell.x - 1f - SimpleUnitCell.SLOP && ptTemp.x <= cell.x + SimpleUnitCell.SLOP
-        && ptTemp.y >= cell.y - 1f - SimpleUnitCell.SLOP && ptTemp.y <= cell.y + SimpleUnitCell.SLOP
-        && ptTemp.z >= cell.z - 1f - SimpleUnitCell.SLOP && ptTemp.z <= cell.z + SimpleUnitCell.SLOP);
-  }
-
-  @Override
-  public void setUnitCell(float[] unitCellParams, boolean setRelative) {
+  public SymmetryInterface setUnitCell(float[] unitCellParams,
+                                       boolean setRelative) {
     unitCell = UnitCell.fromParams(unitCellParams, setRelative);
+    return this;
   }
 
   @Override
@@ -437,7 +450,9 @@ public class Symmetry implements SymmetryInterface {
 
   @Override
   public String getUnitCellState() {
-    return (unitCell == null ? "" : unitCell.getState());
+    if (unitCell == null)
+      return "";
+    return unitCell.getState();
   }
 
   @Override
@@ -447,7 +462,7 @@ public class Symmetry implements SymmetryInterface {
 
   public String getUnitsymmetryInfo() {
     // not used in Jmol?
-    return unitCell.dumpInfo(false);
+    return unitCell.dumpInfo(false, true);
   }
 
   @Override
@@ -465,6 +480,11 @@ public class Symmetry implements SymmetryInterface {
     unitCell.toUnitCell(pt, offset);
   }
 
+  @Override
+  public void toUnitCellRnd(T3 pt, T3 offset) {
+    unitCell.toUnitCellRnd(pt, offset);
+  }
+
   @Override
   public P3 toSupercell(P3 fpt) {
     return unitCell.toSupercell(fpt);
@@ -475,7 +495,7 @@ public class Symmetry implements SymmetryInterface {
     if (!isBio)
       unitCell.toFractional(pt, ignoreOffset);
   }
-  
+
   @Override
   public void toFractionalM(M4 m) {
     if (!isBio)
@@ -539,6 +559,16 @@ public class Symmetry implements SymmetryInterface {
     return unitCell.getUnitCellMultiplier();
   }
 
+  @Override
+  public SymmetryInterface getUnitCellMultiplied() {
+    UnitCell uc = unitCell.getUnitCellMultiplied();
+    if (uc == unitCell)
+      return this;
+    Symmetry s = new Symmetry();
+    s.unitCell = uc;
+    return s;
+  }
+
   @Override
   public P3[] getCanonicalCopy(float scale, boolean withOffset) {
     return unitCell.getCanonicalCopy(scale, withOffset);
@@ -550,8 +580,8 @@ public class Symmetry implements SymmetryInterface {
   }
 
   @Override
-  public String getUnitCellInfo() {
-    return unitCell.dumpInfo(false);
+  public String getUnitCellInfo(boolean scaled) {
+    return unitCell.dumpInfo(false, scaled);
   }
 
   @Override
@@ -566,7 +596,8 @@ public class Symmetry implements SymmetryInterface {
 
   @Override
   public boolean checkDistance(P3 f1, P3 f2, float distance, float dx,
-                               int iRange, int jRange, int kRange, P3 ptOffset) {
+                               int iRange, int jRange, int kRange,
+                               P3 ptOffset) {
     return unitCell.checkDistance(f1, f2, distance, dx, iRange, jRange, kRange,
         ptOffset);
   }
@@ -577,8 +608,10 @@ public class Symmetry implements SymmetryInterface {
   }
 
   /**
-   * @param oabc  [ptorigin, va, vb, vc]
-   * @param setRelative a flag only set true for IsosurfaceMesh
+   * @param oabc
+   *        [ptorigin, va, vb, vc]
+   * @param setRelative
+   *        a flag only set true for IsosurfaceMesh
    * @param name
    * @return this SymmetryInterface
    */
@@ -606,7 +639,8 @@ public class Symmetry implements SymmetryInterface {
       JmolMolecule[] molecules = modelSet.getMolecules();
       int moleculeCount = molecules.length;
       Atom[] atoms = modelSet.at;
-      boolean isOneMolecule = (molecules[moleculeCount - 1].firstAtomIndex == modelSet.am[atoms[iAtom0].mi].firstAtomIndex);
+      boolean isOneMolecule = (molecules[moleculeCount
+          - 1].firstAtomIndex == modelSet.am[atoms[iAtom0].mi].firstAtomIndex);
       P3 center = new P3();
       boolean centroidPacked = (minmax[6] == 1);
       nextMol: for (int i = moleculeCount; --i >= 0
@@ -648,56 +682,70 @@ public class Symmetry implements SymmetryInterface {
           || center.x - 0.000005f > minmax[3]
           || center.y + 0.000005f <= minmax[1]
           || center.y - 0.000005f > minmax[4]
-          || center.z + 0.000005f <= minmax[2] || center.z - 0.000005f > minmax[5]);
+          || center.z + 0.000005f <= minmax[2]
+          || center.z - 0.000005f > minmax[5]);
 
-    return (center.x + 0.000005f <= minmax[0]
-        || center.x + 0.00005f > minmax[3] || center.y + 0.000005f <= minmax[1]
-        || center.y + 0.00005f > minmax[4] || center.z + 0.000005f <= minmax[2] || center.z + 0.00005f > minmax[5]);
+    return (center.x + 0.000005f <= minmax[0] || center.x + 0.00005f > minmax[3]
+        || center.y + 0.000005f <= minmax[1] || center.y + 0.00005f > minmax[4]
+        || center.z + 0.000005f <= minmax[2]
+        || center.z + 0.00005f > minmax[5]);
   }
 
   // info
 
   private SymmetryDesc desc;
+  private static SymmetryDesc nullDesc;
 
   private SymmetryDesc getDesc(ModelSet modelSet) {
-    return (desc == null ? (desc = ((SymmetryDesc) Interface.getInterface(
-        "org.jmol.symmetry.SymmetryDesc", modelSet.vwr, "eval"))) : desc).set(modelSet);
+    if (modelSet == null) {
+      return (nullDesc == null
+          ? (nullDesc = ((SymmetryDesc) Interface.getInterface(
+              "org.jmol.symmetry.SymmetryDesc", null, "modelkit")))
+          : nullDesc);
+    }
+    return (desc == null
+        ? (desc = ((SymmetryDesc) Interface.getInterface(
+            "org.jmol.symmetry.SymmetryDesc", modelSet.vwr, "eval")))
+        : desc).set(modelSet);
   }
 
   @Override
   public Object getSymmetryInfoAtom(ModelSet modelSet, int iatom, String xyz,
-                                    int op, P3 translation, P3 pt, P3 pt2, String id, int type, float scaleFactor, int nth, int options) {
-    return getDesc(modelSet).getSymopInfo(iatom, xyz, op, translation, pt,
-        pt2, id, type, scaleFactor, nth, options);
+                                    int op, P3 translation, P3 pt, P3 pt2,
+                                    String id, int type, float scaleFactor,
+                                    int nth, int options) {
+    return getDesc(modelSet).getSymopInfo(iatom, xyz, op, translation, pt, pt2,
+        id, type, scaleFactor, nth, options);
   }
 
   @Override
-  public Map<String, Object> getSpaceGroupInfo(ModelSet modelSet, String sgName, int modelIndex, boolean isFull, float[] cellParams) {
+  public Map<String, Object> getSpaceGroupInfo(ModelSet modelSet, String sgName,
+                                               int modelIndex, boolean isFull,
+                                               float[] cellParams) {
     boolean isForModel = (sgName == null);
     if (sgName == null) {
-      Map<String, Object> info = modelSet.getModelAuxiliaryInfo(modelSet.vwr.am.cmi);
+      Map<String, Object> info = modelSet
+          .getModelAuxiliaryInfo(modelSet.vwr.am.cmi);
       if (info != null)
         sgName = (String) info.get("spaceGroup");
     }
     SymmetryInterface cellInfo = null;
     if (cellParams != null) {
-      cellInfo = new Symmetry();
-      cellInfo.setUnitCell(cellParams, false);
+      cellInfo = new Symmetry().setUnitCell(cellParams, false);
     }
-    return getDesc(modelSet).getSpaceGroupInfo(this, modelIndex, sgName, 0, null, null,
-        null, 0, -1, isFull, isForModel, 0, cellInfo);
+    return getDesc(modelSet).getSpaceGroupInfo(this, modelIndex, sgName, 0,
+        null, null, null, 0, -1, isFull, isForModel, 0, cellInfo, null);
   }
 
-  
   @Override
   public String fcoord(T3 p) {
     return SymmetryOperation.fcoord(p);
   }
 
   @Override
-  public T3[] getV0abc(Object def) {
-    return (unitCell == null ? null : unitCell.getV0abc(def));
-  } 
+  public T3[] getV0abc(Object def, M4 retMatrix) {
+    return (unitCell == null ? null : unitCell.getV0abc(def, retMatrix));
+  }
 
   @Override
   public Quat getQuaternionRotation(String abc) {
@@ -710,7 +758,7 @@ public class Symmetry implements SymmetryInterface {
   }
 
   @Override
-  public boolean getState(SB commands) {
+  public boolean getState(ModelSet ms, int modelIndex, SB commands) {
     T3 pt = getFractionalOffset();
     boolean loadUC = false;
     if (pt != null && (pt.x != 0 || pt.y != 0 || pt.z != 0)) {
@@ -719,32 +767,42 @@ public class Symmetry implements SymmetryInterface {
     }
     pt = getUnitCellMultiplier();
     if (pt != null) {
-      commands.append("; set unitcell ").append(SimpleUnitCell.escapeMultiplier(pt));
+      commands.append("; set unitcell ")
+          .append(SimpleUnitCell.escapeMultiplier(pt));
+      loadUC = true;
+    }
+    String sg0 = (String) ms.getInfo(modelIndex, "spaceGroupOriginal");
+    String sg = (String) ms.getInfo(modelIndex, "spaceGroup");
+    if (sg0 != null && sg != null && !sg.equals(sg0)) {
+      commands.append("\nMODELKIT SPACEGROUP " + PT.esc(sg));
       loadUC = true;
     }
     return loadUC;
   }
 
   @Override
-  public AtomIndexIterator getIterator(Viewer vwr, Atom atom, Atom[] atoms,
-                                       BS bsAtoms, float radius) {
-    return ((UnitCellIterator) Interface.getInterface("org.jmol.symmetry.UnitCellIterator", vwr, "script"))
-        .set(this, atom, atoms, bsAtoms, radius);
+  public AtomIndexIterator getIterator(Viewer vwr, Atom atom, BS bsAtoms,
+                                       float radius) {
+    return ((UnitCellIterator) Interface
+        .getInterface("org.jmol.symmetry.UnitCellIterator", vwr, "script"))
+            .set(this, atom, vwr.ms.at, bsAtoms, radius);
   }
 
   @Override
-  public boolean toFromPrimitive(boolean toPrimitive, char type, T3[] oabc, M3 primitiveToCrystal) {
+  public boolean toFromPrimitive(boolean toPrimitive, char type, T3[] oabc,
+                                 M3 primitiveToCrystal) {
     if (unitCell == null)
       unitCell = UnitCell.fromOABC(oabc, false);
-    return unitCell.toFromPrimitive(toPrimitive, type, oabc, primitiveToCrystal);
+    return unitCell.toFromPrimitive(toPrimitive, type, oabc,
+        primitiveToCrystal);
   }
 
   @Override
   public Lst<P3> generateCrystalClass(P3 pt0) {
-     M4[] ops = getSymmetryOperations();
+    M4[] ops = getSymmetryOperations();
     Lst<P3> lst = new Lst<P3>();
     boolean isRandom = (pt0 == null);
-    float rand1=0,rand2=0,rand3=0;
+    float rand1 = 0, rand2 = 0, rand3 = 0;
     if (isRandom) {
       rand1 = (float) Math.E;
       rand2 = (float) Math.PI;
@@ -801,30 +859,36 @@ public class Symmetry implements SymmetryInterface {
   public void calculateCIPChiralityForAtoms(Viewer vwr, BS bsAtoms) {
     vwr.setCursor(GenericPlatform.CURSOR_WAIT);
     CIPChirality cip = getCIPChirality(vwr);
-    String dataClass = (vwr.getBoolean(T.testflag1) ? "CIPData" : "CIPDataTracker");
-    CIPData data = ((CIPData) Interface.getInterface("org.jmol.symmetry." + dataClass, vwr, "script")).set(vwr, bsAtoms);
+    String dataClass = (vwr.getBoolean(T.testflag1) ? "CIPData"
+        : "CIPDataTracker");
+    CIPData data = ((CIPData) Interface
+        .getInterface("org.jmol.symmetry." + dataClass, vwr, "script")).set(vwr,
+            bsAtoms);
     data.setRule6Full(vwr.getBoolean(T.ciprule6full));
     cip.getChiralityForAtoms(data);
     vwr.setCursor(GenericPlatform.CURSOR_DEFAULT);
   }
-  
+
   @Override
-  public String[] calculateCIPChiralityForSmiles(Viewer vwr, String smiles) throws Exception {
+  public String[] calculateCIPChiralityForSmiles(Viewer vwr, String smiles)
+      throws Exception {
     vwr.setCursor(GenericPlatform.CURSOR_WAIT);
     CIPChirality cip = getCIPChirality(vwr);
-    CIPDataSmiles data = ((CIPDataSmiles) Interface.getInterface("org.jmol.symmetry.CIPDataSmiles", vwr, "script")).setAtomsForSmiles(vwr, smiles);
+    CIPDataSmiles data = ((CIPDataSmiles) Interface
+        .getInterface("org.jmol.symmetry.CIPDataSmiles", vwr, "script"))
+            .setAtomsForSmiles(vwr, smiles);
     cip.getChiralityForAtoms(data);
     vwr.setCursor(GenericPlatform.CURSOR_DEFAULT);
-       return data.getSmilesChiralityArray();
+    return data.getSmilesChiralityArray();
   }
-  
-  CIPChirality cip;
-  
+
   private CIPChirality getCIPChirality(Viewer vwr) {
-    return (cip == null ? (cip = ((CIPChirality) Interface.getInterface("org.jmol.symmetry.CIPChirality", vwr, "script"))) : cip);
+    return (cip == null
+        ? (cip = ((CIPChirality) Interface
+            .getInterface("org.jmol.symmetry.CIPChirality", vwr, "script")))
+        : cip);
   }
 
-  
   /**
    * return a conventional lattice from a primitive
    * 
@@ -845,8 +909,166 @@ public class Symmetry implements SymmetryInterface {
   }
 
   @Override
-  public void setUnitCell(Symmetry uc) {
-    unitCell = UnitCell.cloneUnitCell(uc.unitCell);   
+  public void setUnitCell(SymmetryInterface uc) {
+    unitCell = UnitCell.cloneUnitCell(((Symmetry) uc).unitCell);
+  }
+
+  @Override
+  public Object findSpaceGroup(Viewer vwr, BS atoms, String opXYZ,
+                               boolean asString) {
+    return ((SpaceGroupFinder) Interface
+        .getInterface("org.jmol.symmetry.SpaceGroupFinder", vwr, "eval"))
+            .findSpaceGroup(vwr, atoms, opXYZ, this, asString);
+  }
+
+  @Override
+  public void setSpaceGroupTo(Object sg) {
+    symmetryInfo = null;
+    if (sg instanceof SpaceGroup) {
+      spaceGroup = (SpaceGroup) sg;
+    } else {
+      spaceGroup = SpaceGroup.getSpaceGroupFromITAName(sg.toString());
+    }
+  }
+
+  @Override
+  public BS removeDuplicates(ModelSet ms, BS bs) {
+    UnitCell uc = this.unitCell;
+    Atom[] atoms = ms.at;
+    float[] occs = ms.occupancies;
+    boolean haveOccupancies = (occs != null);
+    P3 pt = new P3();
+    P3 pt2 = new P3();
+    for (int i = bs.nextSetBit(0); i >= 0; i = bs.nextSetBit(i + 1)) {
+      Atom a = atoms[i];
+      pt.setT(a);
+      uc.toFractional(pt, false);
+      uc.unitizeRnd(pt);
+      int type = a.getAtomicAndIsotopeNumber();
+
+      float occ = (haveOccupancies ? occs[i] : 0);
+      for (int j = bs.nextSetBit(i + 1); j >= 0; j = bs.nextSetBit(j + 1)) {
+        Atom b = atoms[j];
+        if (type != b.getAtomicAndIsotopeNumber()
+            || (haveOccupancies && occ != occs[j]))
+          continue;
+        pt2.setT(b);
+        uc.toFractional(pt2, false);
+        uc.unitizeRnd(pt2);
+        if (pt.distanceSquared(pt2) < JC.UC_TOLERANCE2) {
+          bs.clear(j);
+        }
+      }
+    }
+    return bs;
+  }
+
+  @Override
+  public Lst<P3> getEquivPoints(Lst<P3> pts, P3 pt, String flags) {
+    M4[] ops = getSymmetryOperations();
+    return (ops == null || unitCell == null ? null
+        : unitCell.getEquivPoints(pt, flags, ops,
+            pts == null ? new Lst<P3>() : pts, 0, 0));
+  }
+
+  @Override
+  public void getEquivPointList(Lst<P3> pts, int nIgnored, String flags) {
+    M4[] ops = getSymmetryOperations();
+    boolean newPt = (flags.indexOf("newpt") >= 0);
+    boolean zapped = (flags.indexOf("zapped") >= 0);
+    // we will preserve the points temporarily, then remove them at the end
+    int n = pts.size();
+    boolean tofractional = (flags.indexOf("tofractional") >= 0);
+    // fractionalize all points if necessary
+    if (flags.indexOf("fromfractional") < 0) {
+      for (int i = 0; i < pts.size(); i++) {
+        toFractional(pts.get(i), true);
+      }
+    }
+    // signal to make no changes in points
+    flags += ",fromfractional,tofractional";
+    int check0 = (nIgnored > 0 ? 0 : n);
+    boolean allPoints = (nIgnored == n);
+    int n0 = (nIgnored > 0 ? nIgnored : n);
+    if (allPoints) {
+      nIgnored--;
+      n0--;
+    }
+    if (zapped)
+      n0 = 0;
+    P3 p0 = (nIgnored > 0 ? pts.get(nIgnored) : null);
+    if (ops != null || unitCell != null) {
+      for (int i = nIgnored; i < n; i++) {
+        unitCell.getEquivPoints(pts.get(i), flags, ops, pts, check0, n0);
+      }
+    }
+    // now remove the starting points, checking to see if perhaps our
+    // test point itself has been removed.
+    if (!zapped && (pts.size() == nIgnored || pts.get(nIgnored) != p0
+        || allPoints || newPt))
+      n--;
+    for (int i = n - nIgnored; --i >= 0;)
+      pts.removeItemAt(nIgnored);
+    // final check for removing duplicates
+    //    if (nIgnored > 0)
+    //      UnitCell.checkDuplicate(pts, 0, nIgnored - 1, nIgnored);
+
+    // and turn these to Cartesians if desired
+    if (!tofractional) {
+      for (int i = pts.size(); --i >= nIgnored;)
+        toCartesian(pts.get(i), true);
+    }
+  }
+
+  @Override
+  public int[] getInvariantSymops(P3 pt, int[] v0) {
+    M4[] ops = getSymmetryOperations();
+    if (ops == null)
+      return new int[0];
+    BS bs = new BS();
+    P3 p = new P3();
+    P3 p0 = new P3();
+    int nops = ops.length;
+    for (int i = 1; i < nops; i++) {
+      p.setT(pt);
+      toFractional(p, true);
+      // unitize here should take care of all Wyckoff positions
+      unitCell.unitize(p);
+      p0.setT(p);
+      ops[i].rotTrans(p);
+      unitCell.unitize(p);
+      if (p0.distanceSquared(p) < JC.UC_TOLERANCE2) {
+        bs.set(i);
+      }
+    }
+    int[] ret = new int[bs.cardinality()];
+    if (v0 != null && ret.length != v0.length)
+      return null;
+    for (int k = 0, i = 1; i < nops; i++) {
+      boolean isOK = bs.get(i);
+      if (isOK) {
+        if (v0 != null && v0[k] != i + 1)
+          return null;
+        ret[k++] = i + 1;
+      }
+    }
+    return ret;
+  }
+
+  /**
+   * @param fracA
+   * @param fracB
+   * @return matrix
+   */
+  @Override
+  public M4 getTransform(P3 fracA, P3 fracB, boolean best) {
+    return getDesc(null).getTransform(unitCell, getSymmetryOperations(), fracA,
+        fracB, best);
+  }
+
+  static void newPoint(M4 m, P3 atom1, int x, int y, int z, P3 atom2) {
+    m.rotTrans2(atom1, atom2);
+    atom2.add3(x, y, z);
   }
 
 }
