@@ -30,6 +30,7 @@ import java.util.Map;
 import org.jmol.util.Logger;
 import org.jmol.util.Parser;
 
+import javajs.util.Lst;
 import javajs.util.M3;
 import javajs.util.M4;
 import javajs.util.Matrix;
@@ -67,8 +68,6 @@ public class SymmetryOperation extends M4 {
   private int opId;
   private V3 centering;
 
-  private static P3 atomTest;
-
   private String[] myLabels;
   int modDim;
 
@@ -94,7 +93,7 @@ public class SymmetryOperation extends M4 {
   Matrix rsvs;
 
   boolean isBio;
-  private Matrix sigma;
+  Matrix sigma;
   int number;
   String subsystemCode;
   int timeReversal;
@@ -111,17 +110,14 @@ public class SymmetryOperation extends M4 {
 
   /**
    * 
-   * @param op
-   * @param atoms
-   * @param atomIndex
-   * @param countOrId
-   * @param doNormalize
+   * @param op operation to clone or null
+   * @param id opId for this operation; ignored if cloning
+   * @param doNormalize 
    */
-  SymmetryOperation(SymmetryOperation op, P3[] atoms, int atomIndex,
-      int countOrId, boolean doNormalize) {
+  SymmetryOperation(SymmetryOperation op, int id, boolean doNormalize) {
     this.doNormalize = doNormalize;
     if (op == null) {
-      opId = countOrId;
+      opId = id;
       return;
     }
     /*
@@ -142,8 +138,6 @@ public class SymmetryOperation extends M4 {
     setMatrix(false);
     if (!op.isFinalized)
       doFinalize();
-    if (doNormalize && sigma == null)
-      setOffset(this, atoms, atomIndex, countOrId);
   }
 
   private void setGamma(boolean isReverse) {
@@ -232,11 +226,6 @@ public class SymmetryOperation extends M4 {
     return getXYZFromMatrix(m, false, false, false);
   }
 
-  static void newPoint(M4 m, P3 atom1, P3 atom2, int x, int y, int z) {
-    m.rotTrans2(atom1, atom2);
-    atom2.add3(x, y, z);
-  }
-
   String dumpInfo() {
     return "\n" + xyz + "\ninternal matrix representation:\n" + toString();
   }
@@ -301,7 +290,7 @@ public class SymmetryOperation extends M4 {
       setMatrix(isReverse);
       isFinalized = true;
       isBio = (xyz.indexOf("bio") >= 0);
-      this.xyz = (isBio ? toString()
+      this.xyz = (isBio ? (this.xyzOriginal = super.toString())
           : getXYZFromMatrix(this, false, false, false));
       return true;
     }
@@ -453,6 +442,7 @@ public class SymmetryOperation extends M4 {
     boolean isDenominator = false;
     boolean isDecimal = false;
     boolean isNegative = false;
+    xyz = PT.rep(xyz,  "[bio[", "");
     int modDim = (op == null ? 0 : op.modDim);
     int nRows = 4 + modDim;
     int divisor = (op == null ? setDivisor(xyz) : op.divisor);
@@ -844,8 +834,9 @@ public class SymmetryOperation extends M4 {
       mat.getRow(i, row);
       String term = "";
       for (int j = 0; j < 3; j++)
-        if (row[j] != 0)
+        if (approxF(row[j]) != 0)
           term += plusMinus(term, row[j], labelsXYZ[j + lpt]);
+      if (approxF(row[3]) != 0)
       term += xyzFraction12((is12ths ? row[3] : row[3] * denom), denom, allPositive,
           halfOrLess);
       str += "," + term;
@@ -853,40 +844,7 @@ public class SymmetryOperation extends M4 {
     return str.substring(1);
   }
 
-  public static void setOffset(M4 m, P3[] atoms, int atomIndex, int count) {
-    if (count == 0)
-      return;
-    /*
-     * the center of mass of the full set of atoms is moved into the cell with this
-     *  
-     */
-    float x = 0;
-    float y = 0;
-    float z = 0;
-    if (atomTest == null)
-      atomTest = new P3();
-    for (int i = atomIndex, i2 = i + count; i < i2; i++) {
-      newPoint(m, atoms[i], atomTest, 0, 0, 0);
-      x += atomTest.x;
-      y += atomTest.y;
-      z += atomTest.z;
-    }
-    x /= count;
-    y /= count;
-    z /= count;
-    while (x < -0.001 || x >= 1.001) {
-      m.m03 += (x < 0 ? 1 : -1);
-      x += (x < 0 ? 1 : -1);
-    }
-    while (y < -0.001 || y >= 1.001) {
-      m.m13 += (y < 0 ? 1 : -1);
-      y += (y < 0 ? 1 : -1);
-    }
-    while (z < -0.001 || z >= 1.001) {
-      m.m23 += (z < 0 ? 1 : -1);
-      z += (z < 0 ? 1 : -1);
-    }
-  }
+  
 
   //  // action of this method depends upon setting of unitcell
   //  private void transformCartesian(UnitCell unitcell, P3 pt) {
@@ -1065,6 +1023,7 @@ public class SymmetryOperation extends M4 {
   }
 
   private Hashtable<String, Object> info;
+  static P3 atomTest;
 
   public Map<String, Object> getInfo() {
     if (info == null) {
@@ -1088,4 +1047,59 @@ public class SymmetryOperation extends M4 {
     return info;
   }
 
+  /**
+   * Adjust the translation for this operator so that it moves the center of
+   * mass of the full set of atoms into the cell.
+   * 
+   * @param dim
+   * @param m
+   * @param atoms
+   * @param atomIndex first index
+   * @param count number of atoms
+   */
+  public static void normalizeOperationToCentroid(int dim, M4 m, P3[] atoms, int atomIndex, int count) {
+    if (count <= 0)
+      return;
+    float x = 0;
+    float y = 0;
+    float z = 0;
+    if (SymmetryOperation.atomTest == null)
+      SymmetryOperation.atomTest = new P3();
+    for (int i = atomIndex, i2 = i + count; i < i2; i++) {
+      Symmetry.newPoint(m, atoms[i], 0, 0, 0, SymmetryOperation.atomTest);
+      x += SymmetryOperation.atomTest.x;
+      y += SymmetryOperation.atomTest.y;
+      z += SymmetryOperation.atomTest.z;
+    }
+    x /= count;
+    y /= count;
+    z /= count;
+    while (x < -0.001 || x >= 1.001) {
+      m.m03 += (x < 0 ? 1 : -1);
+      x += (x < 0 ? 1 : -1);
+    }
+    if (dim > 1)
+    while (y < -0.001 || y >= 1.001) {
+      m.m13 += (y < 0 ? 1 : -1);
+      y += (y < 0 ? 1 : -1);
+    }
+    if (dim > 2)
+    while (z < -0.001 || z >= 1.001) {
+      m.m23 += (z < 0 ? 1 : -1);
+      z += (z < 0 ? 1 : -1);
+    }
+  }
+
+  public static Lst<P3> getLatticeCentering(SymmetryOperation[] ops) {
+    // TODO -- check 'R' types
+    Lst<P3> list = new Lst<P3>();
+    for (int i = 0; i < ops.length; i++) {
+      T3 c = (ops[i]  == null ? null : ops[i].getCentering());
+      if (c != null)
+        list.addLast(P3.newP(c));
+    }
+    return list;
+  }
+
+
 }
