@@ -28,6 +28,14 @@ package org.jmol.symmetry;
 import java.util.Hashtable;
 import java.util.Map;
 
+import org.jmol.api.Interface;
+import org.jmol.util.BoxInfo;
+import org.jmol.util.Escape;
+import org.jmol.util.SimpleUnitCell;
+import org.jmol.util.Tensor;
+import org.jmol.viewer.JC;
+import org.jmol.viewer.Viewer;
+
 import javajs.util.Lst;
 import javajs.util.M3;
 import javajs.util.M4;
@@ -39,14 +47,6 @@ import javajs.util.T3;
 import javajs.util.T4;
 import javajs.util.V3;
 
-import org.jmol.api.Interface;
-import org.jmol.util.BoxInfo;
-import org.jmol.util.Escape;
-import org.jmol.util.SimpleUnitCell;
-import org.jmol.util.Tensor;
-import org.jmol.viewer.JC;
-import org.jmol.viewer.Viewer;
-
 /**
  * a class private to the org.jmol.symmetry package
  * to be accessed only through the SymmetryInterface API
@@ -64,19 +64,37 @@ class UnitCell extends SimpleUnitCell implements Cloneable {
   private P3 fractionalOffset;
   /**
    * this flag TRUE causes an update of matrixCtoFNoOffset each time an offset is changed
-   * so that it is updated and the two stay the same; set true only for JmolData, tensors, and isosurfaceMesh
+   * so that it is updated and the two stay the same; set true only for isosurfaceMesh
    * 
-   * it is no longer clear to me exactly why this is necessary, and perhaps it is not for some of these
-   *  
-   *
    */
   private boolean allFractionalRelative;
   
   protected final P3 cartesianOffset = new P3();
+  /**
+   * a P3 or P4; the raw multiplier for the cell from 
+   * 
+   * UNITCELL {ijk ijk scale}
+   * 
+   * UNITCELL {1iiijjjkkk 1iiijjjkkk scale}
+   * 
+   * (encoded as a P4: {1iiijjjkkk 1iiijjjkkk scale 1kkkkkk} )
+   * 
+   */
   protected T3 unitCellMultiplier;
+  
+  /**
+   * the multiplied, offset UnitCell derived from this UnitCell
+   */
+  private UnitCell unitCellMultiplied;
+
   public Lst<String> moreInfo;
+  
   public String name = "";
   
+  /**
+   * Indicate that this is a new unit cell, not one from a file.
+   * 
+   */  
   private UnitCell() {
     super();  
   }
@@ -145,7 +163,7 @@ class UnitCell extends SimpleUnitCell implements Cloneable {
     } else {
       // use original unit cell
       // note that this matrix will be the same as matrixCartesianToFractional
-      // when allFractionalRelative is set true (special cases only)
+      // when allFractionalRelative is set true (isosurfaceMesh special cases only)
       matrixCtoFNoOffset.rotTrans(pt);
       unitize(pt);
       pt.add(offset); 
@@ -153,44 +171,72 @@ class UnitCell extends SimpleUnitCell implements Cloneable {
     }
   }
   
+  /**
+   * when offset is null, use the current cell, otherwise use the original unit cell
+   * 
+   * @param pt
+   * @param offset
+   */
+  final void toUnitCellRnd(T3 pt, T3 offset) {
+    if (matrixCartesianToFractional == null)
+      return;
+    if (offset == null) {
+      // used redefined unitcell 
+      matrixCartesianToFractional.rotTrans(pt);
+      unitizeRnd(pt);
+      matrixFractionalToCartesian.rotTrans(pt);
+    } else {
+      // use original unit cell
+      // note that this matrix will be the same as matrixCartesianToFractional
+      // when allFractionalRelative is set true (isosurfaceMesh special cases only)
+      matrixCtoFNoOffset.rotTrans(pt);
+      unitizeRnd(pt);
+      pt.add(offset); 
+      matrixFtoCNoOffset.rotTrans(pt);
+    }
+  }
+  
   /**
    * returns [0,1)
    * 
    * @param pt
    */
   public void unitize(T3 pt) {
-    switch (dimension) {
-    case 3:
-      pt.z = toFractionalX(pt.z);  
-      //$FALL-THROUGH$
-    case 2:
-      pt.y = toFractionalX(pt.y);
-      //$FALL-THROUGH$
-    case 1:
-      pt.x = toFractionalX(pt.x);
-    }
+	  unitizeDim(dimension, pt);
   }
 
+  /**
+   * returns [0,1) with rounding to 0.0001
+   * 
+   * @param pt
+   */
+  public void unitizeRnd(T3 pt) {
+    unitizeDimRnd(dimension, pt);
+  }
 
   public void reset() {
     unitCellMultiplier = null;
+    unitCellMultiplied = null;
     setOffset(P3.new3(0, 0, 0));
   }
   
   void setOffset(T3 pt) {
     if (pt == null)
       return;
+    unitCellMultiplied = null;
     T4 pt4 = (pt instanceof T4 ? (T4) pt : null);
-    boolean isCell555P4 = (pt4 != null && pt4.w > 999999);
-    if (pt4 != null ? pt4.w <= 0 || isCell555P4 : pt.x >= 100 || pt.y >= 100) {
-      // from "unitcell range {ijk ijk scale}"
-      //   or "unitcell range {1iiijjjkkk 1iiijjjkkk scale}"
-      //     where we have encoded this as a P4: {1iiijjjkkk 1iiijjjkkk scale 1kkkkkk}
-      //   or "unitcell reset"
-      unitCellMultiplier = (pt.z == 0 && pt.x == pt.y && !isCell555P4 ? null : isCell555P4 ? P4.newPt((P4) pt4) : P3.newP(pt));
+    float w = (pt4 == null ? Float.MIN_VALUE : pt4.w);
+    boolean isCell555P4 = (w > 999999);
+    if (pt4 != null ? 
+        w <= 0 || isCell555P4 
+        : pt.x >= 100 || pt.y >= 100) {
+      unitCellMultiplier = (pt.z == 0 && pt.x == pt.y && !isCell555P4 
+          ? null 
+          : isCell555P4 ? P4.newPt((P4) pt4) : P3.newP(pt));
+      unitCellMultiplied = null;
       if (pt4 == null || pt4.w == 0 || isCell555P4)
         return;
-      // from reset, continuing 
+      // pt4.w == -1 from reset, continuing 
     }
     // from "unitcell offset {i j k}"
     if (hasOffset() || pt.lengthSquared() > 0) {
@@ -238,23 +284,39 @@ class UnitCell extends SimpleUnitCell implements Cloneable {
   }
 
   Map<String, Object> getInfo() {
+    UnitCell m = getUnitCellMultiplied();
+    if (m != this)
+      return m.getInfo();       
     Map<String, Object> info = new Hashtable<String, Object>();
     info.put("params", unitCellParams);
-    info.put("vectors", getUnitCellVectors());
+    info.put("oabc", getUnitCellVectors());
     info.put("volume", Double.valueOf(volume));
     info.put("matFtoC", matrixFractionalToCartesian);
     info.put("matCtoF", matrixCartesianToFractional);
     return info;
   }
   
-  String dumpInfo(boolean isFull) {
+  String dumpInfo(boolean isDebug, boolean multiplied) {
+    UnitCell m = (multiplied ? getUnitCellMultiplied() : this);
+    if (m != this)
+      return m.dumpInfo(isDebug, false);
     return "a=" + a + ", b=" + b + ", c=" + c + ", alpha=" + alpha + ", beta=" + beta + ", gamma=" + gamma
-       + "\n" + Escape.eAP(getUnitCellVectors())
+       + "\noabc=" + Escape.eAP(getUnitCellVectors())
        + "\nvolume=" + volume
-       + (isFull ? "\nfractional to cartesian: " + matrixFractionalToCartesian 
+       + (isDebug ? "\nfractional to cartesian: " + matrixFractionalToCartesian 
        + "\ncartesian to fractional: " + matrixCartesianToFractional : "");
   }
 
+  UnitCell getUnitCellMultiplied() {
+    if (unitCellMultiplier == null || unitCellMultiplier.z > 0 && unitCellMultiplier.z == (int) unitCellMultiplier.z)
+      return this;
+    if (unitCellMultiplied == null) {
+      P3[] pts = BoxInfo.toOABC(getScaledCell(true), null);
+      unitCellMultiplied = fromOABC(pts, false);
+    }
+    return unitCellMultiplied;
+  }
+
   P3[] getVertices() {
     return vertices; // does not include offsets
   }
@@ -419,43 +481,41 @@ class UnitCell extends SimpleUnitCell implements Cloneable {
    * @return points in Triangulator order
    */
   P3[] getCanonicalCopy(float scale, boolean withOffset) {
+    P3[] pts = getScaledCell(withOffset);
+    return BoxInfo.getCanonicalCopy(pts, scale);
+  }
+
+  public P3[] getScaledCell(boolean withOffset) {
     P3[] pts  = new P3[8];
     P3 cell0 = null;
     P3 cell1 = null;
-    if (withOffset && unitCellMultiplier != null) {
+    if (withOffset && unitCellMultiplier != null && unitCellMultiplier.z == 0) {
       cell0 = new P3();
       cell1 = new P3();
       ijkToPoint3f((int) unitCellMultiplier.x, cell0, 0, 0);
       ijkToPoint3f((int) unitCellMultiplier.y, cell1, 0, 0);
       cell1.sub(cell0);
     }
+    float scale = (unitCellMultiplier == null || unitCellMultiplier.z == 0 ? 1
+        : Math.abs(unitCellMultiplier.z));
     for (int i = 0; i < 8; i++) {
       P3 pt = pts[i] = P3.newP(BoxInfo.unitCubePoints[i]);
       if (cell0 != null) {
-        scale *= (unitCellMultiplier.z == 0 ? 1 : unitCellMultiplier.z);
         pts[i].add3(cell0.x + cell1.x * pt.x, 
             cell0.y + cell1.y * pt.y,
             cell0.z + cell1.z * pt.z);
       }
+      pts[i].scale(scale);
       matrixFractionalToCartesian.rotTrans(pt);
       if (!withOffset)
         pt.sub(cartesianOffset);
     }
-    return BoxInfo.getCanonicalCopy(pts, scale);
+    return pts;
   }
 
   /// private methods
   
   
-  private static float toFractionalX(float x) {
-    // introduced in Jmol 11.7.36
-    x = (float) (x - Math.floor(x));
-    if (x > 0.9999f || x < 0.0001f) 
-      x = 0;
-    return x;
-  }
-  
-  
   private void initUnitcellVertices() {
     if (matrixFractionalToCartesian == null)
       return;
@@ -530,17 +590,6 @@ class UnitCell extends SimpleUnitCell implements Cloneable {
     return (fractionalOffset != null && fractionalOffset.lengthSquared() != 0);
   }
 
-  public String getState() {
-    String s = "";
-    // unitcell offset {1 1 1}
-    if (fractionalOffset != null && fractionalOffset.lengthSquared() != 0)
-      s += "  unitcell offset " + Escape.eP(fractionalOffset) + ";\n";
-    // unitcell range {444 555 1}
-    if (unitCellMultiplier != null)
-      s += "  unitcell range " + escapeMultiplier(unitCellMultiplier) + ";\n";
-    return s;
-  }
-
   /**
    * Returns a quaternion that will take the standard frame to a view down a
    * particular axis, expressed as its counterparts.
@@ -564,48 +613,74 @@ class UnitCell extends SimpleUnitCell implements Cloneable {
     int mul = (abc.charAt(0) == '-' ? -1 : 1);
     if (mul < 0)
       abc = abc.substring(1);
-    int quadrant = 0;
+    String abc0 = abc;
+    abc = PT.rep(PT.rep(PT.rep(PT.rep(PT.rep(PT.rep(abc, 
+        "ab", "A"), //3
+        "bc", "B"), //4
+        "ca", "C"), //5
+        "ba", "D"), //6
+        "cb", "E"), //7
+        "ac", "F"); //8
+    boolean isFace = !abc0.equals(abc);
+    int quadrant = (isFace ? 1 : 0);
     if (abc.length() == 2) { // a1 a2 a3 a4 b1 b2 b3 b4...
       quadrant = abc.charAt(1) - 48;
       abc = abc.substring(0, 1);
     }
     boolean isEven = (quadrant % 2 == 0);
-    int axis = "abc".indexOf(abc);
+    int axis = "abcABCDEF".indexOf(abc);
 
-    T3 v1, v2;
+    T3 v1, v2, v3;
     switch (axis) {
-    case 0:
+    case 7: // cb
+      mul = -mul;
+      //$FALL-THROUGH$
+    case 4: // bc
+      a.cross(c, b);
+      quadrant = ((5 - quadrant) % 4) + 1;
+          //$FALL-THROUGH$
+    case 0: // a
     default:
       v1 = a;
       v2 = c;
-      if (quadrant > 0) {
-        if (mul > 0 == isEven) {
-          v2 = b;
-          v1.scale(-1);
-        }
-      }
+      v3 = b;
       break;
+    case 8: // ca
+      mul = -mul;
+      //$FALL-THROUGH$
+    case 5: // ac
+      mul = -mul;
+      b.cross(c, a);
+      quadrant = ((2 + quadrant) % 4) + 1;
+    //$FALL-THROUGH$
     case 1: // b
       v1 = b;
       v2 = a;
-      if (quadrant > 0) {
-        if (mul > 0 == isEven) {
-          v2 = c;
-          v1.scale(-1);
-        }
-      }
+      v3 = c;
+      mul = -mul;
       break;
+    case 3: // ab
+      mul = -mul;
+      //$FALL-THROUGH$
+    case 6: // ba
+      c.cross(a,b);
+      if (isEven)
+        quadrant = 6 - quadrant;
+      //$FALL-THROUGH$
     case 2: // c
       v1 = c;
       v2 = a;
-      if (quadrant > 0) {
+      v3 = b;
+      if (!isFace && quadrant > 0) {
         quadrant = 5 - quadrant;
-        if (mul > 0 != isEven) {
-          v2 = b;
-          v1.scale(-1);
-        }
       }
-      break;
+     break;
+    }
+    if (quadrant > 0) {
+      if (mul > 0 != isEven) {
+        v2 = v3;
+        v1.scale(-1);
+      }
     }
     switch (quadrant) {
     case 0:
@@ -640,10 +715,12 @@ class UnitCell extends SimpleUnitCell implements Cloneable {
    *        preceded by ! for "reverse of". For example,
    *        "!a-b,-5a-5b,-c;7/8,0,1/8" offset is optional, and can be a
    *        definition such as "a=3.40,b=4.30,c=5.02,alpha=90,beta=90,gamma=129"
+   * @param retMatrix
+   *        if a string, return the M4 matrix corresponding to this definition
    * 
-   * @return [origin va vb vc]
+   * @return [origin va vb vc] or null if invalid syntax, or null for allowNull and is identity
    */
-  public T3[] getV0abc(Object def) {
+  public T3[] getV0abc(Object def, M4 retMatrix) {
     if (def instanceof T3[])
       return (T3[]) def;
     M4 m;
@@ -676,22 +753,23 @@ class UnitCell extends SimpleUnitCell implements Cloneable {
         return null;
       m = symTemp.getSpaceGroupOperation(i);
       ((SymmetryOperation) m).doFinalize();
-      if (strans != null) {
-        String[] atrans = PT.split(strans, ",");
-        float[] ftrans = new float[3];
-        if (atrans.length == 3)
-          for (int j = 0; j < 3; j++) {
-            String s = atrans[j];
-            int sfpt = s.indexOf("/");
-            if (sfpt >= 0) {
-              ftrans[j] = PT.parseFloat(s.substring(0, sfpt))
-                  / PT.parseFloat(s.substring(sfpt + 1));
-            } else {
-              ftrans[j] = PT.parseFloat(s);
-            }
+      String[] atrans = PT.split(strans, ",");
+      float[] ftrans = new float[3];
+      if (atrans.length == 3)
+        for (int j = 0; j < 3; j++) {
+          String s = atrans[j];
+          int sfpt = s.indexOf("/");
+          if (sfpt >= 0) {
+            ftrans[j] = PT.parseFloat(s.substring(0, sfpt))
+                / PT.parseFloat(s.substring(sfpt + 1));
+          } else {
+            ftrans[j] = PT.parseFloat(s);
           }
-        P3 ptrans = P3.new3(ftrans[0], ftrans[1], ftrans[2]);
-        m.setTranslation(ptrans);
+        }
+      P3 ptrans = P3.new3(ftrans[0], ftrans[1], ftrans[2]);
+      m.setTranslation(ptrans);
+      if (retMatrix != null) {
+        retMatrix.setM4(m);
       }
     } else if (def instanceof M3) {
       m = M4.newMV((M3) def, new P3());
@@ -809,6 +887,7 @@ class UnitCell extends SimpleUnitCell implements Cloneable {
    * return a conventional lattice from a primitive
    * 
    * @param latticeType  "A" "B" "C" "R" etc.
+   * @param primitiveToCrystal 
    * @return [origin va vb vc]
    */
   public T3[] getConventionalUnitCell(String latticeType, M3 primitiveToCrystal) {
@@ -827,4 +906,111 @@ class UnitCell extends SimpleUnitCell implements Cloneable {
     return ucnew;
   }
 
+  /**
+   * 
+   * @param pt
+   *        the point to transform
+   * @param flags
+   *        "tofractional,fromfractional,packed"
+   * @param ops
+   *        space group operations
+   * @param list
+   *        the list to append to
+   * @param i0
+   *        the starting index of the list
+   * @param n0
+   *        the first point that is to be duplicated; prior points are just
+   *        references for removing duplicates
+   * @return augmented list
+   */
+  Lst<P3> getEquivPoints(P3 pt, String flags, M4[] ops, Lst<P3> list,
+                                int i0, int n0) {
+    boolean fromfractional = (flags.indexOf("fromfractional") >= 0);
+    boolean tofractional = (flags.indexOf("tofractional") >= 0);
+    boolean packed = (flags.indexOf("packed") >= 0);
+    if (list == null)
+      list = new Lst<P3>();
+    P3 pf = P3.newP(pt);
+    if (!fromfractional)
+      toFractional(pf, true);
+    int n = list.size();
+    for (int i = 0, nops = ops.length; i < nops; i++) {
+      P3 p = P3.newP(pf);
+      ops[i].rotTrans(p);
+      //not using unitize here, because it does some averaging
+      p.x = (float) (p.x - Math.floor(p.x));
+      p.y = (float) (p.y - Math.floor(p.y));
+      p.z = (float) (p.z - Math.floor(p.z));
+      list.addLast(p);
+      n++;
+    }
+    if (packed) {
+      // duplicate all the points. 
+      for (int i = n0; i < n; i++) {
+        P3 p = list.get(i);
+        unitizeRnd(p);
+        if (p.x == 0) {
+          list.addLast(P3.new3(1, p.y, p.z));
+          if (p.y == 0) {
+            list.addLast(P3.new3(1, 1, p.z));
+            if (p.z == 0) {
+              list.addLast(P3.new3(1, 1, 1));
+            }
+          }
+        }
+        if (p.y == 0) {
+          list.addLast(P3.new3(p.x, 1, p.z));
+          if (p.z == 0) {
+            list.addLast(P3.new3(p.x, 1, 1));
+          }
+        }
+        if (p.z == 0) {
+          list.addLast(P3.new3(p.x, p.y, 1));
+          if (p.x == 0) {
+            list.addLast(P3.new3(1, p.y, 1));
+          }
+        }
+      }
+    }
+    checkDuplicate(list, i0, n0, -1);
+    if (!tofractional) {
+      for (int i = list.size(); --i >= n0;)
+        toCartesian(list.get(i), true);
+    }
+    return list;
+  }
+
+  private static void checkDuplicate(Lst<P3> list, int i0, int n0, int n) {
+    if (n < 0)
+      n = list.size();
+    for (int i = i0; i < n; i++) {
+      P3 p = list.get(i);
+      for (int j = Math.max(i + 1, n0); j < n; j++) {
+        if (list.get(j).distanceSquared(p) < JC.UC_TOLERANCE2) {
+          list.removeItemAt(j);
+          n--;
+          j--;
+        }
+      }
+    }
+  }
+
+  public void normalize(V3 vtrans) {
+    vtrans.x = normalizeXRnd(vtrans.x);
+    vtrans.y = normalizeXRnd(vtrans.y);
+    vtrans.z = normalizeXRnd(vtrans.z);    
+  }
+
+  public String getState() {
+    String s = "";
+    // unitcell offset {1 1 1}
+    if (fractionalOffset != null && fractionalOffset.lengthSquared() != 0)
+      s += "  unitcell offset " + Escape.eP(fractionalOffset) + ";\n";
+    // unitcell range {444 555 1}
+    if (unitCellMultiplier != null)
+      s += "  unitcell range " + escapeMultiplier(unitCellMultiplier) + ";\n";
+    return s;
+  }
+
+
 }
