@@ -24,6 +24,8 @@
 
 package org.jmol.util;
 
+import org.jmol.api.SymmetryInterface;
+
 import javajs.util.AU;
 import javajs.util.M4;
 import javajs.util.P3;
@@ -46,7 +48,15 @@ import javajs.util.V3;
 
 public class SimpleUnitCell {
 
-  protected float[] unitCellParams; //6 parameters + optional 16 matrix items
+
+  public static final int PARAM_STD = 6;
+  public static final int PARAM_VABC = 6;
+  public static final int PARAM_M4 = 6;
+  public static final int PARAM_SUPERCELL = 22;
+  public static final int PARAM_SCALE = 25;
+  public static final int PARAM_COUNT = 26;
+
+  protected float[] unitCellParams;
   public M4 matrixCartesianToFractional;
   public M4 matrixFractionalToCartesian;
   public double volume;
@@ -65,7 +75,7 @@ public class SimpleUnitCell {
   protected double cA_, cB_;
   protected double a_;
   protected double b_, c_;
-  protected int dimension;
+  protected int dimension = 3;
   private P3 fractionalOrigin;
 
   public static boolean isValid(float[] parameters) {
@@ -79,11 +89,26 @@ public class SimpleUnitCell {
 
   /**
    * 
-   * @param params len = 6 [a b c alpha beta gamma] 
-   *            or len = 15 [-1 0 0 0 0 0 va vb vc]
-   *            or len = 22 [a b c alpha beta gamma m00 m01 .. m33]
-   *            and/or len = 25 [......................  na nb nc]
-   *             
+   * @param params
+   * 
+   *        len = 6 [a b c alpha beta gamma]
+   * 
+   *        len = 6 [a b -1 alpha beta gamma] // slab
+   * 
+   *        len = 6 [a -1 -1 alpha beta gamma] // polymer
+   * 
+   *        or len = 15 [-1 -1 -1 -1 -1 -1 va[3] vb[3] vc[3]] // vectors only
+   * 
+   *        or len = 15 [a -1 -1 -1 -1 -1 va[3] vb[3] vc[3]] // polymer, vectors only
+   * 
+   *        or len = 15 [a b -1 -1 -1 -1 va[3] vb[3] vc[3]] // slab, vectors only
+   * 
+   *        or len = 22 [a b c alpha beta gamma m00 m01 .. m33] // matrix included
+   * 
+   *        and/or len = 25 [...................... na nb nc] // supercell
+   * 
+   *        and/or len = 26 [...................... na nb nc scale] // scaled supercell
+   * 
    * @return a simple unit cell
    */
   public static SimpleUnitCell newA(float[] params) {
@@ -92,46 +117,70 @@ public class SimpleUnitCell {
     return c;
   }
   
-  
   protected void init(float[] params) {
     if (params == null)
-      params = new float[] {1, 1, 1, 90, 90, 90};
+      params = new float[] { 1, 1, 1, 90, 90, 90 };
     if (!isValid(params))
       return;
-    unitCellParams = AU.arrayCopyF(params, params.length);
+    params = unitCellParams = AU.arrayCopyF(params, params.length);
 
     boolean rotateHex = false; // special gamma = -1 indicates hex rotation for AFLOW
-    
+
     a = params[0];
     b = params[1];
     c = params[2];
+    if (a > 0) {
+      if (b < 0) {
+        dimension = 1;
+      } else if (c < 0) {
+        dimension = 2;
+      }
+    }
+
     alpha = params[3];
     beta = params[4];
     gamma = params[5];
-    if (gamma == -1) {
+    if (gamma == -1 && c > 0) {
       rotateHex = true;
       gamma = 120;
     }
-    
+
     // (int) Float.NaN == 0 (but not in JavaScript!)
     // supercell
-    float fa = na = Math.max(1, params.length >= 25 && !Float.isNaN(params[22]) ? (int) params[22] : 1);
-    float fb = nb = Math.max(1, params.length >= 25 && !Float.isNaN(params[23]) ? (int) params[23] : 1);
-    float fc = nc = Math.max(1, params.length >= 25 && !Float.isNaN(params[24]) ? (int) params[24] : 1);
-    if (params.length > 25 && !Float.isNaN(params[25])) {
-      float fScale = params[25];
-      fa *= fScale;
-      fb *= fScale;
-      fc *= fScale;
+    float fa = na = Math.max(1,
+        params.length > PARAM_SUPERCELL + 2
+            && !Float.isNaN(params[PARAM_SUPERCELL])
+                ? (int) params[PARAM_SUPERCELL]
+                : 1);
+    float fb = nb = Math.max(1,
+        params.length > PARAM_SUPERCELL + 2
+            && !Float.isNaN(params[PARAM_SUPERCELL + 1])
+                ? (int) params[PARAM_SUPERCELL + 1]
+                : 1);
+    float fc = nc = Math.max(1,
+        params.length > PARAM_SUPERCELL + 2
+            && !Float.isNaN(params[PARAM_SUPERCELL + 2])
+                ? (int) params[PARAM_SUPERCELL + 2]
+                : 1);
+    if (params.length > PARAM_SCALE && !Float.isNaN(params[PARAM_SCALE])) {
+      float fScale = params[PARAM_SCALE];
+      if (fScale > 0) {
+        fa *= fScale;
+        fb *= fScale;
+        fc *= fScale;
+      }
     } else {
       fa = fb = fc = 1;
     }
 
-    if (a <= 0) {
+    if (a <= 0 && c <= 0) {
       // must calculate a, b, c alpha beta gamma from Cartesian vectors;
-      V3 va = V3.new3(params[6], params[7], params[8]);
-      V3 vb = V3.new3(params[9], params[10], params[11]);
-      V3 vc = V3.new3(params[12], params[13], params[14]);
+      V3 va = V3.new3(params[PARAM_VABC], params[PARAM_VABC + 1],
+          params[PARAM_VABC + 2]);
+      V3 vb = V3.new3(params[PARAM_VABC + 3], params[PARAM_VABC + 4],
+          params[PARAM_VABC + 5]);
+      V3 vc = V3.new3(params[PARAM_VABC + 6], params[PARAM_VABC + 7],
+          params[PARAM_VABC + 8]);
       setABC(va, vb, vc);
       if (c < 0) {
         float[] n = AU.arrayCopyF(params, -1);
@@ -155,9 +204,10 @@ public class SimpleUnitCell {
         params = n;
       }
     }
-    //System.out.println("unitcell " + a + " " + b + " " + c);
-    
-    a *= fa; 
+
+    // checking here for still a dimension issue with b or c
+    // was < 0 above; here <= 0
+    a *= fa;
     if (b <= 0) {
       b = c = 1;
       dimension = 1;
@@ -168,11 +218,9 @@ public class SimpleUnitCell {
     } else {
       b *= fb;
       c *= fc;
-      dimension = 3;
     }
-
     setCellParams();
-    
+
     if (params.length > 21 && !Float.isNaN(params[21])) {
       // parameters with a 4x4 matrix
       // [a b c alpha beta gamma m00 m01 m02 m03 m10 m11.... m20...]
@@ -195,10 +243,11 @@ public class SimpleUnitCell {
           break;
         }
         scaleMatrix[i] = params[6 + i] * f;
-      }      
+      }
       matrixCartesianToFractional = M4.newA16(scaleMatrix);
       matrixCartesianToFractional.getTranslation(fractionalOrigin);
-      matrixFractionalToCartesian = M4.newM4(matrixCartesianToFractional).invert();
+      matrixFractionalToCartesian = M4.newM4(matrixCartesianToFractional)
+          .invert();
       if (params[0] == 1)
         setParamsFromMatrix();
     } else if (params.length > 14 && !Float.isNaN(params[14])) {
@@ -209,10 +258,11 @@ public class SimpleUnitCell {
       m.setColumn4(1, params[9] * fb, params[10] * fb, params[11] * fb, 0);
       m.setColumn4(2, params[12] * fc, params[13] * fc, params[14] * fc, 0);
       m.setColumn4(3, 0, 0, 0, 1);
-      matrixCartesianToFractional = M4.newM4(matrixFractionalToCartesian).invert();
+      matrixCartesianToFractional = M4.newM4(matrixFractionalToCartesian)
+          .invert();
     } else {
       M4 m = matrixFractionalToCartesian = new M4();
-      
+
       if (rotateHex) {
         // 1, 2. align a and b symmetrically about the x axis (AFLOW)
         m.setColumn4(0, (float) (-b * cosGamma), (float) (-b * sinGamma), 0, 0);
@@ -226,16 +276,25 @@ public class SimpleUnitCell {
       }
       // 3. now the c axis,
       // http://server.ccl.net/cca/documents/molecular-modeling/node4.html
-      m.setColumn4(2, (float) (c * cosBeta), (float) (c
-          * (cosAlpha - cosBeta * cosGamma) / sinGamma), (float) (volume / (a
-          * b * sinGamma)), 0);
+      m.setColumn4(2, (float) (c * cosBeta),
+          (float) (c * (cosAlpha - cosBeta * cosGamma) / sinGamma),
+          (float) (volume / (a * b * sinGamma)), 0);
       m.setColumn4(3, 0, 0, 0, 1);
-      matrixCartesianToFractional = M4.newM4(matrixFractionalToCartesian).invert();
+      matrixCartesianToFractional = M4.newM4(matrixFractionalToCartesian)
+          .invert();
     }
     matrixCtoFNoOffset = matrixCartesianToFractional;
     matrixFtoCNoOffset = matrixFractionalToCartesian;
   }
 
+  public static void addVectors(float[] params) {
+    SimpleUnitCell c = SimpleUnitCell.newA(params);
+    M4 m = c.matrixFractionalToCartesian;
+    for (int i = 0; i < 9; i++)
+    params[PARAM_VABC + i] = m.getElement(i%3, i/3);
+  }
+
+
   private void setParamsFromMatrix() {
     V3 va = V3.new3(1,  0,  0);
     V3 vb = V3.new3(0,  1,  0);
@@ -260,6 +319,13 @@ public class SimpleUnitCell {
     alpha = (b < 0 || c < 0 ? 90 : vb.angle(vc) / toRadians);
     beta = (c < 0 ? 90 : va.angle(vc) / toRadians);
     gamma = (b < 0 ? 90 : va.angle(vb) / toRadians);
+    float[] p = unitCellParams;
+    p[0] = a;
+    p[1] = b;
+    p[2] = c;
+    p[3] = alpha;
+    p[4] = beta;
+    p[5] = gamma;
   }
 
   private void setCellParams() {
@@ -409,6 +475,21 @@ public class SimpleUnitCell {
     cell.z = (kcode == 0 ? nnn % f 
         : (offset == -500 ? kcode / f : kcode) % f) + offset;
   }
+  
+
+  /**
+   * Convert user's {3 2 1} to {1500500500, 1503502501, 0 or 1, 1500501}
+   * @param pt
+   * @param scale 1 for block of unit cells; 0 for one large supercell
+   * @return converted P4
+   */
+  public static P4 ptToIJK(T3 pt, int scale) {
+    if (pt.x <= 5 && pt.y <= 5 && pt.z <= 5) {
+      return P4.new4(555, (pt.x + 4) * 100 + (pt.y + 4) * 10 + pt.z + 4, scale, 0);
+    } 
+    int i555 = 1500500500;
+    return P4.new4(i555, i555 + pt.x*1000000 + pt.y * 1000 + pt.z, scale, 1500500 + pt.z);
+  }
 
   /**
    * Generally the multiplier is just {ijk ijk scale}, but when we have
@@ -429,7 +510,6 @@ public class SimpleUnitCell {
     }
     return Escape.eP(pt);
   }
-  
 
   public final static float SLOP = 0.02f;
   private final static float SLOP1 = 1 - SLOP;
@@ -451,15 +531,27 @@ public class SimpleUnitCell {
     return f;
   }
   
+  /**
+   * Generate the reciprocal unit cell, scaled as desired
+   * 
+   * @param abc [a,b,c] or [o,a,b,c]
+   * @param ret
+   * @param scale 0 for 2pi, teneral reciprocal lattice
+   * @return oabc
+   */
   public static T3[] getReciprocal(T3[] abc, T3[] ret, float scale) {
+    if (scale == 0)
+      scale = (float)(2 * Math.PI);
     P3[] rabc = new P3[4];
     int off = (abc.length == 4 ? 1 : 0);
     rabc[0] = (off == 1 ? P3.newP(abc[0]) : new P3()); // origin
+    // a' = 2pi/V * b x c  = 2pi * (b x c) / (a . (b x c))
+    // b' = 2pi/V * c x a 
+    // c' = 2pi/V * a x b 
     for (int i = 0; i < 3; i++) {
-      rabc[i + 1] = new P3();
-      rabc[i + 1].cross(abc[((i + off) % 3) + off], abc[((i + off + 1) % 3)
-          + off]);
-      rabc[i + 1].scale(scale / abc[i + off].dot(rabc[i + 1]));
+      P3 v = rabc[i + 1] = new P3();
+      v.cross(abc[((i + 1) % 3) + off], abc[((i + 2) % 3) + off]);
+      v.scale(scale / abc[i + off].dot(v));
     }
     if (ret == null)
       return rabc;
@@ -537,7 +629,85 @@ public class SimpleUnitCell {
   }
 
 
+  @Override
   public String toString() {
     return "[" + a + " " + b + " " + c + " " + alpha + " " + beta + " " + gamma + "]";
   }
+
+  public static void unitizeDim(int dimension, T3 pt) {
+    switch (dimension) {
+    case 3:
+      pt.z = unitizeX(pt.z);  
+      //$FALL-THROUGH$
+    case 2:
+      pt.y = unitizeX(pt.y);
+      //$FALL-THROUGH$
+    case 1:
+      pt.x = unitizeX(pt.x);
+    }
+  }
+
+  public static void unitizeDimRnd(int dimension, T3 pt) {
+    switch (dimension) {
+    case 3:
+      pt.z = unitizeXRnd(pt.z);  
+      //$FALL-THROUGH$
+    case 2:
+      pt.y = unitizeXRnd(pt.y);
+      //$FALL-THROUGH$
+    case 1:
+      pt.x = unitizeXRnd(pt.x);
+    }
+  }
+
+  public static float unitizeX(float x) {
+    // introduced in Jmol 11.7.36
+    x = (float) (x - Math.floor(x));
+    // question - does this cause problems with dragatom?
+    if (x > 0.999f || x < 0.001f)  // 0.9999, 0.0001 was just too tight ams/jolliffeite
+      x = 0;
+    return x;
+  }
+
+  public static float unitizeXRnd(float x) {
+    // introduced in Jmol 11.7.36
+    x = (float) (x - Math.floor(x));
+    if (x > 0.9999f || x < 0.0001f) 
+      x = 0;
+    return x;
+  }
+  
+
+
+  public static float normalizeXRnd(float x) {
+    return Math.round(x*12) / 12;
+  }
+
+  
+  /**
+   * allowance for rounding in [0,1)
+   */
+  private final static float SLOP2 = 0.0001f;
+  
+  /**
+   * check atom position for range [0, 1) allowing for rounding
+
+   * @param pt 
+   * @return true if in [0, 1)
+   */
+  public static boolean checkPeriodic(P3 pt) {
+    return (pt.x >= -SLOP2 && pt.x < 1 - SLOP2
+        && pt.y >= -SLOP2 && pt.y < 1 - SLOP2
+        && pt.z >= -SLOP2 && pt.z < 1 - SLOP2
+        );
+  }
+
+  public static boolean checkUnitCell(SymmetryInterface uc, P3 cell, P3 ptTemp) {
+    uc.toFractional(ptTemp, false);
+    // {1 1 1} here is the original cell
+    return (ptTemp.x >= cell.x - 1f - SLOP && ptTemp.x <= cell.x + SLOP
+        && ptTemp.y >= cell.y - 1f - SLOP && ptTemp.y <= cell.y + SLOP
+        && ptTemp.z >= cell.z - 1f - SLOP && ptTemp.z <= cell.z + SLOP);
+  }
+
 }
