@@ -30,9 +30,6 @@ import org.jmol.api.GenericPlatform;
 import org.jmol.api.Interface;
 import org.jmol.awtjs.Event;
 import org.jmol.i18n.GT;
-
-import javajs.util.AU;
-import javajs.util.BS;
 import org.jmol.modelset.AtomCollection;
 import org.jmol.modelset.MeasurementPending;
 import org.jmol.script.SV;
@@ -44,14 +41,15 @@ import org.jmol.util.C;
 import org.jmol.util.Escape;
 import org.jmol.util.Logger;
 import org.jmol.util.Point3fi;
-
-import javajs.util.P3;
-import javajs.util.PT;
-
 import org.jmol.util.Rectangle;
 import org.jmol.viewer.binding.Binding;
 import org.jmol.viewer.binding.JmolBinding;
 
+import javajs.util.AU;
+import javajs.util.BS;
+import javajs.util.P3;
+import javajs.util.PT;
+
 public class ActionManager implements EventManager {
 
   protected Viewer vwr;
@@ -65,7 +63,7 @@ public class ActionManager implements EventManager {
   private Binding dragBinding;
   private Binding rasmolBinding;
   private Binding predragBinding;
-  private int LEFT_CLICKED;
+  //private int LEFT_CLICKED;
   private int LEFT_DRAGGED;
   
   /**
@@ -78,7 +76,7 @@ public class ActionManager implements EventManager {
     if (!Viewer.isJS)
       createActions();
     setBinding(jmolBinding = new JmolBinding());
-    LEFT_CLICKED = Binding.getMouseAction(1, Binding.LEFT, Event.CLICKED);
+    //LEFT_CLICKED = Binding.getMouseAction(1, Binding.LEFT, Event.CLICKED);
     LEFT_DRAGGED = Binding.getMouseAction(1, Binding.LEFT, Event.DRAGGED);
     dragGesture = new Gesture(20, vwr);
   }
@@ -439,17 +437,6 @@ public class ActionManager implements EventManager {
   public final static int PICKING_DRAG_LIGAND = 36;
   public final static int PICKING_DRAG_MODEL = 37;
 
-  /**
-   * picking styles
-   */
-  public final static int PICKINGSTYLE_SELECT_JMOL = 0;
-  public final static int PICKINGSTYLE_SELECT_CHIME = 0;
-  public final static int PICKINGSTYLE_SELECT_RASMOL = 1;
-  public final static int PICKINGSTYLE_SELECT_PFAAT = 2;
-  public final static int PICKINGSTYLE_SELECT_DRAG = 3;
-  public final static int PICKINGSTYLE_MEASURE_ON = 4;
-  public final static int PICKINGSTYLE_MEASURE_OFF = 5;
-
   private final static String[] pickingModeNames;
   static {
     pickingModeNames = "off identify label center draw spin symmetry deleteatom deletebond atom group chain molecule polymer structure site model element measure distance angle torsion sequence navigate connect struts dragselected dragmolecule dragatom dragminimize dragminimizemolecule invertstereo assignatom assignbond rotatebond identifybond dragligand dragmodel".split(" ");
@@ -467,6 +454,17 @@ public class ActionManager implements EventManager {
     return -1;
   }
 
+  /**
+   * picking styles
+   */
+  public final static int PICKINGSTYLE_SELECT_JMOL = 0;
+  public final static int PICKINGSTYLE_SELECT_CHIME = 0;
+  public final static int PICKINGSTYLE_SELECT_RASMOL = 1;
+  public final static int PICKINGSTYLE_SELECT_PFAAT = 2;
+  public final static int PICKINGSTYLE_SELECT_DRAG = 3;
+  public final static int PICKINGSTYLE_MEASURE_ON = 4;
+  public final static int PICKINGSTYLE_MEASURE_OFF = 5;
+
   private final static String[] pickingStyleNames;
   
   static {
@@ -485,11 +483,11 @@ public class ActionManager implements EventManager {
     return -1;
   }
 
-  int getAtomPickingMode() {
+  public int getAtomPickingMode() {
     return apm;
   }
 
-  void setPickingMode(int pickingMode) {
+  public void setPickingMode(int pickingMode) {
     boolean isNew = false;
     switch (pickingMode) {
     case PICKING_MK_RESET: 
@@ -1049,15 +1047,7 @@ public class ActionManager implements EventManager {
       return;
     }
     if (bnd(pressAction, ACTION_popupMenu)) {
-      char type = 'j';
-      if (vwr.getBoolean(T.modelkitmode)) {
-        Map<String, Object> t = vwr.checkObjectClicked(x, y, LEFT_CLICKED);
-        type = (
-//            t != null && "bond".equals(t.get("type")) ? 'b' : vwr
-//            .findNearestAtomIndex(x, y) >= 0 ? 'a' : 
-              'm');
-      }
-      vwr.popupMenu(x, y, type);
+      doPopup(x, y);
       return;
     }
     if (dragSelectedMode) {
@@ -1074,7 +1064,8 @@ public class ActionManager implements EventManager {
   }
 
   private void checkDragWheelAction(int dragWheelAction, int x, int y,
-                                    int deltaX, int deltaY, long time, int mode) {
+                                    int deltaX, int deltaY, long time,
+                                    int mode) {
     int buttonmods = Binding.getButtonMods(dragWheelAction);
     if (buttonmods != 0) {
       int newAction = vwr.notifyMouseClicked(x, y,
@@ -1093,19 +1084,28 @@ public class ActionManager implements EventManager {
 
     if (checkUserAction(dragWheelAction, x, y, deltaX, deltaY, time, mode))
       return;
-
-    if (vwr.g.modelKitMode && vwr.getModelkit(false).getRotateBondIndex() >= 0) {
+    int bi = (vwr.g.modelKitMode ? vwr.getModelkit(false).getRotateBondIndex()
+        : -1);
+    if (bi >= 0) {
       if (dragAtomIndex >= 0 || mkBondPressed
           || bnd(dragWheelAction, ACTION_rotateBranch)) {
+        if (dragAtomIndex >= 0) {
+          if (measurementQueued == null || measurementQueued.numSet == 0) {
+            vwr.setPendingMeasurement(vwr.getModelkit(false).setBondMeasure(bi,
+                measurementQueued = mp = getMP()));
+          } else {
+            measurementQueued.refresh(null);
+          }
+        }
         vwr.moveSelected(deltaX, deltaY, Integer.MIN_VALUE, x, y, null, false,
             false, dragAtomIndex >= 0 ? 0 : Event.VK_SHIFT);
         return;
       }
     }
-    
+
     BS bs = null;
     if (dragAtomIndex >= 0 && apm != PICKING_LABEL) {
-      
+
       switch (apm) {
       case PICKING_DRAG_SELECTED:
         dragSelected(dragWheelAction, deltaX, deltaY, true);
@@ -1114,7 +1114,8 @@ public class ActionManager implements EventManager {
       case PICKING_DRAG_MODEL:
       case PICKING_DRAG_MOLECULE:
       case PICKING_DRAG_MINIMIZE_MOLECULE:
-        bs = vwr.ms.getAtoms((apm == PICKING_DRAG_MODEL ? T.model : T.molecule), BSUtil.newAndSetBit(dragAtomIndex));
+        bs = vwr.ms.getAtoms((apm == PICKING_DRAG_MODEL ? T.model : T.molecule),
+            BSUtil.newAndSetBit(dragAtomIndex));
         if (apm == PICKING_DRAG_LIGAND)
           bs.and(vwr.getAtomBitSet("ligand"));
         //$FALL-THROUGH$
@@ -1133,14 +1134,12 @@ public class ActionManager implements EventManager {
           case PICKING_DRAG_MODEL:
           case PICKING_DRAG_MOLECULE:
           case PICKING_DRAG_MINIMIZE_MOLECULE:
-            vwr.select(bs, false, 0, true);
+            vwr.selectStatus(bs, false, 0, true, true);
             break;
           }
-          vwr.moveAtomWithHydrogens(
-              dragAtomIndex,
-              deltaX,
-              deltaY,
-              (bnd(dragWheelAction, ACTION_dragZ) ? -deltaY : Integer.MIN_VALUE),
+          vwr.moveAtomWithHydrogens(dragAtomIndex, deltaX, deltaY,
+              (bnd(dragWheelAction, ACTION_dragZ) ? -deltaY
+                  : Integer.MIN_VALUE),
               bs);
         }
         // NAH! if (atomPickingMode == PICKING_DRAG_MINIMIZE_MOLECULE && (dragGesture.getPointCount() % 5 == 0))
@@ -1192,7 +1191,8 @@ public class ActionManager implements EventManager {
         vwr.undoMoveActionClear(iatom, AtomCollection.TAINT_COORD, true);
       else
         vwr.moveSelected(Integer.MAX_VALUE, 0, Integer.MIN_VALUE,
-            Integer.MIN_VALUE, Integer.MIN_VALUE, null, false, false, buttonmods);
+            Integer.MIN_VALUE, Integer.MIN_VALUE, null, false, false,
+            buttonmods);
       dragSelected(dragWheelAction, deltaX, deltaY, false);
       return;
     }
@@ -1204,7 +1204,7 @@ public class ActionManager implements EventManager {
       }
     }
     if (checkMotionRotateZoom(dragWheelAction, x, deltaX, deltaY, true)) {
-      if (vwr.tm.slabEnabled && bnd(dragWheelAction,ACTION_slabAndDepth))
+      if (vwr.tm.slabEnabled && bnd(dragWheelAction, ACTION_slabAndDepth))
         vwr.slabDepthByPixels(deltaY);
       else
         vwr.zoomBy(deltaY);
@@ -1229,7 +1229,7 @@ public class ActionManager implements EventManager {
             Integer.MAX_VALUE);
       }
       return;
-    } 
+    }
     if (vwr.tm.slabEnabled) {
       if (bnd(dragWheelAction, ACTION_depth)) {
         vwr.depthByPixels(deltaY);
@@ -1247,7 +1247,7 @@ public class ActionManager implements EventManager {
     if (bnd(dragWheelAction, ACTION_wheelZoom)) {
       zoomByFactor(deltaY, Integer.MAX_VALUE, Integer.MAX_VALUE);
       return;
-    } 
+    }
     if (bnd(dragWheelAction, ACTION_rotateZ)) {
       setMotion(GenericPlatform.CURSOR_MOVE, true);
       vwr.rotateZBy(-deltaX, Integer.MAX_VALUE, Integer.MAX_VALUE);
@@ -1327,7 +1327,7 @@ public class ActionManager implements EventManager {
         exitMeasurementMode(null);
         return;
       }
-      assignNew(x, y);
+      assignNew();
       return;
     }
     dragAtomIndex = -1;
@@ -1414,6 +1414,10 @@ public class ActionManager implements EventManager {
       return;
     int nearestAtomIndex = findNearestAtom(x, y, nearestPoint, clickedCount > 0);
 
+    if (bnd(clickAction, ACTION_popupMenu)) {
+      doPopup(x, y);
+      return;
+    }
     if (clickedCount == 0 && apm != PICKING_ASSIGN_ATOM) {
       // mouse move
       if (mp == null)
@@ -1487,6 +1491,10 @@ public class ActionManager implements EventManager {
     }
   }
 
+  private void doPopup(int x, int y) {
+    vwr.popupMenu(x, y, vwr.getBoolean(T.modelkitmode) ? 'm' : 'j');
+  }
+
   private void pickLabel(int iatom) {
     String label = vwr.ms.at[iatom].atomPropertyString(vwr,  T.label);
     if (pressedCount == 2) {
@@ -1769,7 +1777,7 @@ public class ActionManager implements EventManager {
   /// methods that utilize vwr.script
 
   private void runScript(String script) {
-    vwr.script(script);
+    vwr.evalStringGUI(script);
   }
 
   private void atomOrPointPicked(int atomIndex, Point3fi ptClicked) {
@@ -1779,7 +1787,7 @@ public class ActionManager implements EventManager {
     if (atomIndex < 0) {
       resetMeasurement(); // for set picking measure only
       if (bnd(clickAction, ACTION_selectNone)) {
-        runScript("select none");
+        runScript("select none"); // note that this sets us up for default labels
         return;
       }
       if (apm != PICKING_SPIN
@@ -1891,7 +1899,7 @@ public class ActionManager implements EventManager {
       return;
     case PICKING_INVERT_STEREO:
       if (bnd(clickAction, ACTION_assignNew)) {
-        vwr.invertRingAt(atomIndex, true);
+        vwr.invertAtomCoord(null, null, null, atomIndex, true);
         vwr.setStatusAtomPicked(atomIndex, "invert stereo for atomIndex=" + atomIndex, null, false);
       }
       return;
@@ -1941,7 +1949,7 @@ public class ActionManager implements EventManager {
     vwr.setStatusAtomPicked(atomIndex, null, null, false);
   }
 
-  private void assignNew(int x, int y) {
+  private void assignNew() {
     if (!vwr.getModelkit(false).handleAssignNew(pressed, dragged, mp, dragAtomIndex)) {
       exitMeasurementMode("bond dropped");      
     }
@@ -2018,13 +2026,13 @@ public class ActionManager implements EventManager {
     if (mp != null || selectionWorking)
       return;
     selectionWorking = true;
-    String s = (rubberbandSelectionMode
-        || bnd(clickAction, ACTION_selectToggle) ? "selected and not ("
-        + item + ") or (not selected) and " : bnd(clickAction,
-        ACTION_selectAndNot) ? "selected and not " : bnd(clickAction,
-        ACTION_selectOr) ? "selected or " : clickAction == 0
-        || bnd(clickAction, ACTION_selectToggleExtended) ? "selected tog "
-        : bnd(clickAction, ACTION_select) ? "" : null);
+    String s = (rubberbandSelectionMode || bnd(clickAction, ACTION_selectToggle) ? 
+        "selected and not (" + item + ") or (not selected) and " 
+        : bnd(clickAction, ACTION_selectAndNot) ? "selected and not " 
+        : bnd(clickAction, ACTION_selectOr) ? "selected or " 
+        : clickAction == 0 || bnd(clickAction, ACTION_selectToggleExtended) ? "selected tog "
+        : bnd(clickAction, ACTION_select) ? "" 
+        : null);
     if (s != null) {
       s += "(" + item + ")";
       try {
@@ -2039,7 +2047,7 @@ public class ActionManager implements EventManager {
   }
 
   private void setAtomsPicked(BS bs, String msg) {
-    vwr.select(bs, false, 0, false);
+    vwr.selectStatus(bs, false, 0, false, true);
     vwr.setStatusAtomPicked(-1, msg, null, false);
   }
 
