@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2021-11-29 20:21:31 -0600 (Mon, 29 Nov 2021) $
- * $Revision: 22264 $
+ * $Date: 2022-04-21 15:54:36 -0500 (Thu, 21 Apr 2022) $
+ * $Revision: 22404 $
  *
  * Copyright (C) 2003-2005  Miguel, Jmol Development Team
  *  
@@ -37,7 +37,6 @@ import java.util.Map;
 import org.jmol.adapter.readers.spartan.SpartanUtil;
 import org.jmol.api.GenericFileInterface;
 import org.jmol.api.Interface;
-import org.jmol.api.JmolDomReaderInterface;
 import org.jmol.api.JmolFilesReaderInterface;
 import org.jmol.io.FileReader;
 import org.jmol.io.JmolUtil;
@@ -150,6 +149,16 @@ public class FileManager implements BytePoster {
     return (!orPrevious || !f.equals(JC.ZAP_TITLE) ? f : lastFullPathName != null ? lastFullPathName : lastNameAsGiven);
   }
 
+  private String lastFileType;
+
+  public String getFileType() {
+    return lastFileType;
+  }
+
+  public void setFileType(String fileType) {
+    lastFileType = fileType;
+  }
+
   public String getFileName() {
     return fileName != null ? fileName : nameAsGiven;
   }
@@ -208,7 +217,10 @@ public class FileManager implements BytePoster {
     if (htParams.get("atomDataOnly") == null)
       setLoadState(htParams);
     String name0 = name;
-    name = vwr.resolveDatabaseFormat(name);
+    int pt = name.indexOf("::");
+    if (pt < 0) {
+      name = vwr.resolveDatabaseFormat(name);
+    }
     if (!name0.equals(name) && name0.indexOf("/") < 0 
         && Viewer.hasDatabasePrefix(name0)) {
       htParams.put("dbName", name0);
@@ -218,8 +230,6 @@ public class FileManager implements BytePoster {
       htParams.put("filter", (filter == null ? "" : filter) + "2D");
       name = name.substring(0, name.length() - 4);
     }
-      
-    int pt = name.indexOf("::");
     String nameAsGiven = (pt >= 0 ? name.substring(pt + 2) : name);
     String fileType = (pt >= 0 ? name.substring(0, pt) : null);
     Logger.info("\nFileManager.getAtomSetCollectionFromFile(" + nameAsGiven
@@ -361,14 +371,6 @@ public class FileManager implements BytePoster {
     return fr;
   }
 
-  Object createAtomSetCollectionFromDOM(Object DOMNode,
-                                        Map<String, Object> htParams) {
-    JmolDomReaderInterface aDOMReader = (JmolDomReaderInterface) Interface.getOption("io.DOMReader", vwr, "file");
-    aDOMReader.set(this, vwr, DOMNode, htParams);
-    aDOMReader.run();
-    return aDOMReader.getAtomSetCollection();
-  }
-
   /**
    * not used in Jmol project -- will close reader
    * 
@@ -983,7 +985,7 @@ public class FileManager implements BytePoster {
   ///// DIRECTORY BUSINESS
   
   private static String addDirectory(String defaultDirectory, String name) {
-    if (defaultDirectory.length() == 0)
+    if (defaultDirectory.length() == 0 || defaultDirectory.equals("."))
       return name;
     char ch = (name.length() > 0 ? name.charAt(0) : ' ');
     String s = defaultDirectory.toLowerCase();
@@ -1132,6 +1134,13 @@ public class FileManager implements BytePoster {
 //    return PT.isOneOf(fname.toLowerCase().substring(fname.lastIndexOf(".")+1), ";jvxl;kin;o;msms;map;pmesh;mrc;efvet;cube;obj;dssr;bcif;");
 //  }
 //  
+  
+  /**
+   * Try to determine a surface file type based on its header. 
+   * Also identifies Menu files. 
+   * @param bufferedReader
+   * @return file type
+   */
   public static String determineSurfaceFileType(BufferedReader bufferedReader) {
     // drag-drop and isosurface command only
     // JVXL should be on the FIRST line of the file, but it may be 
@@ -1211,6 +1220,8 @@ public class FileManager implements BytePoster {
         return "Obj"; // #file: pymol.obj
       if (line.indexOf("MSMS") >= 0)
         return "Msms";
+      if (line.indexOf("Menu") >= 0)
+        return "MENU";
       break;
     case '&':
       if (line.indexOf("&plot") == 0)
@@ -1324,7 +1335,7 @@ public class FileManager implements BytePoster {
       String tag = scriptFilePrefixes[ipt];
       int i = -1;
       while ((i = script.indexOf(tag, i + 1)) >= 0) {
-        String s = stripTypePrefix(PT.getQuotedStringAt(script, i));
+        String s = PT.getQuotedStringAt(script, i);
         if (s.indexOf("\\u") >= 0)
           s = Escape.unescapeUnicode(s);
         fileList.addLast(s);
@@ -1385,8 +1396,15 @@ public class FileManager implements BytePoster {
     for (int iFile = 0; iFile < nFiles; iFile++) {
       String name0 = fileNames.get(iFile);
       String name = name0;
+      int pt = name.indexOf("::");
+      String type = "";
+      if (pt >= 0) {
+        // throwing away the type here. 
+        type = name.substring(pt + 2);
+        name = name.substring(pt + 2);
+      }
       if (isLocal == OC.isLocal(name)) {
-        int pt = (noPath ? -1 : name.indexOf("/" + dataPath + "/"));
+        pt = (noPath ? -1 : name.indexOf("/" + dataPath + "/"));
         if (pt >= 0) {
           name = name.substring(pt + 1);
         } else {
@@ -1398,6 +1416,7 @@ public class FileManager implements BytePoster {
           name = dataPath + name.substring(pt);
         }
       }
+      name = type + name;
       Logger.info("FileManager substituting " + name0 + " --> " + name);
       oldFileNames.addLast("\"" + name0 + "\"");
       newFileNames.addLast("\1\"" + name + "\"");
@@ -1633,4 +1652,9 @@ public class FileManager implements BytePoster {
     return s;
   }
 
+  public boolean isZipStream(Object br) {
+    // TODO
+    return vwr.getJzt().isZipStream(br);
+  }
+
 }
