@@ -52,7 +52,10 @@ public class GlobalSettings {
     resolverResolver = JC.databases.get("resolverresolver");
     macroDirectory = JC.defaultMacroDirectory;
 
-    if (g != null) {
+    if (g == null) {
+      setB("undoAuto", !vwr.isApplet);
+    } else {
+      setB("undoAuto", vwr.getBooleanProperty("undoAuto"));
       //persistent values not reset with the "initialize" command
       if (!clearUserVariables) {
         setO("_pngjFile", g.getParameter("_pngjFile", false));
@@ -63,6 +66,7 @@ public class GlobalSettings {
       debugScript = g.debugScript;
       disablePopupMenu = g.disablePopupMenu;
       messageStyleChime = g.messageStyleChime;
+      measureDistanceUnits = g.measureDistanceUnits;
       defaultDirectory = g.defaultDirectory;
       autoplayMovie = g.autoplayMovie;
       allowAudio = g.allowAudio;
@@ -73,6 +77,7 @@ public class GlobalSettings {
       legacyAutoBonding = g.legacyAutoBonding;
       legacyHAddition = g.legacyHAddition;
       legacyJavaFloat = g.legacyJavaFloat;
+      doublePrecision = g.doublePrecision;
       bondingVersion = g.bondingVersion;
       platformSpeed = g.platformSpeed;
       useScriptQueue = g.useScriptQueue;
@@ -88,7 +93,7 @@ public class GlobalSettings {
     }
     if (nihResolverFormat == null)
       nihResolverFormat = JC.databases.get("nci");
-    setCIR(nihResolverFormat);
+    setCIR(nihResolverFormat, false);
     // beyond these six, they are just in the form load =xxx/id
 
     for (CBK item : CBK.values())
@@ -106,6 +111,7 @@ public class GlobalSettings {
     // also initialized within the managers. 
 
     setF("cameraDepth", TransformManager.DEFAULT_CAMERA_DEPTH);
+    setI("undoMax", vwr.stm.getUndoMax()); // maintained by ScriptEval
     setI("contextDepthMax", 100); // maintained by ScriptEval
     setI("depth", 0); // maintained by TransformManager
     setF("gestureSwipeFactor", ActionManager.DEFAULT_GESTURE_SWIPE_FACTOR);
@@ -179,7 +185,7 @@ public class GlobalSettings {
     setI("propertyAtomNumberField", 0);
     setI("propertyDataColumnCount", 0);
     setI("propertyDataField", 0);
-    setB("undo", true);
+    setI("undoMax", vwr.stm.getUndoMax());
 
     // OK, all of the rest of these are maintained here as global values (below)
 
@@ -265,6 +271,7 @@ public class GlobalSettings {
     //      setParameterValue("_fileCaching", _fileCaching);
     //      setParameterValue("_fileCache", _fileCache);
     setF("exportScale", exportScale);
+    setB("doublePrecision", doublePrecision);
     setB("fontScaling", fontScaling);
     setB("fontCaching", fontCaching);
     setB("forceAutoBond", forceAutoBond);
@@ -292,6 +299,7 @@ public class GlobalSettings {
     setB("isosurfacePropertySmoothing", isosurfacePropertySmoothing);
     setI("isosurfacePropertySmoothingPower", isosurfacePropertySmoothingPower);
     setB("jmolInJSpecView", jmolInJSpecView);
+    setI("labelPointerWidth", labelPointerWidth);
     setB("justifyMeasurements", justifyMeasurements);
     setB("legacyAutoBonding", legacyAutoBonding);
     setB("legacyHAddition", legacyHAddition);
@@ -479,8 +487,13 @@ public class GlobalSettings {
   boolean smartAromatic = true;
   boolean zeroBasedXyzRasmol = false;
   boolean legacyAutoBonding = false;
-  public boolean legacyHAddition = false;
+  boolean legacyHAddition = false;
   public boolean legacyJavaFloat = false; // float/double issue with crystallographic symmetry before Jmol 14.2.5
+  /**
+   * setting doublePrecision will also toggle legacyJavaFloat to doublePrecision
+   * in order to avoid any rounding 
+   */
+  boolean doublePrecision = false; // set true for full double precision (JavaScript only) 
   boolean jmolInJSpecView = true;
 
   boolean modulateOccupancy = true;
@@ -512,7 +525,7 @@ public class GlobalSettings {
   //rendering
 
   int infoFontSize = 20;
-  boolean antialiasDisplay = false;
+  public boolean antialiasDisplay = false;
   boolean antialiasImages = true;
   boolean imageState = true;
   boolean antialiasTranslucent = true;
@@ -607,7 +620,7 @@ public class GlobalSettings {
   float defaultTranslucent = 0.5f;
   int delayMaximumMs = 0;
   float dipoleScale = 1f;
-  float drawFontSize = 14f;
+  float drawFontSize = JC.DRAW_DEFAULT_FONTSIZE;
   boolean disablePopupMenu = false;
   boolean dragSelected = false;
   boolean drawHover = false;
@@ -622,6 +635,7 @@ public class GlobalSettings {
   int helixStep = 1;
   boolean hideNameInPopup = false;
   int hoverDelayMs = 500;
+  public int labelPointerWidth = 2; // BH 2022.01.22
   float loadAtomDataTolerance = 0.01f;
   public boolean logCommands = false;
   public boolean logGestures = false;
@@ -753,7 +767,7 @@ public class GlobalSettings {
   public void setB(String name, boolean value) {
     name = name.toLowerCase();
     if (htNonbooleanParameterValues.containsKey(name))
-      return; // don't allow setting boolean of a numeric
+      return; // don't allow setting boolean of a numeric    
     htBooleanParameterFlags.put(name, value ? Boolean.TRUE : Boolean.FALSE);
   }
 
@@ -973,7 +987,7 @@ public class GlobalSettings {
       + ";scaleangstromsperinch;selectionhalos;showscript;showselections;solvent;strandcount"
       + ";spinx;spiny;spinz;spinfps;navx;navy;navz;navfps;"
       + CBK.getNameList()
-      + ";undo;atompicking;drawpicking;bondpicking;pickspinrate;picklabel"
+      + ";undo;undoauto;undomax;atompicking;drawpicking;bondpicking;pickspinrate;picklabel"
       + ";modelkitmode;autoplaymovie;allowaudio;allowgestures;allowkeystrokes;allowmultitouch;allowmodelkit"
       //  oops these were in some state scripts but should not have been
       + ";dodrop;hovered;historylevel;imagestate;iskiosk;useminimizationthread"
@@ -1076,14 +1090,15 @@ public class GlobalSettings {
     s.append("  ").append(cmd).append(";\n");
   }
 
-  public void setCIR(String template) {
+  public void setCIR(String template, boolean notify) {
     if (template == null || template.equals(nihResolverFormat) && smilesUrlFormat != null)
       return;
     int pt = template.indexOf("/structure");
     if (pt > 0) {
       nihResolverFormat = template.substring(0, pt + 10);
       smilesUrlFormat = nihResolverFormat  + "/%FILE/file?format=sdf&get3d=true";
-      System.out.println("CIR resolver set to " + nihResolverFormat);
+      if (notify)
+        Logger.info("CIR resolver set to " + nihResolverFormat + " template=" + template);
     }
 
   }
