@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2021-09-03 09:05:34 -0500 (Fri, 03 Sep 2021) $
- * $Revision: 22225 $
+ * $Date: 2022-06-03 12:24:02 +0200 (Fri, 03 Jun 2022) $
+ * $Revision: 22425 $
 
  *
  * Copyright (C) 2003-2005  Miguel, Jmol Development, www.jmol.org
@@ -284,7 +284,8 @@ public final class JC {
    * @return https protocol if necessary
    */
   static String fixProtocol(String name) {
-    String newname = (name == null ? null
+    boolean isHttp = (name != null && name.indexOf("http") >= 0);
+    String newname = (name == null || !isHttp ? name
         : name.indexOf("http://www.rcsb.org/pdb/files/") == 0
             ? resolveDataBase(name.indexOf("/ligand/") >= 0 ? "ligand" : "pdb",
                 name.substring(name.lastIndexOf("/") + 1), null)
@@ -293,12 +294,67 @@ public final class JC {
                 || name.indexOf("http://cactus") == 0
                 || name.indexOf("http://www.materialsproject") == 0
                     ? "https://" + name.substring(7)
-                    : name);
+             : name.indexOf("optimade") > 0 ? JC.fixOptimadeCall(name)
+             : name);
     if (newname != name)
       Logger.info("JC.fixProtocol " + name + " --> " + newname);
     return newname;
   }
 
+  private final static String defaultOptimadeFieldsStr = 
+      ",chemical_formula_descriptive"
+    + ",dimension_types"
+    + ",lattice_vectors"
+    + ",cartesian_site_positions"
+    + ",species_at_sites"
+    + ",species,"; 
+  
+  /**
+   * Get all necessary response fields if an optimade call. Note that the only
+   * conditions we have here is that this is http and "optimade" is in the URL.
+   * Obviously not exactly correct.
+   * 
+   * @param url
+   * @return call with added necessary response_fields
+   */
+  static String fixOptimadeCall(String url) {
+    int pt = url.indexOf("response_fields=") + 16;
+    String a = defaultOptimadeFieldsStr;
+    if (pt < 16) {
+      int ptQ = url.indexOf("?");
+      url += (ptQ < 0 ? "?" : "&") + "response_fields="
+          + a.substring(1, a.length() - 1);
+    } else {
+      String fields = "," + url.substring(pt, (url + "&").indexOf('&', pt))
+          + ",";
+      int flen = fields.length();
+      for (int i = 0, b = 0; i >= 0; i = b) {
+        b = a.indexOf(',', i + 1);
+        if (b < 0)
+          break;
+        String k = a.substring(i, b + 1);
+        if (fields.indexOf(k) < 0) {
+          fields += k.substring(1);
+        }
+      }
+      url = url.substring(0, pt) + fields.substring(1, fields.length() - 1)
+          + url.substring(pt + flen - 2);
+    }
+    return url;
+  }
+
+//static {
+//  
+//  System.out.println(fixOptimadeCall("test?filter&response_fields=lkadfs"));
+//  System.out.println(fixOptimadeCall("test?response_fields=species"));
+//  System.out.println(fixOptimadeCall("test?response_fields=lkadfs,species"));
+//  System.out.println(fixOptimadeCall("test?response_fields=lkadfs,species_at_sites,species&more"));
+//  System.out.println(fixOptimadeCall("test?filter&response_fields=species,lkadfs&more"));
+//  System.out.println(fixOptimadeCall("test?response_fields=species,lkadfs&more"));
+//  
+//  
+//}
+
   public static String[] macros = { "aflow",
       "https://chemapps.stolaf.edu/jmol/macros/AFLOW.spt", "AFLOW macros", "bz",
       "https://chemapps.stolaf.edu/jmol/macros/bz.spt",
@@ -389,7 +445,7 @@ public final class JC {
     if (tmpDate != null) {
       tmpDate = tmpDate.substring(7, 23);
       // NOTE : date is updated in the properties by SVN, and is in the format
-      // "$Date: 2021-09-03 09:05:34 -0500 (Fri, 03 Sep 2021) $"
+      // "$Date: 2022-06-03 12:24:02 +0200 (Fri, 03 Jun 2022) $"
       //  0         1         2
       //  012345678901234567890123456789
     }
@@ -454,8 +510,14 @@ public final class JC {
 
   public final static String NOTE_SCRIPT_FILE = "NOTE: file recognized as a script file: ";
 
-  public final static String SCRIPT_EDITOR_IGNORE = "\1## EDITOR_IGNORE ##";
-  public final static String REPAINT_IGNORE = "\1## REPAINT_IGNORE ##";
+  public static final String SCRIPT_EXT = "\1##";
+  public static final String SCRIPT_STEP = SCRIPT_EXT + "SCRIPT_STEP";
+  public static final String SCRIPT_START = SCRIPT_EXT + "SCRIPT_START";
+  public static final String SCRIPT_NOENDCHECK = SCRIPT_EXT + "NOENDCHECK";
+  public final static String SCRIPT_ISEDITOR = SCRIPT_EXT + " ISEDITOR";
+  public final static String SCRIPT_EDITOR_IGNORE = SCRIPT_EXT + " EDITOR_IGNORE ##";
+  public final static String REPAINT_IGNORE = SCRIPT_EXT + " REPAINT_IGNORE ##";
+  public final static String SCRIPT_GUI = "; ## GUI ##";
 
   public final static String LOAD_ATOM_DATA_TYPES = ";xyz;vxyz;vibration;temperature;occupancy;partialcharge;";
 
@@ -876,6 +938,7 @@ public final class JC {
 
   public final static int MEASURE_DEFAULT_FONTSIZE = 18;
   public final static int AXES_DEFAULT_FONTSIZE = 16;
+  public static final float DRAW_DEFAULT_FONTSIZE = 16;
 
   ////////////////////////////////////////////////////////////////
   // do not rearrange/modify these shapes without
@@ -1165,7 +1228,7 @@ public final class JC {
   public final static int LABEL_ZPOS_GROUP = 0x010;
   public final static int LABEL_ZPOS_FRONT = 0x020;
 
-  private final static int LABEL_EXPLICIT = 0x040;
+  public final static int LABEL_EXPLICIT             = 0x040;
 
   private final static int LABEL_CENTERED = 0x100;
 
@@ -1342,6 +1405,7 @@ public final class JC {
   public static final int SMILES_GEN_TOPOLOGY = 0x00004000; // SmilesExt -> generator
   public static final int SMILES_GEN_POLYHEDRAL = 0x00010000; // polyhedron -> generator
   public static final int SMILES_GEN_ATOM_COMMENT = 0x00020000; // polyhedron,Viewer -> generator
+  public static final int SMILES_GEN_NO_BRANCHES               = 0x00040000; // MathExt -> Generator
 
   public static final int SMILES_GEN_BIO = 0x00100000; // MathExt -> generator
   public static final int SMILES_GEN_BIO_ALLOW_UNMATCHED_RINGS = 0x00300000; // MathExt -> generator
@@ -1386,6 +1450,7 @@ public final class JC {
   public final static int UNITID_TRIM = 16;
 
   public static final String DEFAULT_DRAG_DROP_SCRIPT = "zap; load SYNC \"%FILE\";if (%ALLOWCARTOONS && _loadScript == '' && defaultLoadScript == '' && _filetype == 'Pdb') {if ({(protein or nucleic)&*/1.1} && {*/1.1}[1].groupindex != {*/1.1}[0].groupindex){select protein or nucleic;cartoons only;}if ({visible && cartoons > 0}){color structure}else{wireframe -0.1};if (!{visible}){spacefill 23%};select *}";
+  public static final float UC_TOLERANCE2 =  0.0014f*0.0014f; // always use check for LT this, not LTE
 
   /**
    * Get a unitID type
@@ -1409,4 +1474,33 @@ public final class JC {
     return i;
   }
 
+  public final static String[] globalBooleans = {
+  "someModelsHaveFractionalCoordinates", 
+  "someModelsHaveSymmetry",
+  "someModelsHaveUnitcells", 
+  "someModelsHaveCONECT", 
+  "isPDB", 
+  "someModelsHaveDomains", 
+  "someModelsHaveValidations", 
+  "isSupercell",
+  "someModelsHaveAromaticBonds",
+  "someModelsAreModulated"
+  };
+  
+  public final static int GLOBAL_FRACTCOORD = 0;
+  public final static int GLOBAL_SYMMETRY = 1;
+  public final static int GLOBAL_UNITCELLS = 2;
+  public final static int GLOBAL_CONECT = 3;
+  public final static int GLOBAL_ISPDB = 4;
+  public final static int GLOBAL_DOMAINS = 5;
+  public final static int GLOBAL_VALIDATIONS = 6;
+  public final static int GLOBAL_SUPERCELL = 7;
+  public final static int GLOBAL_AROMATICBONDS = 8;
+  public static final int GLOBAL_MODULATED = 9;
+
+  public static String getBoolName(int g) {
+    return globalBooleans[g];
+  }
+  
+
 }
