@@ -572,7 +572,7 @@ abstract class OutputManager {
    * @param fileName
    * @param type
    *        one of: PDB PQR FILE PLOT
-   * @param modelIndex
+   * @param modelIndex for PLOT or PDB or PQR only
    * @param plotParameters
    * @return "OK..." or "" or null
    * 
@@ -585,14 +585,13 @@ abstract class OutputManager {
     if (out == null)
       return "";
     fileName = fullPath[0];
-    String pathName = (type.equals("FILE") ? vwr.fm.getFullPathName(false)
+    String pathName = (type.equals("FILE") ? (String) vwr.getParameter("_modelFile")
         : null);
-    boolean getCurrentFile = (pathName != null && (pathName.equals("string")
+    boolean getStringData = (pathName != null && (pathName.equals("string")
         || pathName.equals("String[]") || pathName.equals("JSNode")));
-    boolean asBytes = (pathName != null && !getCurrentFile);
+    boolean asBytes = (pathName != null && !getStringData);
     if (asBytes) {
-      pathName = vwr.getModelSetPathName();
-      if (pathName == null)
+      if (vwr.getModelSetPathName() == null)
         return null; // zapped
     }
     // The OutputStringBuilder allows us to create strings or byte arrays
@@ -605,7 +604,7 @@ abstract class OutputManager {
         : type.startsWith("PLOT")
             ? vwr.getPdbData(modelIndex, type.substring(5), null,
                 plotParameters, out, true)
-            : getCurrentFile
+            : getStringData
                 ? out.append(vwr.getCurrentFileAsString("write")).toString()
                 : (String) vwr.fm.getFileAsBytes(pathName, out));
     out.closeChannel();
@@ -961,6 +960,13 @@ abstract class OutputManager {
     Lst<String> newFileNames = new Lst<String>();
     for (int iFile = 0; iFile < nFiles; iFile++) {
       String name = fileNamesUTF.get(iFile);
+      int pt = name.indexOf("::");
+      String type = "";
+      if (pt >= 0) {
+        type = name.substring(0, pt + 2);
+        name = name.substring(pt + 2);
+      }
+        
       boolean isLocal = OC.isLocal(name);
       String newName = name;
       // also check that somehow we don't have a local file with the same name as
@@ -989,7 +995,7 @@ abstract class OutputManager {
           newName = addPngFileBytes(name, (byte[]) ret, iFile, crcMap,
               isSparDir, newName, ptSlash, v);
         }
-        name = "$SCRIPT_PATH$" + newName;
+        name = type + "$SCRIPT_PATH$" + newName;
       }
       crcMap.put(newName, newName);
       newFileNames.addLast(PT.escUnicode(name));
