@@ -96,10 +96,16 @@ import javajs.util.SB;
  */
 public class StateCreator extends JmolStateCreator {
 
+
+  private boolean undoWorking = false;
+  private final static int MAX_ACTION_UNDO = 100;
+
+  final Lst<String> actionStates;
+  final Lst<String> actionStatesRedo;
+  
   public StateCreator() {
-
-    // by reflection only!
-
+    actionStates = new Lst<String>();
+    actionStatesRedo = new Lst<String>();
   }
 
   private Viewer vwr;
@@ -107,8 +113,12 @@ public class StateCreator extends JmolStateCreator {
   @Override
   void setViewer(Viewer vwr) {
     this.vwr = vwr;
+
   }
 
+  // by reflection only!
+
+
 
   /////////////////// creating the state script ////////////////////
 
@@ -358,7 +368,7 @@ public class StateCreator extends JmolStateCreator {
           if (symmetry == null)
             continue;
           sb.setLength(0);
-          if (symmetry.getState(sb)) {
+          if (symmetry.getState(ms, i, sb)) {
             loadUC = true;
             commands.append("  frame ").append(ms.getModelNumberDotted(i))
                 .appendSB(sb).append(";\n");
@@ -498,6 +508,7 @@ public class StateCreator extends JmolStateCreator {
             i > 0, null, null);
       } else {
         commands.appendSB(m.loadScript);
+        @SuppressWarnings("unchecked")
         Lst<String> auxFiles = (Lst<String>) m.auxiliaryInfo.get("auxFiles");
         if (auxFiles != null) {
           for (int j = 0; j < auxFiles.size(); j++)
@@ -771,7 +782,7 @@ public class StateCreator extends JmolStateCreator {
       app(commands, "set navigationMode true");
     app(commands, vwr.ms.getBoundBoxCommand(false));
     app(commands, "center " + Escape.eP(tm.fixedRotationCenter));
-    commands.append(vwr.getOrientationText(T.name, null, null).toString());
+    commands.append(vwr.getOrientation(T.name, null, null).toString());
 
     app(commands, moveToText);
 // stereo mode should not be in the state - just a display option
@@ -954,7 +965,7 @@ public class StateCreator extends JmolStateCreator {
 
     Lst<Measurement> mList = shape.measurements;
     int measurementCount = shape.measurementCount;
-    Font font3d = Measures.font3d;
+    Font font3d = shape.font3d;
     TickInfo ti = shape.defaultTickInfo;
     SB commands = new SB();
     app(commands, "measures delete");
@@ -1120,9 +1131,9 @@ public class StateCreator extends JmolStateCreator {
       String st = s = getFontLineShapeState((FontLineShape) shape);
       int iAtom = vwr.am.getUnitCellAtomIndex();
       if (iAtom >= 0)
-        s += "  unitcell ({" + iAtom + "});\n"; 
+        s += "  unitcell ({" + iAtom + "});\n";
       SymmetryInterface uc = vwr.getCurrentUnitCell();
-      if (uc != null) { 
+      if (uc != null) {
         s += uc.getUnitCellState();
         s += st; // needs to be after this state as well.
       }
@@ -1146,37 +1157,40 @@ public class StateCreator extends JmolStateCreator {
     case JC.SHAPE_ECHO:
       Echo es = (Echo) shape;
       SB sb = new SB();
-      sb.append("\n  set echo off;\n");
+      sb.append("\n  set echo off;set echo \"%SCALE\" off;\n");
+      getEchoState(sb, es.scaleObject);
       for (Text t : es.objects.values()) {
-        sb.append(getTextState(t));
-        if (t.hidden)
-          sb.append("  set echo ID ").append(PT.esc(t.target))
-              .append(" hidden;\n");
+        getEchoState(sb, t);
       }
       s = sb.toString();
       break;
     case JC.SHAPE_HALOS:
       Halos hs = (Halos) shape;
-      s = getAtomShapeState(hs)
-          + (hs.colixSelection == C.USE_PALETTE ? ""
-              : hs.colixSelection == C.INHERIT_ALL ? "  color SelectionHalos NONE;\n"
-                  : Shape.getColorCommandUnk("selectionHalos",
-                      hs.colixSelection, hs.translucentAllowed) + ";\n");
+      s = getAtomShapeState(hs) + (hs.colixSelection == C.USE_PALETTE ? ""
+          : hs.colixSelection == C.INHERIT_ALL
+              ? "  color SelectionHalos NONE;\n"
+              : Shape.getColorCommandUnk("selectionHalos", hs.colixSelection,
+                  hs.translucentAllowed) + ";\n");
       if (hs.bsHighlight != null)
-        s += "  set highlight "
-            + Escape.eBS(hs.bsHighlight)
-            + "; "
+        s += "  set highlight " + Escape.eBS(hs.bsHighlight) + "; "
             + Shape.getColorCommandUnk("highlight", hs.colixHighlight,
-                hs.translucentAllowed) + ";\n";
+                hs.translucentAllowed)
+            + ";\n";
       break;
     case JC.SHAPE_HOVER:
       clearTemp();
       Hover h = (Hover) shape;
-      if (h.atomFormats != null)
-        for (int i = vwr.ms.ac; --i >= 0;)
-          if (h.atomFormats[i] != null)
+      if (h.atomFormats != null) {
+        Atom[] at = vwr.ms.at;
+        for (int i = vwr.ms.ac; --i >= 0;) {
+          if (at[i] == null)
+            h.atomFormats[i] = null;
+          if (h.atomFormats[i] != null) {
             BSUtil.setMapBitSet(temp, i, i,
                 "set hoverLabel " + PT.esc(h.atomFormats[i]));
+          }
+        }
+      }
       s = "\n  hover " + PT.esc((h.labelFormat == null ? "" : h.labelFormat))
           + ";\n" + getCommands(temp, null, "select");
       clearTemp();
@@ -1207,17 +1221,15 @@ public class StateCreator extends JmolStateCreator {
         Text text = l.getLabel(i);
         float sppm = (text != null ? text.scalePixelsPerMicron : 0);
         if (sppm > 0)
-          BSUtil.setMapBitSet(temp2, i, i, "set labelScaleReference "
-              + (10000f / sppm));
+          BSUtil.setMapBitSet(temp2, i, i,
+              "set labelScaleReference " + (10000f / sppm));
         if (l.offsets != null && l.offsets.length > i) {
           int offsetFull = l.offsets[i];
-          BSUtil.setMapBitSet(
-              temp2,
-              i,
-              i,
+          BSUtil.setMapBitSet(temp2, i, i,
               "set "
                   + (JC.isOffsetAbsolute(offsetFull) ? "labelOffsetAbsolute "
-                      : "labelOffset ") + JC.getXOffset(offsetFull) + " "
+                      : "labelOffset ")
+                  + JC.getXOffset(offsetFull) + " "
                   + JC.getYOffset(offsetFull));
           String align = JC.getHorizAlignmentName(offsetFull >> 2);
           String pointer = JC.getPointerName(offsetFull);
@@ -1253,7 +1265,10 @@ public class StateCreator extends JmolStateCreator {
       byte[] pids = balls.paletteIDs;
       float r = 0;
       for (int i = 0; i < ac; i++) {
-        if (atoms[i] != null && shape.bsSizeSet != null && shape.bsSizeSet.get(i)) {
+        if (atoms[i] == null)
+          continue;
+        if (shape.bsSizeSet != null
+            && shape.bsSizeSet.get(i)) {
           if ((r = atoms[i].madAtom) < 0)
             BSUtil.setMapBitSet(temp, i, i, "Spacefill on");
           else
@@ -1297,28 +1312,7 @@ public class StateCreator extends JmolStateCreator {
   private String getAxesState(Axes axes) {
     SB sb = new SB();
     sb.append(getFontLineShapeState(axes));
-    sb.append("  axes scale ").appendF(vwr.getFloat(T.axesscale)).append(";\n"); 
-    if (axes.fixedOrigin != null)
-      sb.append("  axes center ")
-          .append(Escape.eP(axes.fixedOrigin)).append(";\n");
-    P3 axisXY = axes.axisXY;
-    if (axisXY.z != 0)
-      sb.append("  axes position [")
-          .appendI((int) axisXY.x).append(" ")
-          .appendI((int) axisXY.y).append(" ")
-          .append(axisXY.z < 0 ? " %" : "").append("];\n");
-    String[] labels = axes.labels;
-    if (labels != null) {
-      sb.append("  axes labels ");
-      for (int i = 0; i < labels.length; i++)
-        if (labels[i] != null)
-          sb.append(PT.esc(labels[i])).append(" ");
-      sb.append(";\n");
-    }
-    if (axes.axisType != null) {
-      sb.append("  axes type " + PT.esc(axes.axisType));
-    }
-    return sb.toString();
+    return axes.getAxesState(sb);
   }
 
 
@@ -1346,13 +1340,14 @@ public class StateCreator extends JmolStateCreator {
     return s;
   }
 
-  private String getTextState(Text t) {
-    SB s = new SB();
-    String text = t.text;
+  private void getEchoState(SB sb, Text t) {
+    // ECHO "%SCALE" uses unformatted text
+    boolean isScale = (t != null && t.barPixels > 0);
+    String text = (t == null ? null : isScale ? t.textUnformatted : t.text); 
     if (text == null || !t.isEcho || t.target.equals("error"))
-      return "";
+      return;
     //set echo top left
-    //set echo myecho x y
+    //set echo ID myecho x y
     //echo .....
     boolean isImage = (t.image != null);
     //    if (isDefine) {
@@ -1373,42 +1368,45 @@ public class StateCreator extends JmolStateCreator {
     case JC.ECHO_XYZ:
       if (strOff == null)
         strOff = Escape.eP(t.xyz);
-      s.append("  ").append(echoCmd).append(" ").append(strOff);
+      sb.append("  ").append(echoCmd).append(" ").append(strOff);
       if (t.align != JC.TEXT_ALIGN_LEFT)
-        s.append(";  ").append(echoCmd).append(" ").append(
+        sb.append(";  ").append(echoCmd).append(" ").append(
             JC.getHorizAlignmentName(t.align));
       break;
     default:
-      s.append("  set echo ").append(JC.getEchoName(t.valign)).append(" ")
+      sb.append("  set echo ").append(JC.getEchoName(t.valign)).append(" ")
           .append(JC.getHorizAlignmentName(t.align));
     }
     if (t.movableZPercent != Integer.MAX_VALUE)
-      s.append(";  ").append(echoCmd).append(" depth ").appendI(
+      sb.append(";  ").append(echoCmd).append(" depth ").appendI(
           t.movableZPercent);
     if (isImage)
-      s.append("; ").append(echoCmd).append(" IMAGE /*file*/");
+      sb.append("; ").append(echoCmd).append(" IMAGE /*file*/");
     else
-      s.append("; echo ");
-    s.append(PT.esc(text)); // was textUnformatted, but that is not really the STATE
-    s.append(";\n");
+      sb.append("; echo ");
+    sb.append(PT.esc(text)); // was textUnformatted, but that is not really the STATE
+    sb.append(";\n");
     if (isImage && t.imageScale != 1)
-      s.append("  ").append(echoCmd).append(" scale ").appendF(t.imageScale)
+      sb.append("  ").append(echoCmd).append(" scale ").appendF(t.imageScale)
           .append(";\n");
     if (t.script != null)
-      s.append("  ").append(echoCmd).append(" script ").append(
+      sb.append("  ").append(echoCmd).append(" script ").append(
           PT.esc(t.script)).append(";\n");
     if (t.modelIndex >= 0)
-      s.append("  ").append(echoCmd).append(" model ").append(
+      sb.append("  ").append(echoCmd).append(" model ").append(
           vwr.getModelNumberDotted(t.modelIndex)).append(";\n");
     if (t.pointerPt != null) {
-      s.append("  ").append(echoCmd).append(" point ").append(
+      sb.append("  ").append(echoCmd).append(" point ").append(
           t.pointerPt instanceof Atom ? "({" + ((Atom) t.pointerPt).i
               + "})" : Escape.eP(t.pointerPt)).append(";\n");
     }
     if (t.pymolOffset != null) {
-      s.append("  ").append(echoCmd).append(" offset ").append(
+      sb.append("  ").append(echoCmd).append(" offset ").append(
           Escape.escapeFloatA(t.pymolOffset, true)).append(";\n");
     }
+    if ("%SCALE".equals(text)) {
+      sb.append("  set measurementUnits " + vwr.getDistanceUnits("")).append(";\n");
+    }
     //    }
     //isDefine and target==top: do all
     //isDefine and target!=top: just start
@@ -1419,19 +1417,22 @@ public class StateCreator extends JmolStateCreator {
     //    if (isDefine != target.equals("top"))
     //      return s.toString();
     // these may not change much:
-    t.appendFontCmd(s);
-    s.append("; color echo");
+    t.appendFontCmd(sb);
+    sb.append("; color echo");
     if (C.isColixTranslucent(t.colix))
-      s.append(C.getColixTranslucencyLabel(t.colix));
-    s.append(" ").append(C.getHexCode(t.colix));
+      sb.append(" ").append(C.getColixTranslucencyLabel(t.colix));
+    sb.append(" ").append(C.getHexCode(t.colix));
     if (t.bgcolix != 0) {
-      s.append("; color echo background ");
+      sb.append("; color echo background ");
       if (C.isColixTranslucent(t.bgcolix))
-        s.append(C.getColixTranslucencyLabel(t.bgcolix)).append(" ");
-      s.append(C.getHexCode(t.bgcolix));
+        sb.append(C.getColixTranslucencyLabel(t.bgcolix)).append(" ");
+      sb.append(C.getHexCode(t.bgcolix));
     }
-    s.append(";\n");
-    return s.toString();
+    sb.append(";\n");
+    if (t.hidden)
+      sb.append("  set echo ID ").append(PT.esc(t.target))
+          .append(" hidden;\n");
+    return;
   }
 
   @Override
@@ -1493,6 +1494,10 @@ public class StateCreator extends JmolStateCreator {
     return sb.toString();
   }
 
+  /**
+   * A relatively static method that is only called on a new StateCreator
+   * so as to all StateCreator to be modular.
+   */
   @Override
   String getFunctionCalls(String f) {
     if (f == null)
@@ -1500,7 +1505,7 @@ public class StateCreator extends JmolStateCreator {
     SB s = new SB();
     int pt = f.indexOf("*");
     boolean isGeneric = (pt >= 0);
-    boolean isStatic = (f.indexOf("static_") == 0);
+    boolean isStatic = Viewer.isStaticFunction(f);
     boolean namesOnly = (f.equalsIgnoreCase("names") || f
         .equalsIgnoreCase("static_names"));
     if (namesOnly)
@@ -1539,8 +1544,6 @@ public class StateCreator extends JmolStateCreator {
 
   @Override
   String getAtomicPropertyState(int taintWhat, BS bsSelected) {
-    if (!vwr.g.preserveState)
-      return "";
     BS bs;
     SB commands = new SB();
     for (int type = 0; type < AtomCollection.TAINT_MAX; type++)
@@ -1567,7 +1570,7 @@ public class StateCreator extends JmolStateCreator {
     BS[] tainted = vwr.ms.tainted;
     if (bs != null)
       for (int i = bs.nextSetBit(0); i >= 0; i = bs.nextSetBit(i + 1)) {
-        if (atoms[i] == null || atoms[i].isDeleted())
+        if (AtomCollection.isDeleted(atoms[i]))
           continue;
         s.appendI(i + 1).append(" ").append(atoms[i].getElementSymbol())
             .append(" ").append(atoms[i].getInfo().replace(' ', '_')).append(
@@ -1611,6 +1614,9 @@ public class StateCreator extends JmolStateCreator {
           else
             s.appendF(PT.FLOAT_MIN_SAFE).append(" ").appendF(PT.FLOAT_MIN_SAFE).append(" ").appendF(v.modScale);
           break;
+        case AtomCollection.TAINT_SITE:
+          s.appendI(atoms[i].getAtomSite());
+          break;
         case AtomCollection.TAINT_ELEMENT:
           s.appendI(atoms[i].getAtomicAndIsotopeNumber());
           break;
@@ -1655,16 +1661,19 @@ public class StateCreator extends JmolStateCreator {
   
   
   @Override
-  void undoMoveAction(int action, int n) {
+  int undoMoveAction(int action, int n) {
     switch (action) {
     case T.undomove:
     case T.redomove:
       switch (n) {
+      case T.count:
+        return (action == T.undomove ? actionStates : actionStatesRedo).size();
       case -2:
-        vwr.undoClear();
+        actionStates.clear();
+        actionStatesRedo.clear();
         break;
       case -1:
-        (action == T.undomove ? vwr.actionStates : vwr.actionStatesRedo)
+        (action == T.undomove ? actionStates : actionStatesRedo)
             .clear();
         break;
       case 0:
@@ -1672,13 +1681,14 @@ public class StateCreator extends JmolStateCreator {
         //$FALL-THROUGH$
       default:
         if (n > MAX_ACTION_UNDO)
-          n = (action == T.undomove ? vwr.actionStates
-              : vwr.actionStatesRedo).size();
+          n = (action == T.undomove ? actionStates
+              : actionStatesRedo).size();
         for (int i = 0; i < n; i++)
           undoMoveActionClear(0, action, true);
       }
       break;
     }
+    return 0;
   }
 
   @Override
@@ -1705,13 +1715,13 @@ public class StateCreator extends JmolStateCreator {
       switch (type) {
       default:
       case T.undomove:
-        list1 = vwr.actionStates;
-        list2 = vwr.actionStatesRedo;
+        list1 = actionStates;
+        list2 = actionStatesRedo;
         break;
       case T.redomove:
-        list1 = vwr.actionStatesRedo;
-        list2 = vwr.actionStates;
-        if (vwr.actionStatesRedo.size() == 1)
+        list1 = actionStatesRedo;
+        list2 = actionStates;
+        if (actionStatesRedo.size() == 1)
           return;
         break;
       }
@@ -1719,7 +1729,7 @@ public class StateCreator extends JmolStateCreator {
         return;
       undoWorking = true;
       list2.add(0, list1.removeItemAt(0));
-      s = vwr.actionStatesRedo.get(0);
+      s = actionStatesRedo.get(0);
       if (type == T.undomove && list2.size() == 1) {
         // must save current state, coord, etc.
         // but this destroys actionStatesRedo
@@ -1738,7 +1748,7 @@ public class StateCreator extends JmolStateCreator {
       } else {
         // if it's not modelkit mode and we are trying to do a zap, then ignore
         // and clear all action states.
-        vwr.actionStates.clear();
+        actionStates.clear();
       }
       break;
     default:
@@ -1765,20 +1775,16 @@ public class StateCreator extends JmolStateCreator {
 
       }
       if (clearRedo) {
-        vwr.actionStates.add(0, sb.toString());
-        vwr.actionStatesRedo.clear();
+        actionStates.add(0, sb.toString());
+        actionStatesRedo.clear();
       } else {
-        vwr.actionStatesRedo.add(1, sb.toString());
+        actionStatesRedo.add(1, sb.toString());
       }
-      if (vwr.actionStates.size() == MAX_ACTION_UNDO) {
-        vwr.actionStates.removeItemAt(MAX_ACTION_UNDO - 1);
+      if (actionStates.size() == MAX_ACTION_UNDO) {
+        actionStates.removeItemAt(MAX_ACTION_UNDO - 1);
       }
     }
     undoWorking = !clearRedo;
   }
 
-  private boolean undoWorking = false;
-  private final static int MAX_ACTION_UNDO = 100;
-  
-
 }
