@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2021-12-04 23:29:43 -0600 (Sat, 04 Dec 2021) $
- * $Revision: 22271 $
+ * $Date: 2022-07-08 20:20:16 +0530 (Fri, 08 Jul 2022) $
+ * $Revision: 22429 $
  *
  * Copyright (C) 2003-2005  The Jmol Development Team
  *
@@ -41,7 +41,9 @@ import org.jmol.api.JmolStatusListener;
 import org.jmol.c.CBK;
 import javajs.util.BS;
 import org.jmol.script.SV;
+import org.jmol.script.ScriptException;
 import org.jmol.script.T;
+import org.jmol.util.BSUtil;
 import org.jmol.util.JmolAudio;
 import org.jmol.util.Logger;
 
@@ -243,21 +245,18 @@ public class StatusManager {
     return list;
   }
 
-  private Map<String, String> jmolScriptCallbacks = new Hashtable<String, String>();
-
-  private String jmolScriptCallback(CBK callback) {
-    String s = jmolScriptCallbacks.get(callback.name());
-    if (s != null)
-      vwr.evalStringQuietSync(s, true, false);
-    if (jmolScriptCallbacks.containsKey("SYNC:" + callback.name())) 
-      s = "SYNC";
-    return s;
-  }
-  
-  synchronized void setCallbackFunction(String callbackType,
-                                        String callbackFunction) {
+  /**
+   * Set a JmolScript or JavaScript or Java callback. 
+   * This method can be called with a JavaScript function, but only 
+   * directly by an applet method, not using Jmol scripting.
+   * @param callbackType
+   * @param callbackObject
+   */
+  synchronized public void setCallbackFunction(String callbackType,
+                                        Object callbackObject) {
     // menu and language setting also use this route
     CBK cbk = CBK.getCallback(callbackType);
+    String callbackFunction = (callbackObject instanceof String ? (String) callbackObject : null);
     if (cbk != null) {
       String callback = CBK.getCallback(callbackType).name();
       Logger.info("StatusManager callback set for " + callbackType + " f="
@@ -275,40 +274,118 @@ public class StatusManager {
         }
       } else {
         String lc = "";
+        // allowing for "script:" or "jmolscript:" here. 
         int pt = (callbackFunction == null ? 0
             : (lc = callbackFunction.toLowerCase()).startsWith("script:") ? 7
                 : lc.startsWith("jmolscript:") ? 11 : 0);
         if (pt == 0) {
-          jmolScriptCallbacks.remove(callback);
+          // could be a function object
+          // note that setting a function or clearing it 
+          // also clears any JmolScript callback
+          if (callbackObject == null)
+            jmolScriptCallbacks.remove(callback);
         } else {
+          // but setting or clearing a JmolScript callback does not have
+          // an impact on JavaScript callbacks. 
           jmolScriptCallbacks.put(callback,
               callbackFunction.substring(pt).trim());
+          return;
         }
       }
     }
-    // allow for specialized callbacks
+    // allow for specialized internal callbacks that may not be part of the CBK system.
+    // note that in JavaScript, the callabackObject may be a function() object.
     if (cbl != null)
-      cbl.setCallbackFunction(callbackType, callbackFunction);
+      cbl.setCallbackFunction(callbackType, (String) callbackObject);
   }
   
   boolean notifyEnabled(CBK type) {
     return cbl != null && cbl.notifyEnabled(type);
   }
 
-  synchronized void setStatusAppletReady(String htmlName, boolean isReady) {
-    String sJmol = (isReady ? jmolScriptCallback(CBK.APPLETREADY) : null);
-    if (notifyEnabled(CBK.APPLETREADY))
-      cbl.notifyCallback(CBK.APPLETREADY,
-          new Object[] { sJmol, htmlName, Boolean.valueOf(isReady), null });
+  private Map<String, String> jmolScriptCallbacks = new Hashtable<String, String>();
+
+  private String getJmolScriptCallback(CBK callback) {
+    return jmolScriptCallbacks.get(callback.name());
   }
 
-  synchronized void setStatusAtomMoved(BS bsMoved) {
-    String sJmol = jmolScriptCallback(CBK.ATOMMOVED);
-    setStatusChanged("atomMoved", -1, bsMoved, false);
-    if (notifyEnabled(CBK.ATOMMOVED))
-      cbl.notifyCallback(CBK.ATOMMOVED,
-          new Object[] { sJmol, bsMoved });
+  private Object[] _args = null;
+
+  public Object getParameter(int i) {
+    if (_args == null || i == Integer.MIN_VALUE)
+      return _args;
+// NO - don't allow (0) to be "last argument" in order to allow for future expansions
+//    if (i < 0)
+//      i += _args.length;
+    return (--i < 0 || i >= _args.length ? null : _args[i]);
   }
+  
+  private void fireJmolScriptCallback(boolean isEnabled, CBK callback,
+                                      Object[] o) {
+    if (o[0] != null) {
+      String cmd = (String) o[0];
+      Object[] args = _args;
+      _args = o;
+      o[0] = callback.name();
+      vwr.evalStringQuietSync(cmd, true, false);
+      o[0] = cmd;
+      _args = args;
+    }
+    if (jmolScriptCallbacks.containsKey("SYNC:" + callback.name()))
+      o[0] = "SYNC";
+    if (isEnabled) {
+      cbl.notifyCallback(callback, o);
+    }
+  }
+  
+  synchronized void setStatusAppletReady(String htmlName, boolean isReady) {
+    String sJmol = (isReady ? getJmolScriptCallback(CBK.APPLETREADY) : null);
+    boolean isEnabled =  notifyEnabled(CBK.APPLETREADY);
+    if (isEnabled || sJmol != null)
+      fireJmolScriptCallback(isEnabled, CBK.APPLETREADY, new Object[] { sJmol, htmlName, Boolean.valueOf(isReady), null });
+  }
+
+  synchronized void setStatusAtomMoved(BS bsMoved) {    
+    String sJmol = getJmolScriptCallback(CBK.ATOMMOVED);
+    setStatusChanged("atomMoved", -1, bsMoved, false);
+    boolean isEnabled =  notifyEnabled(CBK.ATOMMOVED);
+    if (isEnabled || sJmol != null)
+      fireJmolScriptCallback(isEnabled, CBK.ATOMMOVED,
+          new Object[] { sJmol, bsMoved, Integer.valueOf(bsMoved.cardinality()) });   
+  }
+
+  /**
+   * Callback for selection change. Data will include
+   * 
+   * [callbackFuncName, atoms, numberSelected, firstIndex, lastIndex + 1]
+   * 
+   * 
+   * @param atoms 
+   * 
+   */
+  synchronized void setStatusSelect(BS atoms) {
+    String sJmol = getJmolScriptCallback(CBK.SELECT);
+    setStatusChanged("select", -1, atoms, false);
+    boolean isEnabled =  notifyEnabled(CBK.SELECT);
+    if (isEnabled || sJmol != null)
+      fireJmolScriptCallback(isEnabled, CBK.SELECT,
+          new Object[] { sJmol, atoms, Integer.valueOf(atoms.cardinality()), 
+              Integer.valueOf(atoms.nextSetBit(0)), Integer.valueOf(atoms.length()) });
+  }
+
+  public void setStatusStructureModified(int atomIndex, int modelIndex,
+                                         int mode, String msg, int n, BS bsAtoms) {
+    if (atomIndex >= 0 && bsAtoms == null)
+      bsAtoms = BSUtil.newAndSetBit(atomIndex);
+    String sJmol = getJmolScriptCallback(CBK.STRUCTUREMODIFIED);
+    boolean isEnabled =  notifyEnabled(CBK.STRUCTUREMODIFIED);
+    if (isEnabled || sJmol != null)
+      fireJmolScriptCallback(isEnabled, CBK.STRUCTUREMODIFIED,
+          new Object[] { sJmol, Integer.valueOf(mode),
+              Integer.valueOf(atomIndex), Integer.valueOf(modelIndex), msg,
+              Integer.valueOf(n), bsAtoms });
+  }
+  
 
   /**
    * 
@@ -317,48 +394,51 @@ public class StatusManager {
    * @param map 
    */
   synchronized void setStatusAtomPicked(int atomIndex, String strInfo, Map<String, Object> map) {
-    String sJmol = jmolScriptCallback(CBK.PICK);
+    String sJmol = getJmolScriptCallback(CBK.PICK);
     Logger.info("setStatusAtomPicked(" + atomIndex + "," + strInfo + ")");
     setStatusChanged("atomPicked", atomIndex, strInfo, false);
-    if (notifyEnabled(CBK.PICK))
-      cbl.notifyCallback(CBK.PICK,
+    boolean isEnabled =  notifyEnabled(CBK.PICK);
+    if (isEnabled || sJmol != null)
+      fireJmolScriptCallback(isEnabled, CBK.PICK,
           new Object[] { sJmol, strInfo, Integer.valueOf(atomIndex), map });
   }
 
   synchronized int setStatusClicked(int x, int y, int action, int clickCount, int mode) {
     // also called on drag movements
-    String sJmol = jmolScriptCallback(CBK.CLICK);
-    if (!notifyEnabled(CBK.CLICK))
-      return action;
-    // allows modification of action
+    String sJmol = getJmolScriptCallback(CBK.CLICK);
     int[] m = new int[] { action, mode };
-    cbl.notifyCallback(CBK.CLICK,
+    boolean isEnabled =  notifyEnabled(CBK.CLICK);
+    if (isEnabled || sJmol != null)
+      fireJmolScriptCallback(isEnabled, CBK.CLICK,
         new Object[] { sJmol, Integer.valueOf(x), Integer.valueOf(y), Integer.valueOf(action), Integer.valueOf(clickCount), m });
     return m[0];
   }
 
   synchronized void setStatusResized(int width, int height){
-    String sJmol = jmolScriptCallback(CBK.RESIZE);
-    if (notifyEnabled(CBK.RESIZE))
-      cbl.notifyCallback(CBK.RESIZE,
+    String sJmol = getJmolScriptCallback(CBK.RESIZE);
+    boolean isEnabled =  notifyEnabled(CBK.RESIZE);
+    if (isEnabled || sJmol != null)
+      fireJmolScriptCallback(isEnabled, CBK.RESIZE,
           new Object[] { sJmol, Integer.valueOf(width), Integer.valueOf(height) }); 
   }
 
   boolean haveHoverCallback() {
-    return (jmolScriptCallbacks.containsKey(CBK.HOVER) || notifyEnabled(CBK.HOVER));
+    return (jmolScriptCallbacks.containsKey(CBK.HOVER.name()) || notifyEnabled(CBK.HOVER));
   }
   
   synchronized void setStatusAtomHovered(int iatom, String strInfo) {
-    String sJmol = jmolScriptCallback(CBK.HOVER);
-    if (notifyEnabled(CBK.HOVER))
-      cbl.notifyCallback(CBK.HOVER, 
+    String sJmol = getJmolScriptCallback(CBK.HOVER);
+    boolean isEnabled =  notifyEnabled(CBK.HOVER);
+    if (isEnabled || sJmol != null)
+      fireJmolScriptCallback(isEnabled, CBK.HOVER, 
           new Object[] {sJmol, strInfo, Integer.valueOf(iatom) });
   }
   
   synchronized void setStatusObjectHovered(String id, String strInfo, T3 pt) {
-    String sJmol = jmolScriptCallback(CBK.HOVER);
-    if (notifyEnabled(CBK.HOVER))
-      cbl.notifyCallback(CBK.HOVER, 
+    String sJmol = getJmolScriptCallback(CBK.HOVER);
+    boolean isEnabled =  notifyEnabled(CBK.HOVER);
+    if (isEnabled || sJmol != null)
+      fireJmolScriptCallback(isEnabled, CBK.HOVER, 
           new Object[] {sJmol, strInfo, Integer.valueOf(-1), id, Float.valueOf(pt.x), Float.valueOf(pt.y), Float.valueOf(pt.z) });
   }
   
@@ -373,9 +453,10 @@ public class StatusManager {
   synchronized void showImage(String title, Object image) {
     String[] a = PT.split(title,  "\1");
     title = (a.length < 2 ? "Jmol" : a.length < 3 || a[2].equals("null") ? a[1].substring(a[1].lastIndexOf("/") + 1) : a[2]);
-    String sJmol = jmolScriptCallback(CBK.IMAGE);
-    if (notifyEnabled(CBK.IMAGE))
-      cbl.notifyCallback(CBK.IMAGE, new Object[] { sJmol, title, image });
+    String sJmol = getJmolScriptCallback(CBK.IMAGE);
+    boolean isEnabled =  notifyEnabled(CBK.IMAGE);
+    if (isEnabled || sJmol != null)
+      fireJmolScriptCallback(isEnabled, CBK.IMAGE, new Object[] { sJmol, title, image });
     if (Boolean.TRUE.equals(image)) {
       if (imageMap == null)
         return;
@@ -418,13 +499,13 @@ public class StatusManager {
     setStatusChanged("fileLoaded", ptLoad, fullPathName, false);
     if (errorMsg != null)
       setStatusChanged("fileLoadError", ptLoad, errorMsg, false);
-    String sJmol = jmolScriptCallback(CBK.LOADSTRUCT);
-    if (doCallback && notifyEnabled(CBK.LOADSTRUCT)) {
+    String sJmol = getJmolScriptCallback(CBK.LOADSTRUCT);
+    boolean isEnabled =  doCallback && notifyEnabled(CBK.LOADSTRUCT);
+    if (isEnabled || sJmol != null) {
       String name = (String) vwr.getP("_smilesString");
       if (name.length() != 0)
         fileName = name;
-      cbl
-          .notifyCallback(CBK.LOADSTRUCT,
+      fireJmolScriptCallback(isEnabled, CBK.LOADSTRUCT,
               new Object[] { sJmol, fullPathName, fileName, modelName,
                   errorMsg, Integer.valueOf(ptLoad),
                   vwr.getP("_modelNumber"),
@@ -436,9 +517,10 @@ public class StatusManager {
   synchronized void setStatusModelKit(int istate) {
     String state = (istate == 1 ? "ON" : "OFF");
     setStatusChanged("modelkit", istate, state, false);
-    String sJmol = jmolScriptCallback(CBK.MODELKIT);
-    if (notifyEnabled(CBK.MODELKIT))
-      cbl.notifyCallback(CBK.MODELKIT,
+    String sJmol = getJmolScriptCallback(CBK.MODELKIT);
+    boolean isEnabled = notifyEnabled(CBK.MODELKIT);
+    if (isEnabled || sJmol != null)
+      fireJmolScriptCallback(isEnabled, CBK.MODELKIT,
           new Object[] { sJmol, state });
   }
 
@@ -452,27 +534,28 @@ public class StatusManager {
     int frameNo = (animating ? -2 - currentFrame : currentFrame);
     setStatusChanged("frameChanged", frameNo,
         (currentFrame >= 0 ? vwr.getModelNumberDotted(currentFrame) : ""), false);
-    String sJmol = jmolScriptCallback(CBK.ANIMFRAME);
-    if (notifyEnabled(CBK.ANIMFRAME))
-      cbl.notifyCallback(CBK.ANIMFRAME,
+    String sJmol = getJmolScriptCallback(CBK.ANIMFRAME);
+    boolean isEnabled =  notifyEnabled(CBK.ANIMFRAME);
+    if (isEnabled || sJmol != null)
+      fireJmolScriptCallback(isEnabled, CBK.ANIMFRAME,
           new Object[] {
               sJmol,
               new int[] { frameNo, fileNo, modelNo, firstNo, lastNo,
                   currentFrame }, entryName, Float.valueOf(currentMorphModel) });
-    if (!animating)
+    if (!animating && !vwr.isJSNoAWT)
       vwr.checkMenuUpdate();
   }
 
   synchronized boolean setStatusDragDropped(int mode, int x, int y,
-                                            String fileName) {
+                                            String fileName, String[] retType) {
     setStatusChanged("dragDrop", 0, "", false);
-    String sJmol = jmolScriptCallback(CBK.DRAGDROP);
-    if (!notifyEnabled(CBK.DRAGDROP))
-      return false;
-    cbl.notifyCallback(CBK.DRAGDROP,
-        new Object[] { sJmol, Integer.valueOf(mode), Integer.valueOf(x),
-            Integer.valueOf(y), fileName });
-    return true;
+    String sJmol = getJmolScriptCallback(CBK.DRAGDROP);
+    boolean isEnabled = notifyEnabled(CBK.DRAGDROP);
+    if (isEnabled || sJmol != null)
+      fireJmolScriptCallback(isEnabled, CBK.DRAGDROP,
+          new Object[] { sJmol, Integer.valueOf(mode), Integer.valueOf(x),
+              Integer.valueOf(y), fileName, retType });
+    return isEnabled;
   }
 
   synchronized void setScriptEcho(String strEcho,
@@ -480,9 +563,10 @@ public class StatusManager {
     if (strEcho == null)
       return;
     setStatusChanged("scriptEcho", 0, strEcho, false);
-    String sJmol = jmolScriptCallback(CBK.ECHO);
-    if (notifyEnabled(CBK.ECHO))
-      cbl.notifyCallback(CBK.ECHO,
+    String sJmol = getJmolScriptCallback(CBK.ECHO);
+    boolean isEnabled =  notifyEnabled(CBK.ECHO);
+    if (isEnabled || sJmol != null)
+      fireJmolScriptCallback(isEnabled, CBK.ECHO,
           new Object[] { sJmol, strEcho, Integer.valueOf(isScriptQueued ? 1 : 0) });
   }
 
@@ -491,21 +575,23 @@ public class StatusManager {
     String sJmol = null;
     if(status.equals("measureCompleted")) { 
       Logger.info("measurement["+intInfo+"] = "+strMeasure);
-      sJmol = jmolScriptCallback(CBK.MEASURE);
+      sJmol = getJmolScriptCallback(CBK.MEASURE);
     } else if (status.equals("measurePicked")) {
         setStatusChanged("measurePicked", intInfo, strMeasure, false);
         Logger.info("measurePicked " + intInfo + " " + strMeasure);
     }
-    if (notifyEnabled(CBK.MEASURE))
-      cbl.notifyCallback(CBK.MEASURE, 
+    boolean isEnabled =  notifyEnabled(CBK.MEASURE);
+    if (isEnabled || sJmol != null)
+      fireJmolScriptCallback(isEnabled, CBK.MEASURE,
           new Object[] { sJmol, strMeasure,  Integer.valueOf(intInfo), status , Float.valueOf(value)});
   }
   
   synchronized void notifyError(String errType, String errMsg,
                                 String errMsgUntranslated) {
-    String sJmol = jmolScriptCallback(CBK.ERROR);
-    if (notifyEnabled(CBK.ERROR))
-      cbl.notifyCallback(CBK.ERROR,
+    String sJmol = getJmolScriptCallback(CBK.ERROR);
+    boolean isEnabled = notifyEnabled(CBK.ERROR);
+    if (isEnabled || sJmol != null)
+      fireJmolScriptCallback(isEnabled, CBK.ERROR,
           new Object[] { sJmol, errType, errMsg, vwr.getShapeErrorState(),
               errMsgUntranslated });
   }
@@ -513,9 +599,10 @@ public class StatusManager {
   
   synchronized void notifyMinimizationStatus(String minStatus, Integer minSteps, 
                                              Float minEnergy, Float minEnergyDiff, String ff) {
-    String sJmol = jmolScriptCallback(CBK.MINIMIZATION);
-    if (notifyEnabled(CBK.MINIMIZATION))
-      cbl.notifyCallback(CBK.MINIMIZATION,
+    String sJmol = getJmolScriptCallback(CBK.MINIMIZATION);
+    boolean isEnabled = notifyEnabled(CBK.MINIMIZATION);
+    if (isEnabled || sJmol != null)
+      fireJmolScriptCallback(isEnabled, CBK.MINIMIZATION,
           new Object[] { sJmol, minStatus, minSteps, minEnergy, minEnergyDiff, ff });
   }
   
@@ -530,7 +617,7 @@ public class StatusManager {
     } else if (strStatus == null) {
       return;
     }
-    String sJmol = (msWalltime == 0 ? jmolScriptCallback(CBK.SCRIPT)
+    String sJmol = (msWalltime == 0 ? getJmolScriptCallback(CBK.SCRIPT)
         : null);
     boolean isScriptCompletion = (strStatus == JC.SCRIPT_COMPLETED);
 
@@ -547,11 +634,13 @@ public class StatusManager {
     if (isScriptCompletion && vwr.getBoolean(T.messagestylechime)
         && vwr.getBoolean(T.debugscript))
       strStatus = vwr.getChimeMessenger().scriptCompleted(this, statusMessage, strErrorMessageUntranslated);
+
     Object[] data = new Object[] { sJmol, strStatus, statusMessage,
         Integer.valueOf(isScriptCompletion ? -1 : msWalltime),
         strErrorMessageUntranslated };
-    if (notifyEnabled(CBK.SCRIPT))
-      cbl.notifyCallback(CBK.SCRIPT, data);
+    boolean isEnabled = notifyEnabled(CBK.SCRIPT);
+    if (isEnabled || sJmol != null)
+      fireJmolScriptCallback(isEnabled, CBK.SCRIPT, data);
     processScript(data);
   }
 
@@ -650,13 +739,6 @@ public class StatusManager {
     return null;
   }
  
-  public void modifySend(int atomIndex, int modelIndex, int mode, String msg) {
-    String sJmol = jmolScriptCallback(CBK.STRUCTUREMODIFIED);
-    if (notifyEnabled(CBK.STRUCTUREMODIFIED))
-      cbl.notifyCallback(CBK.STRUCTUREMODIFIED,
-          new Object[] { sJmol, Integer.valueOf(mode), Integer.valueOf(atomIndex), Integer.valueOf(modelIndex), msg });
-  }
-  
   /**
    * service is expected to return a value in the "ret" key
    * 
@@ -879,10 +961,11 @@ public class StatusManager {
       vwr.script(script);
     if (status == "ended")
       registerAudio((String) htParams.get("id"), null);
-    String sJmol = jmolScriptCallback(CBK.AUDIO);
-    if (notifyEnabled(CBK.AUDIO))
-      cbl.notifyCallback(CBK.AUDIO,
-          new Object[] { sJmol, htParams });
+    String sJmol = getJmolScriptCallback(CBK.AUDIO);
+    boolean isEnabled = notifyEnabled(CBK.AUDIO);
+    if (isEnabled || sJmol != null)
+      fireJmolScriptCallback(isEnabled, CBK.AUDIO,
+          new Object[] { sJmol, htParams, status });
   }
   
   
@@ -1041,7 +1124,5 @@ public class StatusManager {
     vwr.showString("error reading SYNC command: " + script, false);
   }
 
-
-
 }
 
