@@ -600,8 +600,8 @@ public class TransformManager {
     return 0;
   }
 
-  int angstromsToPixels(float distance) {
-    return (int) Math.floor(scalePixelsPerAngstrom * distance);
+  float angstromsToPixels(float distance) {
+    return scalePixelsPerAngstrom * distance;
   }
 
   void translateXYBy(int xDelta, int yDelta) {
@@ -1562,6 +1562,24 @@ public class TransformManager {
         (Float.isNaN(scale) ? vibrationScale : scale), vwr.g.modulationScale);
   }
 
+  public void transformPt2Df(T3 v, P3 pt) {
+    if (v.z == -Float.MAX_VALUE || v.z == Float.MAX_VALUE) {
+      transformPt2D(v);
+      pt.set(iScrPt.x, iScrPt.y, iScrPt.z);
+    } else {
+      transformPt3f(v, pt);
+    }
+  }
+  
+  public void transformPtScrT32D(T3 v, P3 pt) {
+    if (v.z == -Float.MAX_VALUE || v.z == Float.MAX_VALUE) {
+      transformPt2D(v);
+      pt.set(iScrPt.x, iScrPt.y, iScrPt.z);
+    } else {
+      transformPtScrT3(v, pt);
+    }
+  }
+  
   public synchronized P3i transformPt2D(T3 ptXyp) {
     // axes position [50 50]
     // just does the processing for [x y] and [x y %]
@@ -2395,7 +2413,27 @@ public class TransformManager {
   BS bsSelectedAtoms;
   P3 ptOffset = new P3();
 
-  void setSelectedTranslation(BS bsAtoms, char xyz, int xy) {
+  void setSelectedTranslation(BS bsAtoms, char xyz, float xy, float x) {
+    if (!perspectiveDepth) {
+      V3 v = new V3();
+      switch (xyz) {
+      case 'X':
+      case 'x':
+        v.set(x, 0, 0);
+        break;
+      case 'Y':
+      case 'y':
+        v.set(0, x, 0);
+        break;
+      case 'Z':
+      case 'z':
+        v.set(0, 0, x);
+        break;
+      }
+      vwr.moveAtoms(null, null, matrixRotate, v, internalRotationCenter,
+          false, bsAtoms, true);
+      return;
+    }
     this.bsSelectedAtoms = bsAtoms;
     switch (xyz) {
     case 'X':
