@@ -81,6 +81,7 @@ import org.jmol.api.Interface;
 import org.jmol.api.JmolAbstractButton;
 import org.jmol.api.JmolAdapter;
 import org.jmol.api.JmolScriptManager;
+import org.jmol.api.JmolStatusListener;
 import org.jmol.awt.FileDropper;
 import org.jmol.awt.Platform;
 import org.jmol.console.JmolButton;
@@ -359,7 +360,9 @@ public class JmolPanel extends JPanel
     say(GT.$("Initializing 3D display..."));
     display = new DisplayPanel(this);
     vwrOptions.put("display", display);
+    JmolStatusListener userStatusListener = (JmolStatusListener) vwrOptions.get("statusListener");
     myStatusListener = new StatusListener(this, display);
+    myStatusListener.userStatusListener = userStatusListener;
     vwrOptions.put("statusListener", myStatusListener);
   }
 
@@ -475,8 +478,7 @@ public class JmolPanel extends JPanel
   protected void setupConsole() {
     say(GT.$("Initializing Script Window..."));
     vwr.getProperty("DATA_API", "getAppConsole", Boolean.TRUE);
-    AppConsole console = (AppConsole) vwr.getProperty("DATA_API",
-        "getAppConsole", null);
+    AppConsole console = getConsole();
     if (console != null) {
       if (console.jcd != null && historyFile != null) {
         historyFile.repositionWindow(SCRIPT_WINDOW_NAME, console.jcd, 200, 100,
@@ -675,8 +677,7 @@ public class JmolPanel extends JPanel
     if (frame != null) {
       historyFile.addWindowInfo(windowName, frame, null, display.dimSize);
     }
-    AppConsole console = (AppConsole) vwr.getProperty("DATA_API",
-        "getAppConsole", null);
+    AppConsole console = getConsole();
     if (console != null && console.jcd != null)
       historyFile.addWindowInfo(SCRIPT_WINDOW_NAME, console.jcd, null);
     Component c = (Component) vwr.getProperty("DATA_API", "getScriptEditor",
@@ -847,10 +848,51 @@ public class JmolPanel extends JPanel
 
   protected void addPluginMenu(JMenuBar mb) {
     pluginMenu = guimap.newJMenu("plugins");
+    mb.add(pluginMenu);
+    boolean isOK = false;
     try {
       PropertyResourceBundle bundle = new PropertyResourceBundle(
           getClass().getResourceAsStream(
               "/org/openscience/jmol/app/plugins/plugin.properties"));
+      isOK = bundle.getKeys().hasMoreElements();
+    } catch (IOException e1) {
+      // ignore
+    }
+    if (isOK) {
+      pluginMenu.addMenuListener(new MenuListener() {
+
+        @Override
+        public void menuSelected(MenuEvent e) {
+          setPlugins();
+        }
+
+        @Override
+        public void menuDeselected(MenuEvent e) {
+          // TODO
+
+        }
+
+        @Override
+        public void menuCanceled(MenuEvent e) {
+          // TODO
+
+        }
+
+      });
+    }
+    pluginMenu.setEnabled(isOK);
+  }
+
+  boolean pluginsSet = false;
+  
+  void setPlugins() {
+    if (pluginsSet)
+      return;
+    pluginsSet = true;
+    try {
+      PropertyResourceBundle bundle = new PropertyResourceBundle(getClass()
+          .getResourceAsStream(
+              "/org/openscience/jmol/app/plugins/plugin.properties"));
       Enumeration<String> keys = bundle.getKeys();
       while (keys.hasMoreElements()) {
         final String key = keys.nextElement();
@@ -889,7 +931,6 @@ public class JmolPanel extends JPanel
     } catch (IOException ex) {
       throw new RuntimeException(ex.toString());
     }
-    mb.add(pluginMenu);
     pluginMenu.setEnabled(pluginMenu.getPopupMenu().getComponentCount() > 0);
   }
 
@@ -1494,8 +1535,7 @@ public class JmolPanel extends JPanel
 
     @Override
     public void actionPerformed(ActionEvent e) {
-      AppConsole console = (AppConsole) vwr.getProperty("DATA_API",
-          "getAppConsole", null);
+      AppConsole console = getConsole();
       if (console != null) {
         console.setVisible(true);
       }
@@ -1915,8 +1955,7 @@ public class JmolPanel extends JPanel
   }
 
   public void updateConsoleFont() {
-    AppConsole console = (AppConsole) vwr.getProperty("DATA_API",
-        "getAppConsole", null);
+    AppConsole console = getConsole();
     if (console != null)
       console.updateFontSize();
   }
@@ -1926,6 +1965,11 @@ public class JmolPanel extends JPanel
         : preferencesDialog.currentProperties.get(key));
   }
 
+  public AppConsole getConsole() {
+    return (AppConsole) vwr.getProperty("DATA_API", "getAppConsole", null);
+  }
+	  
+
   public static String getJmolProperty(String key, String defaultValue) {
     return (historyFile == null ? defaultValue
         : historyFile.getProperty(key, defaultValue));
