@@ -47,6 +47,7 @@ import org.openscience.jmol.app.jmolpanel.console.AppConsole;
 import org.openscience.jmol.app.webexport.WebExport;
 
 import javajs.util.PT;
+import jspecview.application.JSpecView;
 import jspecview.application.MainFrame;
 
 public class StatusListener implements JmolStatusListener, JmolSyncInterface, JSVInterface {
@@ -78,6 +79,9 @@ public class StatusListener implements JmolStatusListener, JmolSyncInterface, JS
   private MainFrame jSpecViewFrame;
   private boolean jSpecViewForceNew;
   
+  private String lastSimulate;
+  JmolStatusListener userStatusListener;
+  
   
   public void setViewer(Viewer vwr) {
     this.vwr = vwr;
@@ -92,6 +96,8 @@ public class StatusListener implements JmolStatusListener, JmolSyncInterface, JS
   // / JmolCallbackListener interface ///
   @Override
   public boolean notifyEnabled(CBK type) {
+    if (userStatusListener != null && userStatusListener.notifyEnabled(type))
+      return true;
     switch (type) {
     case ANIMFRAME:
     case LOADSTRUCT:
@@ -111,6 +117,7 @@ public class StatusListener implements JmolStatusListener, JmolSyncInterface, JS
     case RESIZE:
     case CLICK:
     case ATOMMOVED:
+    case SELECT:
     case HOVER:
       return true;
     case APPLETREADY:
@@ -132,6 +139,8 @@ public class StatusListener implements JmolStatusListener, JmolSyncInterface, JS
       // during initialization
       return;
     }
+    if (userStatusListener != null && userStatusListener.notifyEnabled(type))
+      userStatusListener.notifyCallback(type, data);
     if (jmolPanel.isServer() && data != null && "SYNC".equals(data[0])) {
       data[0] = type.toString();
       jmolPanel.sendNioSyncRequest(data, JmolPanel.OUTSOCKET, null);
@@ -201,7 +210,11 @@ public class StatusListener implements JmolStatusListener, JmolSyncInterface, JS
         if (data[2] != null && display.haveDisplay) {
           jmolPanel.setStatus(1, (String) data[2]);
         }
-      }      
+      } else if (msWalltime == -1) {
+        AppConsole console = jmolPanel.getConsole();
+        if (console != null)
+          console.checkUndoEnabled();        
+      }
       break;
     case MODELKIT:
       String state = (String) data[1];
@@ -252,7 +265,7 @@ public class StatusListener implements JmolStatusListener, JmolSyncInterface, JS
       }
       if (strInfo != null && strInfo.toLowerCase().startsWith("nbo:")) {
         if (nboOptions == null)
-          nboOptions= new Hashtable<String, Object>();
+          nboOptions = new Hashtable<String, Object>();
         nboOptions.put("options", strInfo);
         jmolPanel.startNBO(nboOptions);
         return;
@@ -275,6 +288,7 @@ public class StatusListener implements JmolStatusListener, JmolSyncInterface, JS
     case ERROR:
     case ECHO:
     case MINIMIZATION:
+    case SELECT:
       break;
     }
     if (jmolPanel.isServer())
@@ -477,8 +491,6 @@ public class StatusListener implements JmolStatusListener, JmolSyncInterface, JS
     return jmolPanel.resizeInnerPanel(data);
   }
 
-  private String lastSimulate;
-  
   private void checkJSpecView(boolean closeAll) {
     if (jSpecViewFrame != null && modificationMode <= 0) {
       jSpecViewForceNew = jSpecViewFrame.isVisible();
@@ -508,7 +520,8 @@ public class StatusListener implements JmolStatusListener, JmolSyncInterface, JS
         return;
     }
     if (jSpecViewFrame == null) {
-      jSpecViewFrame = new MainFrame(vwr.getBoolean(T.jmolinjspecview) ? (Component) vwr.display : null, this);
+      JSpecView jsv = new JSpecView(true, this);
+      jsv.setMainFrame(jSpecViewFrame = new MainFrame(jsv, vwr.getBoolean(T.jmolinjspecview) ? (Component) vwr.display : null, this));
       jSpecViewFrame.setSize(Math.max(1000, jmolPanel.frame.getWidth() + 50), 600);
       jSpecViewFrame.setLocation(jmolPanel.frame.getLocation().x + 10, jmolPanel.frame
           .getLocation().y + 100);
